namespace NuGet.Tools
{
    static class PkgCmdIDList
    {
        public const int cmdidPowerConsole = 0x0100;
        public const int cmdidAddPackageDialog = 0x100;
        public const int cmdidAddPackageDialogForSolution = 0x200;
        public const int cmdidRestorePackages = 0x300;
        public const int cmdidSourceSettings = 0x0200;
        public const int cmdIdGeneralSettings = 0x0300;
        public const int cmdIdVisualizer = 0x0310;
    }
}
