using System.Reflection;
using System.Runtime.InteropServices;

[assembly: AssemblyTitle("VisualStudio2013")]
[assembly: AssemblyDescription("This assembly contains types that are meant to be loaded only in Visual Studio 2013.")]

[assembly: Guid("E2F5E00C-AABA-49AF-BA9A-977FB158CCBD")]
