using System;
using System.Reflection;
using System.Runtime.CompilerServices;

[assembly: AssemblyTitle("NuGet.Dialog.14")]
[assembly: AssemblyDescription("Manage NuGet Package dialog for Visual Studio 2014.")]
[assembly: CLSCompliant(false)]

[assembly: InternalsVisibleTo("NuGet.TestUI")]
[assembly: InternalsVisibleTo("NuGet.Dialog.Test")]