using System.Reflection;
using System.Runtime.CompilerServices;

[assembly: AssemblyTitle("NuGetVSEvents")]
[assembly: AssemblyDescription("NuGet Visual Studio Events Package")]
[assembly: InternalsVisibleTo("NuGet.VsEvents.Test")]