/* Replace user-splash-screen.png with your own splash screen image */
#msls-id-app-loading .msls-app-loading-img {
    background-image: url(Images/user-splash-screen.png);
}

/* Font families for screen and dialog headers.  Font families for content inside a screen 
    or dialog are determined by the theme's global font family setting. */
.msls-header .ui-title, .msls-screen-tab .ui-btn, .msls-screen-tab .ui-btn-text {
    font-family: 'Segoe Light', 'Segoe UI Light', 'Segoe UI', 'Segoe WP Light', 'Roboto Light','Frutiger','Helvetica Neue', Helvetica, Arial, sans-serif;
}

/* Customizable font styles for screen controls */
.msls-font-style-large {
}

.msls-font-style-normal {
}

.msls-font-style-small {
}

/* Uncomment the following lines if you want to use certain jQuery Mobile listview features such as 
    count bubbles and icons inside LightSwitch lists. jQuery Mobile relies on list items using 
    position:relative for these features.  However, this setting performs poorly on certain mobile 
    devices (webkit based in particular) when there are large numbers of items in the list. */
/*
.msls-ctl-list .ui-listview > .ui-li,
.msls-ctl-list .ui-field-contain.ui-li {
    position: relative;
}
*/

/* Phone media query (portrait and landscape) */
@media 
screen and (max-width: 400px) and (orientation: portrait), 
screen and (max-width: 640px) and (max-height: 400px) and (orientation: landscape) {
}
