//------------------------------------------------------------------------------
// <auto-generated>
//     This code was generated by a tool.
//     Runtime Version:4.0.30319.34014
//
//     Changes to this file may cause incorrect behavior and will be lost if
//     the code is regenerated.
// </auto-generated>
//------------------------------------------------------------------------------

namespace NuGet.Resources {
    using System;
    
    
    /// <summary>
    ///   A strongly-typed resource class, for looking up localized strings, etc.
    /// </summary>
    // This class was auto-generated by the StronglyTypedResourceBuilder
    // class via a tool like ResGen or Visual Studio.
    // To add or remove a member, edit your .ResX file then rerun ResGen
    // with the /str option, or rebuild your VS project.
    [global::System.CodeDom.Compiler.GeneratedCodeAttribute("System.Resources.Tools.StronglyTypedResourceBuilder", "4.0.0.0")]
    [global::System.Diagnostics.DebuggerNonUserCodeAttribute()]
    [global::System.Runtime.CompilerServices.CompilerGeneratedAttribute()]
    public class NuGetResources {
        
        private static global::System.Resources.ResourceManager resourceMan;
        
        private static global::System.Globalization.CultureInfo resourceCulture;
        
        [global::System.Diagnostics.CodeAnalysis.SuppressMessageAttribute("Microsoft.Performance", "CA1811:AvoidUncalledPrivateCode")]
        internal NuGetResources() {
        }
        
        /// <summary>
        ///   Returns the cached ResourceManager instance used by this class.
        /// </summary>
        [global::System.ComponentModel.EditorBrowsableAttribute(global::System.ComponentModel.EditorBrowsableState.Advanced)]
        public static global::System.Resources.ResourceManager ResourceManager {
            get {
                if (object.ReferenceEquals(resourceMan, null)) {
                    global::System.Resources.ResourceManager temp = new global::System.Resources.ResourceManager("NuGet.Resources.NuGetResources", typeof(NuGetResources).Assembly);
                    resourceMan = temp;
                }
                return resourceMan;
            }
        }
        
        /// <summary>
        ///   Overrides the current thread's CurrentUICulture property for all
        ///   resource lookups using this strongly typed resource class.
        /// </summary>
        [global::System.ComponentModel.EditorBrowsableAttribute(global::System.ComponentModel.EditorBrowsableState.Advanced)]
        public static global::System.Globalization.CultureInfo Culture {
            get {
                return resourceCulture;
            }
            set {
                resourceCulture = value;
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to &apos;{0}&apos; has an additional constraint {1} defined in {2}..
        /// </summary>
        public static string AdditonalConstraintsDefined {
            get {
                return ResourceManager.GetString("AdditonalConstraintsDefined", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Aggregate queries require at least one OrderBy..
        /// </summary>
        public static string AggregateQueriesRequireOrder {
            get {
                return ResourceManager.GetString("AggregateQueriesRequireOrder", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Cannot create a package that has no dependencies nor content..
        /// </summary>
        public static string CannotCreateEmptyPackage {
            get {
                return ResourceManager.GetString("CannotCreateEmptyPackage", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Circular dependency detected &apos;{0}&apos;..
        /// </summary>
        public static string CircularDependencyDetected {
            get {
                return ResourceManager.GetString("CircularDependencyDetected", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Updating &apos;{0}&apos; to &apos;{1}&apos; failed. Unable to find a version of &apos;{2}&apos; that is compatible with &apos;{1}&apos;..
        /// </summary>
        public static string ConflictErrorWithDependent {
            get {
                return ResourceManager.GetString("ConflictErrorWithDependent", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Updating &apos;{0}&apos; to &apos;{1}&apos; failed. Unable to find versions of &apos;{2}&apos; that are compatible with &apos;{1}&apos;..
        /// </summary>
        public static string ConflictErrorWithDependents {
            get {
                return ResourceManager.GetString("ConflictErrorWithDependents", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Added file &apos;{0}&apos;..
        /// </summary>
        public static string Debug_AddedFile {
            get {
                return ResourceManager.GetString("Debug_AddedFile", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Added file &apos;{0}&apos; to folder &apos;{1}&apos;..
        /// </summary>
        public static string Debug_AddedFileToFolder {
            get {
                return ResourceManager.GetString("Debug_AddedFileToFolder", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Looking for updates for &apos;{0}&apos;....
        /// </summary>
        public static string Debug_LookingForUpdates {
            get {
                return ResourceManager.GetString("Debug_LookingForUpdates", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Removed file &apos;{0}&apos;..
        /// </summary>
        public static string Debug_RemovedFile {
            get {
                return ResourceManager.GetString("Debug_RemovedFile", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Removed file &apos;{0}&apos; to folder &apos;{1}&apos;..
        /// </summary>
        public static string Debug_RemovedFileFromFolder {
            get {
                return ResourceManager.GetString("Debug_RemovedFileFromFolder", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Removed folder &apos;{0}&apos;..
        /// </summary>
        public static string Debug_RemovedFolder {
            get {
                return ResourceManager.GetString("Debug_RemovedFolder", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to &gt;&gt; {0} are being added from &apos;{1}&apos;{2}.
        /// </summary>
        public static string Debug_TargetFrameworkInfo {
            get {
                return ResourceManager.GetString("Debug_TargetFrameworkInfo", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Assembly references.
        /// </summary>
        public static string Debug_TargetFrameworkInfo_AssemblyReferences {
            get {
                return ResourceManager.GetString("Debug_TargetFrameworkInfo_AssemblyReferences", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Build files.
        /// </summary>
        public static string Debug_TargetFrameworkInfo_BuildFiles {
            get {
                return ResourceManager.GetString("Debug_TargetFrameworkInfo_BuildFiles", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Content files.
        /// </summary>
        public static string Debug_TargetFrameworkInfo_ContentFiles {
            get {
                return ResourceManager.GetString("Debug_TargetFrameworkInfo_ContentFiles", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to  (not framework-specific).
        /// </summary>
        public static string Debug_TargetFrameworkInfo_NotFrameworkSpecific {
            get {
                return ResourceManager.GetString("Debug_TargetFrameworkInfo_NotFrameworkSpecific", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to &gt;&gt; PowerShell scripts are being executed from &apos;{0}&apos;{1}.
        /// </summary>
        public static string Debug_TargetFrameworkInfo_PowershellScripts {
            get {
                return ResourceManager.GetString("Debug_TargetFrameworkInfo_PowershellScripts", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to For adding package &apos;{0}&apos; to project &apos;{1}&apos; that targets &apos;{2}&apos;,.
        /// </summary>
        public static string Debug_TargetFrameworkInfoPrefix {
            get {
                return ResourceManager.GetString("Debug_TargetFrameworkInfoPrefix", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Dependency &apos;{0}&apos; has an invalid version..
        /// </summary>
        public static string DependencyHasInvalidVersion {
            get {
                return ResourceManager.GetString("DependencyHasInvalidVersion", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Child dependencies of dependency only packages cannot mix external and project packages..
        /// </summary>
        public static string DependencyOnlyCannotMixDependencies {
            get {
                return ResourceManager.GetString("DependencyOnlyCannotMixDependencies", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Downloading {0} {1}....
        /// </summary>
        public static string DownloadProgressStatus {
            get {
                return ResourceManager.GetString("DownloadProgressStatus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to &apos;{0}&apos; already has a dependency defined for &apos;{1}&apos;..
        /// </summary>
        public static string DuplicateDependenciesDefined {
            get {
                return ResourceManager.GetString("DuplicateDependenciesDefined", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Unable to load version &apos;{0}&apos; of package &apos;{1}&apos;..
        /// </summary>
        public static string Error_InvalidPackage {
            get {
                return ResourceManager.GetString("Error_InvalidPackage", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to There are no writable config files..
        /// </summary>
        public static string Error_NoWritableConfig {
            get {
                return ResourceManager.GetString("Error_NoWritableConfig", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Package &apos;{0}&apos; already exists..
        /// </summary>
        public static string Error_PackageAlreadyExists {
            get {
                return ResourceManager.GetString("Error_PackageAlreadyExists", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Too many automatic redirections were attempted..
        /// </summary>
        public static string Error_TooManyRedirections {
            get {
                return ResourceManager.GetString("Error_TooManyRedirections", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to An error occurred while loading packages from &apos;{0}&apos;: {1}.
        /// </summary>
        public static string ErrorLoadingPackages {
            get {
                return ResourceManager.GetString("ErrorLoadingPackages", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Error reading &apos;{0}&apos;..
        /// </summary>
        public static string ErrorReadingFile {
            get {
                return ResourceManager.GetString("ErrorReadingFile", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Unable to read package from path &apos;{0}&apos;..
        /// </summary>
        public static string ErrorReadingPackage {
            get {
                return ResourceManager.GetString("ErrorReadingPackage", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to External packages cannot depend on packages that target projects..
        /// </summary>
        public static string ExternalPackagesCannotDependOnProjectLevelPackages {
            get {
                return ResourceManager.GetString("ExternalPackagesCannotDependOnProjectLevelPackages", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to File &apos;{0}&apos; already exists in project &apos;{1}&apos;. Do you want to overwrite it?.
        /// </summary>
        public static string FileConflictMessage {
            get {
                return ResourceManager.GetString("FileConflictMessage", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to File &apos;{0}&apos; does not exist..
        /// </summary>
        public static string FileDoesNotExit {
            get {
                return ResourceManager.GetString("FileDoesNotExit", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to The &apos;packages&apos; and &apos;versionConstraints&apos; parameters must have the same number of elements..
        /// </summary>
        public static string GetUpdatesParameterMismatch {
            get {
                return ResourceManager.GetString("GetUpdatesParameterMismatch", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to The schema version of &apos;{0}&apos; is incompatible with version {1} of NuGet. Please upgrade NuGet to the latest version from http://go.microsoft.com/fwlink/?LinkId=213942..
        /// </summary>
        public static string IncompatibleSchema {
            get {
                return ResourceManager.GetString("IncompatibleSchema", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Overwrite existing file &apos;{0}&apos;..
        /// </summary>
        public static string Info_OverwriteExistingFile {
            get {
                return ResourceManager.GetString("Info_OverwriteExistingFile", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Could not connect to the feed specified at &apos;{0}&apos;. Please verify that the package source (located in the Package Manager Settings) is valid and ensure your network connectivity..
        /// </summary>
        public static string InvalidFeed {
            get {
                return ResourceManager.GetString("InvalidFeed", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Invalid framework name format. Expected {framework}{version}-{profile}..
        /// </summary>
        public static string InvalidFrameworkNameFormat {
            get {
                return ResourceManager.GetString("InvalidFrameworkNameFormat", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to &quot;{0}&quot; cannot be called on a NullSettings. This may be caused on account of insufficient permissions to read or write to &quot;%AppData%\NuGet\NuGet.config&quot;..
        /// </summary>
        public static string InvalidNullSettingsOperation {
            get {
                return ResourceManager.GetString("InvalidNullSettingsOperation", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to The package ID &apos;{0}&apos; contains invalid characters. Examples of valid package IDs include &apos;MyPackage&apos; and &apos;MyPackage.Sample&apos;..
        /// </summary>
        public static string InvalidPackageId {
            get {
                return ResourceManager.GetString("InvalidPackageId", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to &apos;{0}&apos; is not a valid version string..
        /// </summary>
        public static string InvalidVersionString {
            get {
                return ResourceManager.GetString("InvalidVersionString", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Applying constraint &apos;{0} {1}&apos; defined in {2}..
        /// </summary>
        public static string Log_ApplyingConstraints {
            get {
                return ResourceManager.GetString("Log_ApplyingConstraints", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Attempting to resolve dependency &apos;{0}&apos;..
        /// </summary>
        public static string Log_AttemptingToRetrievePackageFromSource {
            get {
                return ResourceManager.GetString("Log_AttemptingToRetrievePackageFromSource", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Adding &apos;{0}&apos; to {1}..
        /// </summary>
        public static string Log_BeginAddPackageReference {
            get {
                return ResourceManager.GetString("Log_BeginAddPackageReference", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Installing &apos;{0}&apos;..
        /// </summary>
        public static string Log_BeginInstallPackage {
            get {
                return ResourceManager.GetString("Log_BeginInstallPackage", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Removing &apos;{0}&apos; from {1}..
        /// </summary>
        public static string Log_BeginRemovePackageReference {
            get {
                return ResourceManager.GetString("Log_BeginRemovePackageReference", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Uninstalling &apos;{0}&apos;..
        /// </summary>
        public static string Log_BeginUninstallPackage {
            get {
                return ResourceManager.GetString("Log_BeginUninstallPackage", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Install &apos;{0}&apos;.
        /// </summary>
        public static string Log_InstallPackage {
            get {
                return ResourceManager.GetString("Log_InstallPackage", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Add &apos;{0}&apos; to project {1}..
        /// </summary>
        public static string Log_InstallPackageIntoProject {
            get {
                return ResourceManager.GetString("Log_InstallPackageIntoProject", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to No updates available for &apos;{0}&apos;..
        /// </summary>
        public static string Log_NoUpdatesAvailable {
            get {
                return ResourceManager.GetString("Log_NoUpdatesAvailable", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to No updates available for &apos;{0}&apos; in project &apos;{1}&apos;..
        /// </summary>
        public static string Log_NoUpdatesAvailableForProject {
            get {
                return ResourceManager.GetString("Log_NoUpdatesAvailableForProject", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to &apos;{0}&apos; already installed..
        /// </summary>
        public static string Log_PackageAlreadyInstalled {
            get {
                return ResourceManager.GetString("Log_PackageAlreadyInstalled", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Successfully installed &apos;{0}&apos;..
        /// </summary>
        public static string Log_PackageInstalledSuccessfully {
            get {
                return ResourceManager.GetString("Log_PackageInstalledSuccessfully", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} already has a reference to &apos;{1}&apos;..
        /// </summary>
        public static string Log_ProjectAlreadyReferencesPackage {
            get {
                return ResourceManager.GetString("Log_ProjectAlreadyReferencesPackage", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Successfully added &apos;{0}&apos; to {1}..
        /// </summary>
        public static string Log_SuccessfullyAddedPackageReference {
            get {
                return ResourceManager.GetString("Log_SuccessfullyAddedPackageReference", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Successfully removed &apos;{0}&apos; from {1}..
        /// </summary>
        public static string Log_SuccessfullyRemovedPackageReference {
            get {
                return ResourceManager.GetString("Log_SuccessfullyRemovedPackageReference", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Successfully uninstalled &apos;{0}&apos;..
        /// </summary>
        public static string Log_SuccessfullyUninstalledPackage {
            get {
                return ResourceManager.GetString("Log_SuccessfullyUninstalledPackage", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Uninstall &apos;{0}&apos;..
        /// </summary>
        public static string Log_UninstallPackage {
            get {
                return ResourceManager.GetString("Log_UninstallPackage", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Remove &apos;{0}&apos; from project {1}..
        /// </summary>
        public static string Log_UninstallPackageFromProject {
            get {
                return ResourceManager.GetString("Log_UninstallPackageFromProject", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Updating &apos;{0}&apos; from version &apos;{1}&apos; to &apos;{2}&apos; in project &apos;{3}&apos;..
        /// </summary>
        public static string Log_UpdatingPackages {
            get {
                return ResourceManager.GetString("Log_UpdatingPackages", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Updating &apos;{0}&apos; to version &apos;{1}&apos; in project &apos;{2}&apos;..
        /// </summary>
        public static string Log_UpdatingPackagesWithoutOldVersion {
            get {
                return ResourceManager.GetString("Log_UpdatingPackagesWithoutOldVersion", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Assembly name is required..
        /// </summary>
        public static string Manifest_AssemblyNameRequired {
            get {
                return ResourceManager.GetString("Manifest_AssemblyNameRequired", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to &lt;dependencies&gt; element must not contain both &lt;group&gt; and &lt;dependency&gt; child elements..
        /// </summary>
        public static string Manifest_DependenciesHasMixedElements {
            get {
                return ResourceManager.GetString("Manifest_DependenciesHasMixedElements", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Dependency Id is required..
        /// </summary>
        public static string Manifest_DependencyIdRequired {
            get {
                return ResourceManager.GetString("Manifest_DependencyIdRequired", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Exclude path &apos;{0}&apos; contains invalid characters..
        /// </summary>
        public static string Manifest_ExcludeContainsInvalidCharacters {
            get {
                return ResourceManager.GetString("Manifest_ExcludeContainsInvalidCharacters", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Id must not exceed 100 characters..
        /// </summary>
        public static string Manifest_IdMaxLengthExceeded {
            get {
                return ResourceManager.GetString("Manifest_IdMaxLengthExceeded", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to The &apos;minClientVersion&apos; attribute in the package manifest has invalid value. It must be a valid version string..
        /// </summary>
        public static string Manifest_InvalidMinClientVersion {
            get {
                return ResourceManager.GetString("Manifest_InvalidMinClientVersion", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to A stable release of a package should not have on a prerelease dependency. Either modify the version spec of dependency &quot;{0}&quot; or update the version field..
        /// </summary>
        public static string Manifest_InvalidPrereleaseDependency {
            get {
                return ResourceManager.GetString("Manifest_InvalidPrereleaseDependency", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Invalid assembly reference &apos;{0}&apos;. Ensure that a file named &apos;{0}&apos; exists in the lib directory..
        /// </summary>
        public static string Manifest_InvalidReference {
            get {
                return ResourceManager.GetString("Manifest_InvalidReference", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Assembly reference &apos;{0}&apos; contains invalid characters..
        /// </summary>
        public static string Manifest_InvalidReferenceFile {
            get {
                return ResourceManager.GetString("Manifest_InvalidReferenceFile", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Invalid schema namespace &apos;{0}&apos;..
        /// </summary>
        public static string Manifest_InvalidSchemaNamespace {
            get {
                return ResourceManager.GetString("Manifest_InvalidSchemaNamespace", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to The manifest of this package has not been read. Call ReadManifest() before calling this method..
        /// </summary>
        public static string Manifest_NotAvailable {
            get {
                return ResourceManager.GetString("Manifest_NotAvailable", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Could not find manifest file at &apos;{0}&apos;..
        /// </summary>
        public static string Manifest_NotFound {
            get {
                return ResourceManager.GetString("Manifest_NotFound", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to &lt;references&gt; element must not contain both &lt;group&gt; and &lt;reference&gt; child elements..
        /// </summary>
        public static string Manifest_ReferencesHasMixedElements {
            get {
                return ResourceManager.GetString("Manifest_ReferencesHasMixedElements", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to The element package\metadata\references\group must contain at least one &lt;reference&gt; child element..
        /// </summary>
        public static string Manifest_ReferencesIsEmpty {
            get {
                return ResourceManager.GetString("Manifest_ReferencesIsEmpty", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to The required element &apos;{0}&apos; is missing from the manifest..
        /// </summary>
        public static string Manifest_RequiredElementMissing {
            get {
                return ResourceManager.GetString("Manifest_RequiredElementMissing", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} is required..
        /// </summary>
        public static string Manifest_RequiredMetadataMissing {
            get {
                return ResourceManager.GetString("Manifest_RequiredMetadataMissing", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Enabling license acceptance requires a license url..
        /// </summary>
        public static string Manifest_RequireLicenseAcceptanceRequiresLicenseUrl {
            get {
                return ResourceManager.GetString("Manifest_RequireLicenseAcceptanceRequiresLicenseUrl", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Source path &apos;{0}&apos; contains invalid characters..
        /// </summary>
        public static string Manifest_SourceContainsInvalidCharacters {
            get {
                return ResourceManager.GetString("Manifest_SourceContainsInvalidCharacters", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Target path &apos;{0}&apos; contains invalid characters..
        /// </summary>
        public static string Manifest_TargetContainsInvalidCharacters {
            get {
                return ResourceManager.GetString("Manifest_TargetContainsInvalidCharacters", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} cannot be empty..
        /// </summary>
        public static string Manifest_UriCannotBeEmpty {
            get {
                return ResourceManager.GetString("Manifest_UriCannotBeEmpty", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Framework name is missing..
        /// </summary>
        public static string MissingFrameworkName {
            get {
                return ResourceManager.GetString("MissingFrameworkName", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Already referencing a newer version of &apos;{0}&apos;..
        /// </summary>
        public static string NewerVersionAlreadyReferenced {
            get {
                return ResourceManager.GetString("NewerVersionAlreadyReferenced", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to File not found: &apos;{0}&apos;..
        /// </summary>
        public static string PackageAuthoring_FileNotFound {
            get {
                return ResourceManager.GetString("PackageAuthoring_FileNotFound", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Package does not contain a manifest..
        /// </summary>
        public static string PackageDoesNotContainManifest {
            get {
                return ResourceManager.GetString("PackageDoesNotContainManifest", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Unable to uninstall &apos;{0}&apos; because &apos;{1}&apos; depends on it..
        /// </summary>
        public static string PackageHasDependent {
            get {
                return ResourceManager.GetString("PackageHasDependent", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Unable to uninstall &apos;{0}&apos; because &apos;{1}&apos; depend on it..
        /// </summary>
        public static string PackageHasDependents {
            get {
                return ResourceManager.GetString("PackageHasDependents", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to The &apos;{0}&apos; package requires NuGet client version &apos;{1}&apos; or above, but the current NuGet version is &apos;{2}&apos;..
        /// </summary>
        public static string PackageMinVersionNotSatisfied {
            get {
                return ResourceManager.GetString("PackageMinVersionNotSatisfied", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to &amp;Allow NuGet to download missing packages.
        /// </summary>
        public static string PackageRestoreConsentCheckBoxText {
            get {
                return ResourceManager.GetString("PackageRestoreConsentCheckBoxText", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Failed to process request. &apos;{0}&apos;. 
        ///{1}..
        /// </summary>
        public static string PackageServerError {
            get {
                return ResourceManager.GetString("PackageServerError", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to The profile part of a portable target framework must not contain empty component..
        /// </summary>
        public static string PortableFrameworkProfileComponentIsEmpty {
            get {
                return ResourceManager.GetString("PortableFrameworkProfileComponentIsEmpty", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to The profile part of a portable target framework must not contain a portable framework component..
        /// </summary>
        public static string PortableFrameworkProfileComponentIsPortable {
            get {
                return ResourceManager.GetString("PortableFrameworkProfileComponentIsPortable", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Portable target framework must not have an empty profile part..
        /// </summary>
        public static string PortableFrameworkProfileEmpty {
            get {
                return ResourceManager.GetString("PortableFrameworkProfileEmpty", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to The profile part of a portable target framework must not contain the &apos;-&apos; character..
        /// </summary>
        public static string PortableFrameworkProfileHasDash {
            get {
                return ResourceManager.GetString("PortableFrameworkProfileHasDash", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to The profile part of a portable target framework must not contain empty space..
        /// </summary>
        public static string PortableFrameworkProfileHasSpace {
            get {
                return ResourceManager.GetString("PortableFrameworkProfileHasSpace", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} does not reference &apos;{1}&apos;..
        /// </summary>
        public static string ProjectDoesNotHaveReference {
            get {
                return ResourceManager.GetString("ProjectDoesNotHaveReference", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Unable to parse developmentDependency value &apos;{0}&apos; from &apos;{1}&apos;..
        /// </summary>
        public static string ReferenceFile_InvalidDevelopmentFlag {
            get {
                return ResourceManager.GetString("ReferenceFile_InvalidDevelopmentFlag", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Unable to parse requireReinstallation value &apos;{0}&apos; from &apos;{1}&apos;..
        /// </summary>
        public static string ReferenceFile_InvalidRequireReinstallationFlag {
            get {
                return ResourceManager.GetString("ReferenceFile_InvalidRequireReinstallationFlag", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Unable to parse version value &apos;{0}&apos; from &apos;{1}&apos;..
        /// </summary>
        public static string ReferenceFile_InvalidVersion {
            get {
                return ResourceManager.GetString("ReferenceFile_InvalidVersion", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to The special version part cannot exceed 20 characters..
        /// </summary>
        public static string SemVerSpecialVersionTooLong {
            get {
                return ResourceManager.GetString("SemVerSpecialVersionTooLong", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Using credentials from config. UserName: {0}.
        /// </summary>
        public static string SettingsCredentials_UsingSavedCredentials {
            get {
                return ResourceManager.GetString("SettingsCredentials_UsingSavedCredentials", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to One of the supported framework is null..
        /// </summary>
        public static string SupportedFrameworkIsNull {
            get {
                return ResourceManager.GetString("SupportedFrameworkIsNull", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to The replacement token &apos;{0}&apos; has no value..
        /// </summary>
        public static string TokenHasNoValue {
            get {
                return ResourceManager.GetString("TokenHasNoValue", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Type to compare must be an instance of SemanticVersion..
        /// </summary>
        public static string TypeMustBeASemanticVersion {
            get {
                return ResourceManager.GetString("TypeMustBeASemanticVersion", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Could not install package &apos;{0}&apos;. You are trying to install this package into a project that targets &apos;{1}&apos;, but the package does not contain any assembly references or content files that are compatible with that framework. For more information, contact the package author..
        /// </summary>
        public static string UnableToFindCompatibleItems {
            get {
                return ResourceManager.GetString("UnableToFindCompatibleItems", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Unable to locate dependency &apos;{0}&apos;. It may have been uninstalled..
        /// </summary>
        public static string UnableToLocateDependency {
            get {
                return ResourceManager.GetString("UnableToLocateDependency", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Connection to feed &apos;{0}&apos; requires the Windows Identity Foundation runtime to be installed. .
        /// </summary>
        public static string UnableToLocateWIF {
            get {
                return ResourceManager.GetString("UnableToLocateWIF", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Unable to resolve dependency &apos;{0}&apos;..
        /// </summary>
        public static string UnableToResolveDependency {
            get {
                return ResourceManager.GetString("UnableToResolveDependency", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Unable to resolve &apos;{0}&apos;..
        /// </summary>
        public static string UnableToResolveUri {
            get {
                return ResourceManager.GetString("UnableToResolveUri", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Unable to find package &apos;{0}&apos;..
        /// </summary>
        public static string UnknownPackage {
            get {
                return ResourceManager.GetString("UnknownPackage", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Unable to find version &apos;{1}&apos; of package &apos;{0}&apos;..
        /// </summary>
        public static string UnknownPackageSpecificVersion {
            get {
                return ResourceManager.GetString("UnknownPackageSpecificVersion", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Unknown schema version &apos;{0}&apos;..
        /// </summary>
        public static string UnknownSchemaVersion {
            get {
                return ResourceManager.GetString("UnknownSchemaVersion", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Hash algorithm &apos;{0}&apos; is unsupported. Supported algorithms include: SHA512 and SHA256..
        /// </summary>
        public static string UnsupportedHashAlgorithm {
            get {
                return ResourceManager.GetString("UnsupportedHashAlgorithm", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Unable to parse config file &apos;{0}&apos;..
        /// </summary>
        public static string UserSettings_UnableToParseConfigFile {
            get {
                return ResourceManager.GetString("UserSettings_UnableToParseConfigFile", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to &apos;{0}&apos; already exists. Skipping....
        /// </summary>
        public static string Warning_FileAlreadyExists {
            get {
                return ResourceManager.GetString("Warning_FileAlreadyExists", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Skipping &apos;{0}&apos; because it was modified..
        /// </summary>
        public static string Warning_FileModified {
            get {
                return ResourceManager.GetString("Warning_FileModified", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to &apos;{0}&apos; skipped, because it is in use by &apos;{1}&apos;..
        /// </summary>
        public static string Warning_PackageSkippedBecauseItIsInUse {
            get {
                return ResourceManager.GetString("Warning_PackageSkippedBecauseItIsInUse", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Uninstalling &apos;{0}&apos; might cause &apos;{1}&apos; to be broken..
        /// </summary>
        public static string Warning_UninstallingPackageWillBreakDependents {
            get {
                return ResourceManager.GetString("Warning_UninstallingPackageWillBreakDependents", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to An error occurred while applying transformation to &apos;{0}&apos; in project &apos;{1}&apos;:.
        /// </summary>
        public static string XdtError {
            get {
                return ResourceManager.GetString("XdtError", resourceCulture);
            }
        }
    }
}
