<ResourceDictionary  xmlns="http://schemas.microsoft.com/winfx/2006/xaml/presentation"
    xmlns:x="http://schemas.microsoft.com/winfx/2006/xaml"
    xmlns:self="clr-namespace:NuGet.Dialog.PackageManagerUI;assembly=NuGet.DialogServices">
    
    <ResourceDictionary.MergedDictionaries>
        <ResourceDictionary Source="CommonStyles.xaml" />
    </ResourceDictionary.MergedDictionaries>
    <Style x:Key="TileTemplateGridStyle" TargetType="{x:Type Grid}">
        <Setter Property="Margin" Value="0,0,-10,0" />
        <Setter Property="MinHeight" Value="62" />
    </Style>
    <Style x:Key="TileTitleTextBoxStyle" TargetType="{x:Type self:TextBlockControl}">
        <Setter Property="ToolTipService.ShowOnDisabled" Value="True" />
        <Setter Property="FontWeight" Value="Bold" />
        <Setter Property="TextTrimming" Value="CharacterEllipsis" />
    </Style>
    <Style x:Key="TileDescriptionTextBoxStyle" TargetType="{x:Type self:TextBlockControl}">
        <Setter Property="ToolTipService.ShowOnDisabled" Value="True" />
        <Setter Property="MaxHeight" Value="44" />
        <Setter Property="TextWrapping" Value="Wrap" />
        <Setter Property="TextTrimming" Value="CharacterEllipsis" />
    </Style>
    <Style x:Key="TileToolTipStyle" TargetType="{x:Type TextBlock}">
        <Setter Property="MaxWidth" Value="300" />
        <Setter Property="TextWrapping" Value="Wrap" />
    </Style>
    <Style x:Key="TileMiddleStackPanelStyle" TargetType="{x:Type StackPanel}">
        <Setter Property="Margin" Value="8,2,0,0" />
    </Style>
    <Style x:Key="TileRightStackPanelStyle" TargetType="{x:Type StackPanel}">
        <Setter Property="Margin" Value="6,2,0,0" />
        <Setter Property="VerticalAlignment" Value="Top" />
        <Setter Property="HorizontalAlignment" Value="Right" />
        <Setter Property="Visibility" Value="Collapsed" />
    </Style>
    <Style x:Key="DetailButtonStyle" TargetType="{x:Type Button}">
        <Setter Property="MinWidth" Value="75" />
        <Setter Property="MinHeight" Value="23" />
        <Setter Property="Margin" Value="6,3" />
        <Setter Property="ToolTipService.ShowOnDisabled" Value="True" />
    </Style>
    <Style TargetType="{x:Type self:TextBlockControl}">
        <Style.Triggers>
            <Trigger Property="UIElement.IsEnabled" Value="False">
                <Setter Property="TextElement.Foreground">
                    <Setter.Value>
                        <DynamicResource ResourceKey="{x:Static SystemColors.GrayTextBrushKey}" />
                    </Setter.Value>
                </Setter>
            </Trigger>
        </Style.Triggers>
    </Style>
    <Style x:Key="DetailBorderStyle" TargetType="{x:Type Border}">
        <Setter Property="BorderThickness" Value="1" />
        <Setter Property="BorderBrush" Value="DarkGray" />
        <Setter Property="CornerRadius" Value="2" />
        <Setter Property="Background" Value="#FFFAFAFA" />
        <Setter Property="Effect">
            <Setter.Value>
                <DropShadowEffect Color="Black" Direction="320" ShadowDepth="4" Opacity="0.5"/>
            </Setter.Value>
        </Setter>
    </Style>
    <Style x:Key="DetailDescriptionTextBoxStyle" TargetType="{x:Type self:TextBlockControl}">
        <Setter Property="Margin" Value="0,8,0,8" />
        <Setter Property="TextWrapping" Value="Wrap" />
        <Setter Property="TextTrimming" Value="CharacterEllipsis" />
    </Style>
    <Style x:Key="NoDependencyTextBoxStyle" TargetType="{x:Type self:TextBlockControl}" BasedOn="{StaticResource DetailDescriptionTextBoxStyle}">
        <Setter Property="FontStyle" Value="Italic" />
    </Style>
    <Style x:Key="DetailDependenciesTextBoxStyle" TargetType="{x:Type self:TextBlockControl}" BasedOn="{StaticResource DetailDescriptionTextBoxStyle}">
    </Style>
    <Style x:Key="DetailImageStyle" TargetType="{x:Type Image}">
        <Setter Property="Width" Value="200" />
        <Setter Property="Height" Value="200" />
    </Style>
    <Style x:Key="DetailMetadataStyle" TargetType="{x:Type StackPanel}">
        <Setter Property="Margin" Value="0,3,0,0" />
        <Setter Property="Orientation" Value="Horizontal" />
    </Style>
    <Style x:Key="DetailHyperlinkTextBlockStyle" TargetType="{x:Type self:TextBlockControl}">
        <Setter Property="Margin" Value="0,3,0,0" />
    </Style>
    <Style x:Key="DetailMetadataLabelStyle" TargetType="{x:Type self:TextBlockControl}">
        <Setter Property="FontWeight" Value="Bold" />
        <Setter Property="Margin" Value="0,0,0,0" />
    </Style>
    <Style x:Key="DetailMetadataValueStyle" TargetType="{x:Type self:TextBlockControl}">
        <Setter Property="Margin" Value="3,0,0,0" />
    </Style>

    <BitmapImage x:Key="BitmapImage_DefaultIcon" UriSource="packageicon.png" />
</ResourceDictionary>