<UserControl x:Class="NuGet.VisualStudio.RestartRequestBar"
             xmlns="http://schemas.microsoft.com/winfx/2006/xaml/presentation"
             xmlns:x="http://schemas.microsoft.com/winfx/2006/xaml"
             xmlns:resx="clr-namespace:NuGet.VisualStudio.Resources"
             xmlns:vsfx="clr-namespace:NuGet.VisualStudio">
    <Grid x:Name="RestartBar" Visibility="Collapsed">
        <Grid.ColumnDefinitions>
            <ColumnDefinition Width="*" />
            <ColumnDefinition Width="Auto"/>
        </Grid.ColumnDefinitions>
        <Border Grid.Column="0" HorizontalAlignment="Stretch" VerticalAlignment="Center" MinHeight="23" Background="{DynamicResource {x:Static vsfx:VsBrushes.InfoBackgroundKey}}">
            <TextBlock Name="RequestRestartMessage" HorizontalAlignment="Center" VerticalAlignment="Center" TextWrapping="WrapWithOverflow" Foreground="{DynamicResource {x:Static vsfx:VsBrushes.InfoTextKey}}" />
        </Border>
        <Button
            x:Name="RestartButton"
            AutomationProperties.AutomationId="RestartButton"
            Grid.Column="1"
            VerticalAlignment="Center"
            Margin="6,12,0,12"
            MinWidth="75" 
            MinHeight="23"
            Padding="4,0,4,0"
            Click="ExecutedRestart" 
            Content="{x:Static resx:VsResources.RestartNow}"/>
        </Grid>
</UserControl>
