using System;
using System.Reflection;
using System.Runtime.CompilerServices;

[assembly: AssemblyTitle("NuGet.Options")]
[assembly: AssemblyDescription("Options page for NuGet package manager")]
[assembly: CLSCompliant(false)]

[assembly: InternalsVisibleTo("NuGet.TestUI")]
