using System.Reflection;

[assembly: AssemblyTitle("NuGet.VisualStudio.Interop")]
[assembly: AssemblyDescription("NuGet/Visual Studio interop assembly")]

// The version of this assembly should never be changed.
[assembly: AssemblyVersion("1.0.0.0")]
