using System.Reflection;
using System.Runtime.InteropServices;

[assembly: AssemblyTitle("VisualStudio2010")]
[assembly: AssemblyDescription("This assembly contains types that are meant to be loaded only in Visual Studio 2010.")]

[assembly: Guid("34795751-1417-402d-9e96-82d16bb3a625")]