using System.Reflection;
using System.Runtime.CompilerServices;

[assembly: AssemblyTitle("NuGetExtension")]

[assembly: InternalsVisibleTo("NuGetExtension.Tests")]
[assembly: InternalsVisibleTo("DynamicProxyGenAssembly2")]
[assembly: AssemblyVersionAttribute("2.6.1.0")]
[assembly: AssemblyFileVersionAttribute("2.6.1.0")]
