using System.Reflection;
using System.Runtime.CompilerServices;

[assembly: AssemblyTitle("NuGet.Server")]
[assembly: AssemblyDescription("Web Application used to host a read-only NuGet feed")]

[assembly: InternalsVisibleTo("Server.Test")]