// PkgCmdID.cs
// MUST match PkgCmdID.h
namespace NuGetConsole.Implementation
{
    static class PkgCmdIDList
    {
        public const int idToolbar = 0x1010;
        public const int cmdidPowerConsole = 0x0100;
        public const int cmdidProjects = 0x0500;
        public const int cmdidProjectsList = 0x0510;
        public const int cmdidSources = 0x0400;
        public const int cmdidSourcesList = 0x0410;
        public const int cmdidClearHost = 0x0300;
        public const int cmdidSourceSettings = 0x0200;
        public const int cmdidStopHost = 0x0600;
    }
}
