namespace NuGet
{
    public static class NuGetConstants
    {
        public static readonly string DefaultFeedUrl = "https://www.nuget.org/api/v2/";
        public static readonly string V2LegacyOfficialPackageSourceUrl = "https://nuget.org/api/v2/";
        public static readonly string V2LegacyFeedUrl = "https://go.microsoft.com/fwlink/?LinkID=230477";
        public static readonly string V1FeedUrl = "https://go.microsoft.com/fwlink/?LinkID=206669";
        public static readonly string VSExpressForWindows8FeedUrl = "https://www.nuget.org/api/v2/curated-feeds/windows8-packages/";

        public static readonly string DefaultGalleryServerUrl = "https://www.nuget.org";

        public static readonly string DefaultSymbolServerUrl = "http://nuget.gw.symbolsource.org/Public/NuGet";
        public static readonly string ReadmeFileName = "readme.txt";

        public static readonly string NuGetSolutionSettingsFolder = ".nuget";
    }
}
