using System.Reflection;
using System.Runtime.CompilerServices;
using System.Runtime.InteropServices;

[assembly: AssemblyTitle("NuGet.Cmdlets")]
[assembly: AssemblyDescription("This assembly contains NuGet PowerShell cmdlets.")]

[assembly: InternalsVisibleTo("NuGet.Cmdlets.Test")]

// The following GUID is for the ID of the typelib if this project is exposed to COM
[assembly: Guid("F3670D5F-A177-4E84-93FD-A9ED25ED9AD6")]
