<pfui:VsDialogWindow 
    x:Class="NuGet.Dialog.PackageManagerUI.ProgressDialog"
    xmlns="http://schemas.microsoft.com/winfx/2006/xaml/presentation"
    xmlns:x="http://schemas.microsoft.com/winfx/2006/xaml"
    xmlns:pfui="clr-namespace:NuGet.VisualStudio;assembly=NuGet.VisualStudio"
    xmlns:vsfx="clr-namespace:NuGet.VisualStudio;assembly=NuGet.VisualStudio"
    xmlns:resources="clr-namespace:NuGet.Dialog"
    x:Name="Root"
    WindowStyle="SingleBorderWindow" 
    WindowStartupLocation="CenterOwner" 
    ShowInTaskbar="False" 
    ResizeMode="CanResize" 
    Width="450"
    Height="350" 
    MinHeight="230" 
    MinWidth="300"
    resources:WindowSizePersistenceHelper.WindowToken="ProgressWindow"
    Background="{DynamicResource {x:Static resources:UIResources.WindowBackgroundBrushKey}}"
    AutomationProperties.AutomationId="NuGetProgressWindow">
    
    <DockPanel>
        <TextBlock DockPanel.Dock="Top" Text="{Binding Title, ElementName=Root}" HorizontalAlignment="Center" TextTrimming="CharacterEllipsis" FontWeight="Bold" Margin="10" x:Name="StatusText" Foreground="{DynamicResource {x:Static vsfx:VsBrushes.WindowTextKey}}" />
        <ProgressBar DockPanel.Dock="Top" IsIndeterminate="True" Margin="20,0,20,5" Height="26" Width="Auto" x:Name="ProgressBar" />
        <DockPanel DockPanel.Dock="Bottom" Margin="20,10,21,10" LastChildFill="False">
            <Button x:Name="UpgradeButton" DockPanel.Dock="Left" Padding="10,2,10,2" Content="{x:Static resources:Resources.Dialog_UpgradeNuGet}" Visibility="Collapsed" Click="UpgradeButton_Click" />
            <Button x:Name="OkButton" DockPanel.Dock="Right" Padding="20,2,20,2" Content="{x:Static resources:Resources.Dialog_Close}" IsCancel="True" IsEnabled="False" Click="OkButton_Click" />
        </DockPanel>
        <RichTextBox Margin="20, 5, 20, 5" AcceptsReturn="True" IsReadOnly="True" ScrollViewer.VerticalScrollBarVisibility="Auto" ScrollViewer.HorizontalScrollBarVisibility="Auto" x:Name="MessagePane">
        </RichTextBox>
    </DockPanel>
</pfui:VsDialogWindow>