<framework:ModalDialogUserControl
    x:Class="NuGet.WebMatrix.FileConflictDialog"
    xmlns="http://schemas.microsoft.com/winfx/2006/xaml/presentation"
    xmlns:x="http://schemas.microsoft.com/winfx/2006/xaml"
    xmlns:mc="http://schemas.openxmlformats.org/markup-compatibility/2006"
    xmlns:d="http://schemas.microsoft.com/expression/blend/2008"
    xmlns:framework="clr-namespace:Microsoft.WebMatrix.Core;assembly=Microsoft.WebMatrix.Core"
    xmlns:self="clr-namespace:NuGet.WebMatrix"
    mc:Ignorable="d" 
    d:DesignHeight="100"
    Height="100"
    Width="500"
    StandardSize="SizeToContent">
    <UserControl.Resources>
        <Style TargetType="{x:Type Button}">
            <Setter Property="Margin" Value="6,6,6,2" />
            <Setter Property="Padding" Value="10,3,10,3" />
            <Setter Property="MinWidth" Value="75" />
        </Style>
    </UserControl.Resources>

    <DockPanel Margin="10">
        <StackPanel DockPanel.Dock="Bottom" Margin="0,0,0,3" Orientation="Horizontal" HorizontalAlignment="Right">
            <Button Content="{x:Static self:Resources.YesLabel}" Tag="Overwrite" AutomationProperties.AutomationId="YesButton" Click="OnButtonClick" />
            <Button Content="{x:Static self:Resources.YesAllLabel}" Tag="OverwriteAll" AutomationProperties.AutomationId="YesToAllButton" Click="OnButtonClick" />
            <Button x:Name="NoButton" Content="{x:Static self:Resources.NoLabel}" Tag="Ignore" AutomationProperties.AutomationId="NoButton" Click="OnButtonClick" IsDefault="True" />
            <Button Content="{x:Static self:Resources.NoAllLabel}" Tag="IgnoreAll" AutomationProperties.AutomationId="NoToAllButton" Click="OnButtonClick" />
        </StackPanel>

        <TextBlock 
            x:Name="MessageText" 
            x:FieldModifier="private" 
            DockPanel.Dock="Top" 
            Margin="0,3,0,3" 
            TextWrapping="Wrap" 
            VerticalAlignment="Top" />
    </DockPanel>
</framework:ModalDialogUserControl>
