using System.Reflection;
using System.Runtime.InteropServices;

[assembly: AssemblyTitle("PowerShellHostProvider")]
[assembly: AssemblyDescription("This assembly loads the PowerShellHost when PS runtime is detected.")]

// The following GUID is for the ID of the typelib if this project is exposed to COM
[assembly: Guid("B37BE991-002E-4D6D-9075-11572F0445FA")]
