namespace NuGet.Test.Integration.NuGetCommandLine
{

    public class NuSpecFileContext
    {

        public static string FileContents =
            @"<?xml version=""1.0"" encoding=""utf-8""?>
<package>
  <metadata>
    <id>Antlr</id>
    <version>3.1.1</version>
    <authors>Terence Parr</authors>
    <description>ANother Tool for Language Recognition, is a language tool that provides a framework for constructing recognizers, interpreters, compilers, and translators from grammatical descriptions containing actions in a variety of target languages.</description>
    <language>en-US</language>
  </metadata>
</package>";
    }
}
