namespace NuGet.WebMatrix.Data
{
    /// <summary>
    ///  A placeholder value in a VirtualizingList
    /// </summary>
    public class VirtualizingListPlaceholder
    {
        public override string ToString()
        {
            return Resources.String_LoadingFeaturedPackages;
        }
    }
}
