namespace NuGetConsole.Implementation.Console
{
    /// <summary>
    /// HostName MEF metadata viewer.
    /// </summary>
    public interface IHostNameMetadata
    {
        string HostName { get; }
    }
}
