<Window x:Class="$safeprojectname$.$safeitemname$"
        xmlns="http://schemas.microsoft.com/winfx/2006/xaml/presentation"
        xmlns:x="http://schemas.microsoft.com/winfx/2006/xaml"
        Title="$safeitemrootname$" Height="350" Width="525">
    <Grid>
        
    </Grid>
</Window>
