//------------------------------------------------------------------------------
// <auto-generated>
//     This code was generated by a tool.
//     Runtime Version:4.0.30319.18051
//
//     Changes to this file may cause incorrect behavior and will be lost if
//     the code is regenerated.
// </auto-generated>
//------------------------------------------------------------------------------

namespace NuGet.Dialog {
    using System;
    
    
    /// <summary>
    ///   A strongly-typed resource class, for looking up localized strings, etc.
    /// </summary>
    // This class was auto-generated by the StronglyTypedResourceBuilder
    // class via a tool like ResGen or Visual Studio.
    // To add or remove a member, edit your .ResX file then rerun ResGen
    // with the /str option, or rebuild your VS project.
    [global::System.CodeDom.Compiler.GeneratedCodeAttribute("System.Resources.Tools.StronglyTypedResourceBuilder", "4.0.0.0")]
    [global::System.Diagnostics.DebuggerNonUserCodeAttribute()]
    [global::System.Runtime.CompilerServices.CompilerGeneratedAttribute()]
    public class Resources {
        
        private static global::System.Resources.ResourceManager resourceMan;
        
        private static global::System.Globalization.CultureInfo resourceCulture;
        
        [global::System.Diagnostics.CodeAnalysis.SuppressMessageAttribute("Microsoft.Performance", "CA1811:AvoidUncalledPrivateCode")]
        internal Resources() {
        }
        
        /// <summary>
        ///   Returns the cached ResourceManager instance used by this class.
        /// </summary>
        [global::System.ComponentModel.EditorBrowsableAttribute(global::System.ComponentModel.EditorBrowsableState.Advanced)]
        public static global::System.Resources.ResourceManager ResourceManager {
            get {
                if (object.ReferenceEquals(resourceMan, null)) {
                    global::System.Resources.ResourceManager temp = new global::System.Resources.ResourceManager("NuGet.Dialog.Resources", typeof(Resources).Assembly);
                    resourceMan = temp;
                }
                return resourceMan;
            }
        }
        
        /// <summary>
        ///   Overrides the current thread's CurrentUICulture property for all
        ///   resource lookups using this strongly typed resource class.
        /// </summary>
        [global::System.ComponentModel.EditorBrowsableAttribute(global::System.ComponentModel.EditorBrowsableState.Advanced)]
        public static global::System.Globalization.CultureInfo Culture {
            get {
                return resourceCulture;
            }
            set {
                resourceCulture = value;
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Cancel.
        /// </summary>
        public static string Dialog_Cancel {
            get {
                return ResourceManager.GetString("Dialog_Cancel", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Close.
        /// </summary>
        public static string Dialog_Close {
            get {
                return ResourceManager.GetString("Dialog_Close", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Created by:.
        /// </summary>
        public static string Dialog_CreatedByLabel {
            get {
                return ResourceManager.GetString("Dialog_CreatedByLabel", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Installed version:.
        /// </summary>
        public static string Dialog_CurrentVersionLabel {
            get {
                return ResourceManager.GetString("Dialog_CurrentVersionLabel", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Dependencies:.
        /// </summary>
        public static string Dialog_DependenciesLabel {
            get {
                return ResourceManager.GetString("Dialog_DependenciesLabel", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Description:.
        /// </summary>
        public static string Dialog_DescriptionLabel {
            get {
                return ResourceManager.GetString("Dialog_DescriptionLabel", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Downloads:.
        /// </summary>
        public static string Dialog_DownloadsLabel {
            get {
                return ResourceManager.GetString("Dialog_DownloadsLabel", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Id:.
        /// </summary>
        public static string Dialog_IdLabel {
            get {
                return ResourceManager.GetString("Dialog_IdLabel", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Installing and uninstalling....
        /// </summary>
        public static string Dialog_InstallAndUninstallProgress {
            get {
                return ResourceManager.GetString("Dialog_InstallAndUninstallProgress", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to _Install.
        /// </summary>
        public static string Dialog_InstallButton {
            get {
                return ResourceManager.GetString("Dialog_InstallButton", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Installed packages.
        /// </summary>
        public static string Dialog_InstalledProvider {
            get {
                return ResourceManager.GetString("Dialog_InstalledProvider", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to No packages are installed in the current project..
        /// </summary>
        public static string Dialog_InstalledProviderNoItem {
            get {
                return ResourceManager.GetString("Dialog_InstalledProviderNoItem", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Install the package into the selected projects. Clear a selection to uninstall the package from that project..
        /// </summary>
        public static string Dialog_InstalledSolutionInstruction {
            get {
                return ResourceManager.GetString("Dialog_InstalledSolutionInstruction", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Installing....
        /// </summary>
        public static string Dialog_InstallProgress {
            get {
                return ResourceManager.GetString("Dialog_InstallProgress", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Last Published:.
        /// </summary>
        public static string Dialog_LastPublishedLabel {
            get {
                return ResourceManager.GetString("Dialog_LastPublishedLabel", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Operation failed.
        /// </summary>
        public static string Dialog_MessageBoxTitle {
            get {
                return ResourceManager.GetString("Dialog_MessageBoxTitle", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Project Information.
        /// </summary>
        public static string Dialog_MoreInfo {
            get {
                return ResourceManager.GetString("Dialog_MoreInfo", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to &apos;{0}&apos; could not be installed because it is not compatible with any project in the solution. The package targets the following frameworks: {1}.
        /// </summary>
        public static string Dialog_NoCompatibleProject {
            get {
                return ResourceManager.GetString("Dialog_NoCompatibleProject", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to &apos;{0}&apos; could not be installed because it is not compatible with any project in the solution..
        /// </summary>
        public static string Dialog_NoCompatibleProjectNoFrameworkNames {
            get {
                return ResourceManager.GetString("Dialog_NoCompatibleProjectNoFrameworkNames", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to No Dependencies.
        /// </summary>
        public static string Dialog_NoDependencyLabel {
            get {
                return ResourceManager.GetString("Dialog_NoDependencyLabel", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to OK.
        /// </summary>
        public static string Dialog_OK {
            get {
                return ResourceManager.GetString("Dialog_OK", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Online.
        /// </summary>
        public static string Dialog_OnlineProvider {
            get {
                return ResourceManager.GetString("Dialog_OnlineProvider", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to No packages found in the current package source..
        /// </summary>
        public static string Dialog_OnlineProviderNoItem {
            get {
                return ResourceManager.GetString("Dialog_OnlineProviderNoItem", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Install the package in the following projects:.
        /// </summary>
        public static string Dialog_OnlineSolutionInstruction {
            get {
                return ResourceManager.GetString("Dialog_OnlineSolutionInstruction", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Operation failed..
        /// </summary>
        public static string Dialog_OperationFailed {
            get {
                return ResourceManager.GetString("Dialog_OperationFailed", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Each item above may have sub-dependencies subject to additional license agreements..
        /// </summary>
        public static string Dialog_PackageDisclaimer {
            get {
                return ResourceManager.GetString("Dialog_PackageDisclaimer", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to This package is already installed..
        /// </summary>
        public static string Dialog_PackageInstalledTooltip {
            get {
                return ResourceManager.GetString("Dialog_PackageInstalledTooltip", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Prerelease.
        /// </summary>
        public static string Dialog_PackagePrereleaseLabel {
            get {
                return ResourceManager.GetString("Dialog_PackagePrereleaseLabel", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to This package is already updated..
        /// </summary>
        public static string Dialog_PackageUpdatedTooltip {
            get {
                return ResourceManager.GetString("Dialog_PackageUpdatedTooltip", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Currently installed in projects:.
        /// </summary>
        public static string Dialog_ProjectListLabel {
            get {
                return ResourceManager.GetString("Dialog_ProjectListLabel", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Select Projects.
        /// </summary>
        public static string Dialog_ProjectSelectorTitle {
            get {
                return ResourceManager.GetString("Dialog_ProjectSelectorTitle", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Release notes:.
        /// </summary>
        public static string Dialog_ReleaseNotesLabel {
            get {
                return ResourceManager.GetString("Dialog_ReleaseNotesLabel", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Package &apos;{0}&apos; depends on the following packages. Do you want to uninstall them too?.
        /// </summary>
        public static string Dialog_RemoveDependencyMessage {
            get {
                return ResourceManager.GetString("Dialog_RemoveDependencyMessage", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Report Abuse.
        /// </summary>
        public static string Dialog_ReportAbuse {
            get {
                return ResourceManager.GetString("Dialog_ReportAbuse", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to All.
        /// </summary>
        public static string Dialog_RootNodeAll {
            get {
                return ResourceManager.GetString("Dialog_RootNodeAll", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Search Results.
        /// </summary>
        public static string Dialog_RootNodeSearch {
            get {
                return ResourceManager.GetString("Dialog_RootNodeSearch", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to _Settings.
        /// </summary>
        public static string Dialog_Settings {
            get {
                return ResourceManager.GetString("Dialog_Settings", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to No packages are installed in the current solution..
        /// </summary>
        public static string Dialog_SolutionInstalledProviderNoItem {
            get {
                return ResourceManager.GetString("Dialog_SolutionInstalledProviderNoItem", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to _Manage.
        /// </summary>
        public static string Dialog_SolutionManageButton {
            get {
                return ResourceManager.GetString("Dialog_SolutionManageButton", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Solution &apos;{0}&apos;.
        /// </summary>
        public static string Dialog_SolutionNode {
            get {
                return ResourceManager.GetString("Dialog_SolutionNode", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Ascending.
        /// </summary>
        public static string Dialog_SortAscending {
            get {
                return ResourceManager.GetString("Dialog_SortAscending", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Descending.
        /// </summary>
        public static string Dialog_SortDescending {
            get {
                return ResourceManager.GetString("Dialog_SortDescending", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Most Downloads.
        /// </summary>
        public static string Dialog_SortOption_MostDownloads {
            get {
                return ResourceManager.GetString("Dialog_SortOption_MostDownloads", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Name.
        /// </summary>
        public static string Dialog_SortOption_Name {
            get {
                return ResourceManager.GetString("Dialog_SortOption_Name", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Published Date.
        /// </summary>
        public static string Dialog_SortOption_PublishedDate {
            get {
                return ResourceManager.GetString("Dialog_SortOption_PublishedDate", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Relevance.
        /// </summary>
        public static string Dialog_SortOption_Relevance {
            get {
                return ResourceManager.GetString("Dialog_SortOption_Relevance", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to NuGet failed to install or uninstall the selected package in the following project(s)..
        /// </summary>
        public static string Dialog_SummaryErrorMessage {
            get {
                return ResourceManager.GetString("Dialog_SummaryErrorMessage", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Summary.
        /// </summary>
        public static string Dialog_SummaryWindowTitle {
            get {
                return ResourceManager.GetString("Dialog_SummaryWindowTitle", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Tags:.
        /// </summary>
        public static string Dialog_TagLabel {
            get {
                return ResourceManager.GetString("Dialog_TagLabel", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} - Manage NuGet Packages.
        /// </summary>
        public static string Dialog_Title {
            get {
                return ResourceManager.GetString("Dialog_Title", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to _Uninstall.
        /// </summary>
        public static string Dialog_UninstallButton {
            get {
                return ResourceManager.GetString("Dialog_UninstallButton", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Uninstalling....
        /// </summary>
        public static string Dialog_UninstallProgress {
            get {
                return ResourceManager.GetString("Dialog_UninstallProgress", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Update _All.
        /// </summary>
        public static string Dialog_UpdateAllButton {
            get {
                return ResourceManager.GetString("Dialog_UpdateAllButton", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to There are multiple updates available..
        /// </summary>
        public static string Dialog_UpdateAllLabel {
            get {
                return ResourceManager.GetString("Dialog_UpdateAllLabel", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Update all packages....
        /// </summary>
        public static string Dialog_UpdateAllProgress {
            get {
                return ResourceManager.GetString("Dialog_UpdateAllProgress", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to _Update.
        /// </summary>
        public static string Dialog_UpdateButton {
            get {
                return ResourceManager.GetString("Dialog_UpdateButton", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Updating....
        /// </summary>
        public static string Dialog_UpdateProgress {
            get {
                return ResourceManager.GetString("Dialog_UpdateProgress", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Updates.
        /// </summary>
        public static string Dialog_UpdateProvider {
            get {
                return ResourceManager.GetString("Dialog_UpdateProvider", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to No package updates are available..
        /// </summary>
        public static string Dialog_UpdatesProviderNoItem {
            get {
                return ResourceManager.GetString("Dialog_UpdatesProviderNoItem", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Update the package in the following projects:.
        /// </summary>
        public static string Dialog_UpdatesSolutionInstruction {
            get {
                return ResourceManager.GetString("Dialog_UpdatesSolutionInstruction", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to _Upgrade NuGet.
        /// </summary>
        public static string Dialog_UpgradeNuGet {
            get {
                return ResourceManager.GetString("Dialog_UpgradeNuGet", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Version:.
        /// </summary>
        public static string Dialog_VersionLabel {
            get {
                return ResourceManager.GetString("Dialog_VersionLabel", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Error: {0}.
        /// </summary>
        public static string ErrorStringFormat {
            get {
                return ResourceManager.GetString("ErrorStringFormat", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Include Prerelease.
        /// </summary>
        public static string Filter_IncludePrerelease {
            get {
                return ResourceManager.GetString("Filter_IncludePrerelease", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Stable Only.
        /// </summary>
        public static string Filter_StablePackages {
            get {
                return ResourceManager.GetString("Filter_StablePackages", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to I _Accept.
        /// </summary>
        public static string License_Accept {
            get {
                return ResourceManager.GetString("License_Accept", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to (Author: {0}).
        /// </summary>
        public static string License_AuthorFormatString {
            get {
                return ResourceManager.GetString("License_AuthorFormatString", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to I _Decline.
        /// </summary>
        public static string License_Decline {
            get {
                return ResourceManager.GetString("License_Decline", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to License Acceptance.
        /// </summary>
        public static string License_DialogTitle {
            get {
                return ResourceManager.GetString("License_DialogTitle", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to License.
        /// </summary>
        public static string License_Header {
            get {
                return ResourceManager.GetString("License_Header", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to The following package(s) require a click-to-accept license:.
        /// </summary>
        public static string License_HeaderText {
            get {
                return ResourceManager.GetString("License_HeaderText", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to By clicking &quot;I Accept,&quot; you agree to the license terms for the package(s) listed above. If you do not agree to the license terms, click &quot;I Decline.&quot;.
        /// </summary>
        public static string License_LicenseText {
            get {
                return ResourceManager.GetString("License_LicenseText", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to View License.
        /// </summary>
        public static string License_ViewLicense {
            get {
                return ResourceManager.GetString("License_ViewLicense", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Component report.
        /// </summary>
        public static string License_ViewLicenseReport {
            get {
                return ResourceManager.GetString("License_ViewLicenseReport", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to No to A_ll.
        /// </summary>
        public static string NoAllLabel {
            get {
                return ResourceManager.GetString("NoAllLabel", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to (not framework specific).
        /// </summary>
        public static string NoFxDependencyGroup {
            get {
                return ResourceManager.GetString("NoFxDependencyGroup", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to _No.
        /// </summary>
        public static string NoLabel {
            get {
                return ResourceManager.GetString("NoLabel", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Unable to uninstall &apos;{0}&apos; because &apos;{1}&apos; depend(s) on it..
        /// </summary>
        public static string PackageHasDependents {
            get {
                return ResourceManager.GetString("PackageHasDependents", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Installed Package.
        /// </summary>
        public static string Visualizer_InstalledPackage {
            get {
                return ResourceManager.GetString("Visualizer_InstalledPackage", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Package.
        /// </summary>
        public static string Visualizer_Package {
            get {
                return ResourceManager.GetString("Visualizer_Package", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Package Dependency.
        /// </summary>
        public static string Visualizer_PackageDependency {
            get {
                return ResourceManager.GetString("Visualizer_PackageDependency", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Project.
        /// </summary>
        public static string Visualizer_Project {
            get {
                return ResourceManager.GetString("Visualizer_Project", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Yes to _All.
        /// </summary>
        public static string YesAllLabel {
            get {
                return ResourceManager.GetString("YesAllLabel", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to _Yes.
        /// </summary>
        public static string YesLabel {
            get {
                return ResourceManager.GetString("YesLabel", resourceCulture);
            }
        }
    }
}
