// Assembly Microsoft.VisualStudio.ExtensionsExplorer.UI, Version 12.0.0.0

[assembly: System.Reflection.AssemblyVersion("11.0.0.0")]

[assembly: System.Runtime.InteropServices.ComVisible(false)]
[assembly: System.CLSCompliant(false)]
[assembly: System.Reflection.AssemblyTitle("Microsoft.VisualStudio.ExtensionsExplorer.UI.dll")]
[assembly: System.Reflection.AssemblyDescription("Microsoft.VisualStudio.ExtensionsExplorer.UI.dll")]
[assembly: System.Reflection.AssemblyDefaultAlias("Microsoft.VisualStudio.ExtensionsExplorer.UI.dll")]
[assembly: System.Reflection.AssemblyCompany("Microsoft Corporation")]
[assembly: System.Reflection.AssemblyProduct("Microsoft\x00ae Visual Studio\x00ae Next")]
[assembly: System.Reflection.AssemblyCopyright("\x00a9 Microsoft Corporation. All rights reserved.")]
[assembly: System.Reflection.AssemblyFileVersion("11.0.20304.0")]
[assembly: System.Reflection.AssemblyInformationalVersion("11.0.20304.0")]
[assembly: System.Resources.SatelliteContractVersion("11.0.0.0")]
[assembly: System.Resources.NeutralResourcesLanguage("en-US")]
