using System.Reflection;
using System.Runtime.InteropServices;

[assembly: AssemblyTitle("VisualStudio2014")]
[assembly: AssemblyDescription("This assembly contains types that are meant to be loaded only in Visual Studio 2014.")]
