//------------------------------------------------------------------------------
// <auto-generated>
//     This code was generated by a tool.
//     Runtime Version:4.0.30319.18213
//
//     Changes to this file may cause incorrect behavior and will be lost if
//     the code is regenerated.
// </auto-generated>
//------------------------------------------------------------------------------

namespace NuGet.WebMatrix {
    using System;
    
    
    /// <summary>
    ///   A strongly-typed resource class, for looking up localized strings, etc.
    /// </summary>
    // This class was auto-generated by the StronglyTypedResourceBuilder
    // class via a tool like ResGen or Visual Studio.
    // To add or remove a member, edit your .ResX file then rerun ResGen
    // with the /str option, or rebuild your VS project.
    [global::System.CodeDom.Compiler.GeneratedCodeAttribute("System.Resources.Tools.StronglyTypedResourceBuilder", "4.0.0.0")]
    [global::System.Diagnostics.DebuggerNonUserCodeAttribute()]
    [global::System.Runtime.CompilerServices.CompilerGeneratedAttribute()]
    public class Resources {
        
        private static global::System.Resources.ResourceManager resourceMan;
        
        private static global::System.Globalization.CultureInfo resourceCulture;
        
        [global::System.Diagnostics.CodeAnalysis.SuppressMessageAttribute("Microsoft.Performance", "CA1811:AvoidUncalledPrivateCode")]
        internal Resources() {
        }
        
        /// <summary>
        ///   Returns the cached ResourceManager instance used by this class.
        /// </summary>
        [global::System.ComponentModel.EditorBrowsableAttribute(global::System.ComponentModel.EditorBrowsableState.Advanced)]
        public static global::System.Resources.ResourceManager ResourceManager {
            get {
                if (object.ReferenceEquals(resourceMan, null)) {
                    global::System.Resources.ResourceManager temp = new global::System.Resources.ResourceManager("NuGet.WebMatrix.Resources", typeof(Resources).Assembly);
                    resourceMan = temp;
                }
                return resourceMan;
            }
        }
        
        /// <summary>
        ///   Overrides the current thread's CurrentUICulture property for all
        ///   resource lookups using this strongly typed resource class.
        /// </summary>
        [global::System.ComponentModel.EditorBrowsableAttribute(global::System.ComponentModel.EditorBrowsableState.Advanced)]
        public static global::System.Globalization.CultureInfo Culture {
            get {
                return resourceCulture;
            }
            set {
                resourceCulture = value;
            }
        }
        
        /// <summary>
        ///   Looks up a localized resource of type System.Drawing.Bitmap.
        /// </summary>
        public static System.Drawing.Bitmap ClearSearch {
            get {
                object obj = ResourceManager.GetObject("ClearSearch", resourceCulture);
                return ((System.Drawing.Bitmap)(obj));
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Readme.txt.
        /// </summary>
        public static string ClickForReadme {
            get {
                return ResourceManager.GetString("ClickForReadme", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to NuGet Web Content Package Source.
        /// </summary>
        public static string ContentPackageSourceTitle {
            get {
                return ResourceManager.GetString("ContentPackageSourceTitle", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to WebMatrix Package Source.
        /// </summary>
        public static string CuratedFeedSourceTitle {
            get {
                return ResourceManager.GetString("CuratedFeedSourceTitle", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Downloads.
        /// </summary>
        public static string Downloads {
            get {
                return ResourceManager.GetString("Downloads", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to NuGet.
        /// </summary>
        public static string ExtensionName {
            get {
                return ResourceManager.GetString("ExtensionName", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to File Conflict Resolution.
        /// </summary>
        public static string FileConflictResolution {
            get {
                return ResourceManager.GetString("FileConflictResolution", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to All.
        /// </summary>
        public static string Filter_All {
            get {
                return ResourceManager.GetString("Filter_All", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to All {0}.
        /// </summary>
        public static string Filter_AllDescription {
            get {
                return ResourceManager.GetString("Filter_AllDescription", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Disabled.
        /// </summary>
        public static string Filter_Disabled {
            get {
                return ResourceManager.GetString("Filter_Disabled", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Disabled {0}.
        /// </summary>
        public static string Filter_DisabledDescription {
            get {
                return ResourceManager.GetString("Filter_DisabledDescription", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Installed.
        /// </summary>
        public static string Filter_Installed {
            get {
                return ResourceManager.GetString("Filter_Installed", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Installed {0}.
        /// </summary>
        public static string Filter_InstalledDescription {
            get {
                return ResourceManager.GetString("Filter_InstalledDescription", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Updates.
        /// </summary>
        public static string Filter_Updated {
            get {
                return ResourceManager.GetString("Filter_Updated", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Updates available for installed {0}.
        /// </summary>
        public static string Filter_UpdatedDescription {
            get {
                return ResourceManager.GetString("Filter_UpdatedDescription", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to NuGet.
        /// </summary>
        public static string GalleryCommandName {
            get {
                return ResourceManager.GetString("GalleryCommandName", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Open NuGet Gallery.
        /// </summary>
        public static string GalleryCommandTooltip {
            get {
                return ResourceManager.GetString("GalleryCommandTooltip", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized resource of type System.Drawing.Bitmap.
        /// </summary>
        public static System.Drawing.Bitmap MagGlass {
            get {
                object obj = ResourceManager.GetObject("MagGlass", resourceCulture);
                return ((System.Drawing.Bitmap)(obj));
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} ({1}).
        /// </summary>
        public static string MultiFilter_DisplayName {
            get {
                return ResourceManager.GetString("MultiFilter_DisplayName", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to No to A_ll.
        /// </summary>
        public static string NoAllLabel {
            get {
                return ResourceManager.GetString("NoAllLabel", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to _No.
        /// </summary>
        public static string NoLabel {
            get {
                return ResourceManager.GetString("NoLabel", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to The {0} &apos;{1}&apos; was successfully installed..
        /// </summary>
        public static string Notification_Installed {
            get {
                return ResourceManager.GetString("Notification_Installed", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to NuGet package.
        /// </summary>
        public static string Notification_NugetPackage {
            get {
                return ResourceManager.GetString("Notification_NugetPackage", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to The {0} &apos;{1}&apos; was successfully uninstalled..
        /// </summary>
        public static string Notification_Uninstalled {
            get {
                return ResourceManager.GetString("Notification_Uninstalled", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to The {0} &apos;{1}&apos; was successfully updated..
        /// </summary>
        public static string Notification_Updated {
            get {
                return ResourceManager.GetString("Notification_Updated", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to All the packages were updated.
        /// </summary>
        public static string Notification_UpdatedAll {
            get {
                return ResourceManager.GetString("Notification_UpdatedAll", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized resource of type System.Drawing.Bitmap.
        /// </summary>
        public static System.Drawing.Bitmap nuget_32 {
            get {
                object obj = ResourceManager.GetObject("nuget_32", resourceCulture);
                return ((System.Drawing.Bitmap)(obj));
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to NuGet Official Package Source.
        /// </summary>
        public static string NuGet_PackageSourceName {
            get {
                return ResourceManager.GetString("NuGet_PackageSourceName", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized resource of type System.Drawing.Bitmap.
        /// </summary>
        public static System.Drawing.Bitmap NugetIcon96 {
            get {
                object obj = ResourceManager.GetObject("NugetIcon96", resourceCulture);
                return ((System.Drawing.Bitmap)(obj));
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Package Id.
        /// </summary>
        public static string PackageId {
            get {
                return ResourceManager.GetString("PackageId", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Package Id: {0}.
        /// </summary>
        public static string PackageIdToolTipFormat {
            get {
                return ResourceManager.GetString("PackageIdToolTipFormat", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Configure Package Sources.
        /// </summary>
        public static string PackageSourcesManagerHeader {
            get {
                return ResourceManager.GetString("PackageSourcesManagerHeader", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Include Prerelease.
        /// </summary>
        public static string Prerelease_Filter_IncludePrerelease {
            get {
                return ResourceManager.GetString("Prerelease_Filter_IncludePrerelease", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Stable Only.
        /// </summary>
        public static string Prerelease_Filter_StableOnly {
            get {
                return ResourceManager.GetString("Prerelease_Filter_StableOnly", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to U.
        /// </summary>
        public static string RibbonButtonKeyTip {
            get {
                return ResourceManager.GetString("RibbonButtonKeyTip", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to NuGet.
        /// </summary>
        public static string RibbonGroupName {
            get {
                return ResourceManager.GetString("RibbonGroupName", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to _Close.
        /// </summary>
        public static string String__Close {
            get {
                return ResourceManager.GetString("String__Close", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to _Disable.
        /// </summary>
        public static string String__Disable {
            get {
                return ResourceManager.GetString("String__Disable", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to _Enable.
        /// </summary>
        public static string String__Enable {
            get {
                return ResourceManager.GetString("String__Enable", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to _Install.
        /// </summary>
        public static string String__Install {
            get {
                return ResourceManager.GetString("String__Install", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to _No.
        /// </summary>
        public static string String__No {
            get {
                return ResourceManager.GetString("String__No", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to _OK.
        /// </summary>
        public static string String__Ok {
            get {
                return ResourceManager.GetString("String__Ok", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to _Uninstall.
        /// </summary>
        public static string String__Uninstall {
            get {
                return ResourceManager.GetString("String__Uninstall", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to U_pdate.
        /// </summary>
        public static string String__Update {
            get {
                return ResourceManager.GetString("String__Update", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Up_date All.
        /// </summary>
        public static string String__Update_All {
            get {
                return ResourceManager.GetString("String__Update_All", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to _Yes.
        /// </summary>
        public static string String__Yes {
            get {
                return ResourceManager.GetString("String__Yes", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Accept EULA.
        /// </summary>
        public static string String_AcceptEula {
            get {
                return ResourceManager.GetString("String_AcceptEula", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Add.
        /// </summary>
        public static string String_Add {
            get {
                return ResourceManager.GetString("String_Add", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Add Source.
        /// </summary>
        public static string String_AddSource {
            get {
                return ResourceManager.GetString("String_AddSource", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Add source....
        /// </summary>
        public static string String_AddSourceEllipse {
            get {
                return ResourceManager.GetString("String_AddSourceEllipse", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Authors.
        /// </summary>
        public static string String_Authors {
            get {
                return ResourceManager.GetString("String_Authors", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Back.
        /// </summary>
        public static string String_Back {
            get {
                return ResourceManager.GetString("String_Back", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Clear Search Results.
        /// </summary>
        public static string String_ClearSearchResults {
            get {
                return ResourceManager.GetString("String_ClearSearchResults", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to The current version is.
        /// </summary>
        public static string String_CurrentVersion {
            get {
                return ResourceManager.GetString("String_CurrentVersion", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Delete.
        /// </summary>
        public static string String_Delete {
            get {
                return ResourceManager.GetString("String_Delete", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Dependencies which will be installed:.
        /// </summary>
        public static string String_DependenciesToBeInstall {
            get {
                return ResourceManager.GetString("String_DependenciesToBeInstall", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Enter Search String.
        /// </summary>
        public static string String_EnterSearchString {
            get {
                return ResourceManager.GetString("String_EnterSearchString", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Error.
        /// </summary>
        public static string String_Error {
            get {
                return ResourceManager.GetString("String_Error", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to .
        /// </summary>
        public static string String_ErrorGettingInstalledPackages {
            get {
                return ResourceManager.GetString("String_ErrorGettingInstalledPackages", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to An error occurred..
        /// </summary>
        public static string String_ErrorOccurred {
            get {
                return ResourceManager.GetString("String_ErrorOccurred", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Featured Packages.
        /// </summary>
        public static string String_FeaturedCategory {
            get {
                return ResourceManager.GetString("String_FeaturedCategory", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Each package is licensed to you by its owner. Microsoft is not responsible for, nor does it grant any licenses to, third-party packages. Some packages may include dependencies which are governed by additional licenses..
        /// </summary>
        public static string String_HeaderText {
            get {
                return ResourceManager.GetString("String_HeaderText", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to I _Accept.
        /// </summary>
        public static string String_I__Accept {
            get {
                return ResourceManager.GetString("String_I__Accept", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to I _Decline.
        /// </summary>
        public static string String_I__Decline {
            get {
                return ResourceManager.GetString("String_I__Decline", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Installed Packages.
        /// </summary>
        public static string String_InstalledCategory {
            get {
                return ResourceManager.GetString("String_InstalledCategory", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Installed Version.
        /// </summary>
        public static string String_InstalledVersion {
            get {
                return ResourceManager.GetString("String_InstalledVersion", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Installing....
        /// </summary>
        public static string String_Installing {
            get {
                return ResourceManager.GetString("String_Installing", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Would you like to install this package?.
        /// </summary>
        public static string String_InstallPrompt {
            get {
                return ResourceManager.GetString("String_InstallPrompt", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Last Updated.
        /// </summary>
        public static string String_LastUpdated {
            get {
                return ResourceManager.GetString("String_LastUpdated", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to By clicking &quot;I Accept,&quot; you agree to the license terms for the package(s) listed above. If you do not agree to the license terms, click &quot;I Decline.&quot;.
        /// </summary>
        public static string String_LicenseText {
            get {
                return ResourceManager.GetString("String_LicenseText", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Loading....
        /// </summary>
        public static string String_LoadingFeaturedPackages {
            get {
                return ResourceManager.GetString("String_LoadingFeaturedPackages", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Loading NuGet gallery....
        /// </summary>
        public static string String_LoadingNuGetGallery {
            get {
                return ResourceManager.GetString("String_LoadingNuGetGallery", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to More information:.
        /// </summary>
        public static string String_MoreInformation {
            get {
                return ResourceManager.GetString("String_MoreInformation", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Name.
        /// </summary>
        public static string String_Name {
            get {
                return ResourceManager.GetString("String_Name", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to .
        /// </summary>
        public static string String_NoInstalledPackages {
            get {
                return ResourceManager.GetString("String_NoInstalledPackages", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to None.
        /// </summary>
        public static string String_None {
            get {
                return ResourceManager.GetString("String_None", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Not Installed.
        /// </summary>
        public static string String_NotInstalled {
            get {
                return ResourceManager.GetString("String_NotInstalled", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to &lt;Bold&gt;NuGet&lt;/Bold&gt; Gallery.
        /// </summary>
        public static string String_NuGetGalleryHeading {
            get {
                return ResourceManager.GetString("String_NuGetGalleryHeading", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Retrieving package information....
        /// </summary>
        public static string String_PackageInformation {
            get {
                return ResourceManager.GetString("String_PackageInformation", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to The following packages will be updated.
        /// </summary>
        public static string String_PackagesForUpdateAll {
            get {
                return ResourceManager.GetString("String_PackagesForUpdateAll", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to people have downloaded this package.
        /// </summary>
        public static string String_PeopleHaveDownloaded {
            get {
                return ResourceManager.GetString("String_PeopleHaveDownloaded", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Please wait....
        /// </summary>
        public static string String_PleaseWait {
            get {
                return ResourceManager.GetString("String_PleaseWait", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Report abuse....
        /// </summary>
        public static string String_ReportAbuse {
            get {
                return ResourceManager.GetString("String_ReportAbuse", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Searching....
        /// </summary>
        public static string String_Searching {
            get {
                return ResourceManager.GetString("String_Searching", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Manage your &lt;Bold&gt; Package Sources&lt;/Bold&gt;.
        /// </summary>
        public static string String_SourceManagerb {
            get {
                return ResourceManager.GetString("String_SourceManagerb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to The current operating system does not meet the package system requirements..
        /// </summary>
        public static string String_SystemRequirementsNotMet {
            get {
                return ResourceManager.GetString("String_SystemRequirementsNotMet", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to .
        /// </summary>
        public static string String_UnableToLoadPackages {
            get {
                return ResourceManager.GetString("String_UnableToLoadPackages", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Uninstall.
        /// </summary>
        public static string String_Uninstall {
            get {
                return ResourceManager.GetString("String_Uninstall", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Uninstalling....
        /// </summary>
        public static string String_Uninstalling {
            get {
                return ResourceManager.GetString("String_Uninstalling", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Are you sure you want to uninstall the package? Doing so may break your site if it uses functionality provided by this package..
        /// </summary>
        public static string String_UninstallMessage {
            get {
                return ResourceManager.GetString("String_UninstallMessage", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Unknown.
        /// </summary>
        public static string String_Unknown {
            get {
                return ResourceManager.GetString("String_Unknown", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Would you like to update this package?.
        /// </summary>
        public static string String_UpdatePrompt {
            get {
                return ResourceManager.GetString("String_UpdatePrompt", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Updating....
        /// </summary>
        public static string String_Updating {
            get {
                return ResourceManager.GetString("String_Updating", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to URL.
        /// </summary>
        public static string String_URL {
            get {
                return ResourceManager.GetString("String_URL", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Version.
        /// </summary>
        public static string String_Version {
            get {
                return ResourceManager.GetString("String_Version", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to View license terms.
        /// </summary>
        public static string String_ViewLicense {
            get {
                return ResourceManager.GetString("String_ViewLicense", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to View package details online....
        /// </summary>
        public static string String_ViewPackageDetailsOnline {
            get {
                return ResourceManager.GetString("String_ViewPackageDetailsOnline", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to View project site....
        /// </summary>
        public static string String_ViewProjectSite {
            get {
                return ResourceManager.GetString("String_ViewProjectSite", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Your query returned zero results.
        /// </summary>
        public static string String_YourQueryReturnedZeroResults {
            get {
                return ResourceManager.GetString("String_YourQueryReturnedZeroResults", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to .
        /// </summary>
        public static string String1 {
            get {
                return ResourceManager.GetString("String1", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Windows 7.
        /// </summary>
        public static string Windows7 {
            get {
                return ResourceManager.GetString("Windows7", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Windows 8.
        /// </summary>
        public static string Windows8 {
            get {
                return ResourceManager.GetString("Windows8", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} or newer operating system..
        /// </summary>
        public static string WindowsNamedAnyAnyAnyNewer {
            get {
                return ResourceManager.GetString("WindowsNamedAnyAnyAnyNewer", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} operating system..
        /// </summary>
        public static string WindowsNamedAnyAnyAnySpecified {
            get {
                return ResourceManager.GetString("WindowsNamedAnyAnyAnySpecified", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} x64 or newer operating system..
        /// </summary>
        public static string WindowsNamedAnyAnyx64Newer {
            get {
                return ResourceManager.GetString("WindowsNamedAnyAnyx64Newer", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} x64 operating system..
        /// </summary>
        public static string WindowsNamedAnyAnyx64Specified {
            get {
                return ResourceManager.GetString("WindowsNamedAnyAnyx64Specified", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} x86 or newer operating system..
        /// </summary>
        public static string WindowsNamedAnyAnyx86Newer {
            get {
                return ResourceManager.GetString("WindowsNamedAnyAnyx86Newer", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} x86 operating system..
        /// </summary>
        public static string WindowsNamedAnyAnyx86Specified {
            get {
                return ResourceManager.GetString("WindowsNamedAnyAnyx86Specified", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} client or newer operating system..
        /// </summary>
        public static string WindowsNamedAnyClientAnyNewer {
            get {
                return ResourceManager.GetString("WindowsNamedAnyClientAnyNewer", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} client operating system..
        /// </summary>
        public static string WindowsNamedAnyClientAnySpecified {
            get {
                return ResourceManager.GetString("WindowsNamedAnyClientAnySpecified", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} x64 client or newer operating system..
        /// </summary>
        public static string WindowsNamedAnyClientx64Newer {
            get {
                return ResourceManager.GetString("WindowsNamedAnyClientx64Newer", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} x64 client operating system..
        /// </summary>
        public static string WindowsNamedAnyClientx64Specified {
            get {
                return ResourceManager.GetString("WindowsNamedAnyClientx64Specified", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} x86 client or newer operating system..
        /// </summary>
        public static string WindowsNamedAnyClientx86Newer {
            get {
                return ResourceManager.GetString("WindowsNamedAnyClientx86Newer", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} x86 client operating system..
        /// </summary>
        public static string WindowsNamedAnyClientx86Specified {
            get {
                return ResourceManager.GetString("WindowsNamedAnyClientx86Specified", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} server or newer operating system..
        /// </summary>
        public static string WindowsNamedAnyServerAnyNewer {
            get {
                return ResourceManager.GetString("WindowsNamedAnyServerAnyNewer", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} server operating system..
        /// </summary>
        public static string WindowsNamedAnyServerAnySpecified {
            get {
                return ResourceManager.GetString("WindowsNamedAnyServerAnySpecified", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} x64 server or newer operating system..
        /// </summary>
        public static string WindowsNamedAnyServerx64Newer {
            get {
                return ResourceManager.GetString("WindowsNamedAnyServerx64Newer", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} x64 server operating system..
        /// </summary>
        public static string WindowsNamedAnyServerx64Specified {
            get {
                return ResourceManager.GetString("WindowsNamedAnyServerx64Specified", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} x86 server or newer operating system..
        /// </summary>
        public static string WindowsNamedAnyServerx86Newer {
            get {
                return ResourceManager.GetString("WindowsNamedAnyServerx86Newer", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} x86 server operating system..
        /// </summary>
        public static string WindowsNamedAnyServerx86Specified {
            get {
                return ResourceManager.GetString("WindowsNamedAnyServerx86Specified", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} minimum Service Pack {2} or newer operating system..
        /// </summary>
        public static string WindowsNamedMinimumAnyAnyNewer {
            get {
                return ResourceManager.GetString("WindowsNamedMinimumAnyAnyNewer", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} minimum Service Pack {2} operating system..
        /// </summary>
        public static string WindowsNamedMinimumAnyAnySpecified {
            get {
                return ResourceManager.GetString("WindowsNamedMinimumAnyAnySpecified", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} minimum Service Pack {2} x64 or newer operating system..
        /// </summary>
        public static string WindowsNamedMinimumAnyx64Newer {
            get {
                return ResourceManager.GetString("WindowsNamedMinimumAnyx64Newer", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} minimum Service Pack {2} x64 operating system..
        /// </summary>
        public static string WindowsNamedMinimumAnyx64Specified {
            get {
                return ResourceManager.GetString("WindowsNamedMinimumAnyx64Specified", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} minimum Service Pack {2} x86 or newer operating system..
        /// </summary>
        public static string WindowsNamedMinimumAnyx86Newer {
            get {
                return ResourceManager.GetString("WindowsNamedMinimumAnyx86Newer", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} minimum Service Pack {2} x86 operating system..
        /// </summary>
        public static string WindowsNamedMinimumAnyx86Specified {
            get {
                return ResourceManager.GetString("WindowsNamedMinimumAnyx86Specified", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} minimum Service Pack {2} client or newer operating system..
        /// </summary>
        public static string WindowsNamedMinimumClientAnyNewer {
            get {
                return ResourceManager.GetString("WindowsNamedMinimumClientAnyNewer", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} minimum Service Pack {2} client operating system..
        /// </summary>
        public static string WindowsNamedMinimumClientAnySpecified {
            get {
                return ResourceManager.GetString("WindowsNamedMinimumClientAnySpecified", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} minimum Service Pack {2} x64 client or newer operating system..
        /// </summary>
        public static string WindowsNamedMinimumClientx64Newer {
            get {
                return ResourceManager.GetString("WindowsNamedMinimumClientx64Newer", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} minimum Service Pack {2} x64 client operating system..
        /// </summary>
        public static string WindowsNamedMinimumClientx64Specified {
            get {
                return ResourceManager.GetString("WindowsNamedMinimumClientx64Specified", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} minimum Service Pack {2} x86 client or newer operating system..
        /// </summary>
        public static string WindowsNamedMinimumClientx86Newer {
            get {
                return ResourceManager.GetString("WindowsNamedMinimumClientx86Newer", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} minimum Service Pack {2} x86 client operating system..
        /// </summary>
        public static string WindowsNamedMinimumClientx86Specified {
            get {
                return ResourceManager.GetString("WindowsNamedMinimumClientx86Specified", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} minimum Service Pack {2} server or newer operating system..
        /// </summary>
        public static string WindowsNamedMinimumServerAnyNewer {
            get {
                return ResourceManager.GetString("WindowsNamedMinimumServerAnyNewer", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} minimum Service Pack {2} server operating system..
        /// </summary>
        public static string WindowsNamedMinimumServerAnySpecified {
            get {
                return ResourceManager.GetString("WindowsNamedMinimumServerAnySpecified", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} minimum Service Pack {2} x64 server or newer operating system..
        /// </summary>
        public static string WindowsNamedMinimumServerx64Newer {
            get {
                return ResourceManager.GetString("WindowsNamedMinimumServerx64Newer", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} minimum Service Pack {2} x64 server operating system..
        /// </summary>
        public static string WindowsNamedMinimumServerx64Specified {
            get {
                return ResourceManager.GetString("WindowsNamedMinimumServerx64Specified", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} minimum Service Pack {2} x86 server or newer operating system..
        /// </summary>
        public static string WindowsNamedMinimumServerx86Newer {
            get {
                return ResourceManager.GetString("WindowsNamedMinimumServerx86Newer", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} minimum Service Pack {2} x86 server operating system..
        /// </summary>
        public static string WindowsNamedMinimumServerx86Specified {
            get {
                return ResourceManager.GetString("WindowsNamedMinimumServerx86Specified", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Windows version {1} or newer operating system..
        /// </summary>
        public static string WindowsNumericAnyAnyAnyNewer {
            get {
                return ResourceManager.GetString("WindowsNumericAnyAnyAnyNewer", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Windows version {1} operating system..
        /// </summary>
        public static string WindowsNumericAnyAnyAnySpecified {
            get {
                return ResourceManager.GetString("WindowsNumericAnyAnyAnySpecified", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Windows version {1} x64 or newer operating system..
        /// </summary>
        public static string WindowsNumericAnyAnyx64Newer {
            get {
                return ResourceManager.GetString("WindowsNumericAnyAnyx64Newer", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Windows version {1} x64 operating system..
        /// </summary>
        public static string WindowsNumericAnyAnyx64Specified {
            get {
                return ResourceManager.GetString("WindowsNumericAnyAnyx64Specified", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Windows version {1} x86 or newer operating system..
        /// </summary>
        public static string WindowsNumericAnyAnyx86Newer {
            get {
                return ResourceManager.GetString("WindowsNumericAnyAnyx86Newer", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Windows version {1} x86 operating system..
        /// </summary>
        public static string WindowsNumericAnyAnyx86Specified {
            get {
                return ResourceManager.GetString("WindowsNumericAnyAnyx86Specified", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Windows version {1} client or newer operating system..
        /// </summary>
        public static string WindowsNumericAnyClientAnyNewer {
            get {
                return ResourceManager.GetString("WindowsNumericAnyClientAnyNewer", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Windows version {1} client operating system..
        /// </summary>
        public static string WindowsNumericAnyClientAnySpecified {
            get {
                return ResourceManager.GetString("WindowsNumericAnyClientAnySpecified", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Windows version {1} x64 client or newer operating system..
        /// </summary>
        public static string WindowsNumericAnyClientx64Newer {
            get {
                return ResourceManager.GetString("WindowsNumericAnyClientx64Newer", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Windows version {1} x64 client operating system..
        /// </summary>
        public static string WindowsNumericAnyClientx64Specified {
            get {
                return ResourceManager.GetString("WindowsNumericAnyClientx64Specified", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Windows version {1} x86 client or newer operating system..
        /// </summary>
        public static string WindowsNumericAnyClientx86Newer {
            get {
                return ResourceManager.GetString("WindowsNumericAnyClientx86Newer", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Windows version {1} x86 client operating system..
        /// </summary>
        public static string WindowsNumericAnyClientx86Specified {
            get {
                return ResourceManager.GetString("WindowsNumericAnyClientx86Specified", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Windows version {1} server or newer operating system..
        /// </summary>
        public static string WindowsNumericAnyServerAnyNewer {
            get {
                return ResourceManager.GetString("WindowsNumericAnyServerAnyNewer", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Windows version {1} server operating system..
        /// </summary>
        public static string WindowsNumericAnyServerAnySpecified {
            get {
                return ResourceManager.GetString("WindowsNumericAnyServerAnySpecified", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Windows version {1} x64 server or newer operating system..
        /// </summary>
        public static string WindowsNumericAnyServerx64Newer {
            get {
                return ResourceManager.GetString("WindowsNumericAnyServerx64Newer", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Windows version {1} x64 server operating system..
        /// </summary>
        public static string WindowsNumericAnyServerx64Specified {
            get {
                return ResourceManager.GetString("WindowsNumericAnyServerx64Specified", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Windows version {1} x86 server or newer operating system..
        /// </summary>
        public static string WindowsNumericAnyServerx86Newer {
            get {
                return ResourceManager.GetString("WindowsNumericAnyServerx86Newer", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Windows version {1} x86 server operating system..
        /// </summary>
        public static string WindowsNumericAnyServerx86Specified {
            get {
                return ResourceManager.GetString("WindowsNumericAnyServerx86Specified", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Windows version {1} minimum Service Pack {2} or newer operating system..
        /// </summary>
        public static string WindowsNumericMinimumAnyAnyNewer {
            get {
                return ResourceManager.GetString("WindowsNumericMinimumAnyAnyNewer", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Windows version {1} minimum Service Pack {2} operating system..
        /// </summary>
        public static string WindowsNumericMinimumAnyAnySpecified {
            get {
                return ResourceManager.GetString("WindowsNumericMinimumAnyAnySpecified", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Windows version {1} minimum Service Pack {2} x64 or newer operating system..
        /// </summary>
        public static string WindowsNumericMinimumAnyx64Newer {
            get {
                return ResourceManager.GetString("WindowsNumericMinimumAnyx64Newer", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Windows version {1} minimum Service Pack {2} x64 operating system..
        /// </summary>
        public static string WindowsNumericMinimumAnyx64Specified {
            get {
                return ResourceManager.GetString("WindowsNumericMinimumAnyx64Specified", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Windows version {1} minimum Service Pack {2} x86 or newer operating system..
        /// </summary>
        public static string WindowsNumericMinimumAnyx86Newer {
            get {
                return ResourceManager.GetString("WindowsNumericMinimumAnyx86Newer", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Windows version {1} minimum Service Pack {2} x86 operating system..
        /// </summary>
        public static string WindowsNumericMinimumAnyx86Specified {
            get {
                return ResourceManager.GetString("WindowsNumericMinimumAnyx86Specified", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Windows version {1} minimum Service Pack {2} client or newer operating system..
        /// </summary>
        public static string WindowsNumericMinimumClientAnyNewer {
            get {
                return ResourceManager.GetString("WindowsNumericMinimumClientAnyNewer", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Windows version {1} minimum Service Pack {2} client operating system..
        /// </summary>
        public static string WindowsNumericMinimumClientAnySpecified {
            get {
                return ResourceManager.GetString("WindowsNumericMinimumClientAnySpecified", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Windows version {1} minimum Service Pack {2} x64 client or newer operating system..
        /// </summary>
        public static string WindowsNumericMinimumClientx64Newer {
            get {
                return ResourceManager.GetString("WindowsNumericMinimumClientx64Newer", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Windows version {1} minimum Service Pack {2} x64 client operating system..
        /// </summary>
        public static string WindowsNumericMinimumClientx64Specified {
            get {
                return ResourceManager.GetString("WindowsNumericMinimumClientx64Specified", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Windows version {1} minimum Service Pack {2} x86 client or newer operating system..
        /// </summary>
        public static string WindowsNumericMinimumClientx86Newer {
            get {
                return ResourceManager.GetString("WindowsNumericMinimumClientx86Newer", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Windows version {1} minimum Service Pack {2} x86 client operating system..
        /// </summary>
        public static string WindowsNumericMinimumClientx86Specified {
            get {
                return ResourceManager.GetString("WindowsNumericMinimumClientx86Specified", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Windows version {1} minimum Service Pack {2} server or newer operating system..
        /// </summary>
        public static string WindowsNumericMinimumServerAnyNewer {
            get {
                return ResourceManager.GetString("WindowsNumericMinimumServerAnyNewer", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Windows version {1} minimum Service Pack {2} server operating system..
        /// </summary>
        public static string WindowsNumericMinimumServerAnySpecified {
            get {
                return ResourceManager.GetString("WindowsNumericMinimumServerAnySpecified", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Windows version {1} minimum Service Pack {2} x64 server or newer operating system..
        /// </summary>
        public static string WindowsNumericMinimumServerx64Newer {
            get {
                return ResourceManager.GetString("WindowsNumericMinimumServerx64Newer", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Windows version {1} minimum Service Pack {2} x64 server operating system..
        /// </summary>
        public static string WindowsNumericMinimumServerx64Specified {
            get {
                return ResourceManager.GetString("WindowsNumericMinimumServerx64Specified", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Windows version {1} minimum Service Pack {2} x86 server or newer operating system..
        /// </summary>
        public static string WindowsNumericMinimumServerx86Newer {
            get {
                return ResourceManager.GetString("WindowsNumericMinimumServerx86Newer", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Windows version {1} minimum Service Pack {2} x86 server operating system..
        /// </summary>
        public static string WindowsNumericMinimumServerx86Specified {
            get {
                return ResourceManager.GetString("WindowsNumericMinimumServerx86Specified", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Windows Vista.
        /// </summary>
        public static string WindowsVista {
            get {
                return ResourceManager.GetString("WindowsVista", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Windows XP.
        /// </summary>
        public static string WindowsXP {
            get {
                return ResourceManager.GetString("WindowsXP", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Yes to _All.
        /// </summary>
        public static string YesAllLabel {
            get {
                return ResourceManager.GetString("YesAllLabel", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to _Yes.
        /// </summary>
        public static string YesLabel {
            get {
                return ResourceManager.GetString("YesLabel", resourceCulture);
            }
        }
    }
}
