using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
$if$ ($targetframeworkversion$ >= 3.5)using System.Linq;
$endif$using System.Text;
using System.Windows.Forms;

namespace $safeprojectname$
{
	public partial class Form1: Form
	{
		public Form1()
		{
			InitializeComponent();
		}    
	}
}
