using System;
using System.Reflection;
using System.Runtime.CompilerServices;
using System.Runtime.InteropServices;

[assembly: AssemblyTitle("Microsoft.VisualStudio.Project.V11Only.dll")]

[assembly: ComVisible(false)]
[assembly: CLSCompliant(true)]

[assembly: AssemblyVersion("11.0.0.0")]
[assembly: AssemblyFileVersion("11.0.50727.1")]
[assembly: AssemblyInformationalVersion("11.0.50727.1")]
