using System.Reflection;
using System.Runtime.CompilerServices;

[assembly: AssemblyTitle("NuGet")]
[assembly: AssemblyDescription("NuGet Command Line Tool")]

[assembly: InternalsVisibleTo("NuGet.Test")]
[assembly: InternalsVisibleTo("NuGet.Test.Integration")]