<pfui:VsDialogWindow
     x:Class="NuGet.Dialog.PackageManagerUI.FileConflictDialog"
     xmlns="http://schemas.microsoft.com/winfx/2006/xaml/presentation"
     xmlns:x="http://schemas.microsoft.com/winfx/2006/xaml"
     xmlns:mc="http://schemas.openxmlformats.org/markup-compatibility/2006" 
     xmlns:d="http://schemas.microsoft.com/expression/blend/2008" 
     mc:Ignorable="d" 
     xmlns:resources="clr-namespace:NuGet.Dialog"
     xmlns:vsresources="clr-namespace:NuGet.VisualStudio.Resources;assembly=NuGet.VisualStudio"
     xmlns:pfui="clr-namespace:NuGet.VisualStudio;assembly=NuGet.VisualStudio"
     xmlns:vsfx="clr-namespace:NuGet.VisualStudio;assembly=NuGet.VisualStudio"
     ShowInTaskbar="False"
     ResizeMode="NoResize"
     MinHeight="180"
     MinWidth="500"
     MaxWidth="900"
     WindowStyle="SingleBorderWindow"
     WindowStartupLocation="CenterOwner"
     SizeToContent="WidthAndHeight"
     FocusManager.FocusedElement="{Binding ElementName=NoButton, Mode=OneTime}"
     Title="{x:Static vsresources:VsResources.FileConflictTitle}"
     Background="{DynamicResource {x:Static vsfx:VsBrushes.WindowKey}}"
     AutomationProperties.AutomationId="NuGetFileConflictWindow">
    
    <Window.Resources>
        <Style TargetType="{x:Type Button}">
            <Setter Property="Margin" Value="6,6,6,2" />
            <Setter Property="Padding" Value="10,3,10,3" />
            <Setter Property="MinWidth" Value="75" />
        </Style>
    </Window.Resources>
    
    <DockPanel Margin="10">
        <StackPanel DockPanel.Dock="Bottom" Margin="0,0,0,3" Orientation="Horizontal" HorizontalAlignment="Right">
            <Button Content="{x:Static resources:Resources.YesLabel}" Tag="Overwrite" AutomationProperties.AutomationId="YesButton" Click="OnButtonClick" />
            <Button Content="{x:Static resources:Resources.YesAllLabel}" Tag="OverwriteAll" AutomationProperties.AutomationId="YesToAllButton" Click="OnButtonClick" />
            <Button x:Name="NoButton" Content="{x:Static resources:Resources.NoLabel}" Tag="Ignore" AutomationProperties.AutomationId="NoButton" Click="OnButtonClick" IsDefault="True" />
            <Button Content="{x:Static resources:Resources.NoAllLabel}" Tag="IgnoreAll" AutomationProperties.AutomationId="NoToAllButton" Click="OnButtonClick" />
        </StackPanel>

        <TextBlock 
            x:Name="QuestionText" 
            x:FieldModifier="private" 
            DockPanel.Dock="Top" 
            Margin="0,3,0,3" 
            TextWrapping="Wrap" 
            VerticalAlignment="Top"
            Foreground="{DynamicResource {x:Static vsfx:VsBrushes.WindowTextKey}}" />
    </DockPanel>
</pfui:VsDialogWindow>