//------------------------------------------------------------------------------
// <auto-generated>
//     This code was generated by a tool.
//     Runtime Version:4.0.30319.18408
//
//     Changes to this file may cause incorrect behavior and will be lost if
//     the code is regenerated.
// </auto-generated>
//------------------------------------------------------------------------------

namespace NuGet.VisualStudio.Resources {
    using System;
    
    
    /// <summary>
    ///   A strongly-typed resource class, for looking up localized strings, etc.
    /// </summary>
    // This class was auto-generated by the StronglyTypedResourceBuilder
    // class via a tool like ResGen or Visual Studio.
    // To add or remove a member, edit your .ResX file then rerun ResGen
    // with the /str option, or rebuild your VS project.
    [global::System.CodeDom.Compiler.GeneratedCodeAttribute("System.Resources.Tools.StronglyTypedResourceBuilder", "4.0.0.0")]
    [global::System.Diagnostics.DebuggerNonUserCodeAttribute()]
    [global::System.Runtime.CompilerServices.CompilerGeneratedAttribute()]
    public class VsResources {
        
        private static global::System.Resources.ResourceManager resourceMan;
        
        private static global::System.Globalization.CultureInfo resourceCulture;
        
        [global::System.Diagnostics.CodeAnalysis.SuppressMessageAttribute("Microsoft.Performance", "CA1811:AvoidUncalledPrivateCode")]
        internal VsResources() {
        }
        
        /// <summary>
        ///   Returns the cached ResourceManager instance used by this class.
        /// </summary>
        [global::System.ComponentModel.EditorBrowsableAttribute(global::System.ComponentModel.EditorBrowsableState.Advanced)]
        public static global::System.Resources.ResourceManager ResourceManager {
            get {
                if (object.ReferenceEquals(resourceMan, null)) {
                    global::System.Resources.ResourceManager temp = new global::System.Resources.ResourceManager("NuGet.VisualStudio.Resources.VsResources", typeof(VsResources).Assembly);
                    resourceMan = temp;
                }
                return resourceMan;
            }
        }
        
        /// <summary>
        ///   Overrides the current thread's CurrentUICulture property for all
        ///   resource lookups using this strongly typed resource class.
        /// </summary>
        [global::System.ComponentModel.EditorBrowsableAttribute(global::System.ComponentModel.EditorBrowsableState.Advanced)]
        public static global::System.Globalization.CultureInfo Culture {
            get {
                return resourceCulture;
            }
            set {
                resourceCulture = value;
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to All.
        /// </summary>
        public static string AggregateSourceName {
            get {
                return ResourceManager.GetString("AggregateSourceName", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Unable to find &apos;{0}&apos; in &apos;{1}&apos; and found multiple versions of &apos;{0}&apos; installed. Please specify a version..
        /// </summary>
        public static string AmbiguousProjectLevelUninstal {
            get {
                return ResourceManager.GetString("AmbiguousProjectLevelUninstal", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Found multiple versions of &apos;{0}&apos; installed. Please specify a version..
        /// </summary>
        public static string AmbiguousUninstall {
            get {
                return ResourceManager.GetString("AmbiguousUninstall", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Unable to update &apos;{0}&apos;. Found multiple versions installed..
        /// </summary>
        public static string AmbiguousUpdate {
            get {
                return ResourceManager.GetString("AmbiguousUpdate", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Some NuGet packages are missing from this solution. Click to restore from your online package sources..
        /// </summary>
        public static string AskForRestoreMessage {
            get {
                return ResourceManager.GetString("AskForRestoreMessage", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to A new version of NuGet Package Manager is available..
        /// </summary>
        public static string AskForUpdateMessage {
            get {
                return ResourceManager.GetString("AskForUpdateMessage", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Each package is licensed to you by its owner. NuGet is not responsible for, nor does it grant any licenses to, third-party packages. Some packages may include dependencies which are governed by additional licenses. Follow the package source (feed) URL to determine any dependencies..
        /// </summary>
        public static string Console_DisclaimerText {
            get {
                return ResourceManager.GetString("Console_DisclaimerText", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Type &apos;get-help NuGet&apos; to see all available NuGet commands..
        /// </summary>
        public static string Console_HelpText {
            get {
                return ResourceManager.GetString("Console_HelpText", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Failed to initialize the PowerShell host. If your PowerShell execution policy setting is set to AllSigned, open the Package Manager Console to initialize the host first..
        /// </summary>
        public static string Console_InitializeHostFails {
            get {
                return ResourceManager.GetString("Console_InitializeHostFails", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Added file &apos;{0}&apos; to project &apos;{1}&apos;.
        /// </summary>
        public static string Debug_AddedFileToProject {
            get {
                return ResourceManager.GetString("Debug_AddedFileToProject", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Added reference &apos;{0}&apos; to project &apos;{1}&apos;.
        /// </summary>
        public static string Debug_AddReference {
            get {
                return ResourceManager.GetString("Debug_AddReference", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Removed file &apos;{0}&apos;.
        /// </summary>
        public static string Debug_RemovedFile {
            get {
                return ResourceManager.GetString("Debug_RemovedFile", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Removed file &apos;{0}&apos; from folder &apos;{1}&apos;.
        /// </summary>
        public static string Debug_RemovedFileFromFolder {
            get {
                return ResourceManager.GetString("Debug_RemovedFileFromFolder", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Removed folder &apos;{0}&apos;.
        /// </summary>
        public static string Debug_RemovedFolder {
            get {
                return ResourceManager.GetString("Debug_RemovedFolder", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Removed reference &apos;{0}&apos; from project &apos;{1}&apos;.
        /// </summary>
        public static string Debug_RemoveReference {
            get {
                return ResourceManager.GetString("Debug_RemoveReference", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Each package is licensed to you by its owner. NuGet is not responsible for, nor does it grant any licenses to, third-party packages..
        /// </summary>
        public static string Dialog_DisclaimerText {
            get {
                return ResourceManager.GetString("Dialog_DisclaimerText", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to NuGet Package Manager.
        /// </summary>
        public static string DialogTitle {
            get {
                return ResourceManager.GetString("DialogTitle", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Don&apos;t show again.
        /// </summary>
        public static string DoNotRemindAgain {
            get {
                return ResourceManager.GetString("DoNotRemindAgain", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to The project is invalid.
        /// </summary>
        public static string DTE_InvalidProject {
            get {
                return ResourceManager.GetString("DTE_InvalidProject", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to The project &apos;{0}&apos; is unsupported.
        /// </summary>
        public static string DTE_ProjectUnsupported {
            get {
                return ResourceManager.GetString("DTE_ProjectUnsupported", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Failed to create parent folder for file &apos;{0}&apos; in project {1}..
        /// </summary>
        public static string Error_FailedToCreateParentFolder {
            get {
                return ResourceManager.GetString("Error_FailedToCreateParentFolder", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Error reading &apos;{0}&apos;..
        /// </summary>
        public static string ErrorReadingFile {
            get {
                return ResourceManager.GetString("ErrorReadingFile", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Executing script file &apos;{0}&apos;..
        /// </summary>
        public static string ExecutingScript {
            get {
                return ResourceManager.GetString("ExecutingScript", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Failed to add reference to &apos;{0}&apos;. Please make sure that it is in the Global Assembly Cache..
        /// </summary>
        public static string FailedToAddGacReference {
            get {
                return ResourceManager.GetString("FailedToAddGacReference", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Failed to add reference to &apos;{0}&apos;..
        /// </summary>
        public static string FailedToAddReference {
            get {
                return ResourceManager.GetString("FailedToAddReference", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Case insensitive search for reference &apos;{0}&apos;, to be removed, returned more than 1 match. Request could not be performed..
        /// </summary>
        public static string FailedToRemoveReference {
            get {
                return ResourceManager.GetString("FailedToRemoveReference", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to File Conflict.
        /// </summary>
        public static string FileConflictTitle {
            get {
                return ResourceManager.GetString("FileConflictTitle", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Ignore for now.
        /// </summary>
        public static string IgnoreUpgrade {
            get {
                return ResourceManager.GetString("IgnoreUpgrade", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to The provided version string is not a valid semantic version..
        /// </summary>
        public static string InvalidSemanticVersionString {
            get {
                return ResourceManager.GetString("InvalidSemanticVersionString", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to &apos;{0}&apos; already installed..
        /// </summary>
        public static string Log_PackageAlreadyInstalled {
            get {
                return ResourceManager.GetString("Log_PackageAlreadyInstalled", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Unable to retrieve package list because no source was specified..
        /// </summary>
        public static string NoActivePackageSource {
            get {
                return ResourceManager.GetString("NoActivePackageSource", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to No updates available for &apos;{0}&apos;..
        /// </summary>
        public static string NoUpdatesAvailable {
            get {
                return ResourceManager.GetString("NoUpdatesAvailable", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to NuGet official package source.
        /// </summary>
        public static string NuGetLegacyOfficialSourceName {
            get {
                return ResourceManager.GetString("NuGetLegacyOfficialSourceName", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to nuget.org.
        /// </summary>
        public static string NuGetOfficialSourceName {
            get {
                return ResourceManager.GetString("NuGetOfficialSourceName", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to nuget.
        /// </summary>
        public static string NuGetSearchProvider_CategoryShortcut {
            get {
                return ResourceManager.GetString("NuGetSearchProvider_CategoryShortcut", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Search NuGet packages.
        /// </summary>
        public static string NuGetSearchProvider_Description {
            get {
                return ResourceManager.GetString("NuGetSearchProvider_Description", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to NuGet Packages.
        /// </summary>
        public static string NuGetSearchProvider_DisplayText {
            get {
                return ResourceManager.GetString("NuGetSearchProvider_DisplayText", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to NuGet Search.
        /// </summary>
        public static string NuGetSearchProvider_Name {
            get {
                return ResourceManager.GetString("NuGetSearchProvider_Name", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Search Online for NuGet Packages matching &apos;{0}&apos;.
        /// </summary>
        public static string NuGetStaticResult_DisplayText {
            get {
                return ResourceManager.GetString("NuGetStaticResult_DisplayText", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Operation failed..
        /// </summary>
        public static string OperationFailed {
            get {
                return ResourceManager.GetString("OperationFailed", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Package Manager.
        /// </summary>
        public static string OutputConsolePaneName {
            get {
                return ResourceManager.GetString("OutputConsolePaneName", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to &apos;{0}&apos; cannot be removed because it in use by &apos;{1}&apos;.
        /// </summary>
        public static string PackageCannotBeRemovedBecauseItIsInUse {
            get {
                return ResourceManager.GetString("PackageCannotBeRemovedBecauseItIsInUse", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to &apos;{0}&apos; was not installed in any project. Update failed..
        /// </summary>
        public static string PackageNotInstalledInAnyProject {
            get {
                return ResourceManager.GetString("PackageNotInstalledInAnyProject", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Finished configuring this solution to restore NuGet packages on build..
        /// </summary>
        public static string PackageRestoreCompleted {
            get {
                return ResourceManager.GetString("PackageRestoreCompleted", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Do you want to configure this solution to download and restore missing NuGet packages during build? A .nuget folder will be added to the root of the solution that contains files that enable package restore.
        ///
        ///Packages installed into Website projects will not be restored during build. Consider converting those into Web application projects if necessary..
        /// </summary>
        public static string PackageRestoreConfirmation {
            get {
                return ResourceManager.GetString("PackageRestoreConfirmation", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Downloading package &apos;{0}&apos; failed..
        /// </summary>
        public static string PackageRestoreDownloadPackageFailed {
            get {
                return ResourceManager.GetString("PackageRestoreDownloadPackageFailed", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to An error occurred while configuring the solution to restore NuGet packages on build.
        /// </summary>
        public static string PackageRestoreErrorMessage {
            get {
                return ResourceManager.GetString("PackageRestoreErrorMessage", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to An error occurred while trying to restore packages:.
        /// </summary>
        public static string PackageRestoreErrorTryAgain {
            get {
                return ResourceManager.GetString("PackageRestoreErrorTryAgain", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Downloading missing packages....
        /// </summary>
        public static string PackageRestoreProgressMessage {
            get {
                return ResourceManager.GetString("PackageRestoreProgressMessage", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Skipped reinstalling package &apos;{0}&apos; in project &apos;{1}&apos; because the package does not exist in the package source..
        /// </summary>
        public static string PackageRestoreSkipForProject {
            get {
                return ResourceManager.GetString("PackageRestoreSkipForProject", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Skipped reinstalling package &apos;{0}&apos; because the package does not exist in the package source..
        /// </summary>
        public static string PackageRestoreSkipForSolution {
            get {
                return ResourceManager.GetString("PackageRestoreSkipForSolution", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Configuring the solution to restore NuGet packages on build....
        /// </summary>
        public static string PackageRestoreWaitMessage {
            get {
                return ResourceManager.GetString("PackageRestoreWaitMessage", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to The package source does not belong to the collection of available sources..
        /// </summary>
        public static string PackageSource_Invalid {
            get {
                return ResourceManager.GetString("PackageSource_Invalid", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to &apos;{0}&apos; updated successfully..
        /// </summary>
        public static string PackageUpdatedSuccessfully {
            get {
                return ResourceManager.GetString("PackageUpdatedSuccessfully", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Error accessing Extension Manager..
        /// </summary>
        public static string PreinstalledPackages_ExtensionManagerError {
            get {
                return ResourceManager.GetString("PreinstalledPackages_ExtensionManagerError", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Could not add all required packages to the project. The following packages failed to install from &apos;{0}&apos;:.
        /// </summary>
        public static string PreinstalledPackages_FailedToInstallPackage {
            get {
                return ResourceManager.GetString("PreinstalledPackages_FailedToInstallPackage", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to The preinstalled packages have a reference to a missing Extension. Could not find an Extension with ID &apos;{0}&apos;..
        /// </summary>
        public static string PreinstalledPackages_InvalidExtensionId {
            get {
                return ResourceManager.GetString("PreinstalledPackages_InvalidExtensionId", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to The preinstalled packages have a reference to a missing Registry value. Could not find a Registry key with name &apos;{0}&apos; under &apos;{1}&apos;..
        /// </summary>
        public static string PreinstalledPackages_InvalidRegistryValue {
            get {
                return ResourceManager.GetString("PreinstalledPackages_InvalidRegistryValue", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Adding {0}.{1} to project....
        /// </summary>
        public static string PreinstalledPackages_PackageInstallStatus {
            get {
                return ResourceManager.GetString("PreinstalledPackages_PackageInstallStatus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to The preinstalled packages are configured to use a Registry-provided package repository but there was an error accessing Registry key &apos;{0}&apos;..
        /// </summary>
        public static string PreinstalledPackages_RegistryKeyError {
            get {
                return ResourceManager.GetString("PreinstalledPackages_RegistryKeyError", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Attempting to install version &apos;{1}&apos; of &apos;{0}&apos; but the project already includes a different version. Skipping....
        /// </summary>
        public static string PreinstalledPackages_VersionConflict {
            get {
                return ResourceManager.GetString("PreinstalledPackages_VersionConflict", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to No project was specified..
        /// </summary>
        public static string ProjectNotSpecified {
            get {
                return ResourceManager.GetString("ProjectNotSpecified", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Reinstalling &apos;{0}&apos; in project &apos;{1}&apos;..
        /// </summary>
        public static string ReinstallProjectPackage {
            get {
                return ResourceManager.GetString("ReinstallProjectPackage", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Reinstalling &apos;{0}&apos;..
        /// </summary>
        public static string ReinstallSolutionPackage {
            get {
                return ResourceManager.GetString("ReinstallSolutionPackage", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to One or more packages could not be completely uninstalled: {0}. Restart Visual Studio to finish uninstall..
        /// </summary>
        public static string RequestRestartToCompleteUninstall {
            get {
                return ResourceManager.GetString("RequestRestartToCompleteUninstall", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Restart Now.
        /// </summary>
        public static string RestartNow {
            get {
                return ResourceManager.GetString("RestartNow", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to _Restore.
        /// </summary>
        public static string RestoreButtonLabel {
            get {
                return ResourceManager.GetString("RestoreButtonLabel", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Show Details.
        /// </summary>
        public static string ShowDetails {
            get {
                return ResourceManager.GetString("ShowDetails", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Unable to locate the solution directory. Please ensure that the solution has been saved..
        /// </summary>
        public static string SolutionDirectoryNotAvailable {
            get {
                return ResourceManager.GetString("SolutionDirectoryNotAvailable", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to The current environment does not have a solution loaded..
        /// </summary>
        public static string SolutionNotAvailable {
            get {
                return ResourceManager.GetString("SolutionNotAvailable", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Package Installation Error.
        /// </summary>
        public static string TemplateWizard_ErrorDialogTitle {
            get {
                return ResourceManager.GetString("TemplateWizard_ErrorDialogTitle", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to The project template lists one or more packages with missing, empty, or invalid values for the &quot;id&quot; or &quot;version&quot; attributes. Both attributes are required and must have valid values..
        /// </summary>
        public static string TemplateWizard_InvalidPackageElementAttributes {
            get {
                return ResourceManager.GetString("TemplateWizard_InvalidPackageElementAttributes", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to The &quot;repository&quot; attribute of the package element has an invalid value: &apos;{0}&apos;. Valid values are: &apos;template&apos; or &apos;extension&apos;..
        /// </summary>
        public static string TemplateWizard_InvalidRepositoryAttribute {
            get {
                return ResourceManager.GetString("TemplateWizard_InvalidRepositoryAttribute", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to This template wizard can only be applied to single-project or project-item templates..
        /// </summary>
        public static string TemplateWizard_InvalidWizardRunKind {
            get {
                return ResourceManager.GetString("TemplateWizard_InvalidWizardRunKind", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to The project template is configured to use an Extension-specific package repository but the Extension ID has not been specified. Use the &quot;repositoryId&quot; attribute to specify the Extension ID..
        /// </summary>
        public static string TemplateWizard_MissingExtensionId {
            get {
                return ResourceManager.GetString("TemplateWizard_MissingExtensionId", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to The project template is configured to use a Registry-provided package repository but the Registry value name has not been specified. Use the &quot;keyName&quot; attribute to specify the Registry value..
        /// </summary>
        public static string TemplateWizard_MissingRegistryKeyName {
            get {
                return ResourceManager.GetString("TemplateWizard_MissingRegistryKeyName", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Unable to find package &apos;{0}&apos; version &apos;{1}&apos; from the source &apos;{2}&apos;..
        /// </summary>
        public static string UnableToFindPackageFromSource {
            get {
                return ResourceManager.GetString("UnableToFindPackageFromSource", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Unable to find package &apos;{0}&apos;..
        /// </summary>
        public static string UnknownPackage {
            get {
                return ResourceManager.GetString("UnknownPackage", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Unable to find package &apos;{0}&apos; in &apos;{1}&apos;..
        /// </summary>
        public static string UnknownPackageInProject {
            get {
                return ResourceManager.GetString("UnknownPackageInProject", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Windows 8 Packages.
        /// </summary>
        public static string VisualStudioExpressForWindows8SourceName {
            get {
                return ResourceManager.GetString("VisualStudioExpressForWindows8SourceName", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Directory &apos;{0}&apos; is not empty. Skipping....
        /// </summary>
        public static string Warning_DirectoryNotEmpty {
            get {
                return ResourceManager.GetString("Warning_DirectoryNotEmpty", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Failed to generate binding redirects for &apos;{0}&apos;. {1}.
        /// </summary>
        public static string Warning_FailedToAddBindingRedirects {
            get {
                return ResourceManager.GetString("Warning_FailedToAddBindingRedirects", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Failed to remove package directories that were marked for deletion on Visual Studio restart: {0}.
        /// </summary>
        public static string Warning_FailedToDeleteMarkedPackageDirectories {
            get {
                return ResourceManager.GetString("Warning_FailedToDeleteMarkedPackageDirectories", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Failed to remove package directory &apos;{0}&apos; that was marked for deletion on Visual Studio restart..
        /// </summary>
        public static string Warning_FailedToDeleteMarkedPackageDirectory {
            get {
                return ResourceManager.GetString("Warning_FailedToDeleteMarkedPackageDirectory", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Failed to find match for removing reference &apos;{0}&apos;..
        /// </summary>
        public static string Warning_FailedToFindMatchForRemoveReference {
            get {
                return ResourceManager.GetString("Warning_FailedToFindMatchForRemoveReference", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Failed to mark package directory &apos;{0}&apos; for deletion on Visual Studio restart: {1}.
        /// </summary>
        public static string Warning_FailedToMarkPackageDirectoryForDeletion {
            get {
                return ResourceManager.GetString("Warning_FailedToMarkPackageDirectoryForDeletion", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to &apos;{0}&apos; already exists. Skipping....
        /// </summary>
        public static string Warning_FileAlreadyExists {
            get {
                return ResourceManager.GetString("Warning_FileAlreadyExists", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to No exact match found for removing reference &apos;{0}&apos;. Trying case-insensitive search....
        /// </summary>
        public static string Warning_NoExactMatchForReference {
            get {
                return ResourceManager.GetString("Warning_NoExactMatchForReference", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Package {0} of version {1} is not referenced by any project..
        /// </summary>
        public static string Warning_PackageNotReferencedByAnyProject {
            get {
                return ResourceManager.GetString("Warning_PackageNotReferencedByAnyProject", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Install failed. Rolling back....
        /// </summary>
        public static string Warning_RollingBack {
            get {
                return ResourceManager.GetString("Warning_RollingBack", resourceCulture);
            }
        }
    }
}
