
namespace NuGetConsole
{
    /// <summary>
    /// MEF interface to interact with the PowerConsole tool window.
    /// </summary>
    public interface IPowerConsoleWindow
    {
        /// <summary>
        /// Show the tool window
        /// </summary>
        void Show();
    }
}
