<UserControl x:Class="NuGet.VisualStudio.PackageRestoreBar"
             xmlns="http://schemas.microsoft.com/winfx/2006/xaml/presentation"
             xmlns:x="http://schemas.microsoft.com/winfx/2006/xaml"
             Loaded="UserControl_Loaded"
             xmlns:resx="clr-namespace:NuGet.VisualStudio.Resources">    
    <Border x:Name="RestoreBar" VerticalAlignment="Center" Visibility="Collapsed" BorderThickness="0,0,0,1">
        <Grid Margin="0,4,0,6">
            <Grid.ColumnDefinitions>
                <ColumnDefinition Width="*" />
                <ColumnDefinition Width="Auto" />
            </Grid.ColumnDefinitions>
            
            <TextBlock x:Name="StatusMessage" Grid.Column="0" Text="{x:Static resx:VsResources.AskForRestoreMessage}" TextWrapping="Wrap" TextOptions.TextFormattingMode="Display" VerticalAlignment="Center" Margin="5,0,5,0" />
            <Button x:Name="RestoreButton" VerticalAlignment="Center" Grid.Column="1" Content="{x:Static resx:VsResources.RestoreButtonLabel}" Click="OnRestoreLinkClick" Margin="5,0,3,0" Padding="8,2,8,2" />
            <ProgressBar x:Name="ProgressBar" Grid.Column="1" IsIndeterminate="True" Height="{Binding ActualHeight, ElementName=RestoreButton, Mode=OneWay}" Width="170" Margin="5,0,3,0" />
        </Grid>
    </Border>
</UserControl>