using System;

namespace NuGetConsole.Implementation
{
    public static class GuidList
    {
        // IMPORTANT: this GUID has to match the one declared in NuGet.Tools\Guid.cs
        internal const string guidNuGetConsoleCmdSetString = "1E8A55F6-C18D-407F-91C8-94B02AE1CED6";
        internal static readonly Guid guidNuGetCmdSet = new Guid(guidNuGetConsoleCmdSetString);

        // GUID for the Package Manager Console category in the Font and Colors options page
        public const string GuidPackageManagerConsoleFontAndColorCategoryString = "{F9D6BCE6-C669-41DB-8EE7-DD953828685B}";
        internal static readonly Guid guidPackageManagerConsoleFontAndColorCategory = new Guid(GuidPackageManagerConsoleFontAndColorCategoryString);
    }
}
