using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace GenerateTestPackages {
    partial class AssemblySourceFileGenerator {
        internal PackageInfo Package { get; set; }
    }
}
