namespace NuGet.Common
{
    internal static class CommandLineConstants
    {
        internal static readonly string UserAgent = "NuGet Command Line";

        internal static string NuGetDocs = "http://docs.nuget.org/";

        internal static string NuGetDocsCommandLineReference = "http://docs.nuget.org/docs/reference/command-line-reference";

        internal static string PackagesDirectoryName = "packages";
    }
}
