<ResourceDictionary
    xmlns="http://schemas.microsoft.com/winfx/2006/xaml/presentation"
    xmlns:x="http://schemas.microsoft.com/winfx/2006/xaml"
    xmlns:vsshell="clr-namespace:NuGet.VisualStudio;assembly=NuGet.VisualStudio">

    <Style TargetType="{x:Type Hyperlink}" x:Key="CommonHyperlinkStyle">
        <Setter Property="Foreground" Value="{DynamicResource {x:Static vsshell:VsBrushes.ControlLinkTextKey}}" />
        
        <Style.Triggers>
            <Trigger Property="IsMouseOver" Value="True">
                <Setter Property="Foreground" Value="{DynamicResource {x:Static vsshell:VsBrushes.ControlLinkTextHoverKey}}" />
            </Trigger>
            
            <Trigger Property="IsMouseOver" Value="False">
                <Setter Property="TextBlock.TextDecorations" Value="{x:Null}" />
            </Trigger>

            <Trigger Property="NavigateUri" Value="{x:Null}">
                <Setter Property="Foreground" Value="{DynamicResource {x:Static vsshell:VsBrushes.WindowTextKey}}" />
                <Setter Property="TextBlock.TextDecorations" Value="{x:Null}" />
                <Setter Property="IsEnabled" Value="False" />
            </Trigger>
        </Style.Triggers>
    </Style>
</ResourceDictionary>
