<UserControl x:Class="NuGet.Dialog.PackageManagerUI.UpdateAllUI"
             xmlns="http://schemas.microsoft.com/winfx/2006/xaml/presentation"
             xmlns:x="http://schemas.microsoft.com/winfx/2006/xaml"
             xmlns:mc="http://schemas.openxmlformats.org/markup-compatibility/2006" 
             xmlns:d="http://schemas.microsoft.com/expression/blend/2008" 
             mc:Ignorable="d" 
             xmlns:self="clr-namespace:NuGet.Dialog.PackageManagerUI"
             xmlns:local="clr-namespace:NuGet.Dialog"
             xmlns:vsfx="clr-namespace:NuGet.VisualStudio;assembly=NuGet.VisualStudio"
             d:DesignHeight="30" d:DesignWidth="400">
    <UserControl.CommandBindings>
        <CommandBinding Command="Open" CanExecute="CommandBinding_CanExecute" Executed="CommandBinding_Executed" />
    </UserControl.CommandBindings>
    <DockPanel LastChildFill="True">
        <Button
            Command="Open"
            Content="{x:Static local:Resources.Dialog_UpdateAllButton}"
            MinWidth="75"
            MinHeight="23"
            Margin="5,0,25,0"
            DockPanel.Dock="Right">
        </Button>

        <self:TextBlockControl
            Foreground="{DynamicResource {x:Static vsfx:VsBrushes.WindowTextKey}}"
            Text="{x:Static local:Resources.Dialog_UpdateAllLabel}"
            HorizontalAlignment="Left"
            VerticalAlignment="Center" />
    </DockPanel>
</UserControl>