namespace NuGet.VisualStudio
{
    public static class NuGetEvent
    {
        public static readonly int SelectProjectDialogBegin = 25700;
        public static readonly int SelectProjectDialogEnd = 25701;
        public static readonly int LicenseWindowBegin = 25702;
        public static readonly int LicenseWindowEnd = 25703;
        public static readonly int ProgressDialogBegin = 25704;
        public static readonly int ProgressDialogEnd = 25705;
        public static readonly int PackageLoadBegin = 25706;
        public static readonly int PackageLoadEnd = 25707;
        public static readonly int PackageRestoreCompleted = 25708;
        public static readonly int LinkOpened = 25709;
        public static readonly int PackageOperationBegin = 25710;
        public static readonly int PackageOperationEnd = 25711;
        public static readonly int PackageManagerConsoleLoaded = 25712;
        public static readonly int PackageManagerConsoleCommandExecutionBegin = 25713;
        public static readonly int PackageManagerConsoleCommandExecutionEnd = 25714;
    }    
}
