using System.Reflection;
using System.Runtime.InteropServices;

// General Information about an assembly is controlled through the following 
// set of attributes. Change these attribute values to modify the information
// associated with an assembly.
[assembly: AssemblyTitle("NuGet.Console.Types")]
[assembly: AssemblyDescription("Package Manager Powershell Console Types")]

// The following GUID is for the ID of the typelib if this project is exposed to COM
[assembly: ImportedFromTypeLib("NuGet.Console.Types")]
[assembly: Guid("b5b74c80-0d1f-43c8-8b01-f3c28bd92a45")]
