//------------------------------------------------------------------------------
// <auto-generated>
//     This code was generated by a tool.
//     Runtime Version:4.0.30319.18051
//
//     Changes to this file may cause incorrect behavior and will be lost if
//     the code is regenerated.
// </auto-generated>
//------------------------------------------------------------------------------

namespace NuGet {
    using System;
    
    
    /// <summary>
    ///   A strongly-typed resource class, for looking up localized strings, etc.
    /// </summary>
    // This class was auto-generated by the StronglyTypedResourceBuilder
    // class via a tool like ResGen or Visual Studio.
    // To add or remove a member, edit your .ResX file then rerun ResGen
    // with the /str option, or rebuild your VS project.
    [global::System.CodeDom.Compiler.GeneratedCodeAttribute("System.Resources.Tools.StronglyTypedResourceBuilder", "4.0.0.0")]
    [global::System.Diagnostics.DebuggerNonUserCodeAttribute()]
    [global::System.Runtime.CompilerServices.CompilerGeneratedAttribute()]
    internal class NuGetResources {
        
        private static global::System.Resources.ResourceManager resourceMan;
        
        private static global::System.Globalization.CultureInfo resourceCulture;
        
        [global::System.Diagnostics.CodeAnalysis.SuppressMessageAttribute("Microsoft.Performance", "CA1811:AvoidUncalledPrivateCode")]
        internal NuGetResources() {
        }
        
        /// <summary>
        ///   Returns the cached ResourceManager instance used by this class.
        /// </summary>
        [global::System.ComponentModel.EditorBrowsableAttribute(global::System.ComponentModel.EditorBrowsableState.Advanced)]
        internal static global::System.Resources.ResourceManager ResourceManager {
            get {
                if (object.ReferenceEquals(resourceMan, null)) {
                    global::System.Resources.ResourceManager temp = new global::System.Resources.ResourceManager("NuGet.NuGetResources", typeof(NuGetResources).Assembly);
                    resourceMan = temp;
                }
                return resourceMan;
            }
        }
        
        /// <summary>
        ///   Overrides the current thread's CurrentUICulture property for all
        ///   resource lookups using this strongly typed resource class.
        /// </summary>
        [global::System.ComponentModel.EditorBrowsableAttribute(global::System.ComponentModel.EditorBrowsableState.Advanced)]
        internal static global::System.Globalization.CultureInfo Culture {
            get {
                return resourceCulture;
            }
            set {
                resourceCulture = value;
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Add file &apos;{0}&apos; to package as &apos;{1}&apos;.
        /// </summary>
        internal static string AddFileToPackage {
            get {
                return ResourceManager.GetString("AddFileToPackage", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 将文件“{0}”作为“{1}”添加到程序包.
        /// </summary>
        internal static string AddFileToPackage_chs {
            get {
                return ResourceManager.GetString("AddFileToPackage_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 新增檔案 &apos;{0}&apos; 到封裝以做為 &apos;{1}&apos;.
        /// </summary>
        internal static string AddFileToPackage_cht {
            get {
                return ResourceManager.GetString("AddFileToPackage_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Přidat soubor {0} do balíčku jako {1}.
        /// </summary>
        internal static string AddFileToPackage_csy {
            get {
                return ResourceManager.GetString("AddFileToPackage_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Datei &quot;{0}&quot; dem Paket als &quot;{1}&quot; hinzufügen.
        /// </summary>
        internal static string AddFileToPackage_deu {
            get {
                return ResourceManager.GetString("AddFileToPackage_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Agregar archivo &apos;{0}&apos; al paquete como &apos;{1}&apos;.
        /// </summary>
        internal static string AddFileToPackage_esp {
            get {
                return ResourceManager.GetString("AddFileToPackage_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Ajouter le fichier &apos;{0}&apos; au package comme &apos;{1}&apos;.
        /// </summary>
        internal static string AddFileToPackage_fra {
            get {
                return ResourceManager.GetString("AddFileToPackage_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Aggiungere file &apos;{0}&apos; a pacchetto come &apos;{1}&apos;.
        /// </summary>
        internal static string AddFileToPackage_ita {
            get {
                return ResourceManager.GetString("AddFileToPackage_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to ファイル &apos;{0}&apos; を &apos;{1}&apos; としてパッケージに追加します.
        /// </summary>
        internal static string AddFileToPackage_jpn {
            get {
                return ResourceManager.GetString("AddFileToPackage_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 패키지에 &apos;{1}&apos;(으)로 &apos;{0}&apos; 파일 추가.
        /// </summary>
        internal static string AddFileToPackage_kor {
            get {
                return ResourceManager.GetString("AddFileToPackage_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Dodaj plik „{0}” do pakietu jako „{1}”.
        /// </summary>
        internal static string AddFileToPackage_plk {
            get {
                return ResourceManager.GetString("AddFileToPackage_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Adicionar arquivo &apos;{0}&apos; ao pacote como &apos;{1}&apos;.
        /// </summary>
        internal static string AddFileToPackage_ptb {
            get {
                return ResourceManager.GetString("AddFileToPackage_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Добавление файла &quot;{0}&quot; в пакет как &quot;{1}&quot;.
        /// </summary>
        internal static string AddFileToPackage_rus {
            get {
                return ResourceManager.GetString("AddFileToPackage_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0}&apos; dosyasını pakete &apos;{1}&apos; olarak ekle.
        /// </summary>
        internal static string AddFileToPackage_trk {
            get {
                return ResourceManager.GetString("AddFileToPackage_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Ambiguous command &apos;{0}&apos;. Possible values: {1}..
        /// </summary>
        internal static string AmbiguousCommand {
            get {
                return ResourceManager.GetString("AmbiguousCommand", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 命令“{0}”不明确。可能的值: {1}。.
        /// </summary>
        internal static string AmbiguousCommand_chs {
            get {
                return ResourceManager.GetString("AmbiguousCommand_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 模糊的命令 &apos;{0}&apos;。可能值為: {1}。.
        /// </summary>
        internal static string AmbiguousCommand_cht {
            get {
                return ResourceManager.GetString("AmbiguousCommand_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Nejednoznačný příkaz {0}. Možné hodnoty: {1}..
        /// </summary>
        internal static string AmbiguousCommand_csy {
            get {
                return ResourceManager.GetString("AmbiguousCommand_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Mehrdeutiger Befehl &quot;{0}&quot;. Mögliche Werte: {1}..
        /// </summary>
        internal static string AmbiguousCommand_deu {
            get {
                return ResourceManager.GetString("AmbiguousCommand_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Comando ambiguo &apos;{0}&apos;. Valores posibles: {1}..
        /// </summary>
        internal static string AmbiguousCommand_esp {
            get {
                return ResourceManager.GetString("AmbiguousCommand_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Commande ambiguë &apos;{0}&apos;. Valeurs possibles : {1}..
        /// </summary>
        internal static string AmbiguousCommand_fra {
            get {
                return ResourceManager.GetString("AmbiguousCommand_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Comando ambiguo &apos;{0}&apos;. Possibili valori: {1}..
        /// </summary>
        internal static string AmbiguousCommand_ita {
            get {
                return ResourceManager.GetString("AmbiguousCommand_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to あいまいなコマンド &apos;{0}&apos;。指定可能な値: {1}..
        /// </summary>
        internal static string AmbiguousCommand_jpn {
            get {
                return ResourceManager.GetString("AmbiguousCommand_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 모호한 &apos;{0}&apos; 명령입니다. 사용 가능한 값: {1}..
        /// </summary>
        internal static string AmbiguousCommand_kor {
            get {
                return ResourceManager.GetString("AmbiguousCommand_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Niejednoznaczne polecenie „{0}”. Możliwe wartości: {1}..
        /// </summary>
        internal static string AmbiguousCommand_plk {
            get {
                return ResourceManager.GetString("AmbiguousCommand_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Comando ambíguo &apos;{0}&apos;. Valores possíveis: {1}..
        /// </summary>
        internal static string AmbiguousCommand_ptb {
            get {
                return ResourceManager.GetString("AmbiguousCommand_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Неоднозначная команда &quot;{0}&quot;. Возможные значения: {1}..
        /// </summary>
        internal static string AmbiguousCommand_rus {
            get {
                return ResourceManager.GetString("AmbiguousCommand_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Belirsiz komut &apos;{0}&apos;. Olası değerler: {1}..
        /// </summary>
        internal static string AmbiguousCommand_trk {
            get {
                return ResourceManager.GetString("AmbiguousCommand_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Ambiguous option &apos;{0}&apos;. Possible values: {1}..
        /// </summary>
        internal static string AmbiguousOption {
            get {
                return ResourceManager.GetString("AmbiguousOption", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 选项“{0}”不明确。可能的值: {1}。.
        /// </summary>
        internal static string AmbiguousOption_chs {
            get {
                return ResourceManager.GetString("AmbiguousOption_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 模糊的選項 &apos;{0}&apos;。可能值為: {1}。.
        /// </summary>
        internal static string AmbiguousOption_cht {
            get {
                return ResourceManager.GetString("AmbiguousOption_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Nejednoznačná možnost {0}. Možné hodnoty: {1}..
        /// </summary>
        internal static string AmbiguousOption_csy {
            get {
                return ResourceManager.GetString("AmbiguousOption_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Mehrdeutige Option &quot;{0}&quot;. Mögliche Werte: {1}..
        /// </summary>
        internal static string AmbiguousOption_deu {
            get {
                return ResourceManager.GetString("AmbiguousOption_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Opción ambigua &apos;{0}&apos;. Valores posibles: {1}..
        /// </summary>
        internal static string AmbiguousOption_esp {
            get {
                return ResourceManager.GetString("AmbiguousOption_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Option ambiguë &apos;{0}&apos;. Valeurs possibles : {1}..
        /// </summary>
        internal static string AmbiguousOption_fra {
            get {
                return ResourceManager.GetString("AmbiguousOption_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Opzione ambigua &apos;{0}&apos;. Possibili valori: {1}..
        /// </summary>
        internal static string AmbiguousOption_ita {
            get {
                return ResourceManager.GetString("AmbiguousOption_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to あいまいなオプション &apos;{0}&apos;。指定可能な値: {1}..
        /// </summary>
        internal static string AmbiguousOption_jpn {
            get {
                return ResourceManager.GetString("AmbiguousOption_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 모호한 &apos;{0}&apos; 옵션입니다. 사용 가능한 값: {1}..
        /// </summary>
        internal static string AmbiguousOption_kor {
            get {
                return ResourceManager.GetString("AmbiguousOption_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Niejednoznaczna opcja „{0}”. Możliwe wartości: {1}..
        /// </summary>
        internal static string AmbiguousOption_plk {
            get {
                return ResourceManager.GetString("AmbiguousOption_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Opção ambígua &apos;{0}&apos;. Valores possíveis: {1}..
        /// </summary>
        internal static string AmbiguousOption_ptb {
            get {
                return ResourceManager.GetString("AmbiguousOption_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Неоднозначный параметр &quot;{0}&quot;. Возможные значения: {1}..
        /// </summary>
        internal static string AmbiguousOption_rus {
            get {
                return ResourceManager.GetString("AmbiguousOption_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Belirsiz seçenek &apos;{0}&apos;. Olası değerler: {1}..
        /// </summary>
        internal static string AmbiguousOption_trk {
            get {
                return ResourceManager.GetString("AmbiguousOption_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Building project &apos;{0}&apos; for target framework &apos;{1}&apos;..
        /// </summary>
        internal static string BuildingProjectTargetingFramework {
            get {
                return ResourceManager.GetString("BuildingProjectTargetingFramework", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 正在为目标框架“{1}”生成项目“{0}”。.
        /// </summary>
        internal static string BuildingProjectTargetingFramework_chs {
            get {
                return ResourceManager.GetString("BuildingProjectTargetingFramework_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 正在為目標架構 &apos;{1}&apos; 建置專案 &apos;{0}&apos;。.
        /// </summary>
        internal static string BuildingProjectTargetingFramework_cht {
            get {
                return ResourceManager.GetString("BuildingProjectTargetingFramework_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Vytváří se projekt {0} pro cílovou architekturu {1}..
        /// </summary>
        internal static string BuildingProjectTargetingFramework_csy {
            get {
                return ResourceManager.GetString("BuildingProjectTargetingFramework_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Projekt &quot;{0}&quot; wird für das Zielframework &quot;{1}&quot; erstellt..
        /// </summary>
        internal static string BuildingProjectTargetingFramework_deu {
            get {
                return ResourceManager.GetString("BuildingProjectTargetingFramework_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Compilando proyecto &apos;{0}&apos; para el marco de trabajo de destino &apos;{1}&apos;..
        /// </summary>
        internal static string BuildingProjectTargetingFramework_esp {
            get {
                return ResourceManager.GetString("BuildingProjectTargetingFramework_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Création du projet &apos;{0}&apos; pour le Framework cible &apos;{1}&apos;..
        /// </summary>
        internal static string BuildingProjectTargetingFramework_fra {
            get {
                return ResourceManager.GetString("BuildingProjectTargetingFramework_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Creazione progetto &apos;{0}&apos; per quadro target &apos;{1}&apos;..
        /// </summary>
        internal static string BuildingProjectTargetingFramework_ita {
            get {
                return ResourceManager.GetString("BuildingProjectTargetingFramework_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to ターゲット フレームワーク &apos;{1}&apos; のプロジェクト &apos;{0}&apos; をビルドしています.
        /// </summary>
        internal static string BuildingProjectTargetingFramework_jpn {
            get {
                return ResourceManager.GetString("BuildingProjectTargetingFramework_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 대상 프레임워크 &apos;{1}&apos;에 대한 &apos;{0}&apos; 프로젝트를 빌드하고 있습니다..
        /// </summary>
        internal static string BuildingProjectTargetingFramework_kor {
            get {
                return ResourceManager.GetString("BuildingProjectTargetingFramework_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Kompilowanie projektu „{0}” dla struktury docelowej „{1}”..
        /// </summary>
        internal static string BuildingProjectTargetingFramework_plk {
            get {
                return ResourceManager.GetString("BuildingProjectTargetingFramework_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Criar projeto &apos;{0}&apos; para a estrutura de destino &apos;{1}&apos;..
        /// </summary>
        internal static string BuildingProjectTargetingFramework_ptb {
            get {
                return ResourceManager.GetString("BuildingProjectTargetingFramework_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Выполнение сборки проекта &quot;{0}&quot; для целевой платформы &quot;{1}&quot;..
        /// </summary>
        internal static string BuildingProjectTargetingFramework_rus {
            get {
                return ResourceManager.GetString("BuildingProjectTargetingFramework_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {1}&apos; hedef framework&apos;ü için &apos;{0}&apos; projesi oluşturuluyor..
        /// </summary>
        internal static string BuildingProjectTargetingFramework_trk {
            get {
                return ResourceManager.GetString("BuildingProjectTargetingFramework_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to WARNING: {0}.
        /// </summary>
        internal static string CommandLine_Warning {
            get {
                return ResourceManager.GetString("CommandLine_Warning", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 警告: {0}.
        /// </summary>
        internal static string CommandLine_Warning_chs {
            get {
                return ResourceManager.GetString("CommandLine_Warning_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 警告: {0}.
        /// </summary>
        internal static string CommandLine_Warning_cht {
            get {
                return ResourceManager.GetString("CommandLine_Warning_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to UPOZORNĚNÍ: {0}.
        /// </summary>
        internal static string CommandLine_Warning_csy {
            get {
                return ResourceManager.GetString("CommandLine_Warning_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to WARNUNG: {0}.
        /// </summary>
        internal static string CommandLine_Warning_deu {
            get {
                return ResourceManager.GetString("CommandLine_Warning_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to ADVERTENCIA: &apos;{0}&apos;.
        /// </summary>
        internal static string CommandLine_Warning_esp {
            get {
                return ResourceManager.GetString("CommandLine_Warning_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to AVERTISSEMENT : {0}.
        /// </summary>
        internal static string CommandLine_Warning_fra {
            get {
                return ResourceManager.GetString("CommandLine_Warning_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to AVVISO: {0}.
        /// </summary>
        internal static string CommandLine_Warning_ita {
            get {
                return ResourceManager.GetString("CommandLine_Warning_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 警告: {0}.
        /// </summary>
        internal static string CommandLine_Warning_jpn {
            get {
                return ResourceManager.GetString("CommandLine_Warning_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 경고: {0}.
        /// </summary>
        internal static string CommandLine_Warning_kor {
            get {
                return ResourceManager.GetString("CommandLine_Warning_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to OSTRZEŻENIE: {0}.
        /// </summary>
        internal static string CommandLine_Warning_plk {
            get {
                return ResourceManager.GetString("CommandLine_Warning_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to AVISO: {0}.
        /// </summary>
        internal static string CommandLine_Warning_ptb {
            get {
                return ResourceManager.GetString("CommandLine_Warning_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to ПРЕДУПРЕЖДЕНИЕ. {0}.
        /// </summary>
        internal static string CommandLine_Warning_rus {
            get {
                return ResourceManager.GetString("CommandLine_Warning_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to UYARI: {0}.
        /// </summary>
        internal static string CommandLine_Warning_trk {
            get {
                return ResourceManager.GetString("CommandLine_Warning_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Key &apos;{0}&apos; not found..
        /// </summary>
        internal static string ConfigCommandKeyNotFound {
            get {
                return ResourceManager.GetString("ConfigCommandKeyNotFound", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 找不到键“{0}”。.
        /// </summary>
        internal static string ConfigCommandKeyNotFound_chs {
            get {
                return ResourceManager.GetString("ConfigCommandKeyNotFound_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 找不到索引鍵 &apos;{0}&apos;。.
        /// </summary>
        internal static string ConfigCommandKeyNotFound_cht {
            get {
                return ResourceManager.GetString("ConfigCommandKeyNotFound_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Klíč {0} nebyl nalezen..
        /// </summary>
        internal static string ConfigCommandKeyNotFound_csy {
            get {
                return ResourceManager.GetString("ConfigCommandKeyNotFound_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Der Schlüssel &quot;{0}&quot; wurde nicht gefunden..
        /// </summary>
        internal static string ConfigCommandKeyNotFound_deu {
            get {
                return ResourceManager.GetString("ConfigCommandKeyNotFound_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to No se encuentra la clave &apos;{0}&apos;..
        /// </summary>
        internal static string ConfigCommandKeyNotFound_esp {
            get {
                return ResourceManager.GetString("ConfigCommandKeyNotFound_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Clé &apos;{0}&apos; introuvable..
        /// </summary>
        internal static string ConfigCommandKeyNotFound_fra {
            get {
                return ResourceManager.GetString("ConfigCommandKeyNotFound_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Chiave &apos;{0}&apos; non trovata..
        /// </summary>
        internal static string ConfigCommandKeyNotFound_ita {
            get {
                return ResourceManager.GetString("ConfigCommandKeyNotFound_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to キー &apos;{0}&apos; が見つかりません。.
        /// </summary>
        internal static string ConfigCommandKeyNotFound_jpn {
            get {
                return ResourceManager.GetString("ConfigCommandKeyNotFound_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to &apos;{0}&apos; 키를 찾을 수 없습니다..
        /// </summary>
        internal static string ConfigCommandKeyNotFound_kor {
            get {
                return ResourceManager.GetString("ConfigCommandKeyNotFound_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Nie odnaleziono klucza „{0}”..
        /// </summary>
        internal static string ConfigCommandKeyNotFound_plk {
            get {
                return ResourceManager.GetString("ConfigCommandKeyNotFound_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Chave &apos;{0}&apos; não encontrada..
        /// </summary>
        internal static string ConfigCommandKeyNotFound_ptb {
            get {
                return ResourceManager.GetString("ConfigCommandKeyNotFound_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Ключ &quot;{0}&quot; не найден..
        /// </summary>
        internal static string ConfigCommandKeyNotFound_rus {
            get {
                return ResourceManager.GetString("ConfigCommandKeyNotFound_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0}&apos; anahtarı bulunamadı..
        /// </summary>
        internal static string ConfigCommandKeyNotFound_trk {
            get {
                return ResourceManager.GetString("ConfigCommandKeyNotFound_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} (y/N) .
        /// </summary>
        internal static string ConsoleConfirmMessage {
            get {
                return ResourceManager.GetString("ConsoleConfirmMessage", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} (y/N) .
        /// </summary>
        internal static string ConsoleConfirmMessage_chs {
            get {
                return ResourceManager.GetString("ConsoleConfirmMessage_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} (是/否) .
        /// </summary>
        internal static string ConsoleConfirmMessage_cht {
            get {
                return ResourceManager.GetString("ConsoleConfirmMessage_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} (A/N): .
        /// </summary>
        internal static string ConsoleConfirmMessage_csy {
            get {
                return ResourceManager.GetString("ConsoleConfirmMessage_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} (j/N) .
        /// </summary>
        internal static string ConsoleConfirmMessage_deu {
            get {
                return ResourceManager.GetString("ConsoleConfirmMessage_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} (Sí/No) .
        /// </summary>
        internal static string ConsoleConfirmMessage_esp {
            get {
                return ResourceManager.GetString("ConsoleConfirmMessage_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} (o/N) .
        /// </summary>
        internal static string ConsoleConfirmMessage_fra {
            get {
                return ResourceManager.GetString("ConsoleConfirmMessage_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} (y/N) .
        /// </summary>
        internal static string ConsoleConfirmMessage_ita {
            get {
                return ResourceManager.GetString("ConsoleConfirmMessage_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} (y/N) .
        /// </summary>
        internal static string ConsoleConfirmMessage_jpn {
            get {
                return ResourceManager.GetString("ConsoleConfirmMessage_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0}(y/N) .
        /// </summary>
        internal static string ConsoleConfirmMessage_kor {
            get {
                return ResourceManager.GetString("ConsoleConfirmMessage_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} (t/N) .
        /// </summary>
        internal static string ConsoleConfirmMessage_plk {
            get {
                return ResourceManager.GetString("ConsoleConfirmMessage_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} (y / N).
        /// </summary>
        internal static string ConsoleConfirmMessage_ptb {
            get {
                return ResourceManager.GetString("ConsoleConfirmMessage_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} (д/н) .
        /// </summary>
        internal static string ConsoleConfirmMessage_rus {
            get {
                return ResourceManager.GetString("ConsoleConfirmMessage_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} (e/H) .
        /// </summary>
        internal static string ConsoleConfirmMessage_trk {
            get {
                return ResourceManager.GetString("ConsoleConfirmMessage_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to y.
        /// </summary>
        internal static string ConsoleConfirmMessageAccept {
            get {
                return ResourceManager.GetString("ConsoleConfirmMessageAccept", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to y.
        /// </summary>
        internal static string ConsoleConfirmMessageAccept_chs {
            get {
                return ResourceManager.GetString("ConsoleConfirmMessageAccept_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 是.
        /// </summary>
        internal static string ConsoleConfirmMessageAccept_cht {
            get {
                return ResourceManager.GetString("ConsoleConfirmMessageAccept_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to A.
        /// </summary>
        internal static string ConsoleConfirmMessageAccept_csy {
            get {
                return ResourceManager.GetString("ConsoleConfirmMessageAccept_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to j.
        /// </summary>
        internal static string ConsoleConfirmMessageAccept_deu {
            get {
                return ResourceManager.GetString("ConsoleConfirmMessageAccept_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to sí.
        /// </summary>
        internal static string ConsoleConfirmMessageAccept_esp {
            get {
                return ResourceManager.GetString("ConsoleConfirmMessageAccept_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to o.
        /// </summary>
        internal static string ConsoleConfirmMessageAccept_fra {
            get {
                return ResourceManager.GetString("ConsoleConfirmMessageAccept_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to y.
        /// </summary>
        internal static string ConsoleConfirmMessageAccept_ita {
            get {
                return ResourceManager.GetString("ConsoleConfirmMessageAccept_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to y.
        /// </summary>
        internal static string ConsoleConfirmMessageAccept_jpn {
            get {
                return ResourceManager.GetString("ConsoleConfirmMessageAccept_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to y.
        /// </summary>
        internal static string ConsoleConfirmMessageAccept_kor {
            get {
                return ResourceManager.GetString("ConsoleConfirmMessageAccept_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to t.
        /// </summary>
        internal static string ConsoleConfirmMessageAccept_plk {
            get {
                return ResourceManager.GetString("ConsoleConfirmMessageAccept_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to y.
        /// </summary>
        internal static string ConsoleConfirmMessageAccept_ptb {
            get {
                return ResourceManager.GetString("ConsoleConfirmMessageAccept_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to д.
        /// </summary>
        internal static string ConsoleConfirmMessageAccept_rus {
            get {
                return ResourceManager.GetString("ConsoleConfirmMessageAccept_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to e.
        /// </summary>
        internal static string ConsoleConfirmMessageAccept_trk {
            get {
                return ResourceManager.GetString("ConsoleConfirmMessageAccept_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Password: .
        /// </summary>
        internal static string Credentials_Password {
            get {
                return ResourceManager.GetString("Credentials_Password", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 密码: .
        /// </summary>
        internal static string Credentials_Password_chs {
            get {
                return ResourceManager.GetString("Credentials_Password_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 密碼: .
        /// </summary>
        internal static string Credentials_Password_cht {
            get {
                return ResourceManager.GetString("Credentials_Password_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Heslo: .
        /// </summary>
        internal static string Credentials_Password_csy {
            get {
                return ResourceManager.GetString("Credentials_Password_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Kennwort: .
        /// </summary>
        internal static string Credentials_Password_deu {
            get {
                return ResourceManager.GetString("Credentials_Password_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Contraseña:.
        /// </summary>
        internal static string Credentials_Password_esp {
            get {
                return ResourceManager.GetString("Credentials_Password_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Mot de passe : .
        /// </summary>
        internal static string Credentials_Password_fra {
            get {
                return ResourceManager.GetString("Credentials_Password_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Password:.
        /// </summary>
        internal static string Credentials_Password_ita {
            get {
                return ResourceManager.GetString("Credentials_Password_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to パスワード: .
        /// </summary>
        internal static string Credentials_Password_jpn {
            get {
                return ResourceManager.GetString("Credentials_Password_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 암호: .
        /// </summary>
        internal static string Credentials_Password_kor {
            get {
                return ResourceManager.GetString("Credentials_Password_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Hasło:.
        /// </summary>
        internal static string Credentials_Password_plk {
            get {
                return ResourceManager.GetString("Credentials_Password_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Senha:.
        /// </summary>
        internal static string Credentials_Password_ptb {
            get {
                return ResourceManager.GetString("Credentials_Password_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Пароль: .
        /// </summary>
        internal static string Credentials_Password_rus {
            get {
                return ResourceManager.GetString("Credentials_Password_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Parola: .
        /// </summary>
        internal static string Credentials_Password_trk {
            get {
                return ResourceManager.GetString("Credentials_Password_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Please provide proxy credentials:.
        /// </summary>
        internal static string Credentials_ProxyCredentials {
            get {
                return ResourceManager.GetString("Credentials_ProxyCredentials", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 请提供代理凭据:.
        /// </summary>
        internal static string Credentials_ProxyCredentials_chs {
            get {
                return ResourceManager.GetString("Credentials_ProxyCredentials_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 請提供 Proxy 認證:.
        /// </summary>
        internal static string Credentials_ProxyCredentials_cht {
            get {
                return ResourceManager.GetString("Credentials_ProxyCredentials_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Zadejte proxy přihlašovací údaje:.
        /// </summary>
        internal static string Credentials_ProxyCredentials_csy {
            get {
                return ResourceManager.GetString("Credentials_ProxyCredentials_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Bitte stellen Sie Proxyanmeldeinformationen zur Verfügung:.
        /// </summary>
        internal static string Credentials_ProxyCredentials_deu {
            get {
                return ResourceManager.GetString("Credentials_ProxyCredentials_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Proporcione las credenciales de proxy:.
        /// </summary>
        internal static string Credentials_ProxyCredentials_esp {
            get {
                return ResourceManager.GetString("Credentials_ProxyCredentials_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Veuillez fournir les informations d&apos;identification du proxy :.
        /// </summary>
        internal static string Credentials_ProxyCredentials_fra {
            get {
                return ResourceManager.GetString("Credentials_ProxyCredentials_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Fornire credenziali proxy:.
        /// </summary>
        internal static string Credentials_ProxyCredentials_ita {
            get {
                return ResourceManager.GetString("Credentials_ProxyCredentials_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to プロキシの資格情報を指定してください:.
        /// </summary>
        internal static string Credentials_ProxyCredentials_jpn {
            get {
                return ResourceManager.GetString("Credentials_ProxyCredentials_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 프록시 자격 증명을 제공하십시오..
        /// </summary>
        internal static string Credentials_ProxyCredentials_kor {
            get {
                return ResourceManager.GetString("Credentials_ProxyCredentials_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Podaj poświadczenia serwera proxy:.
        /// </summary>
        internal static string Credentials_ProxyCredentials_plk {
            get {
                return ResourceManager.GetString("Credentials_ProxyCredentials_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Forneça as credenciais de proxy:.
        /// </summary>
        internal static string Credentials_ProxyCredentials_ptb {
            get {
                return ResourceManager.GetString("Credentials_ProxyCredentials_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Укажите учетные данные прокси-сервера:.
        /// </summary>
        internal static string Credentials_ProxyCredentials_rus {
            get {
                return ResourceManager.GetString("Credentials_ProxyCredentials_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Lütfen proxy kimlik bilgilerini belirtin:.
        /// </summary>
        internal static string Credentials_ProxyCredentials_trk {
            get {
                return ResourceManager.GetString("Credentials_ProxyCredentials_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Please provide credentials for: {0}.
        /// </summary>
        internal static string Credentials_RequestCredentials {
            get {
                return ResourceManager.GetString("Credentials_RequestCredentials", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 请提供以下人员的凭据: {0}.
        /// </summary>
        internal static string Credentials_RequestCredentials_chs {
            get {
                return ResourceManager.GetString("Credentials_RequestCredentials_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 請提供以下項目的認證: {0}.
        /// </summary>
        internal static string Credentials_RequestCredentials_cht {
            get {
                return ResourceManager.GetString("Credentials_RequestCredentials_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Zadejte přihlašovací údaje pro: {0}.
        /// </summary>
        internal static string Credentials_RequestCredentials_csy {
            get {
                return ResourceManager.GetString("Credentials_RequestCredentials_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Bitte stellen Sie Anmeldeinformationen zur Verfügung für: {0}.
        /// </summary>
        internal static string Credentials_RequestCredentials_deu {
            get {
                return ResourceManager.GetString("Credentials_RequestCredentials_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Proporcione las credenciales para: {0}.
        /// </summary>
        internal static string Credentials_RequestCredentials_esp {
            get {
                return ResourceManager.GetString("Credentials_RequestCredentials_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Veuillez fournir les informations d&apos;identification relative à : {0}.
        /// </summary>
        internal static string Credentials_RequestCredentials_fra {
            get {
                return ResourceManager.GetString("Credentials_RequestCredentials_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Fornire credenziali per: {0}.
        /// </summary>
        internal static string Credentials_RequestCredentials_ita {
            get {
                return ResourceManager.GetString("Credentials_RequestCredentials_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 次の資格情報を指定してください: {0}.
        /// </summary>
        internal static string Credentials_RequestCredentials_jpn {
            get {
                return ResourceManager.GetString("Credentials_RequestCredentials_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0}에 대한 자격 증명을 제공하십시오..
        /// </summary>
        internal static string Credentials_RequestCredentials_kor {
            get {
                return ResourceManager.GetString("Credentials_RequestCredentials_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Podaj poświadczenia dla: {0}.
        /// </summary>
        internal static string Credentials_RequestCredentials_plk {
            get {
                return ResourceManager.GetString("Credentials_RequestCredentials_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Forneça as credenciais para: {0}.
        /// </summary>
        internal static string Credentials_RequestCredentials_ptb {
            get {
                return ResourceManager.GetString("Credentials_RequestCredentials_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Укажите учетные данные для: {0}.
        /// </summary>
        internal static string Credentials_RequestCredentials_rus {
            get {
                return ResourceManager.GetString("Credentials_RequestCredentials_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Lütfen buna ait kimlik bilgilerini belirtin: {0}.
        /// </summary>
        internal static string Credentials_RequestCredentials_trk {
            get {
                return ResourceManager.GetString("Credentials_RequestCredentials_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to UserName: .
        /// </summary>
        internal static string Credentials_UserName {
            get {
                return ResourceManager.GetString("Credentials_UserName", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 用户名: .
        /// </summary>
        internal static string Credentials_UserName_chs {
            get {
                return ResourceManager.GetString("Credentials_UserName_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 使用者名稱: .
        /// </summary>
        internal static string Credentials_UserName_cht {
            get {
                return ResourceManager.GetString("Credentials_UserName_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Uživatelské jméno: .
        /// </summary>
        internal static string Credentials_UserName_csy {
            get {
                return ResourceManager.GetString("Credentials_UserName_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to UserName: .
        /// </summary>
        internal static string Credentials_UserName_deu {
            get {
                return ResourceManager.GetString("Credentials_UserName_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Nombre de usuario:.
        /// </summary>
        internal static string Credentials_UserName_esp {
            get {
                return ResourceManager.GetString("Credentials_UserName_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Nom d&apos;utilisateur : .
        /// </summary>
        internal static string Credentials_UserName_fra {
            get {
                return ResourceManager.GetString("Credentials_UserName_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Nome utente:.
        /// </summary>
        internal static string Credentials_UserName_ita {
            get {
                return ResourceManager.GetString("Credentials_UserName_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to UserName: .
        /// </summary>
        internal static string Credentials_UserName_jpn {
            get {
                return ResourceManager.GetString("Credentials_UserName_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 사용자 이름: .
        /// </summary>
        internal static string Credentials_UserName_kor {
            get {
                return ResourceManager.GetString("Credentials_UserName_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Nazwa użytkownika: .
        /// </summary>
        internal static string Credentials_UserName_plk {
            get {
                return ResourceManager.GetString("Credentials_UserName_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Nome de usuário:.
        /// </summary>
        internal static string Credentials_UserName_ptb {
            get {
                return ResourceManager.GetString("Credentials_UserName_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Имя пользователя: .
        /// </summary>
        internal static string Credentials_UserName_rus {
            get {
                return ResourceManager.GetString("Credentials_UserName_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to KullanıcıAdı: .
        /// </summary>
        internal static string Credentials_UserName_trk {
            get {
                return ResourceManager.GetString("Credentials_UserName_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to No description was provided for this command..
        /// </summary>
        internal static string DefaultCommandDescription {
            get {
                return ResourceManager.GetString("DefaultCommandDescription", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 未提供此命令的说明。.
        /// </summary>
        internal static string DefaultCommandDescription_chs {
            get {
                return ResourceManager.GetString("DefaultCommandDescription_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 並未提供此命令的描述。.
        /// </summary>
        internal static string DefaultCommandDescription_cht {
            get {
                return ResourceManager.GetString("DefaultCommandDescription_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Pro tento příkaz nebyl zadán žádný popis..
        /// </summary>
        internal static string DefaultCommandDescription_csy {
            get {
                return ResourceManager.GetString("DefaultCommandDescription_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Für diesen Befehl wurde keine Beschreibung bereitgestellt..
        /// </summary>
        internal static string DefaultCommandDescription_deu {
            get {
                return ResourceManager.GetString("DefaultCommandDescription_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to No se ha proporcionado ninguna descripción para este comando..
        /// </summary>
        internal static string DefaultCommandDescription_esp {
            get {
                return ResourceManager.GetString("DefaultCommandDescription_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Aucune description n&apos;a été fournie pour cette commande..
        /// </summary>
        internal static string DefaultCommandDescription_fra {
            get {
                return ResourceManager.GetString("DefaultCommandDescription_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Non è stata fornita alcuna descrizione per questo comando..
        /// </summary>
        internal static string DefaultCommandDescription_ita {
            get {
                return ResourceManager.GetString("DefaultCommandDescription_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to このコマンドに説明は指定されていません。.
        /// </summary>
        internal static string DefaultCommandDescription_jpn {
            get {
                return ResourceManager.GetString("DefaultCommandDescription_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 이 명령에 대한 설명이 제공되지 않았습니다..
        /// </summary>
        internal static string DefaultCommandDescription_kor {
            get {
                return ResourceManager.GetString("DefaultCommandDescription_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Nie podano opisu dla tego polecenia..
        /// </summary>
        internal static string DefaultCommandDescription_plk {
            get {
                return ResourceManager.GetString("DefaultCommandDescription_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Nenhuma descrição foi fornecida para este comando..
        /// </summary>
        internal static string DefaultCommandDescription_ptb {
            get {
                return ResourceManager.GetString("DefaultCommandDescription_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Описание для этой команды отсутствует..
        /// </summary>
        internal static string DefaultCommandDescription_rus {
            get {
                return ResourceManager.GetString("DefaultCommandDescription_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Bu komut için açıklama belirtilmedi..
        /// </summary>
        internal static string DefaultCommandDescription_trk {
            get {
                return ResourceManager.GetString("DefaultCommandDescription_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to the symbol server.
        /// </summary>
        internal static string DefaultSymbolServer {
            get {
                return ResourceManager.GetString("DefaultSymbolServer", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 符号服务器.
        /// </summary>
        internal static string DefaultSymbolServer_chs {
            get {
                return ResourceManager.GetString("DefaultSymbolServer_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 符號伺服器.
        /// </summary>
        internal static string DefaultSymbolServer_cht {
            get {
                return ResourceManager.GetString("DefaultSymbolServer_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to server symbolů.
        /// </summary>
        internal static string DefaultSymbolServer_csy {
            get {
                return ResourceManager.GetString("DefaultSymbolServer_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to der Symbolserver.
        /// </summary>
        internal static string DefaultSymbolServer_deu {
            get {
                return ResourceManager.GetString("DefaultSymbolServer_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to servidor de símbolos.
        /// </summary>
        internal static string DefaultSymbolServer_esp {
            get {
                return ResourceManager.GetString("DefaultSymbolServer_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to serveur de symboles.
        /// </summary>
        internal static string DefaultSymbolServer_fra {
            get {
                return ResourceManager.GetString("DefaultSymbolServer_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to il symbol server.
        /// </summary>
        internal static string DefaultSymbolServer_ita {
            get {
                return ResourceManager.GetString("DefaultSymbolServer_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to シンボル サーバー.
        /// </summary>
        internal static string DefaultSymbolServer_jpn {
            get {
                return ResourceManager.GetString("DefaultSymbolServer_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 기호 서버.
        /// </summary>
        internal static string DefaultSymbolServer_kor {
            get {
                return ResourceManager.GetString("DefaultSymbolServer_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to serwer symboli.
        /// </summary>
        internal static string DefaultSymbolServer_plk {
            get {
                return ResourceManager.GetString("DefaultSymbolServer_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to o servidor de símbolos.
        /// </summary>
        internal static string DefaultSymbolServer_ptb {
            get {
                return ResourceManager.GetString("DefaultSymbolServer_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to сервер символов.
        /// </summary>
        internal static string DefaultSymbolServer_rus {
            get {
                return ResourceManager.GetString("DefaultSymbolServer_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to simge sunucusu.
        /// </summary>
        internal static string DefaultSymbolServer_trk {
            get {
                return ResourceManager.GetString("DefaultSymbolServer_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Delete canceled.
        /// </summary>
        internal static string DeleteCommandCanceled {
            get {
                return ResourceManager.GetString("DeleteCommandCanceled", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 已取消删除.
        /// </summary>
        internal static string DeleteCommandCanceled_chs {
            get {
                return ResourceManager.GetString("DeleteCommandCanceled_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 已取消刪除.
        /// </summary>
        internal static string DeleteCommandCanceled_cht {
            get {
                return ResourceManager.GetString("DeleteCommandCanceled_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Odstranit zrušené.
        /// </summary>
        internal static string DeleteCommandCanceled_csy {
            get {
                return ResourceManager.GetString("DeleteCommandCanceled_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Der Löschvorgang wurde abgebrochen..
        /// </summary>
        internal static string DeleteCommandCanceled_deu {
            get {
                return ResourceManager.GetString("DeleteCommandCanceled_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Eliminación cancelada.
        /// </summary>
        internal static string DeleteCommandCanceled_esp {
            get {
                return ResourceManager.GetString("DeleteCommandCanceled_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Suppression annulée.
        /// </summary>
        internal static string DeleteCommandCanceled_fra {
            get {
                return ResourceManager.GetString("DeleteCommandCanceled_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to cancellazione teminata.
        /// </summary>
        internal static string DeleteCommandCanceled_ita {
            get {
                return ResourceManager.GetString("DeleteCommandCanceled_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 削除は取り消されました.
        /// </summary>
        internal static string DeleteCommandCanceled_jpn {
            get {
                return ResourceManager.GetString("DeleteCommandCanceled_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 삭제가 취소됨.
        /// </summary>
        internal static string DeleteCommandCanceled_kor {
            get {
                return ResourceManager.GetString("DeleteCommandCanceled_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Usuwanie anulowane.
        /// </summary>
        internal static string DeleteCommandCanceled_plk {
            get {
                return ResourceManager.GetString("DeleteCommandCanceled_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Excluir cancelado.
        /// </summary>
        internal static string DeleteCommandCanceled_ptb {
            get {
                return ResourceManager.GetString("DeleteCommandCanceled_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Удаление отменено.
        /// </summary>
        internal static string DeleteCommandCanceled_rus {
            get {
                return ResourceManager.GetString("DeleteCommandCanceled_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Silme iptal edildi.
        /// </summary>
        internal static string DeleteCommandCanceled_trk {
            get {
                return ResourceManager.GetString("DeleteCommandCanceled_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} {1} will be deleted from the {2}. Would you like to continue?.
        /// </summary>
        internal static string DeleteCommandConfirm {
            get {
                return ResourceManager.GetString("DeleteCommandConfirm", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 将从 {2} 中删除 {0} {1}。是否要继续?.
        /// </summary>
        internal static string DeleteCommandConfirm_chs {
            get {
                return ResourceManager.GetString("DeleteCommandConfirm_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} {1} 會從 {2} 刪除。您要繼續嗎?.
        /// </summary>
        internal static string DeleteCommandConfirm_cht {
            get {
                return ResourceManager.GetString("DeleteCommandConfirm_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} {1} bude odstraněno z {2}. Chcete pokračovat?.
        /// </summary>
        internal static string DeleteCommandConfirm_csy {
            get {
                return ResourceManager.GetString("DeleteCommandConfirm_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} {1} wird aus &quot;{2}&quot; gelöscht. Möchten Sie den Vorgang fortsetzen?.
        /// </summary>
        internal static string DeleteCommandConfirm_deu {
            get {
                return ResourceManager.GetString("DeleteCommandConfirm_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} {1} se eliminará de {2}. ¿Desea continuar?.
        /// </summary>
        internal static string DeleteCommandConfirm_esp {
            get {
                return ResourceManager.GetString("DeleteCommandConfirm_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} {1} sera supprimé de {2}. Souhaitez-vous continuer ?.
        /// </summary>
        internal static string DeleteCommandConfirm_fra {
            get {
                return ResourceManager.GetString("DeleteCommandConfirm_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} {1} sarà cancellato da {2}. Vuoi continuare?.
        /// </summary>
        internal static string DeleteCommandConfirm_ita {
            get {
                return ResourceManager.GetString("DeleteCommandConfirm_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} {1} は {2} から削除されます。続行しますか?.
        /// </summary>
        internal static string DeleteCommandConfirm_jpn {
            get {
                return ResourceManager.GetString("DeleteCommandConfirm_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} {1}이(가) {2}에서 삭제됩니다. 계속하시겠습니까?.
        /// </summary>
        internal static string DeleteCommandConfirm_kor {
            get {
                return ResourceManager.GetString("DeleteCommandConfirm_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Element {0} {1} zostanie usunięty z {2}. Czy chcesz kontynuować?.
        /// </summary>
        internal static string DeleteCommandConfirm_plk {
            get {
                return ResourceManager.GetString("DeleteCommandConfirm_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} {1} será excluído do {2}. Deseja continuar?.
        /// </summary>
        internal static string DeleteCommandConfirm_ptb {
            get {
                return ResourceManager.GetString("DeleteCommandConfirm_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Будет выполнено удаление {0} {1} из {2}. Продолжить?.
        /// </summary>
        internal static string DeleteCommandConfirm_rus {
            get {
                return ResourceManager.GetString("DeleteCommandConfirm_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} {1} öğeleri {2} konumundan silinecek. Devam etmek istiyor musunuz?.
        /// </summary>
        internal static string DeleteCommandConfirm_trk {
            get {
                return ResourceManager.GetString("DeleteCommandConfirm_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} {1} was deleted successfully..
        /// </summary>
        internal static string DeleteCommandDeletedPackage {
            get {
                return ResourceManager.GetString("DeleteCommandDeletedPackage", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 已成功删除 {0} {1}。.
        /// </summary>
        internal static string DeleteCommandDeletedPackage_chs {
            get {
                return ResourceManager.GetString("DeleteCommandDeletedPackage_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} {1} 已成功刪除。.
        /// </summary>
        internal static string DeleteCommandDeletedPackage_cht {
            get {
                return ResourceManager.GetString("DeleteCommandDeletedPackage_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} {1} bylo úspěšně odstraněno..
        /// </summary>
        internal static string DeleteCommandDeletedPackage_csy {
            get {
                return ResourceManager.GetString("DeleteCommandDeletedPackage_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} {1} wurde erfolgreich gelöscht..
        /// </summary>
        internal static string DeleteCommandDeletedPackage_deu {
            get {
                return ResourceManager.GetString("DeleteCommandDeletedPackage_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} {1} se ha eliminado correctamente..
        /// </summary>
        internal static string DeleteCommandDeletedPackage_esp {
            get {
                return ResourceManager.GetString("DeleteCommandDeletedPackage_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} {1} a été correctement supprimé..
        /// </summary>
        internal static string DeleteCommandDeletedPackage_fra {
            get {
                return ResourceManager.GetString("DeleteCommandDeletedPackage_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} {1} è stato correttamente cancellato..
        /// </summary>
        internal static string DeleteCommandDeletedPackage_ita {
            get {
                return ResourceManager.GetString("DeleteCommandDeletedPackage_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} {1} は正常に削除されました。.
        /// </summary>
        internal static string DeleteCommandDeletedPackage_jpn {
            get {
                return ResourceManager.GetString("DeleteCommandDeletedPackage_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} {1}이(가) 삭제되었습니다..
        /// </summary>
        internal static string DeleteCommandDeletedPackage_kor {
            get {
                return ResourceManager.GetString("DeleteCommandDeletedPackage_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Pomyślnie usunięto element {0} {1}..
        /// </summary>
        internal static string DeleteCommandDeletedPackage_plk {
            get {
                return ResourceManager.GetString("DeleteCommandDeletedPackage_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} {1} foi excluído com sucesso..
        /// </summary>
        internal static string DeleteCommandDeletedPackage_ptb {
            get {
                return ResourceManager.GetString("DeleteCommandDeletedPackage_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Удаление {0} {1} выполнено..
        /// </summary>
        internal static string DeleteCommandDeletedPackage_rus {
            get {
                return ResourceManager.GetString("DeleteCommandDeletedPackage_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} {1} başarıyla silindi..
        /// </summary>
        internal static string DeleteCommandDeletedPackage_trk {
            get {
                return ResourceManager.GetString("DeleteCommandDeletedPackage_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Deleting {0} {1} from the {2}..
        /// </summary>
        internal static string DeleteCommandDeletingPackage {
            get {
                return ResourceManager.GetString("DeleteCommandDeletingPackage", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 正在从 {2} 中删除 {0} {1}。.
        /// </summary>
        internal static string DeleteCommandDeletingPackage_chs {
            get {
                return ResourceManager.GetString("DeleteCommandDeletingPackage_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 正在從 {2} 刪除 {0} {1}。.
        /// </summary>
        internal static string DeleteCommandDeletingPackage_cht {
            get {
                return ResourceManager.GetString("DeleteCommandDeletingPackage_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Probíhá odstraňování {0} {1} z {2}..
        /// </summary>
        internal static string DeleteCommandDeletingPackage_csy {
            get {
                return ResourceManager.GetString("DeleteCommandDeletingPackage_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} {1} wird aus &quot;{2}&quot; gelöscht..
        /// </summary>
        internal static string DeleteCommandDeletingPackage_deu {
            get {
                return ResourceManager.GetString("DeleteCommandDeletingPackage_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Eliminando {0} {1} de {2}..
        /// </summary>
        internal static string DeleteCommandDeletingPackage_esp {
            get {
                return ResourceManager.GetString("DeleteCommandDeletingPackage_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Suppression de {0} {1} de {2}..
        /// </summary>
        internal static string DeleteCommandDeletingPackage_fra {
            get {
                return ResourceManager.GetString("DeleteCommandDeletingPackage_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Cancellando {0} {1} da {2}..
        /// </summary>
        internal static string DeleteCommandDeletingPackage_ita {
            get {
                return ResourceManager.GetString("DeleteCommandDeletingPackage_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {2} から {0} {1} を削除しています。.
        /// </summary>
        internal static string DeleteCommandDeletingPackage_jpn {
            get {
                return ResourceManager.GetString("DeleteCommandDeletingPackage_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} {1}을(를) {2}에서 삭제하고 있습니다..
        /// </summary>
        internal static string DeleteCommandDeletingPackage_kor {
            get {
                return ResourceManager.GetString("DeleteCommandDeletingPackage_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Usuwanie elementu {0} {1} z {2}..
        /// </summary>
        internal static string DeleteCommandDeletingPackage_plk {
            get {
                return ResourceManager.GetString("DeleteCommandDeletingPackage_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Excluindo {0} {1} do {2}..
        /// </summary>
        internal static string DeleteCommandDeletingPackage_ptb {
            get {
                return ResourceManager.GetString("DeleteCommandDeletingPackage_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Удаление {0} {1} из {2}..
        /// </summary>
        internal static string DeleteCommandDeletingPackage_rus {
            get {
                return ResourceManager.GetString("DeleteCommandDeletingPackage_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} {1} öğeleri {2} konumundan siliniyor..
        /// </summary>
        internal static string DeleteCommandDeletingPackage_trk {
            get {
                return ResourceManager.GetString("DeleteCommandDeletingPackage_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Cannot get the GetAllProjectFileNamesMethod from type  Mono.XBuild.CommandLine.SolutionParser..
        /// </summary>
        internal static string Error_CannotGetGetAllProjectFileNamesMethod {
            get {
                return ResourceManager.GetString("Error_CannotGetGetAllProjectFileNamesMethod", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Cannot get type Mono.XBuild.CommandLine.SolutionParser..
        /// </summary>
        internal static string Error_CannotGetXBuildSolutionParser {
            get {
                return ResourceManager.GetString("Error_CannotGetXBuildSolutionParser", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Cannot load type Microsoft.Build.Construction.ProjectInSolution from Microsoft.Build.dll.
        /// </summary>
        internal static string Error_CannotLoadTypeProjectInSolution {
            get {
                return ResourceManager.GetString("Error_CannotLoadTypeProjectInSolution", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 无法从 Microsoft.Build.dll 中加载类型 Microsoft.Build.Construction.ProjectInSolution.
        /// </summary>
        internal static string Error_CannotLoadTypeProjectInSolution_chs {
            get {
                return ResourceManager.GetString("Error_CannotLoadTypeProjectInSolution_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 無法從 Microsoft.Build.dll 載入類型  Microsoft.Build.Construction.ProjectInSolution.
        /// </summary>
        internal static string Error_CannotLoadTypeProjectInSolution_cht {
            get {
                return ResourceManager.GetString("Error_CannotLoadTypeProjectInSolution_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Nelze načíst typ Microsoft.Build.Construction.ProjectInSolution z knihovny Microsoft.Build.dll..
        /// </summary>
        internal static string Error_CannotLoadTypeProjectInSolution_csy {
            get {
                return ResourceManager.GetString("Error_CannotLoadTypeProjectInSolution_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Der Typ &quot;Microsoft.Build.Construction.ProjectInSolution&quot; kann nicht aus &quot;Microsoft.Build.dll&quot; geladen werden..
        /// </summary>
        internal static string Error_CannotLoadTypeProjectInSolution_deu {
            get {
                return ResourceManager.GetString("Error_CannotLoadTypeProjectInSolution_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to No se puede cargar el tipo Microsoft.Build.Construction.ProjectInSolution desde Microsoft.Build.dll.
        /// </summary>
        internal static string Error_CannotLoadTypeProjectInSolution_esp {
            get {
                return ResourceManager.GetString("Error_CannotLoadTypeProjectInSolution_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Chargement impossible du type Microsoft.Build.Construction.ProjectInSolution depuis Microsoft.Build.dll.
        /// </summary>
        internal static string Error_CannotLoadTypeProjectInSolution_fra {
            get {
                return ResourceManager.GetString("Error_CannotLoadTypeProjectInSolution_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Impossibile caricare Microsoft.Build.Construction.SolutionParser da Microsoft.Build.dll.
        /// </summary>
        internal static string Error_CannotLoadTypeProjectInSolution_ita {
            get {
                return ResourceManager.GetString("Error_CannotLoadTypeProjectInSolution_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Microsoft.Build.dll から Microsoft.Build.Construction.ProjectInSolution 型を読み込めません.
        /// </summary>
        internal static string Error_CannotLoadTypeProjectInSolution_jpn {
            get {
                return ResourceManager.GetString("Error_CannotLoadTypeProjectInSolution_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Microsoft.Build.dll에서 Microsoft.Build.Construction.ProjectInSolution 형식을 로드할 수 없습니다..
        /// </summary>
        internal static string Error_CannotLoadTypeProjectInSolution_kor {
            get {
                return ResourceManager.GetString("Error_CannotLoadTypeProjectInSolution_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Nie można załadować typu rozwiązania Microsoft.Build.Construction.ProjectInSolution z biblioteki Microsoft.Build.dll.
        /// </summary>
        internal static string Error_CannotLoadTypeProjectInSolution_plk {
            get {
                return ResourceManager.GetString("Error_CannotLoadTypeProjectInSolution_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Não foi possível carregar o tipo Microsoft.Build.Construction.ProjectInSolution do Microsoft.Build.dll.
        /// </summary>
        internal static string Error_CannotLoadTypeProjectInSolution_ptb {
            get {
                return ResourceManager.GetString("Error_CannotLoadTypeProjectInSolution_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Не удается загрузить тип Microsoft.Build.Construction.ProjectInSolution из Microsoft.Build.dll.
        /// </summary>
        internal static string Error_CannotLoadTypeProjectInSolution_rus {
            get {
                return ResourceManager.GetString("Error_CannotLoadTypeProjectInSolution_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Microsoft.Build.Construction.ProjectInSolution türü Microsoft.Build.dll üzerinden yüklenemiyor.
        /// </summary>
        internal static string Error_CannotLoadTypeProjectInSolution_trk {
            get {
                return ResourceManager.GetString("Error_CannotLoadTypeProjectInSolution_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Cannot load type Microsoft.Build.Construction.SolutionParser from Microsoft.Build.dll.
        /// </summary>
        internal static string Error_CannotLoadTypeSolutionParser {
            get {
                return ResourceManager.GetString("Error_CannotLoadTypeSolutionParser", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 无法从 Microsoft.Build.dll 中加载类型 Microsoft.Build.Construction.SolutionParser.
        /// </summary>
        internal static string Error_CannotLoadTypeSolutionParser_chs {
            get {
                return ResourceManager.GetString("Error_CannotLoadTypeSolutionParser_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 無法從  Microsoft.Build.dll 載入類型 Microsoft.Build.Construction.SolutionParser.
        /// </summary>
        internal static string Error_CannotLoadTypeSolutionParser_cht {
            get {
                return ResourceManager.GetString("Error_CannotLoadTypeSolutionParser_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Nelze načíst typ Microsoft.Build.Construction.SolutionParser z knihovny Microsoft.Build.dll..
        /// </summary>
        internal static string Error_CannotLoadTypeSolutionParser_csy {
            get {
                return ResourceManager.GetString("Error_CannotLoadTypeSolutionParser_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Der Typ &quot;Microsoft.Build.Construction.SolutionParser&quot; kann nicht aus &quot;Microsoft.Build.dll&quot; geladen werden..
        /// </summary>
        internal static string Error_CannotLoadTypeSolutionParser_deu {
            get {
                return ResourceManager.GetString("Error_CannotLoadTypeSolutionParser_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to No se puede cargar el tipo Microsoft.Build.Construction.SolutionParser desde Microsoft.Build.dll.
        /// </summary>
        internal static string Error_CannotLoadTypeSolutionParser_esp {
            get {
                return ResourceManager.GetString("Error_CannotLoadTypeSolutionParser_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Chargement impossible du type Microsoft.Build.Construction.SolutionParser depuis Microsoft.Build.dll.
        /// </summary>
        internal static string Error_CannotLoadTypeSolutionParser_fra {
            get {
                return ResourceManager.GetString("Error_CannotLoadTypeSolutionParser_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Impossibile caricare Microsoft.Build.Construction.SolutionParser da Microsoft.Build.dll.
        /// </summary>
        internal static string Error_CannotLoadTypeSolutionParser_ita {
            get {
                return ResourceManager.GetString("Error_CannotLoadTypeSolutionParser_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Microsoft.Build.dll から Microsoft.Build.Construction.SolutionParser 型を読み込めません.
        /// </summary>
        internal static string Error_CannotLoadTypeSolutionParser_jpn {
            get {
                return ResourceManager.GetString("Error_CannotLoadTypeSolutionParser_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Microsoft.Build.dll에서 Microsoft.Build.Construction.SolutionParser 형식을 로드할 수 없습니다..
        /// </summary>
        internal static string Error_CannotLoadTypeSolutionParser_kor {
            get {
                return ResourceManager.GetString("Error_CannotLoadTypeSolutionParser_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Nie można załadować typu parsera Microsoft.Build.Construction.SolutionParser z biblioteki Microsoft.Build.dll.
        /// </summary>
        internal static string Error_CannotLoadTypeSolutionParser_plk {
            get {
                return ResourceManager.GetString("Error_CannotLoadTypeSolutionParser_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Não foi possível carregar o tipo Microsoft.Build.Construction.SolutionParser do Microsoft.Build.dll.
        /// </summary>
        internal static string Error_CannotLoadTypeSolutionParser_ptb {
            get {
                return ResourceManager.GetString("Error_CannotLoadTypeSolutionParser_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Не удается загрузить тип Microsoft.Build.Construction.SolutionParser из Microsoft.Build.dll.
        /// </summary>
        internal static string Error_CannotLoadTypeSolutionParser_rus {
            get {
                return ResourceManager.GetString("Error_CannotLoadTypeSolutionParser_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Microsoft.Build.Construction.SolutionParser türü Microsoft.Build.dll konumundan yüklenemiyor.
        /// </summary>
        internal static string Error_CannotLoadTypeSolutionParser_trk {
            get {
                return ResourceManager.GetString("Error_CannotLoadTypeSolutionParser_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Cannot locate a solution file..
        /// </summary>
        internal static string Error_CannotLocateSolutionFile {
            get {
                return ResourceManager.GetString("Error_CannotLocateSolutionFile", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Cannot prompt for input in non-interactive mode..
        /// </summary>
        internal static string Error_CannotPromptForInput {
            get {
                return ResourceManager.GetString("Error_CannotPromptForInput", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 无法在非交互式模式下提示输入。.
        /// </summary>
        internal static string Error_CannotPromptForInput_chs {
            get {
                return ResourceManager.GetString("Error_CannotPromptForInput_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 無法在非互動模式下提供提示。.
        /// </summary>
        internal static string Error_CannotPromptForInput_cht {
            get {
                return ResourceManager.GetString("Error_CannotPromptForInput_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to V neinteraktivním režimu nelze zobrazit výzvu pro zadání vstupu..
        /// </summary>
        internal static string Error_CannotPromptForInput_csy {
            get {
                return ResourceManager.GetString("Error_CannotPromptForInput_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Im nicht interaktiven Modus kann keine Eingabeaufforderung erfolgen..
        /// </summary>
        internal static string Error_CannotPromptForInput_deu {
            get {
                return ResourceManager.GetString("Error_CannotPromptForInput_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to No se puede pedir confirmación de la entrada en un modo no interactivo..
        /// </summary>
        internal static string Error_CannotPromptForInput_esp {
            get {
                return ResourceManager.GetString("Error_CannotPromptForInput_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Le mode non interactif ne permet pas l&apos;invite de saisie de données..
        /// </summary>
        internal static string Error_CannotPromptForInput_fra {
            get {
                return ResourceManager.GetString("Error_CannotPromptForInput_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Impossibile richiedere input in modalità non interattiva.
        /// </summary>
        internal static string Error_CannotPromptForInput_ita {
            get {
                return ResourceManager.GetString("Error_CannotPromptForInput_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 非対話モードの場合、入力のプロンプトを表示できません。.
        /// </summary>
        internal static string Error_CannotPromptForInput_jpn {
            get {
                return ResourceManager.GetString("Error_CannotPromptForInput_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 비대화형 모드에서는 입력을 요청할 수 없습니다..
        /// </summary>
        internal static string Error_CannotPromptForInput_kor {
            get {
                return ResourceManager.GetString("Error_CannotPromptForInput_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Nie można monitować o dane wejściowe w trybie nieinterakcyjnym..
        /// </summary>
        internal static string Error_CannotPromptForInput_plk {
            get {
                return ResourceManager.GetString("Error_CannotPromptForInput_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Não foi possível solicitar a entrada no modo não interativo..
        /// </summary>
        internal static string Error_CannotPromptForInput_ptb {
            get {
                return ResourceManager.GetString("Error_CannotPromptForInput_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Запрос ввода в неинтерактивном режиме невозможен..
        /// </summary>
        internal static string Error_CannotPromptForInput_rus {
            get {
                return ResourceManager.GetString("Error_CannotPromptForInput_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Etkileşimli olmayan modda girdi istenemiyor..
        /// </summary>
        internal static string Error_CannotPromptForInput_trk {
            get {
                return ResourceManager.GetString("Error_CannotPromptForInput_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to This folder contains more than one solution file..
        /// </summary>
        internal static string Error_MultipleSolutions {
            get {
                return ResourceManager.GetString("Error_MultipleSolutions", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 此文件夹包含多个解决方案文件。.
        /// </summary>
        internal static string Error_MultipleSolutions_chs {
            get {
                return ResourceManager.GetString("Error_MultipleSolutions_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 此資料夾包含一個以上的方案檔案。.
        /// </summary>
        internal static string Error_MultipleSolutions_cht {
            get {
                return ResourceManager.GetString("Error_MultipleSolutions_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Tato složka obsahuje více než jeden soubor řešení..
        /// </summary>
        internal static string Error_MultipleSolutions_csy {
            get {
                return ResourceManager.GetString("Error_MultipleSolutions_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Dieser Ordner enthält mehrere Projektdateien..
        /// </summary>
        internal static string Error_MultipleSolutions_deu {
            get {
                return ResourceManager.GetString("Error_MultipleSolutions_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Esta carpeta contiene más de un archivo de la solución..
        /// </summary>
        internal static string Error_MultipleSolutions_esp {
            get {
                return ResourceManager.GetString("Error_MultipleSolutions_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Ce dossier contient plus d&apos;un fichier solution..
        /// </summary>
        internal static string Error_MultipleSolutions_fra {
            get {
                return ResourceManager.GetString("Error_MultipleSolutions_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to La cartella contiene più di un solution file..
        /// </summary>
        internal static string Error_MultipleSolutions_ita {
            get {
                return ResourceManager.GetString("Error_MultipleSolutions_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to このフォルダーには、複数のソリューション ファイルが含まれています。.
        /// </summary>
        internal static string Error_MultipleSolutions_jpn {
            get {
                return ResourceManager.GetString("Error_MultipleSolutions_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 이 폴더에 솔루션 파일이 두 개 이상 포함되어 있습니다..
        /// </summary>
        internal static string Error_MultipleSolutions_kor {
            get {
                return ResourceManager.GetString("Error_MultipleSolutions_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Ten folder zawiera więcej niż jeden plik rozwiązania..
        /// </summary>
        internal static string Error_MultipleSolutions_plk {
            get {
                return ResourceManager.GetString("Error_MultipleSolutions_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Esta pasta contém mais de um arquivo de solução..
        /// </summary>
        internal static string Error_MultipleSolutions_ptb {
            get {
                return ResourceManager.GetString("Error_MultipleSolutions_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Эта папка содержит более одного файла решения..
        /// </summary>
        internal static string Error_MultipleSolutions_rus {
            get {
                return ResourceManager.GetString("Error_MultipleSolutions_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Bu klasör birden çok çözüm dosyası içeriyor..
        /// </summary>
        internal static string Error_MultipleSolutions_trk {
            get {
                return ResourceManager.GetString("Error_MultipleSolutions_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to This folder contains no solution files, nor packages.config files..
        /// </summary>
        internal static string Error_NoSolutionFileNorePackagesConfigFile {
            get {
                return ResourceManager.GetString("Error_NoSolutionFileNorePackagesConfigFile", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 此文件夹既不包含解决方案文件，也不包含 packages.config 文件。.
        /// </summary>
        internal static string Error_NoSolutionFileNorePackagesConfigFile_chs {
            get {
                return ResourceManager.GetString("Error_NoSolutionFileNorePackagesConfigFile_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 此資料夾不包含方案檔案，也沒有 packages.config 檔案。.
        /// </summary>
        internal static string Error_NoSolutionFileNorePackagesConfigFile_cht {
            get {
                return ResourceManager.GetString("Error_NoSolutionFileNorePackagesConfigFile_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Tato složka neobsahuje žádné soubory řešení ani soubory packages.config..
        /// </summary>
        internal static string Error_NoSolutionFileNorePackagesConfigFile_csy {
            get {
                return ResourceManager.GetString("Error_NoSolutionFileNorePackagesConfigFile_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Dieser Ordner enthält keine Projektdateien und keine packages.config-Dateien..
        /// </summary>
        internal static string Error_NoSolutionFileNorePackagesConfigFile_deu {
            get {
                return ResourceManager.GetString("Error_NoSolutionFileNorePackagesConfigFile_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Esta carpeta no contiene ningún archivo de la solución ni archivos packages.config..
        /// </summary>
        internal static string Error_NoSolutionFileNorePackagesConfigFile_esp {
            get {
                return ResourceManager.GetString("Error_NoSolutionFileNorePackagesConfigFile_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Ce dossier ne contient ni fichier solution ni fichier packages.config..
        /// </summary>
        internal static string Error_NoSolutionFileNorePackagesConfigFile_fra {
            get {
                return ResourceManager.GetString("Error_NoSolutionFileNorePackagesConfigFile_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to La cartella non contiene solution file né pacchetti. Config..
        /// </summary>
        internal static string Error_NoSolutionFileNorePackagesConfigFile_ita {
            get {
                return ResourceManager.GetString("Error_NoSolutionFileNorePackagesConfigFile_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to このフォルダーには、ソリューション ファイルと packages.config ファイルが含まれていません。.
        /// </summary>
        internal static string Error_NoSolutionFileNorePackagesConfigFile_jpn {
            get {
                return ResourceManager.GetString("Error_NoSolutionFileNorePackagesConfigFile_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 이 폴더에는 솔루션 파일과 packages.config 파일이 모두 없습니다..
        /// </summary>
        internal static string Error_NoSolutionFileNorePackagesConfigFile_kor {
            get {
                return ResourceManager.GetString("Error_NoSolutionFileNorePackagesConfigFile_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Ten folder nie zawiera ani plików rozwiązań, ani plików packages.config..
        /// </summary>
        internal static string Error_NoSolutionFileNorePackagesConfigFile_plk {
            get {
                return ResourceManager.GetString("Error_NoSolutionFileNorePackagesConfigFile_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Esta pasta não contém arquivos de solução, nem arquivos packages.config..
        /// </summary>
        internal static string Error_NoSolutionFileNorePackagesConfigFile_ptb {
            get {
                return ResourceManager.GetString("Error_NoSolutionFileNorePackagesConfigFile_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Эта папка не содержит ни файлы решений, ни файлы packages.config..
        /// </summary>
        internal static string Error_NoSolutionFileNorePackagesConfigFile_rus {
            get {
                return ResourceManager.GetString("Error_NoSolutionFileNorePackagesConfigFile_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Bu klasör herhangi çözüm dosyası veya packages.config dosyası içermiyor..
        /// </summary>
        internal static string Error_NoSolutionFileNorePackagesConfigFile_trk {
            get {
                return ResourceManager.GetString("Error_NoSolutionFileNorePackagesConfigFile_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Error occurred when processing file &apos;{0}&apos;: {1}.
        /// </summary>
        internal static string Error_ProcessingNuspecFile {
            get {
                return ResourceManager.GetString("Error_ProcessingNuspecFile", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 处理文件“{0}”时出错: {1}.
        /// </summary>
        internal static string Error_ProcessingNuspecFile_chs {
            get {
                return ResourceManager.GetString("Error_ProcessingNuspecFile_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 處理檔案 &apos;{0}&apos; 時發生錯誤: {1}.
        /// </summary>
        internal static string Error_ProcessingNuspecFile_cht {
            get {
                return ResourceManager.GetString("Error_ProcessingNuspecFile_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Při zpracování souboru {0} došlo k chybě: {1}.
        /// </summary>
        internal static string Error_ProcessingNuspecFile_csy {
            get {
                return ResourceManager.GetString("Error_ProcessingNuspecFile_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Fehler beim Verarbeiten der Datei &quot;{0}&quot;: {1}.
        /// </summary>
        internal static string Error_ProcessingNuspecFile_deu {
            get {
                return ResourceManager.GetString("Error_ProcessingNuspecFile_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Error al procesar el archivo &apos;{0}&apos;: {1}.
        /// </summary>
        internal static string Error_ProcessingNuspecFile_esp {
            get {
                return ResourceManager.GetString("Error_ProcessingNuspecFile_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Une erreur est survenue lors du traitement du fichier &apos;{0}&apos; : {1}.
        /// </summary>
        internal static string Error_ProcessingNuspecFile_fra {
            get {
                return ResourceManager.GetString("Error_ProcessingNuspecFile_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Si è verificato un errore nell&apos;elaborazione del file &apos;{0}&apos;: {1}.
        /// </summary>
        internal static string Error_ProcessingNuspecFile_ita {
            get {
                return ResourceManager.GetString("Error_ProcessingNuspecFile_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to ファイル &apos;{0}&apos; の処理中にエラーが発生しました: {1}.
        /// </summary>
        internal static string Error_ProcessingNuspecFile_jpn {
            get {
                return ResourceManager.GetString("Error_ProcessingNuspecFile_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to &apos;{0}&apos; 파일을 처리하는 동안 오류가 발생했습니다. {1}.
        /// </summary>
        internal static string Error_ProcessingNuspecFile_kor {
            get {
                return ResourceManager.GetString("Error_ProcessingNuspecFile_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Wystąpił błąd podczas przetwarzania pliku „{0}”: {1}.
        /// </summary>
        internal static string Error_ProcessingNuspecFile_plk {
            get {
                return ResourceManager.GetString("Error_ProcessingNuspecFile_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Falha ao processar o arquivo &apos;{0}: {1}.
        /// </summary>
        internal static string Error_ProcessingNuspecFile_ptb {
            get {
                return ResourceManager.GetString("Error_ProcessingNuspecFile_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to При обработке файла&quot;{0}&quot; произошла ошибка: {1}.
        /// </summary>
        internal static string Error_ProcessingNuspecFile_rus {
            get {
                return ResourceManager.GetString("Error_ProcessingNuspecFile_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0}&apos; dosyası işlenirken hata oluştu: {1}.
        /// </summary>
        internal static string Error_ProcessingNuspecFile_trk {
            get {
                return ResourceManager.GetString("Error_ProcessingNuspecFile_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Property Settings is null..
        /// </summary>
        internal static string Error_SettingsIsNull {
            get {
                return ResourceManager.GetString("Error_SettingsIsNull", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 属性设置为 null。.
        /// </summary>
        internal static string Error_SettingsIsNull_chs {
            get {
                return ResourceManager.GetString("Error_SettingsIsNull_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 屬性設定為 Null。.
        /// </summary>
        internal static string Error_SettingsIsNull_cht {
            get {
                return ResourceManager.GetString("Error_SettingsIsNull_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Vlastnost Settings má hodnotu null..
        /// </summary>
        internal static string Error_SettingsIsNull_csy {
            get {
                return ResourceManager.GetString("Error_SettingsIsNull_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Die Eigenschafteneinstellungen sind null..
        /// </summary>
        internal static string Error_SettingsIsNull_deu {
            get {
                return ResourceManager.GetString("Error_SettingsIsNull_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to La configuración de propiedad es nula..
        /// </summary>
        internal static string Error_SettingsIsNull_esp {
            get {
                return ResourceManager.GetString("Error_SettingsIsNull_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Les paramètres de propriété sont nuls..
        /// </summary>
        internal static string Error_SettingsIsNull_fra {
            get {
                return ResourceManager.GetString("Error_SettingsIsNull_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Property Setting è nullo..
        /// </summary>
        internal static string Error_SettingsIsNull_ita {
            get {
                return ResourceManager.GetString("Error_SettingsIsNull_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to プロパティ設定が null です。.
        /// </summary>
        internal static string Error_SettingsIsNull_jpn {
            get {
                return ResourceManager.GetString("Error_SettingsIsNull_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Settings 속성이 null입니다..
        /// </summary>
        internal static string Error_SettingsIsNull_kor {
            get {
                return ResourceManager.GetString("Error_SettingsIsNull_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Właściwość Settings ma wartość null..
        /// </summary>
        internal static string Error_SettingsIsNull_plk {
            get {
                return ResourceManager.GetString("Error_SettingsIsNull_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to O item Property Settings é nulo..
        /// </summary>
        internal static string Error_SettingsIsNull_ptb {
            get {
                return ResourceManager.GetString("Error_SettingsIsNull_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Settings свойства имеет значение NULL..
        /// </summary>
        internal static string Error_SettingsIsNull_rus {
            get {
                return ResourceManager.GetString("Error_SettingsIsNull_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Özellik Ayarları null..
        /// </summary>
        internal static string Error_SettingsIsNull_trk {
            get {
                return ResourceManager.GetString("Error_SettingsIsNull_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Property SourceProvider is null..
        /// </summary>
        internal static string Error_SourceProviderIsNull {
            get {
                return ResourceManager.GetString("Error_SourceProviderIsNull", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to SourceProvider 属性为 null。.
        /// </summary>
        internal static string Error_SourceProviderIsNull_chs {
            get {
                return ResourceManager.GetString("Error_SourceProviderIsNull_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 屬性 SourceProvider 為 Null。.
        /// </summary>
        internal static string Error_SourceProviderIsNull_cht {
            get {
                return ResourceManager.GetString("Error_SourceProviderIsNull_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Vlastnost SourceProvider má hodnotu null..
        /// </summary>
        internal static string Error_SourceProviderIsNull_csy {
            get {
                return ResourceManager.GetString("Error_SourceProviderIsNull_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Die Eigenschaft &quot;SourceProvider&quot; ist null..
        /// </summary>
        internal static string Error_SourceProviderIsNull_deu {
            get {
                return ResourceManager.GetString("Error_SourceProviderIsNull_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to La propiedad SourceProvider es nula..
        /// </summary>
        internal static string Error_SourceProviderIsNull_esp {
            get {
                return ResourceManager.GetString("Error_SourceProviderIsNull_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Le @@@fournisseur de source de propriété est nul..
        /// </summary>
        internal static string Error_SourceProviderIsNull_fra {
            get {
                return ResourceManager.GetString("Error_SourceProviderIsNull_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Property sSourceProvider è nullo..
        /// </summary>
        internal static string Error_SourceProviderIsNull_ita {
            get {
                return ResourceManager.GetString("Error_SourceProviderIsNull_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to プロパティ SourceProvider は null です。.
        /// </summary>
        internal static string Error_SourceProviderIsNull_jpn {
            get {
                return ResourceManager.GetString("Error_SourceProviderIsNull_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to SourceProvider 속성이 null입니다..
        /// </summary>
        internal static string Error_SourceProviderIsNull_kor {
            get {
                return ResourceManager.GetString("Error_SourceProviderIsNull_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Właściwość SourceProvider ma wartość null..
        /// </summary>
        internal static string Error_SourceProviderIsNull_plk {
            get {
                return ResourceManager.GetString("Error_SourceProviderIsNull_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to O item SourceProvider de propriedade é nulo..
        /// </summary>
        internal static string Error_SourceProviderIsNull_ptb {
            get {
                return ResourceManager.GetString("Error_SourceProviderIsNull_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to SourceProvider свойства имеет значение NULL..
        /// </summary>
        internal static string Error_SourceProviderIsNull_rus {
            get {
                return ResourceManager.GetString("Error_SourceProviderIsNull_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Özellik SourceProvider null..
        /// </summary>
        internal static string Error_SourceProviderIsNull_trk {
            get {
                return ResourceManager.GetString("Error_SourceProviderIsNull_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Failed to build &apos;{0}&apos;..
        /// </summary>
        internal static string FailedToBuildProject {
            get {
                return ResourceManager.GetString("FailedToBuildProject", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 无法生成“{0}”。.
        /// </summary>
        internal static string FailedToBuildProject_chs {
            get {
                return ResourceManager.GetString("FailedToBuildProject_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 無法建置 &apos;{0}&apos;。.
        /// </summary>
        internal static string FailedToBuildProject_cht {
            get {
                return ResourceManager.GetString("FailedToBuildProject_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Sestavení {0} se nezdařilo..
        /// </summary>
        internal static string FailedToBuildProject_csy {
            get {
                return ResourceManager.GetString("FailedToBuildProject_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Fehler beim Erstellen von &quot;{0}&quot;..
        /// </summary>
        internal static string FailedToBuildProject_deu {
            get {
                return ResourceManager.GetString("FailedToBuildProject_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Error al compilar &apos;{0}&apos;..
        /// </summary>
        internal static string FailedToBuildProject_esp {
            get {
                return ResourceManager.GetString("FailedToBuildProject_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Échec de la création de &apos;{0}&apos;..
        /// </summary>
        internal static string FailedToBuildProject_fra {
            get {
                return ResourceManager.GetString("FailedToBuildProject_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Impossibile sviluppare &apos;{0}&apos;..
        /// </summary>
        internal static string FailedToBuildProject_ita {
            get {
                return ResourceManager.GetString("FailedToBuildProject_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to &apos;{0}&apos; をビルドできませんでした。.
        /// </summary>
        internal static string FailedToBuildProject_jpn {
            get {
                return ResourceManager.GetString("FailedToBuildProject_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to &apos;{0}&apos;을(를) 빌드하지 못했습니다..
        /// </summary>
        internal static string FailedToBuildProject_kor {
            get {
                return ResourceManager.GetString("FailedToBuildProject_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Nie można skompilować „{0}”..
        /// </summary>
        internal static string FailedToBuildProject_plk {
            get {
                return ResourceManager.GetString("FailedToBuildProject_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Falha ao construir &apos;{0}&apos;..
        /// </summary>
        internal static string FailedToBuildProject_ptb {
            get {
                return ResourceManager.GetString("FailedToBuildProject_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Ошибка при сборке &quot;{0}&quot;..
        /// </summary>
        internal static string FailedToBuildProject_rus {
            get {
                return ResourceManager.GetString("FailedToBuildProject_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0}&apos; oluşturulamadı..
        /// </summary>
        internal static string FailedToBuildProject_trk {
            get {
                return ResourceManager.GetString("FailedToBuildProject_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to [Y] Yes  [A] Yes to All  [N] No  [L] No to All ?.
        /// </summary>
        internal static string FileConflictChoiceText {
            get {
                return ResourceManager.GetString("FileConflictChoiceText", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to [Y] 是  [A] 全部是  [N] 否  [L] 全部否 ?.
        /// </summary>
        internal static string FileConflictChoiceText_chs {
            get {
                return ResourceManager.GetString("FileConflictChoiceText_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to [Y] 是  [A] 全部皆是  [N] 否  [L] 全部皆否 ?.
        /// </summary>
        internal static string FileConflictChoiceText_cht {
            get {
                return ResourceManager.GetString("FileConflictChoiceText_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to [Y] Ano [A] Ano pro všechny  [N] Ne  [L] Ne pro všechny?.
        /// </summary>
        internal static string FileConflictChoiceText_csy {
            get {
                return ResourceManager.GetString("FileConflictChoiceText_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to [Y] Ja  [A] Ja, alle  [N] Nein  [L] Nein für alle?.
        /// </summary>
        internal static string FileConflictChoiceText_deu {
            get {
                return ResourceManager.GetString("FileConflictChoiceText_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to [Y] Sí  [A] Sí a todo  [N] No  [L] ¿No a todo?.
        /// </summary>
        internal static string FileConflictChoiceText_esp {
            get {
                return ResourceManager.GetString("FileConflictChoiceText_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to [Y] Oui  [A] Oui pour tout  [N] Non  [L] Non pour tout ?.
        /// </summary>
        internal static string FileConflictChoiceText_fra {
            get {
                return ResourceManager.GetString("FileConflictChoiceText_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to [Y] Sì  [A] Sì a tutto  [N] No  [L] No a tutti ?.
        /// </summary>
        internal static string FileConflictChoiceText_ita {
            get {
                return ResourceManager.GetString("FileConflictChoiceText_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to [Y] はい  [A] すべてはい  [N] いいえ  [L] すべていいえ.
        /// </summary>
        internal static string FileConflictChoiceText_jpn {
            get {
                return ResourceManager.GetString("FileConflictChoiceText_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to [Y] 예  [A] 모두 예  [N] 아니요  [L] 모두 아니요 ?.
        /// </summary>
        internal static string FileConflictChoiceText_kor {
            get {
                return ResourceManager.GetString("FileConflictChoiceText_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to [Y] Tak  [A] Tak dla wszystkich  [N] Nie  [L] Nie dla wszystkich ?.
        /// </summary>
        internal static string FileConflictChoiceText_plk {
            get {
                return ResourceManager.GetString("FileConflictChoiceText_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to [Y] Sim  [A] Sim para todos  [N] Não  [L] Não para todos?.
        /// </summary>
        internal static string FileConflictChoiceText_ptb {
            get {
                return ResourceManager.GetString("FileConflictChoiceText_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to [Y] Да  [A] Да для всех  [N] Нет  [L] Нет для всех ?.
        /// </summary>
        internal static string FileConflictChoiceText_rus {
            get {
                return ResourceManager.GetString("FileConflictChoiceText_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to [Y] Evet  [A] Tümüne Evet  [N] Hayır  [L] Tümüne Hayır ?.
        /// </summary>
        internal static string FileConflictChoiceText_trk {
            get {
                return ResourceManager.GetString("FileConflictChoiceText_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to File &apos;{0}&apos; is not added because the package already contains file &apos;{1}&apos;.
        /// </summary>
        internal static string FileNotAddedToPackage {
            get {
                return ResourceManager.GetString("FileNotAddedToPackage", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 未添加文件“{0}”，因为程序包已包含文件“{1}”.
        /// </summary>
        internal static string FileNotAddedToPackage_chs {
            get {
                return ResourceManager.GetString("FileNotAddedToPackage_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 並未新增檔案 &apos;{0}&apos;，因為封裝已經包含檔案 &apos;{1}&apos;.
        /// </summary>
        internal static string FileNotAddedToPackage_cht {
            get {
                return ResourceManager.GetString("FileNotAddedToPackage_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Soubor {0} není přidán, protože balíček již obsahuje soubor {1}..
        /// </summary>
        internal static string FileNotAddedToPackage_csy {
            get {
                return ResourceManager.GetString("FileNotAddedToPackage_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Die Datei &quot;{0}&quot; wird nicht hinzugefügt, weil das Paket die Datei &quot;{1}&quot; bereits enthält..
        /// </summary>
        internal static string FileNotAddedToPackage_deu {
            get {
                return ResourceManager.GetString("FileNotAddedToPackage_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to No se ha agregado el archivo &apos;{0}&apos; porque el paquete aún contiene el archivo &apos;{1}&apos;.
        /// </summary>
        internal static string FileNotAddedToPackage_esp {
            get {
                return ResourceManager.GetString("FileNotAddedToPackage_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Le fichier &apos;{0}&apos; n&apos;a pas été ajouté car le package contient déjà le fichier &apos;{1}&apos;..
        /// </summary>
        internal static string FileNotAddedToPackage_fra {
            get {
                return ResourceManager.GetString("FileNotAddedToPackage_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Impossibile aggiungere file &apos;{0}&apos; perché il pacchetto contiene già file &apos;{1}&apos;.
        /// </summary>
        internal static string FileNotAddedToPackage_ita {
            get {
                return ResourceManager.GetString("FileNotAddedToPackage_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to パッケージには既にファイル &apos;{1}&apos; が含まれているため、ファイル &apos;{0}&apos; は追加されません.
        /// </summary>
        internal static string FileNotAddedToPackage_jpn {
            get {
                return ResourceManager.GetString("FileNotAddedToPackage_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 패키지에 &apos;{1}&apos; 파일이 이미 있으므로 &apos;{0}&apos; 파일이 추가되지 않았습니다..
        /// </summary>
        internal static string FileNotAddedToPackage_kor {
            get {
                return ResourceManager.GetString("FileNotAddedToPackage_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Plik „{0}” nie został dodany, ponieważ pakiet zawiera już plik „{1}”.
        /// </summary>
        internal static string FileNotAddedToPackage_plk {
            get {
                return ResourceManager.GetString("FileNotAddedToPackage_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to O arquivo &apos;{0}&apos; não é adicionado porque o pacote já contém o arquivo &apos;{1}&apos;.
        /// </summary>
        internal static string FileNotAddedToPackage_ptb {
            get {
                return ResourceManager.GetString("FileNotAddedToPackage_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Файл &quot;{0}&quot; не добавлен, так как в пакете уже есть файл &quot;{1}&quot;.
        /// </summary>
        internal static string FileNotAddedToPackage_rus {
            get {
                return ResourceManager.GetString("FileNotAddedToPackage_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Paket zaten &apos;{1}&apos;dosyası içerdiğinden &apos;{0}&apos; eklenmedi.
        /// </summary>
        internal static string FileNotAddedToPackage_trk {
            get {
                return ResourceManager.GetString("FileNotAddedToPackage_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Found 1 project with a packages.config file. ({0}).
        /// </summary>
        internal static string FoundProject {
            get {
                return ResourceManager.GetString("FoundProject", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 找到一个包含 packages.config 文件的项目。({0}).
        /// </summary>
        internal static string FoundProject_chs {
            get {
                return ResourceManager.GetString("FoundProject_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 找到具有 packages.config 檔案的專案。({0}).
        /// </summary>
        internal static string FoundProject_cht {
            get {
                return ResourceManager.GetString("FoundProject_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Počet nalezených projektů se souborem packages.config: 1. ({0}).
        /// </summary>
        internal static string FoundProject_csy {
            get {
                return ResourceManager.GetString("FoundProject_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Es wurde ein Projekt mit einer Datei &quot;packages.config&quot; gefunden. ({0}).
        /// </summary>
        internal static string FoundProject_deu {
            get {
                return ResourceManager.GetString("FoundProject_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Se ha encontrado 1 proyecto con un archivo packages.config. ({0}).
        /// </summary>
        internal static string FoundProject_esp {
            get {
                return ResourceManager.GetString("FoundProject_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Un projet contenant packages.config a été trouvé. ({0}).
        /// </summary>
        internal static string FoundProject_fra {
            get {
                return ResourceManager.GetString("FoundProject_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Trovato 1 progetto con packages.config file. ({0}).
        /// </summary>
        internal static string FoundProject_ita {
            get {
                return ResourceManager.GetString("FoundProject_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to packages.config ファイルが指定されたプロジェクトが 1 個見つかりました。({0}).
        /// </summary>
        internal static string FoundProject_jpn {
            get {
                return ResourceManager.GetString("FoundProject_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to packages.config 파일이 있는 프로젝트를 1개 찾았습니다. ({0}).
        /// </summary>
        internal static string FoundProject_kor {
            get {
                return ResourceManager.GetString("FoundProject_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Znaleziono 1 projekt z plikiem packages.config. ({0}).
        /// </summary>
        internal static string FoundProject_plk {
            get {
                return ResourceManager.GetString("FoundProject_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Encontrado um projeto com um arquivo packages.config. ({0}).
        /// </summary>
        internal static string FoundProject_ptb {
            get {
                return ResourceManager.GetString("FoundProject_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Обнаружен 1 проект с файлом packages.config. ({0}).
        /// </summary>
        internal static string FoundProject_rus {
            get {
                return ResourceManager.GetString("FoundProject_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to packages.config dosyası içeren 1 proje bulundu. ({0}).
        /// </summary>
        internal static string FoundProject_trk {
            get {
                return ResourceManager.GetString("FoundProject_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Found {0} projects with a packages.config file. ({1}).
        /// </summary>
        internal static string FoundProjects {
            get {
                return ResourceManager.GetString("FoundProjects", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 找到 {0} 个包含 packages.config 文件的项目。({1}).
        /// </summary>
        internal static string FoundProjects_chs {
            get {
                return ResourceManager.GetString("FoundProjects_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 找到具有 packages.confi 檔案的 {0} 個專案。({1}).
        /// </summary>
        internal static string FoundProjects_cht {
            get {
                return ResourceManager.GetString("FoundProjects_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Počet nalezených projektů se souborem packages.config: {0}. ({1}).
        /// </summary>
        internal static string FoundProjects_csy {
            get {
                return ResourceManager.GetString("FoundProjects_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Es wurden {0} Projekte mit einer Datei &quot;packages.config&quot; gefunden. ({1}).
        /// </summary>
        internal static string FoundProjects_deu {
            get {
                return ResourceManager.GetString("FoundProjects_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Se han encontrado {0} proyectos con un archivo packages.config. ({1}).
        /// </summary>
        internal static string FoundProjects_esp {
            get {
                return ResourceManager.GetString("FoundProjects_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} projets contenant un fichier packages.config ont été trouvés. ({1}).
        /// </summary>
        internal static string FoundProjects_fra {
            get {
                return ResourceManager.GetString("FoundProjects_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Trovati {0} progetti con packages.config file. ({1}).
        /// </summary>
        internal static string FoundProjects_ita {
            get {
                return ResourceManager.GetString("FoundProjects_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to packages.config ファイルが指定されたプロジェクトが {0} 個見つかりました。({1}).
        /// </summary>
        internal static string FoundProjects_jpn {
            get {
                return ResourceManager.GetString("FoundProjects_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to packages.config 파일이 있는 프로젝트를 {0}개 찾았습니다. ({1}).
        /// </summary>
        internal static string FoundProjects_kor {
            get {
                return ResourceManager.GetString("FoundProjects_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Liczba znalezionych projektów z plikiem  packages.config: {0}. ({1}).
        /// </summary>
        internal static string FoundProjects_plk {
            get {
                return ResourceManager.GetString("FoundProjects_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Encontrados projetos {0} com um arquivo packages.config. ({1}).
        /// </summary>
        internal static string FoundProjects_ptb {
            get {
                return ResourceManager.GetString("FoundProjects_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Обнаружено проектов с файлом packages.config: {0}. ({1}).
        /// </summary>
        internal static string FoundProjects_rus {
            get {
                return ResourceManager.GetString("FoundProjects_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to packages.config dosyası içeren {0} proje bulundu. ({1}).
        /// </summary>
        internal static string FoundProjects_trk {
            get {
                return ResourceManager.GetString("FoundProjects_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to &apos;{0}&apos; contains invalid package references. .
        /// </summary>
        internal static string InstallCommandInvalidPackageReference {
            get {
                return ResourceManager.GetString("InstallCommandInvalidPackageReference", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to “{0}”包含的程序包引用无效。.
        /// </summary>
        internal static string InstallCommandInvalidPackageReference_chs {
            get {
                return ResourceManager.GetString("InstallCommandInvalidPackageReference_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0}&apos; 包含無效的封裝參照。.
        /// </summary>
        internal static string InstallCommandInvalidPackageReference_cht {
            get {
                return ResourceManager.GetString("InstallCommandInvalidPackageReference_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} obsahuje neplatné odkazy na balíčky. .
        /// </summary>
        internal static string InstallCommandInvalidPackageReference_csy {
            get {
                return ResourceManager.GetString("InstallCommandInvalidPackageReference_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to &quot;{0}&quot; enthält ungültige Paketverweise. .
        /// </summary>
        internal static string InstallCommandInvalidPackageReference_deu {
            get {
                return ResourceManager.GetString("InstallCommandInvalidPackageReference_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0}&apos; contiene referencias de paquete no válidas..
        /// </summary>
        internal static string InstallCommandInvalidPackageReference_esp {
            get {
                return ResourceManager.GetString("InstallCommandInvalidPackageReference_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to &apos;{0}&apos; contient des références de package non valides. .
        /// </summary>
        internal static string InstallCommandInvalidPackageReference_fra {
            get {
                return ResourceManager.GetString("InstallCommandInvalidPackageReference_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to &apos;{0}&apos; contains invalid package references. .
        /// </summary>
        internal static string InstallCommandInvalidPackageReference_ita {
            get {
                return ResourceManager.GetString("InstallCommandInvalidPackageReference_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to &apos;{0}&apos; には無効なパッケージ参照が含まれています。.
        /// </summary>
        internal static string InstallCommandInvalidPackageReference_jpn {
            get {
                return ResourceManager.GetString("InstallCommandInvalidPackageReference_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to &apos;{0}&apos;에 잘못된 패키지 참조가 포함되어 있습니다. .
        /// </summary>
        internal static string InstallCommandInvalidPackageReference_kor {
            get {
                return ResourceManager.GetString("InstallCommandInvalidPackageReference_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to „{0}” zawiera nieprawidłowe odwołania do pakietów. .
        /// </summary>
        internal static string InstallCommandInvalidPackageReference_plk {
            get {
                return ResourceManager.GetString("InstallCommandInvalidPackageReference_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to &apos;{0}&apos; contém referências de pacotes inválidos..
        /// </summary>
        internal static string InstallCommandInvalidPackageReference_ptb {
            get {
                return ResourceManager.GetString("InstallCommandInvalidPackageReference_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to &quot;{0}&quot; содержит недопустимые ссылки на пакеты. .
        /// </summary>
        internal static string InstallCommandInvalidPackageReference_rus {
            get {
                return ResourceManager.GetString("InstallCommandInvalidPackageReference_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0}&apos; geçersiz paket başvuruları içeriyor. .
        /// </summary>
        internal static string InstallCommandInvalidPackageReference_trk {
            get {
                return ResourceManager.GetString("InstallCommandInvalidPackageReference_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to All packages listed in {0} are already installed..
        /// </summary>
        internal static string InstallCommandNothingToInstall {
            get {
                return ResourceManager.GetString("InstallCommandNothingToInstall", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} 中列出的所有程序包均已安装。.
        /// </summary>
        internal static string InstallCommandNothingToInstall_chs {
            get {
                return ResourceManager.GetString("InstallCommandNothingToInstall_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} 中列出的所有封裝均已安裝。.
        /// </summary>
        internal static string InstallCommandNothingToInstall_cht {
            get {
                return ResourceManager.GetString("InstallCommandNothingToInstall_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Všechny balíčky, které uvádí {0}, jsou již nainstalované..
        /// </summary>
        internal static string InstallCommandNothingToInstall_csy {
            get {
                return ResourceManager.GetString("InstallCommandNothingToInstall_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Alle in &quot;{0}&quot; aufgeführten Pakete sind bereits installiert..
        /// </summary>
        internal static string InstallCommandNothingToInstall_deu {
            get {
                return ResourceManager.GetString("InstallCommandNothingToInstall_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Ya se han instalado todos los paquetes mostrados en &apos;{0}&apos;..
        /// </summary>
        internal static string InstallCommandNothingToInstall_esp {
            get {
                return ResourceManager.GetString("InstallCommandNothingToInstall_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Tous les packages répertoriés dans {0} sont déjà installés..
        /// </summary>
        internal static string InstallCommandNothingToInstall_fra {
            get {
                return ResourceManager.GetString("InstallCommandNothingToInstall_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Tutti i pacchetti elencati in {0}sono già installati..
        /// </summary>
        internal static string InstallCommandNothingToInstall_ita {
            get {
                return ResourceManager.GetString("InstallCommandNothingToInstall_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} に含まれるすべてのパッケージは、既にインストールされています。.
        /// </summary>
        internal static string InstallCommandNothingToInstall_jpn {
            get {
                return ResourceManager.GetString("InstallCommandNothingToInstall_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0}에 나열된 모든 패키지가 이미 설치되었습니다..
        /// </summary>
        internal static string InstallCommandNothingToInstall_kor {
            get {
                return ResourceManager.GetString("InstallCommandNothingToInstall_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Wszystkie pakiety wymienione w {0} zostały już zainstalowane..
        /// </summary>
        internal static string InstallCommandNothingToInstall_plk {
            get {
                return ResourceManager.GetString("InstallCommandNothingToInstall_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Todos os pacotes listados em {0} já estão instalados..
        /// </summary>
        internal static string InstallCommandNothingToInstall_ptb {
            get {
                return ResourceManager.GetString("InstallCommandNothingToInstall_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Все пакеты, перечисленные в {0}, уже установлены..
        /// </summary>
        internal static string InstallCommandNothingToInstall_rus {
            get {
                return ResourceManager.GetString("InstallCommandNothingToInstall_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} içinde sıralanan tüm paketler zaten yüklü..
        /// </summary>
        internal static string InstallCommandNothingToInstall_trk {
            get {
                return ResourceManager.GetString("InstallCommandNothingToInstall_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Package &quot;{0}&quot; is already installed..
        /// </summary>
        internal static string InstallCommandPackageAlreadyExists {
            get {
                return ResourceManager.GetString("InstallCommandPackageAlreadyExists", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 已安装程序包“{0}”。.
        /// </summary>
        internal static string InstallCommandPackageAlreadyExists_chs {
            get {
                return ResourceManager.GetString("InstallCommandPackageAlreadyExists_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 已安裝封裝 &quot;{0}&quot;。.
        /// </summary>
        internal static string InstallCommandPackageAlreadyExists_cht {
            get {
                return ResourceManager.GetString("InstallCommandPackageAlreadyExists_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Balíček {0} je již nainstalován..
        /// </summary>
        internal static string InstallCommandPackageAlreadyExists_csy {
            get {
                return ResourceManager.GetString("InstallCommandPackageAlreadyExists_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Das Paket &quot;{0}&quot; ist bereits installiert..
        /// </summary>
        internal static string InstallCommandPackageAlreadyExists_deu {
            get {
                return ResourceManager.GetString("InstallCommandPackageAlreadyExists_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Ya se ha instalado el paquete &quot;{0}&quot;..
        /// </summary>
        internal static string InstallCommandPackageAlreadyExists_esp {
            get {
                return ResourceManager.GetString("InstallCommandPackageAlreadyExists_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Le package « {0} » est déjà installé..
        /// </summary>
        internal static string InstallCommandPackageAlreadyExists_fra {
            get {
                return ResourceManager.GetString("InstallCommandPackageAlreadyExists_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Pacchetto &quot;{0}&quot; è già installato.
        /// </summary>
        internal static string InstallCommandPackageAlreadyExists_ita {
            get {
                return ResourceManager.GetString("InstallCommandPackageAlreadyExists_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to パッケージ &quot;{0}&quot; は既にインストールされています。.
        /// </summary>
        internal static string InstallCommandPackageAlreadyExists_jpn {
            get {
                return ResourceManager.GetString("InstallCommandPackageAlreadyExists_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} 패키지는 이미 설치되었습니다..
        /// </summary>
        internal static string InstallCommandPackageAlreadyExists_kor {
            get {
                return ResourceManager.GetString("InstallCommandPackageAlreadyExists_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Pakiet „{0}” został już zainstalowany..
        /// </summary>
        internal static string InstallCommandPackageAlreadyExists_plk {
            get {
                return ResourceManager.GetString("InstallCommandPackageAlreadyExists_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to O pacote &quot;{0}&quot; já está instalado..
        /// </summary>
        internal static string InstallCommandPackageAlreadyExists_ptb {
            get {
                return ResourceManager.GetString("InstallCommandPackageAlreadyExists_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Пакет &quot;{0}&quot; уже установлен..
        /// </summary>
        internal static string InstallCommandPackageAlreadyExists_rus {
            get {
                return ResourceManager.GetString("InstallCommandPackageAlreadyExists_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to &quot;{0}&quot; paketi zaten yüklü..
        /// </summary>
        internal static string InstallCommandPackageAlreadyExists_trk {
            get {
                return ResourceManager.GetString("InstallCommandPackageAlreadyExists_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Version string specified for package reference &apos;{0}&apos; is invalid..
        /// </summary>
        internal static string InstallCommandPackageReferenceInvalidVersion {
            get {
                return ResourceManager.GetString("InstallCommandPackageReferenceInvalidVersion", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 为程序包引用“{0}”指定的版本字符串无效。.
        /// </summary>
        internal static string InstallCommandPackageReferenceInvalidVersion_chs {
            get {
                return ResourceManager.GetString("InstallCommandPackageReferenceInvalidVersion_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 封裝參照 &apos;{0}&apos; 指定的版本字串無效。.
        /// </summary>
        internal static string InstallCommandPackageReferenceInvalidVersion_cht {
            get {
                return ResourceManager.GetString("InstallCommandPackageReferenceInvalidVersion_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Řetězec verze zadaný pro odkaz na balíček {0} je neplatný..
        /// </summary>
        internal static string InstallCommandPackageReferenceInvalidVersion_csy {
            get {
                return ResourceManager.GetString("InstallCommandPackageReferenceInvalidVersion_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Die für den Paketverweis &quot;{0}&quot; angegebene Versionszeichenfolge ist ungültig..
        /// </summary>
        internal static string InstallCommandPackageReferenceInvalidVersion_deu {
            get {
                return ResourceManager.GetString("InstallCommandPackageReferenceInvalidVersion_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to La cadena de versión especificada para la referencia de paquete &apos;{0}&apos; no es válida..
        /// </summary>
        internal static string InstallCommandPackageReferenceInvalidVersion_esp {
            get {
                return ResourceManager.GetString("InstallCommandPackageReferenceInvalidVersion_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to La chaîne de version spécifiée pour la référence de package &apos;{0}&apos; n&apos;est pas valide..
        /// </summary>
        internal static string InstallCommandPackageReferenceInvalidVersion_fra {
            get {
                return ResourceManager.GetString("InstallCommandPackageReferenceInvalidVersion_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Stringa versione specificata per il pacchetto di riferimento &apos;{0}&apos; non è valida..
        /// </summary>
        internal static string InstallCommandPackageReferenceInvalidVersion_ita {
            get {
                return ResourceManager.GetString("InstallCommandPackageReferenceInvalidVersion_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to パッケージ参照 &apos;{0}&apos; に指定されたバージョン文字列は無効です。.
        /// </summary>
        internal static string InstallCommandPackageReferenceInvalidVersion_jpn {
            get {
                return ResourceManager.GetString("InstallCommandPackageReferenceInvalidVersion_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 패키지 참조 &apos;{0}&apos;에 대해 지정된 버전 문자열이 잘못되었습니다..
        /// </summary>
        internal static string InstallCommandPackageReferenceInvalidVersion_kor {
            get {
                return ResourceManager.GetString("InstallCommandPackageReferenceInvalidVersion_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Ciąg wersji określony dla odwołania do pakietu „{0}” jest nieprawidłowy..
        /// </summary>
        internal static string InstallCommandPackageReferenceInvalidVersion_plk {
            get {
                return ResourceManager.GetString("InstallCommandPackageReferenceInvalidVersion_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to A cadeia de caracteres de versão especificada para a referência do pacote &apos;{0}&apos; é inválida..
        /// </summary>
        internal static string InstallCommandPackageReferenceInvalidVersion_ptb {
            get {
                return ResourceManager.GetString("InstallCommandPackageReferenceInvalidVersion_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Строка версии, указанная для ссылки на пакет &quot;{0}&quot;, является недопустимой..
        /// </summary>
        internal static string InstallCommandPackageReferenceInvalidVersion_rus {
            get {
                return ResourceManager.GetString("InstallCommandPackageReferenceInvalidVersion_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0}&apos; paket referansı için belirtilen sürüm dizesi geçersiz..
        /// </summary>
        internal static string InstallCommandPackageReferenceInvalidVersion_trk {
            get {
                return ResourceManager.GetString("InstallCommandPackageReferenceInvalidVersion_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Package restore is disabled. To enable it, open the Visual Studio Options dialog, click on Package Manager node and check &apos;{0}&apos;. You can also enable package restore by setting the environment variable &apos;EnableNuGetPackageRestore&apos; to &apos;true&apos;..
        /// </summary>
        internal static string InstallCommandPackageRestoreConsentNotFound {
            get {
                return ResourceManager.GetString("InstallCommandPackageRestoreConsentNotFound", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 已禁用程序包还原。若要启用它，请打开“Visual Studio 选项”对话框，单击“程序包管理器”节点并检查“{0}”。你还可以通过将环境变量 &quot;EnableNuGetPackageRestore&quot; 设置为 &quot;true&quot; 来启用程序包还原。.
        /// </summary>
        internal static string InstallCommandPackageRestoreConsentNotFound_chs {
            get {
                return ResourceManager.GetString("InstallCommandPackageRestoreConsentNotFound_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 已停用封裝還原。若要啟用，開啟 Visual Studio 選項對話，按一下 [封裝管理員] 節點並勾選 &apos;{0}&apos;。您也可以設定環境變數 &apos;EnableNuGetPackageRestore&apos; 為 &apos;true&apos; 以啟用封裝還原。.
        /// </summary>
        internal static string InstallCommandPackageRestoreConsentNotFound_cht {
            get {
                return ResourceManager.GetString("InstallCommandPackageRestoreConsentNotFound_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Obnovení balíčku je zakázáno. Chcete-li je povolit, otevřete dialogové okno Možnosti sady Visual Studio, klikněte na uzel Správce balíčků a zaškrtněte položku {0}. Obnovení balíčku můžete rovněž povolit nastavením proměnné prostředí EnableNuGetPackageRestore na hodnotu true..
        /// </summary>
        internal static string InstallCommandPackageRestoreConsentNotFound_csy {
            get {
                return ResourceManager.GetString("InstallCommandPackageRestoreConsentNotFound_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Die Paketwiederherstellung ist deaktiviert. Öffnen Sie zum Aktivieren das Dialogfeld &quot;Optionen&quot; von Visual Studio, klicken Sie auf den Knoten &quot;Paket-Manager&quot;, und aktivieren Sie dann &quot;{0}&quot;. Sie können die Paketwiederherstellung auch aktivieren, indem Sie die Umgebungsvariable &quot;EnableNuGetPackageRestore&quot; auf &quot;true&quot; festlegen..
        /// </summary>
        internal static string InstallCommandPackageRestoreConsentNotFound_deu {
            get {
                return ResourceManager.GetString("InstallCommandPackageRestoreConsentNotFound_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Se ha deshabilitado la restauración del paquete. Para habilitarla, abra el cuadro de diálogo Opciones de Visual Studio, haga clic en el nodo del Administrador de paquetes y compruebe &apos;{0}&apos;. Para habilitar la restauración de paquetes, también puede configurar la variable de entorno &apos;EnableNuGetPackageRestore&apos; a &apos;true&apos;..
        /// </summary>
        internal static string InstallCommandPackageRestoreConsentNotFound_esp {
            get {
                return ResourceManager.GetString("InstallCommandPackageRestoreConsentNotFound_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to La restauration de package est désactivée. Pour l&apos;activer, ouvrez la boîte de dialogue Options Visual Studio, cliquez sur le nœud Gestionnaire de package et cochez &apos;{0}&apos;. Vous pouvez également activer la restauration de package en définissant la variable d&apos;environnement &apos;EnableNuGetPackageRestore&apos; avec la valeur &apos;true&apos;..
        /// </summary>
        internal static string InstallCommandPackageRestoreConsentNotFound_fra {
            get {
                return ResourceManager.GetString("InstallCommandPackageRestoreConsentNotFound_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Ripristino pacchetto disabilitato. Per abilitarlo, aprire la finestra di dialogo Visual Studio Options, fare clic su Package Manager node e verificare &apos;{0}&apos;. È possibile anche abilitare il pacchetto impostando la variabile ambiente &apos;EnableNuGetPackageRestore&apos; su &apos;vero&apos;..
        /// </summary>
        internal static string InstallCommandPackageRestoreConsentNotFound_ita {
            get {
                return ResourceManager.GetString("InstallCommandPackageRestoreConsentNotFound_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to パッケージの復元は無効です。有効にするには、Visual Studio の [オプション] ダイアログを開き、パッケージ マネージャー ノードをクリックし、&apos;{0}&apos; を確認します。環境変数 &apos;EnableNuGetPackageRestore&apos; を &apos;true&apos; に設定して、パッケージの復元を有効にすることもできます。.
        /// </summary>
        internal static string InstallCommandPackageRestoreConsentNotFound_jpn {
            get {
                return ResourceManager.GetString("InstallCommandPackageRestoreConsentNotFound_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 패키지 복원을 사용할 수 없습니다. 패키지 복원을 사용하도록 설정하려면 [Visual Studio 옵션] 대화 상자를 열고 [패키지 관리자] 노드를 클릭한 후 &apos;{0}&apos;을(를) 선택합니다. 또한 환경 변수 &apos;EnableNuGetPackageRestore&apos;를 &apos;true&apos;로 설정하여 패키지 복원을 사용하도록 설정할 수 있습니다..
        /// </summary>
        internal static string InstallCommandPackageRestoreConsentNotFound_kor {
            get {
                return ResourceManager.GetString("InstallCommandPackageRestoreConsentNotFound_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Przywracanie pakietu jest wyłączone. Aby je włączyć, otwórz okno dialogowe opcji programu Visual Studio, kliknij węzeł Menedżera pakietów i zaznacz pozycję „{0}”. Przywracanie pakietów możesz także włączyć, ustawiając wartość „true” dla zmiennej środowiskowej „EnableNuGetPackageRestore”..
        /// </summary>
        internal static string InstallCommandPackageRestoreConsentNotFound_plk {
            get {
                return ResourceManager.GetString("InstallCommandPackageRestoreConsentNotFound_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to A restauração do pacote está desativada. Para ativá-la, abra a caixa de diálogo Opções do Visual Studio, clique no nó do Gerenciador de Pacotes e marque &apos;{0}&apos;. Você também pode ativar a restauração do pacote definindo a variável de ambiente &apos;EnableNuGetPackageRestore&apos; como &quot;true&quot;..
        /// </summary>
        internal static string InstallCommandPackageRestoreConsentNotFound_ptb {
            get {
                return ResourceManager.GetString("InstallCommandPackageRestoreConsentNotFound_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Восстановление пакетов отключено. Чтобы включить его, в Visual Studio откройте диалоговое окно &quot;Параметры&quot;, выберите узел &quot;Диспетчер пакетов&quot; и проверьте &quot;{0}&quot;. Кроме того, чтобы включить восстановление пакетов, можно для переменной среды &quot;EnableNuGetPackageRestore&quot; задать значение &quot;true&quot;..
        /// </summary>
        internal static string InstallCommandPackageRestoreConsentNotFound_rus {
            get {
                return ResourceManager.GetString("InstallCommandPackageRestoreConsentNotFound_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Paket geri yükleme devre dışı. Etkinleştirmek için Visual Studio Seçenekler iletişim kutusunu açın, Paket Yöneticisi düğümünü tıklayın ve &apos;{0}&apos; öğesini kontrol edin. Paket geri yüklemesini ayrıca &apos;EnableNuGetPackageRestore&apos; ortam değişkenini &apos;doğru&apos; olarak ayarlayarak da etkinleştirebilirsiniz..
        /// </summary>
        internal static string InstallCommandPackageRestoreConsentNotFound_trk {
            get {
                return ResourceManager.GetString("InstallCommandPackageRestoreConsentNotFound_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0}: invalid arguments..
        /// </summary>
        internal static string InvalidArguments {
            get {
                return ResourceManager.GetString("InvalidArguments", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0}: 参数无效。.
        /// </summary>
        internal static string InvalidArguments_chs {
            get {
                return ResourceManager.GetString("InvalidArguments_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0}: 無效的引數。.
        /// </summary>
        internal static string InvalidArguments_cht {
            get {
                return ResourceManager.GetString("InvalidArguments_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0}: neplatné argumenty.
        /// </summary>
        internal static string InvalidArguments_csy {
            get {
                return ResourceManager.GetString("InvalidArguments_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0}: ungültige Argumente..
        /// </summary>
        internal static string InvalidArguments_deu {
            get {
                return ResourceManager.GetString("InvalidArguments_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0}: argumentos no válidos..
        /// </summary>
        internal static string InvalidArguments_esp {
            get {
                return ResourceManager.GetString("InvalidArguments_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} : arguments non valides..
        /// </summary>
        internal static string InvalidArguments_fra {
            get {
                return ResourceManager.GetString("InvalidArguments_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0}: argomenti non validi..
        /// </summary>
        internal static string InvalidArguments_ita {
            get {
                return ResourceManager.GetString("InvalidArguments_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0}: 引数が無効です。.
        /// </summary>
        internal static string InvalidArguments_jpn {
            get {
                return ResourceManager.GetString("InvalidArguments_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0}: 잘못된 인수입니다..
        /// </summary>
        internal static string InvalidArguments_kor {
            get {
                return ResourceManager.GetString("InvalidArguments_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0}: nieprawidłowe argumenty..
        /// </summary>
        internal static string InvalidArguments_plk {
            get {
                return ResourceManager.GetString("InvalidArguments_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0}: argumentos inválidos..
        /// </summary>
        internal static string InvalidArguments_ptb {
            get {
                return ResourceManager.GetString("InvalidArguments_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0}: недопустимые аргументы..
        /// </summary>
        internal static string InvalidArguments_rus {
            get {
                return ResourceManager.GetString("InvalidArguments_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0}: geçersiz bağımsız değişken..
        /// </summary>
        internal static string InvalidArguments_trk {
            get {
                return ResourceManager.GetString("InvalidArguments_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to No packages.config, project or solution file specified. Use the -self switch to update NuGet.exe..
        /// </summary>
        internal static string InvalidFile {
            get {
                return ResourceManager.GetString("InvalidFile", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 未指定 packages.config 或解决方案文件。请使用 -self 开关更新 NuGet.exe。.
        /// </summary>
        internal static string InvalidFile_chs {
            get {
                return ResourceManager.GetString("InvalidFile_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 未指定 packages.config 或方案檔案。使用 -self 切換以更新 NuGet.exe。.
        /// </summary>
        internal static string InvalidFile_cht {
            get {
                return ResourceManager.GetString("InvalidFile_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Nebyl zadán žádný soubor packages.config ani soubor řešení. K aktualizaci souboru NuGet.exe použijte přepínač -self..
        /// </summary>
        internal static string InvalidFile_csy {
            get {
                return ResourceManager.GetString("InvalidFile_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Es wurde keine Datei &quot;packages.config&quot; oder Projektdatei angegeben. Verwenden Sie den Schalter &quot;-self&quot;, um &quot;NuGet.exe&quot; zu aktualisieren..
        /// </summary>
        internal static string InvalidFile_deu {
            get {
                return ResourceManager.GetString("InvalidFile_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to No se ha especificado ningún archivo de soluciones ni packages.config. Use el conmutador automático para actualizar NuGet.exe..
        /// </summary>
        internal static string InvalidFile_esp {
            get {
                return ResourceManager.GetString("InvalidFile_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Aucun fichier packages.config ou solution spécifié. Utilisez le commutateur @@@-self pour mettre à jour NuGet.exe..
        /// </summary>
        internal static string InvalidFile_fra {
            get {
                return ResourceManager.GetString("InvalidFile_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Nessun packages.config o solution file specificati. Usare il self switch per aggiornare a NuGet.exe..
        /// </summary>
        internal static string InvalidFile_ita {
            get {
                return ResourceManager.GetString("InvalidFile_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to packages.config またはソリューション ファイルが指定されていません。NuGet.exe を更新するには、-self スイッチを使用してください。.
        /// </summary>
        internal static string InvalidFile_jpn {
            get {
                return ResourceManager.GetString("InvalidFile_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to packages.config 또는 솔루션 파일이 지정되지 않았습니다. -self 스위치를 사용하여 NuGet.exe를 업데이트하십시오..
        /// </summary>
        internal static string InvalidFile_kor {
            get {
                return ResourceManager.GetString("InvalidFile_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Nie określono pliku packages.config ani pliku rozwiązania. Użyj przełącznika -self, aby zaktualizować pakiet NuGet.exe..
        /// </summary>
        internal static string InvalidFile_plk {
            get {
                return ResourceManager.GetString("InvalidFile_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Nenhum arquivo packages.config ou de solução especificada. Use o botão automático para atualizar o NuGet.exe..
        /// </summary>
        internal static string InvalidFile_ptb {
            get {
                return ResourceManager.GetString("InvalidFile_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Не указан файл packages.config или файл решения. Используйте параметр -self для обновления NuGet.exe..
        /// </summary>
        internal static string InvalidFile_rus {
            get {
                return ResourceManager.GetString("InvalidFile_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Herhangi bir packages.config veya çözüm dosyası belirtilmedi. NuGet.exe öğesini güncellemek için -self anahtarını kullanın ..
        /// </summary>
        internal static string InvalidFile_trk {
            get {
                return ResourceManager.GetString("InvalidFile_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Invalid option value: &apos;{0} {1}&apos;.
        /// </summary>
        internal static string InvalidOptionValueError {
            get {
                return ResourceManager.GetString("InvalidOptionValueError", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 无效的选项值:“{0} {1}”.
        /// </summary>
        internal static string InvalidOptionValueError_chs {
            get {
                return ResourceManager.GetString("InvalidOptionValueError_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 無效的選項值: &apos;{0} {1}&apos;.
        /// </summary>
        internal static string InvalidOptionValueError_cht {
            get {
                return ResourceManager.GetString("InvalidOptionValueError_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Neplatná hodnota možnosti: &apos;{0} {1}&apos;.
        /// </summary>
        internal static string InvalidOptionValueError_csy {
            get {
                return ResourceManager.GetString("InvalidOptionValueError_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Ungültiger Optionswert: &quot;{0} {1}&quot;.
        /// </summary>
        internal static string InvalidOptionValueError_deu {
            get {
                return ResourceManager.GetString("InvalidOptionValueError_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Valor de opción no válido: &apos;{0} {1}&apos;.
        /// </summary>
        internal static string InvalidOptionValueError_esp {
            get {
                return ResourceManager.GetString("InvalidOptionValueError_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Valeur de l&apos;option non valide : &apos;{0} {1}&apos;.
        /// </summary>
        internal static string InvalidOptionValueError_fra {
            get {
                return ResourceManager.GetString("InvalidOptionValueError_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Valore opzione non valido: &apos;{0} {1}&apos;.
        /// </summary>
        internal static string InvalidOptionValueError_ita {
            get {
                return ResourceManager.GetString("InvalidOptionValueError_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 無効なオプションの値: &apos;{0} {1}&apos;.
        /// </summary>
        internal static string InvalidOptionValueError_jpn {
            get {
                return ResourceManager.GetString("InvalidOptionValueError_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 잘못된 옵션 값: &apos;{0} {1}&apos;.
        /// </summary>
        internal static string InvalidOptionValueError_kor {
            get {
                return ResourceManager.GetString("InvalidOptionValueError_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Nieprawidłowa wartość opcji: „{0} {1}”.
        /// </summary>
        internal static string InvalidOptionValueError_plk {
            get {
                return ResourceManager.GetString("InvalidOptionValueError_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Valor de opção inválida: &apos;{0} {1}&apos;.
        /// </summary>
        internal static string InvalidOptionValueError_ptb {
            get {
                return ResourceManager.GetString("InvalidOptionValueError_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Недопустимое значение параметра: &apos;{0} {1}&apos;.
        /// </summary>
        internal static string InvalidOptionValueError_rus {
            get {
                return ResourceManager.GetString("InvalidOptionValueError_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Geçersiz seçenek değeri: &apos;{0} {1}&apos;.
        /// </summary>
        internal static string InvalidOptionValueError_trk {
            get {
                return ResourceManager.GetString("InvalidOptionValueError_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to The specified source &apos;{0}&apos; is invalid. Please provide a valid source..
        /// </summary>
        internal static string InvalidSource {
            get {
                return ResourceManager.GetString("InvalidSource", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 指定的源“{0}”无效。请提供有效源。.
        /// </summary>
        internal static string InvalidSource_chs {
            get {
                return ResourceManager.GetString("InvalidSource_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 指定的來源 &apos;{0}&apos; 無效。請提供有效的來源。.
        /// </summary>
        internal static string InvalidSource_cht {
            get {
                return ResourceManager.GetString("InvalidSource_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Zadaný zdroj {0} je neplatný. Zadejte platný zdroj..
        /// </summary>
        internal static string InvalidSource_csy {
            get {
                return ResourceManager.GetString("InvalidSource_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Die angegebene Quelle &quot;{0}&quot; ist ungültig. Bitte stellen Sie eine gültige Quelle zur Verfügung..
        /// </summary>
        internal static string InvalidSource_deu {
            get {
                return ResourceManager.GetString("InvalidSource_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to El origen especificado &apos;{0}&apos; no es válido. Proporcione un origen válido..
        /// </summary>
        internal static string InvalidSource_esp {
            get {
                return ResourceManager.GetString("InvalidSource_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to La source spécifiée &apos;{0}&apos; n&apos;est pas valide. Veuillez fournir une source valide..
        /// </summary>
        internal static string InvalidSource_fra {
            get {
                return ResourceManager.GetString("InvalidSource_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to La fonte specificata &apos;{0}&apos; non è valida. Fornire una fonte valida..
        /// </summary>
        internal static string InvalidSource_ita {
            get {
                return ResourceManager.GetString("InvalidSource_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 指定されたソース &apos;{0}&apos; は無効です。有効なソースを指定してください。.
        /// </summary>
        internal static string InvalidSource_jpn {
            get {
                return ResourceManager.GetString("InvalidSource_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 지정된 소스 &apos;{0}&apos;이(가) 잘못되었습니다. 올바른 소스를 제공하십시오..
        /// </summary>
        internal static string InvalidSource_kor {
            get {
                return ResourceManager.GetString("InvalidSource_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Określone źródło „{0}” jest nieprawidłowe. Podaj prawidłowe źródło..
        /// </summary>
        internal static string InvalidSource_plk {
            get {
                return ResourceManager.GetString("InvalidSource_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to A origem especificada &apos;{0}&apos; é inválida. Forneça uma origem válida..
        /// </summary>
        internal static string InvalidSource_ptb {
            get {
                return ResourceManager.GetString("InvalidSource_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Указанный источник &quot;{0}&quot; недопустим. Укажите допустимый источник..
        /// </summary>
        internal static string InvalidSource_rus {
            get {
                return ResourceManager.GetString("InvalidSource_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Belirtilen &apos;{0}&apos; kaynağı geçersiz. Lütfen geçerli bir kaynak belirtin..
        /// </summary>
        internal static string InvalidSource_trk {
            get {
                return ResourceManager.GetString("InvalidSource_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to License url: {0}.
        /// </summary>
        internal static string ListCommand_LicenseUrl {
            get {
                return ResourceManager.GetString("ListCommand_LicenseUrl", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 许可证 URL: {0}.
        /// </summary>
        internal static string ListCommand_LicenseUrl_chs {
            get {
                return ResourceManager.GetString("ListCommand_LicenseUrl_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 授權 URL: {0}.
        /// </summary>
        internal static string ListCommand_LicenseUrl_cht {
            get {
                return ResourceManager.GetString("ListCommand_LicenseUrl_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Adresa URL licence: {0}.
        /// </summary>
        internal static string ListCommand_LicenseUrl_csy {
            get {
                return ResourceManager.GetString("ListCommand_LicenseUrl_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Lizenz-URL: {0}.
        /// </summary>
        internal static string ListCommand_LicenseUrl_deu {
            get {
                return ResourceManager.GetString("ListCommand_LicenseUrl_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to URL de la licencia: {0}.
        /// </summary>
        internal static string ListCommand_LicenseUrl_esp {
            get {
                return ResourceManager.GetString("ListCommand_LicenseUrl_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to URL de la licence : {0}.
        /// </summary>
        internal static string ListCommand_LicenseUrl_fra {
            get {
                return ResourceManager.GetString("ListCommand_LicenseUrl_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Licenze url: {0}.
        /// </summary>
        internal static string ListCommand_LicenseUrl_ita {
            get {
                return ResourceManager.GetString("ListCommand_LicenseUrl_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to ライセンス URL: {0}.
        /// </summary>
        internal static string ListCommand_LicenseUrl_jpn {
            get {
                return ResourceManager.GetString("ListCommand_LicenseUrl_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 라이선스 URL: {0}.
        /// </summary>
        internal static string ListCommand_LicenseUrl_kor {
            get {
                return ResourceManager.GetString("ListCommand_LicenseUrl_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Adres URL licencji: {0}.
        /// </summary>
        internal static string ListCommand_LicenseUrl_plk {
            get {
                return ResourceManager.GetString("ListCommand_LicenseUrl_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Url de licença: {0}.
        /// </summary>
        internal static string ListCommand_LicenseUrl_ptb {
            get {
                return ResourceManager.GetString("ListCommand_LicenseUrl_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to URL-адрес лицензии: {0}.
        /// </summary>
        internal static string ListCommand_LicenseUrl_rus {
            get {
                return ResourceManager.GetString("ListCommand_LicenseUrl_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Lisans url&apos;si: {0}.
        /// </summary>
        internal static string ListCommand_LicenseUrl_trk {
            get {
                return ResourceManager.GetString("ListCommand_LicenseUrl_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to No packages found..
        /// </summary>
        internal static string ListCommandNoPackages {
            get {
                return ResourceManager.GetString("ListCommandNoPackages", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 找不到程序包。.
        /// </summary>
        internal static string ListCommandNoPackages_chs {
            get {
                return ResourceManager.GetString("ListCommandNoPackages_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 找不到封裝。.
        /// </summary>
        internal static string ListCommandNoPackages_cht {
            get {
                return ResourceManager.GetString("ListCommandNoPackages_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Nebyly nalezeny žádné balíčky..
        /// </summary>
        internal static string ListCommandNoPackages_csy {
            get {
                return ResourceManager.GetString("ListCommandNoPackages_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Es wurden keine Pakete gefunden..
        /// </summary>
        internal static string ListCommandNoPackages_deu {
            get {
                return ResourceManager.GetString("ListCommandNoPackages_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to No se han encontrado paquetes..
        /// </summary>
        internal static string ListCommandNoPackages_esp {
            get {
                return ResourceManager.GetString("ListCommandNoPackages_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Aucun package trouvé..
        /// </summary>
        internal static string ListCommandNoPackages_fra {
            get {
                return ResourceManager.GetString("ListCommandNoPackages_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Nessun pacchetto trovato.
        /// </summary>
        internal static string ListCommandNoPackages_ita {
            get {
                return ResourceManager.GetString("ListCommandNoPackages_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to パッケージが見つかりませんでした。.
        /// </summary>
        internal static string ListCommandNoPackages_jpn {
            get {
                return ResourceManager.GetString("ListCommandNoPackages_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 패키지를 찾을 수 없습니다..
        /// </summary>
        internal static string ListCommandNoPackages_kor {
            get {
                return ResourceManager.GetString("ListCommandNoPackages_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Nie znaleziono żadnych pakietów..
        /// </summary>
        internal static string ListCommandNoPackages_plk {
            get {
                return ResourceManager.GetString("ListCommandNoPackages_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Nenhum pacote encontrado..
        /// </summary>
        internal static string ListCommandNoPackages_ptb {
            get {
                return ResourceManager.GetString("ListCommandNoPackages_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Пакеты не найдены..
        /// </summary>
        internal static string ListCommandNoPackages_rus {
            get {
                return ResourceManager.GetString("ListCommandNoPackages_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Hiçbir paket bulunamadı..
        /// </summary>
        internal static string ListCommandNoPackages_trk {
            get {
                return ResourceManager.GetString("ListCommandNoPackages_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to the NuGet gallery.
        /// </summary>
        internal static string LiveFeed {
            get {
                return ResourceManager.GetString("LiveFeed", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to NuGet 库.
        /// </summary>
        internal static string LiveFeed_chs {
            get {
                return ResourceManager.GetString("LiveFeed_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to NuGet 陳列庫.
        /// </summary>
        internal static string LiveFeed_cht {
            get {
                return ResourceManager.GetString("LiveFeed_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to galerie NuGet.
        /// </summary>
        internal static string LiveFeed_csy {
            get {
                return ResourceManager.GetString("LiveFeed_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to der NuGet-Katalog.
        /// </summary>
        internal static string LiveFeed_deu {
            get {
                return ResourceManager.GetString("LiveFeed_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to la galería NuGet.
        /// </summary>
        internal static string LiveFeed_esp {
            get {
                return ResourceManager.GetString("LiveFeed_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to galerie NuGet.
        /// </summary>
        internal static string LiveFeed_fra {
            get {
                return ResourceManager.GetString("LiveFeed_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to La galleria NuGet.
        /// </summary>
        internal static string LiveFeed_ita {
            get {
                return ResourceManager.GetString("LiveFeed_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to NuGet ギャラリー.
        /// </summary>
        internal static string LiveFeed_jpn {
            get {
                return ResourceManager.GetString("LiveFeed_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to NuGet 갤러리.
        /// </summary>
        internal static string LiveFeed_kor {
            get {
                return ResourceManager.GetString("LiveFeed_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to galeria NuGet.
        /// </summary>
        internal static string LiveFeed_plk {
            get {
                return ResourceManager.GetString("LiveFeed_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to a galeria de NuGet.
        /// </summary>
        internal static string LiveFeed_ptb {
            get {
                return ResourceManager.GetString("LiveFeed_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to коллекция NuGet.
        /// </summary>
        internal static string LiveFeed_rus {
            get {
                return ResourceManager.GetString("LiveFeed_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to NuGet galerisi.
        /// </summary>
        internal static string LiveFeed_trk {
            get {
                return ResourceManager.GetString("LiveFeed_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Looking for installed packages in &apos;{0}&apos;..
        /// </summary>
        internal static string LookingForInstalledPackages {
            get {
                return ResourceManager.GetString("LookingForInstalledPackages", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 正在“{0}”中查找已安装的程序包。.
        /// </summary>
        internal static string LookingForInstalledPackages_chs {
            get {
                return ResourceManager.GetString("LookingForInstalledPackages_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 在 &apos;{0}&apos; 中尋找已安裝的封裝。.
        /// </summary>
        internal static string LookingForInstalledPackages_cht {
            get {
                return ResourceManager.GetString("LookingForInstalledPackages_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Probíhá vyhledávání nainstalovaných balíčků v {0}..
        /// </summary>
        internal static string LookingForInstalledPackages_csy {
            get {
                return ResourceManager.GetString("LookingForInstalledPackages_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to In &quot;{0}&quot; wird nach installierten Paketen gesucht..
        /// </summary>
        internal static string LookingForInstalledPackages_deu {
            get {
                return ResourceManager.GetString("LookingForInstalledPackages_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Buscando paquetes instalados en &apos;{0}&apos;..
        /// </summary>
        internal static string LookingForInstalledPackages_esp {
            get {
                return ResourceManager.GetString("LookingForInstalledPackages_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Recherche en cours des packages installés sur &apos;{0}&apos;..
        /// </summary>
        internal static string LookingForInstalledPackages_fra {
            get {
                return ResourceManager.GetString("LookingForInstalledPackages_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Ricerca pacchetti installati in &apos;{0}&apos;..
        /// </summary>
        internal static string LookingForInstalledPackages_ita {
            get {
                return ResourceManager.GetString("LookingForInstalledPackages_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to &apos;{0}&apos; にインストールされているパッケージを探しています。.
        /// </summary>
        internal static string LookingForInstalledPackages_jpn {
            get {
                return ResourceManager.GetString("LookingForInstalledPackages_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to &apos;{0}&apos;에서 설치된 패키지를 찾고 있습니다..
        /// </summary>
        internal static string LookingForInstalledPackages_kor {
            get {
                return ResourceManager.GetString("LookingForInstalledPackages_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Wyszukiwanie zainstalowanych pakietów w „{0}”..
        /// </summary>
        internal static string LookingForInstalledPackages_plk {
            get {
                return ResourceManager.GetString("LookingForInstalledPackages_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Buscar pacotes instalados em &apos;{0}&apos;..
        /// </summary>
        internal static string LookingForInstalledPackages_ptb {
            get {
                return ResourceManager.GetString("LookingForInstalledPackages_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Поиск установленных пакетов в &quot;{0}&quot;..
        /// </summary>
        internal static string LookingForInstalledPackages_rus {
            get {
                return ResourceManager.GetString("LookingForInstalledPackages_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0}&apos; içindeki yüklü paketler aranıyor..
        /// </summary>
        internal static string LookingForInstalledPackages_trk {
            get {
                return ResourceManager.GetString("LookingForInstalledPackages_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Missing option value for: &apos;{0}&apos;.
        /// </summary>
        internal static string MissingOptionValueError {
            get {
                return ResourceManager.GetString("MissingOptionValueError", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 缺少以下项的选项值:“{0}”.
        /// </summary>
        internal static string MissingOptionValueError_chs {
            get {
                return ResourceManager.GetString("MissingOptionValueError_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 缺少下列項目的選項值: &apos;{0}&apos;.
        /// </summary>
        internal static string MissingOptionValueError_cht {
            get {
                return ResourceManager.GetString("MissingOptionValueError_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Chybějící hodnota možnosti: &apos;{0}&apos;.
        /// </summary>
        internal static string MissingOptionValueError_csy {
            get {
                return ResourceManager.GetString("MissingOptionValueError_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Fehlender Optionswert für: &quot;{0}&quot;.
        /// </summary>
        internal static string MissingOptionValueError_deu {
            get {
                return ResourceManager.GetString("MissingOptionValueError_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Falta el valor de opción para &apos;{0}&apos;.
        /// </summary>
        internal static string MissingOptionValueError_esp {
            get {
                return ResourceManager.GetString("MissingOptionValueError_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Valeur de l&apos;option manquante pour : &apos;{0}&apos;.
        /// </summary>
        internal static string MissingOptionValueError_fra {
            get {
                return ResourceManager.GetString("MissingOptionValueError_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Valore opzione mancante per: &apos;{0}&apos;.
        /// </summary>
        internal static string MissingOptionValueError_ita {
            get {
                return ResourceManager.GetString("MissingOptionValueError_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 次のオプションの値が見つかりません: &apos;{0}&apos;.
        /// </summary>
        internal static string MissingOptionValueError_jpn {
            get {
                return ResourceManager.GetString("MissingOptionValueError_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to &apos;{0}&apos;에 대한 옵션 값이 없습니다..
        /// </summary>
        internal static string MissingOptionValueError_kor {
            get {
                return ResourceManager.GetString("MissingOptionValueError_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Brak wartości opcji dla: „{0}”.
        /// </summary>
        internal static string MissingOptionValueError_plk {
            get {
                return ResourceManager.GetString("MissingOptionValueError_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Valor da opção ausente para: &apos;{0}&apos;.
        /// </summary>
        internal static string MissingOptionValueError_ptb {
            get {
                return ResourceManager.GetString("MissingOptionValueError_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Отсутствует значение параметра для: &quot;{0}&quot;.
        /// </summary>
        internal static string MissingOptionValueError_rus {
            get {
                return ResourceManager.GetString("MissingOptionValueError_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Bu öğe için seçenek değeri eksik: &apos;{0}&apos;.
        /// </summary>
        internal static string MissingOptionValueError_trk {
            get {
                return ResourceManager.GetString("MissingOptionValueError_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Found multiple project files for &apos;{0}&apos;..
        /// </summary>
        internal static string MultipleProjectFilesFound {
            get {
                return ResourceManager.GetString("MultipleProjectFilesFound", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to No API Key was provided and no API Key could be found for {0}. To save an API Key for a source use the &apos;setApiKey&apos; command..
        /// </summary>
        internal static string NoApiKeyFound {
            get {
                return ResourceManager.GetString("NoApiKeyFound", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 未提供 API 密钥并且找不到 {0} 的 API 密钥。若要保存源的 API 密钥，请使用 &quot;setApiKey&quot; 命令。.
        /// </summary>
        internal static string NoApiKeyFound_chs {
            get {
                return ResourceManager.GetString("NoApiKeyFound_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 未提供 API 索引鍵，也找不到 {0} 的 API 索引鍵。如要儲存來源的 API 索引鍵，請使用 &apos;setApiKey&apos; 命令。.
        /// </summary>
        internal static string NoApiKeyFound_cht {
            get {
                return ResourceManager.GetString("NoApiKeyFound_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Nebyl zadán žádný klíč API a nebyl nalezen žádný klíč API Key pro {0}. Chcete-li uložit klíč API pro určitý zdroj, použijte příkaz setApiKey..
        /// </summary>
        internal static string NoApiKeyFound_csy {
            get {
                return ResourceManager.GetString("NoApiKeyFound_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Es wurde kein API-Schlüssel bereitgestellt bzw. für &quot;{0}&quot; gefunden. Verwenden Sie zum Speichern eines API-Schlüssels für eine Quelle den Befehl &quot;setApiKey&quot;..
        /// </summary>
        internal static string NoApiKeyFound_deu {
            get {
                return ResourceManager.GetString("NoApiKeyFound_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to No se ha proporcionado ninguna clave API ni tampoco se ha podido encontrar una para {0}. Para guardar una clave API para un origen use el comando &apos;setApiKey&apos;..
        /// </summary>
        internal static string NoApiKeyFound_esp {
            get {
                return ResourceManager.GetString("NoApiKeyFound_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Aucune clé API n&apos;a été fournie et aucune clé API n&apos;a été trouvée pour {0}. Utilisez la commande &apos;setApiKey&apos; pour enregistrer une clé API pour une source..
        /// </summary>
        internal static string NoApiKeyFound_fra {
            get {
                return ResourceManager.GetString("NoApiKeyFound_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Non è stato fornito alcun API Key  e non è stato possibile trovare API Key per {0}. Per salvare un API Key per una fonte usare il comando &apos;setApiKey&apos;..
        /// </summary>
        internal static string NoApiKeyFound_ita {
            get {
                return ResourceManager.GetString("NoApiKeyFound_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to API キーが指定されておらず、{0} の API キーが見つかりませんでした。ソースの API キーを保存するには、&apos;setApiKey&apos; コマンドを使用してください。.
        /// </summary>
        internal static string NoApiKeyFound_jpn {
            get {
                return ResourceManager.GetString("NoApiKeyFound_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to API 키가 제공되지 않았으므로 {0}에서 API 키를 찾을 수 없습니다. 소스에 대한 API 키를 저장하려면 &apos;setApiKey&apos; 명령을 사용하십시오..
        /// </summary>
        internal static string NoApiKeyFound_kor {
            get {
                return ResourceManager.GetString("NoApiKeyFound_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Nie podano klucza interfejsu API i nie można znaleźć klucza interfejsu API dla {0}. Aby zapisać klucz interfejsu API dla źródła, użyj polecenia „setApiKey”..
        /// </summary>
        internal static string NoApiKeyFound_plk {
            get {
                return ResourceManager.GetString("NoApiKeyFound_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Nenhuma chave de API foi fornecida e nenhuma chave de API foi encontrada para {0}. Para salvar uma chave de API para uma origem, use o comando &quot;setApiKey..
        /// </summary>
        internal static string NoApiKeyFound_ptb {
            get {
                return ResourceManager.GetString("NoApiKeyFound_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Ключ API не указан, также не удалось найти ключ API для {0}. Чтобы сохранить ключ API для источника, используйте команду &quot;setApiKey&quot;..
        /// </summary>
        internal static string NoApiKeyFound_rus {
            get {
                return ResourceManager.GetString("NoApiKeyFound_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to API Anahtarı belirtilmemiş ve {0} için hiçbir API Anahtarı bulunamadı. Bir kaynak için API Anahtarını kaydetmek için &apos;setApiKey&apos; komutunu kullanın..
        /// </summary>
        internal static string NoApiKeyFound_trk {
            get {
                return ResourceManager.GetString("NoApiKeyFound_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to No projects found with packages.config..
        /// </summary>
        internal static string NoProjectsFound {
            get {
                return ResourceManager.GetString("NoProjectsFound", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 找不到包含 packages.config 的项目。.
        /// </summary>
        internal static string NoProjectsFound_chs {
            get {
                return ResourceManager.GetString("NoProjectsFound_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 找不到具有 packages.config 的專案。.
        /// </summary>
        internal static string NoProjectsFound_cht {
            get {
                return ResourceManager.GetString("NoProjectsFound_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Nebyly nalezeny žádné projekty se souborem packages.config..
        /// </summary>
        internal static string NoProjectsFound_csy {
            get {
                return ResourceManager.GetString("NoProjectsFound_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Es wurden keine Projekte mit &quot;packages.config&quot; gefunden..
        /// </summary>
        internal static string NoProjectsFound_deu {
            get {
                return ResourceManager.GetString("NoProjectsFound_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to No se han encontrado proyectos con packages.config..
        /// </summary>
        internal static string NoProjectsFound_esp {
            get {
                return ResourceManager.GetString("NoProjectsFound_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Aucun projet contenant le fichier packages.config n&apos;a été trouvé..
        /// </summary>
        internal static string NoProjectsFound_fra {
            get {
                return ResourceManager.GetString("NoProjectsFound_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Nessun progetto trovato per packages.config..
        /// </summary>
        internal static string NoProjectsFound_ita {
            get {
                return ResourceManager.GetString("NoProjectsFound_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to packages.config が指定されたプロジェクトが見つかりませんでした。.
        /// </summary>
        internal static string NoProjectsFound_jpn {
            get {
                return ResourceManager.GetString("NoProjectsFound_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to packages.config가 있는 프로젝트를 찾을 수 없습니다..
        /// </summary>
        internal static string NoProjectsFound_kor {
            get {
                return ResourceManager.GetString("NoProjectsFound_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Nie odnaleziono żadnych projektów z plikiem packages.config..
        /// </summary>
        internal static string NoProjectsFound_plk {
            get {
                return ResourceManager.GetString("NoProjectsFound_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Nenhum projeto encontrado com packages.config..
        /// </summary>
        internal static string NoProjectsFound_ptb {
            get {
                return ResourceManager.GetString("NoProjectsFound_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Проекты с файлом packages.config не найдены..
        /// </summary>
        internal static string NoProjectsFound_rus {
            get {
                return ResourceManager.GetString("NoProjectsFound_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Packages.config içeren hiçbir paket bulunamadı..
        /// </summary>
        internal static string NoProjectsFound_trk {
            get {
                return ResourceManager.GetString("NoProjectsFound_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to NuGet official package source.
        /// </summary>
        internal static string OfficialPackageSourceName {
            get {
                return ResourceManager.GetString("OfficialPackageSourceName", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to NuGet 官方程序包源.
        /// </summary>
        internal static string OfficialPackageSourceName_chs {
            get {
                return ResourceManager.GetString("OfficialPackageSourceName_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to NuGet 官方封裝來源.
        /// </summary>
        internal static string OfficialPackageSourceName_cht {
            get {
                return ResourceManager.GetString("OfficialPackageSourceName_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Oficiální zdroj balíčků NuGet.
        /// </summary>
        internal static string OfficialPackageSourceName_csy {
            get {
                return ResourceManager.GetString("OfficialPackageSourceName_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Offizielle NuGet-Paketquelle.
        /// </summary>
        internal static string OfficialPackageSourceName_deu {
            get {
                return ResourceManager.GetString("OfficialPackageSourceName_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to origen del paquete oficial de NuGet.
        /// </summary>
        internal static string OfficialPackageSourceName_esp {
            get {
                return ResourceManager.GetString("OfficialPackageSourceName_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Source du package officiel NuGet.
        /// </summary>
        internal static string OfficialPackageSourceName_fra {
            get {
                return ResourceManager.GetString("OfficialPackageSourceName_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Fonte pacchetto ufficiale NuGet .
        /// </summary>
        internal static string OfficialPackageSourceName_ita {
            get {
                return ResourceManager.GetString("OfficialPackageSourceName_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to NuGet の正式なパッケージ ソース.
        /// </summary>
        internal static string OfficialPackageSourceName_jpn {
            get {
                return ResourceManager.GetString("OfficialPackageSourceName_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to NuGet 공식 패키지 소스.
        /// </summary>
        internal static string OfficialPackageSourceName_kor {
            get {
                return ResourceManager.GetString("OfficialPackageSourceName_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Oficjalne źródło pakietów NuGet.
        /// </summary>
        internal static string OfficialPackageSourceName_plk {
            get {
                return ResourceManager.GetString("OfficialPackageSourceName_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Origem de pacote oficial NuGet.
        /// </summary>
        internal static string OfficialPackageSourceName_ptb {
            get {
                return ResourceManager.GetString("OfficialPackageSourceName_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Официальный источник пакетов NuGet.
        /// </summary>
        internal static string OfficialPackageSourceName_rus {
            get {
                return ResourceManager.GetString("OfficialPackageSourceName_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to NuGet resmi paket kaynağı.
        /// </summary>
        internal static string OfficialPackageSourceName_trk {
            get {
                return ResourceManager.GetString("OfficialPackageSourceName_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Option &apos;Verbose&apos; has been deprecated. Use &apos;Verbosity&apos; instead..
        /// </summary>
        internal static string Option_VerboseDeprecated {
            get {
                return ResourceManager.GetString("Option_VerboseDeprecated", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 选项 &quot;Verbose&quot; 已弃用。请改用 &quot;Verbosity&quot;。.
        /// </summary>
        internal static string Option_VerboseDeprecated_chs {
            get {
                return ResourceManager.GetString("Option_VerboseDeprecated_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Verbose&apos; 選項已過時。請使用 &apos;Verbosity&apos;。.
        /// </summary>
        internal static string Option_VerboseDeprecated_cht {
            get {
                return ResourceManager.GetString("Option_VerboseDeprecated_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Možnost Podrobný se již nepoužívá. Použijte místo toho možnost Podrobnosti..
        /// </summary>
        internal static string Option_VerboseDeprecated_csy {
            get {
                return ResourceManager.GetString("Option_VerboseDeprecated_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Die Option &quot;Verbose&quot; ist veraltet. Verwenden Sie stattdessen &quot;Verbosity&quot;..
        /// </summary>
        internal static string Option_VerboseDeprecated_deu {
            get {
                return ResourceManager.GetString("Option_VerboseDeprecated_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to La opción &apos;Verbose&apos; ha dejado de usarse. Use &apos;Verbosity&apos; en su lugar..
        /// </summary>
        internal static string Option_VerboseDeprecated_esp {
            get {
                return ResourceManager.GetString("Option_VerboseDeprecated_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to L&apos;option &apos;Verbose&apos; a été dépréciée. Préférez plutôt &apos;Verbosity&apos;..
        /// </summary>
        internal static string Option_VerboseDeprecated_fra {
            get {
                return ResourceManager.GetString("Option_VerboseDeprecated_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Opzione &apos;Verbose&apos; non approvata. Usare &apos;Verbosity&apos; invece..
        /// </summary>
        internal static string Option_VerboseDeprecated_ita {
            get {
                return ResourceManager.GetString("Option_VerboseDeprecated_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to オプション &apos;Verbose&apos; は使用しないでください。代わりに &apos;Verbosity&apos; を使用してください。.
        /// </summary>
        internal static string Option_VerboseDeprecated_jpn {
            get {
                return ResourceManager.GetString("Option_VerboseDeprecated_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to &apos;Verbose&apos; 옵션은 사용되지 않습니다. 대신 &apos;Verbosity&apos;를 사용하십시오..
        /// </summary>
        internal static string Option_VerboseDeprecated_kor {
            get {
                return ResourceManager.GetString("Option_VerboseDeprecated_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Opcja „Verbose” jest przestarzała. Zamiast niej użyj opcji „Verbosity”..
        /// </summary>
        internal static string Option_VerboseDeprecated_plk {
            get {
                return ResourceManager.GetString("Option_VerboseDeprecated_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to A opção &quot;Detalhe&quot; foi substituída. Use &quot;Detalhamento&quot; como alternativa..
        /// </summary>
        internal static string Option_VerboseDeprecated_ptb {
            get {
                return ResourceManager.GetString("Option_VerboseDeprecated_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Параметр &quot;Verbose&quot; удален из этой версии. Вместо него используйте &quot;Verbosity&quot;..
        /// </summary>
        internal static string Option_VerboseDeprecated_rus {
            get {
                return ResourceManager.GetString("Option_VerboseDeprecated_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Ayrıntılı&apos; seçeneği kullanım dışı bırakılmış. Bunun yerine &apos;Ayrıntı Düzeyi&apos; seçimini kullanın..
        /// </summary>
        internal static string Option_VerboseDeprecated_trk {
            get {
                return ResourceManager.GetString("Option_VerboseDeprecated_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to [option] on &apos;{0}&apos; is invalid without a setter..
        /// </summary>
        internal static string OptionInvalidWithoutSetter {
            get {
                return ResourceManager.GetString("OptionInvalidWithoutSetter", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to “{0}”上的 [option] 没有 setter，是无效的。.
        /// </summary>
        internal static string OptionInvalidWithoutSetter_chs {
            get {
                return ResourceManager.GetString("OptionInvalidWithoutSetter_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0}&apos; 上的 [選項] 因為沒有 Setter 因此無效。.
        /// </summary>
        internal static string OptionInvalidWithoutSetter_cht {
            get {
                return ResourceManager.GetString("OptionInvalidWithoutSetter_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to [možnost] pro {0} je bez metody Setter neplatná..
        /// </summary>
        internal static string OptionInvalidWithoutSetter_csy {
            get {
                return ResourceManager.GetString("OptionInvalidWithoutSetter_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to [Option] für &quot;{0}&quot; ist ohne einen Setter ungültig..
        /// </summary>
        internal static string OptionInvalidWithoutSetter_deu {
            get {
                return ResourceManager.GetString("OptionInvalidWithoutSetter_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to [option] en &apos;{0}&apos; no es válida si no se ha establecido de ningún modo..
        /// </summary>
        internal static string OptionInvalidWithoutSetter_esp {
            get {
                return ResourceManager.GetString("OptionInvalidWithoutSetter_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to [option] sur &apos;{0}&apos; n&apos;est pas valide sans un setter..
        /// </summary>
        internal static string OptionInvalidWithoutSetter_fra {
            get {
                return ResourceManager.GetString("OptionInvalidWithoutSetter_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to [option] su &apos;{0}&apos; non valido senza setter..
        /// </summary>
        internal static string OptionInvalidWithoutSetter_ita {
            get {
                return ResourceManager.GetString("OptionInvalidWithoutSetter_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to セッターがない &apos;{0}&apos; の [option] は無効です。.
        /// </summary>
        internal static string OptionInvalidWithoutSetter_jpn {
            get {
                return ResourceManager.GetString("OptionInvalidWithoutSetter_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to setter가 없으므로 &apos;{0}&apos;의 [옵션]이 잘못되었습니다..
        /// </summary>
        internal static string OptionInvalidWithoutSetter_kor {
            get {
                return ResourceManager.GetString("OptionInvalidWithoutSetter_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Opcja [opcja] dla „{0}” jest nieprawidłowa bez metody ustawiającej..
        /// </summary>
        internal static string OptionInvalidWithoutSetter_plk {
            get {
                return ResourceManager.GetString("OptionInvalidWithoutSetter_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to [option] em &apos;{0}&apos; não é válido sem um setter..
        /// </summary>
        internal static string OptionInvalidWithoutSetter_ptb {
            get {
                return ResourceManager.GetString("OptionInvalidWithoutSetter_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to [параметр] в &quot;{0}&quot; является недопустимым без метода задания значения..
        /// </summary>
        internal static string OptionInvalidWithoutSetter_rus {
            get {
                return ResourceManager.GetString("OptionInvalidWithoutSetter_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0}&apos; üzerindeki [option] ayarlayıcı olmadan geçersizdir..
        /// </summary>
        internal static string OptionInvalidWithoutSetter_trk {
            get {
                return ResourceManager.GetString("OptionInvalidWithoutSetter_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Added file &apos;{0}&apos;..
        /// </summary>
        internal static string PackageCommandAddedFile {
            get {
                return ResourceManager.GetString("PackageCommandAddedFile", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 已添加文件“{0}”。.
        /// </summary>
        internal static string PackageCommandAddedFile_chs {
            get {
                return ResourceManager.GetString("PackageCommandAddedFile_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 已新增檔案 &apos;{0}&apos;。.
        /// </summary>
        internal static string PackageCommandAddedFile_cht {
            get {
                return ResourceManager.GetString("PackageCommandAddedFile_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Byl přidán soubor {0}..
        /// </summary>
        internal static string PackageCommandAddedFile_csy {
            get {
                return ResourceManager.GetString("PackageCommandAddedFile_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Die Datei &quot;{0}&quot; wurde hinzugefügt..
        /// </summary>
        internal static string PackageCommandAddedFile_deu {
            get {
                return ResourceManager.GetString("PackageCommandAddedFile_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Se ha agregado el archivo &apos;{0}&apos;..
        /// </summary>
        internal static string PackageCommandAddedFile_esp {
            get {
                return ResourceManager.GetString("PackageCommandAddedFile_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Fichier ajouté &apos;{0}&apos;..
        /// </summary>
        internal static string PackageCommandAddedFile_fra {
            get {
                return ResourceManager.GetString("PackageCommandAddedFile_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Aggiungere file &apos;{0}&apos;..
        /// </summary>
        internal static string PackageCommandAddedFile_ita {
            get {
                return ResourceManager.GetString("PackageCommandAddedFile_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to ファイル &apos;{0}&apos; が追加されました。.
        /// </summary>
        internal static string PackageCommandAddedFile_jpn {
            get {
                return ResourceManager.GetString("PackageCommandAddedFile_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to &apos;{0}&apos; 파일을 추가했습니다..
        /// </summary>
        internal static string PackageCommandAddedFile_kor {
            get {
                return ResourceManager.GetString("PackageCommandAddedFile_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Dodano plik „{0}”..
        /// </summary>
        internal static string PackageCommandAddedFile_plk {
            get {
                return ResourceManager.GetString("PackageCommandAddedFile_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Arquivo adicionado &apos;{0}&apos;..
        /// </summary>
        internal static string PackageCommandAddedFile_ptb {
            get {
                return ResourceManager.GetString("PackageCommandAddedFile_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Добавлен файл &quot;{0}&quot;..
        /// </summary>
        internal static string PackageCommandAddedFile_rus {
            get {
                return ResourceManager.GetString("PackageCommandAddedFile_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0}&apos; dosyası eklendi..
        /// </summary>
        internal static string PackageCommandAddedFile_trk {
            get {
                return ResourceManager.GetString("PackageCommandAddedFile_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Attempting to build package from &apos;{0}&apos;..
        /// </summary>
        internal static string PackageCommandAttemptingToBuildPackage {
            get {
                return ResourceManager.GetString("PackageCommandAttemptingToBuildPackage", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 正在尝试从“{0}”生成程序包。.
        /// </summary>
        internal static string PackageCommandAttemptingToBuildPackage_chs {
            get {
                return ResourceManager.GetString("PackageCommandAttemptingToBuildPackage_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 正在嘗試從 &apos;{0}&apos; 建置封裝。.
        /// </summary>
        internal static string PackageCommandAttemptingToBuildPackage_cht {
            get {
                return ResourceManager.GetString("PackageCommandAttemptingToBuildPackage_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Probíhá pokus o sestavení balíčku z {0}..
        /// </summary>
        internal static string PackageCommandAttemptingToBuildPackage_csy {
            get {
                return ResourceManager.GetString("PackageCommandAttemptingToBuildPackage_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Es wird versucht, das Paket aus &quot;{0}&quot; zu erstellen..
        /// </summary>
        internal static string PackageCommandAttemptingToBuildPackage_deu {
            get {
                return ResourceManager.GetString("PackageCommandAttemptingToBuildPackage_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Intentando compilar el paquete desde &apos;{0}&apos;. .
        /// </summary>
        internal static string PackageCommandAttemptingToBuildPackage_esp {
            get {
                return ResourceManager.GetString("PackageCommandAttemptingToBuildPackage_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Tentative de création du package depuis &apos;{0}&apos;..
        /// </summary>
        internal static string PackageCommandAttemptingToBuildPackage_fra {
            get {
                return ResourceManager.GetString("PackageCommandAttemptingToBuildPackage_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Componendo pacchetto da&apos;{0}&apos;..
        /// </summary>
        internal static string PackageCommandAttemptingToBuildPackage_ita {
            get {
                return ResourceManager.GetString("PackageCommandAttemptingToBuildPackage_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to &apos;{0}&apos; からパッケージをビルドしています。.
        /// </summary>
        internal static string PackageCommandAttemptingToBuildPackage_jpn {
            get {
                return ResourceManager.GetString("PackageCommandAttemptingToBuildPackage_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to &apos;{0}&apos;에서 패키지를 빌드하고 있습니다..
        /// </summary>
        internal static string PackageCommandAttemptingToBuildPackage_kor {
            get {
                return ResourceManager.GetString("PackageCommandAttemptingToBuildPackage_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Próba skompilowania pakietu z „{0}”..
        /// </summary>
        internal static string PackageCommandAttemptingToBuildPackage_plk {
            get {
                return ResourceManager.GetString("PackageCommandAttemptingToBuildPackage_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Tentando construir o pacote de &apos;{0}&apos;..
        /// </summary>
        internal static string PackageCommandAttemptingToBuildPackage_ptb {
            get {
                return ResourceManager.GetString("PackageCommandAttemptingToBuildPackage_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Попытка выполнить сборку пакета из &quot;{0}&quot;..
        /// </summary>
        internal static string PackageCommandAttemptingToBuildPackage_rus {
            get {
                return ResourceManager.GetString("PackageCommandAttemptingToBuildPackage_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Paket &apos;{0}&apos; konumundan oluşturulmaya çalışılıyor. .
        /// </summary>
        internal static string PackageCommandAttemptingToBuildPackage_trk {
            get {
                return ResourceManager.GetString("PackageCommandAttemptingToBuildPackage_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Attempting to build symbols package for &apos;{0}&apos;..
        /// </summary>
        internal static string PackageCommandAttemptingToBuildSymbolsPackage {
            get {
                return ResourceManager.GetString("PackageCommandAttemptingToBuildSymbolsPackage", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 正在尝试为“{0}”生成符号程序包。.
        /// </summary>
        internal static string PackageCommandAttemptingToBuildSymbolsPackage_chs {
            get {
                return ResourceManager.GetString("PackageCommandAttemptingToBuildSymbolsPackage_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 正在嘗試為 &apos;{0}&apos; 建置符號封裝。.
        /// </summary>
        internal static string PackageCommandAttemptingToBuildSymbolsPackage_cht {
            get {
                return ResourceManager.GetString("PackageCommandAttemptingToBuildSymbolsPackage_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Probíhá pokus o sestavení balíčku symbolů pro {0}..
        /// </summary>
        internal static string PackageCommandAttemptingToBuildSymbolsPackage_csy {
            get {
                return ResourceManager.GetString("PackageCommandAttemptingToBuildSymbolsPackage_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Es wird versucht, das Symbolpaket für &quot;{0}&quot; zu erstellen..
        /// </summary>
        internal static string PackageCommandAttemptingToBuildSymbolsPackage_deu {
            get {
                return ResourceManager.GetString("PackageCommandAttemptingToBuildSymbolsPackage_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Intentando compilar el paquete de símbolos para &apos;{0}&apos;. .
        /// </summary>
        internal static string PackageCommandAttemptingToBuildSymbolsPackage_esp {
            get {
                return ResourceManager.GetString("PackageCommandAttemptingToBuildSymbolsPackage_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Tentative de création du package de symboles pour &apos;{0}&apos;..
        /// </summary>
        internal static string PackageCommandAttemptingToBuildSymbolsPackage_fra {
            get {
                return ResourceManager.GetString("PackageCommandAttemptingToBuildSymbolsPackage_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Componendo pacchetti simboli &apos;{0}&apos;..
        /// </summary>
        internal static string PackageCommandAttemptingToBuildSymbolsPackage_ita {
            get {
                return ResourceManager.GetString("PackageCommandAttemptingToBuildSymbolsPackage_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to &apos;{0}&apos; のシンボル パッケージをビルドしています。.
        /// </summary>
        internal static string PackageCommandAttemptingToBuildSymbolsPackage_jpn {
            get {
                return ResourceManager.GetString("PackageCommandAttemptingToBuildSymbolsPackage_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to &apos;{0}&apos;에 대한 기호 패키지를 빌드하고 있습니다..
        /// </summary>
        internal static string PackageCommandAttemptingToBuildSymbolsPackage_kor {
            get {
                return ResourceManager.GetString("PackageCommandAttemptingToBuildSymbolsPackage_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Próba skompilowania pakietu symboli dla „{0}”..
        /// </summary>
        internal static string PackageCommandAttemptingToBuildSymbolsPackage_plk {
            get {
                return ResourceManager.GetString("PackageCommandAttemptingToBuildSymbolsPackage_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Tentando construir o pacote de símbolos para &apos;{0}&apos;..
        /// </summary>
        internal static string PackageCommandAttemptingToBuildSymbolsPackage_ptb {
            get {
                return ResourceManager.GetString("PackageCommandAttemptingToBuildSymbolsPackage_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Попытка выполнить сборку пакета символов для &quot;{0}&quot;..
        /// </summary>
        internal static string PackageCommandAttemptingToBuildSymbolsPackage_rus {
            get {
                return ResourceManager.GetString("PackageCommandAttemptingToBuildSymbolsPackage_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0}&apos; için simge paketi oluşturulmaya çalışılıyor..
        /// </summary>
        internal static string PackageCommandAttemptingToBuildSymbolsPackage_trk {
            get {
                return ResourceManager.GetString("PackageCommandAttemptingToBuildSymbolsPackage_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to File from dependency is changed. Adding file &apos;{0}&apos;..
        /// </summary>
        internal static string PackageCommandFileFromDependencyIsChanged {
            get {
                return ResourceManager.GetString("PackageCommandFileFromDependencyIsChanged", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 依赖关系中的文件已更改。正在添加文件“{0}”。.
        /// </summary>
        internal static string PackageCommandFileFromDependencyIsChanged_chs {
            get {
                return ResourceManager.GetString("PackageCommandFileFromDependencyIsChanged_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 相依性中的檔案已變更。正在新增檔案 &apos;{0}&apos;。.
        /// </summary>
        internal static string PackageCommandFileFromDependencyIsChanged_cht {
            get {
                return ResourceManager.GetString("PackageCommandFileFromDependencyIsChanged_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Soubor ze závislosti se změnil. Probíhá přidávání souboru {0}..
        /// </summary>
        internal static string PackageCommandFileFromDependencyIsChanged_csy {
            get {
                return ResourceManager.GetString("PackageCommandFileFromDependencyIsChanged_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Die Datei aus der Abhängigkeit wurde geändert. Die Datei &quot;{0}&quot; wird hinzugefügt..
        /// </summary>
        internal static string PackageCommandFileFromDependencyIsChanged_deu {
            get {
                return ResourceManager.GetString("PackageCommandFileFromDependencyIsChanged_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Se ha cambiado el archivo de la dependencia. Agregando archivo &apos;{0}&apos;..
        /// </summary>
        internal static string PackageCommandFileFromDependencyIsChanged_esp {
            get {
                return ResourceManager.GetString("PackageCommandFileFromDependencyIsChanged_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Le @@@fichier de dépendance est modifié. Ajout du fichier &apos;{0}&apos;..
        /// </summary>
        internal static string PackageCommandFileFromDependencyIsChanged_fra {
            get {
                return ResourceManager.GetString("PackageCommandFileFromDependencyIsChanged_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to File da dipendenza modificato. Aggiungere file &apos;{0}&apos;..
        /// </summary>
        internal static string PackageCommandFileFromDependencyIsChanged_ita {
            get {
                return ResourceManager.GetString("PackageCommandFileFromDependencyIsChanged_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 依存関係のファイルが変更されます。ファイル &apos;{0}&apos; を追加しています。.
        /// </summary>
        internal static string PackageCommandFileFromDependencyIsChanged_jpn {
            get {
                return ResourceManager.GetString("PackageCommandFileFromDependencyIsChanged_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 종속성에서 파일이 변경되었습니다. &apos;{0}&apos; 파일을 추가하고 있습니다..
        /// </summary>
        internal static string PackageCommandFileFromDependencyIsChanged_kor {
            get {
                return ResourceManager.GetString("PackageCommandFileFromDependencyIsChanged_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Plik z zależności został zmieniony. Dodawanie pliku „{0}”..
        /// </summary>
        internal static string PackageCommandFileFromDependencyIsChanged_plk {
            get {
                return ResourceManager.GetString("PackageCommandFileFromDependencyIsChanged_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to O arquivo de dependência foi alterado. Adicionando o arquivo &apos;{0}&apos;..
        /// </summary>
        internal static string PackageCommandFileFromDependencyIsChanged_ptb {
            get {
                return ResourceManager.GetString("PackageCommandFileFromDependencyIsChanged_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Файл из зависимости был изменен. Добавление файла &quot;{0}&quot;..
        /// </summary>
        internal static string PackageCommandFileFromDependencyIsChanged_rus {
            get {
                return ResourceManager.GetString("PackageCommandFileFromDependencyIsChanged_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Bağımlılık dosyası değiştirildi. &apos;{0}&apos; dosyası ekleniyor..
        /// </summary>
        internal static string PackageCommandFileFromDependencyIsChanged_trk {
            get {
                return ResourceManager.GetString("PackageCommandFileFromDependencyIsChanged_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to File from dependency is not changed. File &apos;{0}&apos; is not added..
        /// </summary>
        internal static string PackageCommandFileFromDependencyIsNotChanged {
            get {
                return ResourceManager.GetString("PackageCommandFileFromDependencyIsNotChanged", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 依赖关系中的文件未更改。未添加文件“{0}”。.
        /// </summary>
        internal static string PackageCommandFileFromDependencyIsNotChanged_chs {
            get {
                return ResourceManager.GetString("PackageCommandFileFromDependencyIsNotChanged_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 相依性中的檔案並未變更。並未新增檔案 &apos;{0}&apos;。.
        /// </summary>
        internal static string PackageCommandFileFromDependencyIsNotChanged_cht {
            get {
                return ResourceManager.GetString("PackageCommandFileFromDependencyIsNotChanged_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Soubor ze závislosti se nezměnil. Soubor {0} není přidán..
        /// </summary>
        internal static string PackageCommandFileFromDependencyIsNotChanged_csy {
            get {
                return ResourceManager.GetString("PackageCommandFileFromDependencyIsNotChanged_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Die Datei aus der Abhängigkeit wurde nicht geändert. Die Datei &quot;{0}&quot; wird nicht hinzugefügt..
        /// </summary>
        internal static string PackageCommandFileFromDependencyIsNotChanged_deu {
            get {
                return ResourceManager.GetString("PackageCommandFileFromDependencyIsNotChanged_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to No se ha cambiado el archivo de la dependencia. El archivo &apos;{0}&apos; no se ha agregado..
        /// </summary>
        internal static string PackageCommandFileFromDependencyIsNotChanged_esp {
            get {
                return ResourceManager.GetString("PackageCommandFileFromDependencyIsNotChanged_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Le @@@fichier de dépendance n&apos;a pas été modifié. Le fichier &apos;{0}&apos; n&apos;a pas été ajouté..
        /// </summary>
        internal static string PackageCommandFileFromDependencyIsNotChanged_fra {
            get {
                return ResourceManager.GetString("PackageCommandFileFromDependencyIsNotChanged_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to File da dipendenza non modificato. File &apos;{0}&apos; non aggiunto..
        /// </summary>
        internal static string PackageCommandFileFromDependencyIsNotChanged_ita {
            get {
                return ResourceManager.GetString("PackageCommandFileFromDependencyIsNotChanged_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 依存関係のファイルは変更されません。ファイル &apos;{0}&apos; は追加されません。.
        /// </summary>
        internal static string PackageCommandFileFromDependencyIsNotChanged_jpn {
            get {
                return ResourceManager.GetString("PackageCommandFileFromDependencyIsNotChanged_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 종속성에서 파일이 변경되지 않았습니다. &apos;{0}&apos; 파일이 추가되지 않습니다..
        /// </summary>
        internal static string PackageCommandFileFromDependencyIsNotChanged_kor {
            get {
                return ResourceManager.GetString("PackageCommandFileFromDependencyIsNotChanged_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Plik z zależności nie został zmieniony. Nie dodano pliku „{0}”..
        /// </summary>
        internal static string PackageCommandFileFromDependencyIsNotChanged_plk {
            get {
                return ResourceManager.GetString("PackageCommandFileFromDependencyIsNotChanged_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to O arquivo da dependência não foi alterado. O arquivo &apos;{0}&apos; não foi adicionado..
        /// </summary>
        internal static string PackageCommandFileFromDependencyIsNotChanged_ptb {
            get {
                return ResourceManager.GetString("PackageCommandFileFromDependencyIsNotChanged_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Файл из зависимости не был изменен. Файл &quot;{0}&quot; не добавлен..
        /// </summary>
        internal static string PackageCommandFileFromDependencyIsNotChanged_rus {
            get {
                return ResourceManager.GetString("PackageCommandFileFromDependencyIsNotChanged_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Bağımlılık dosyası değiştirilmedi. &apos;{0}&apos; dosyası eklenmedi..
        /// </summary>
        internal static string PackageCommandFileFromDependencyIsNotChanged_trk {
            get {
                return ResourceManager.GetString("PackageCommandFileFromDependencyIsNotChanged_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to The value of MinClientVersion argument is not a valid version..
        /// </summary>
        internal static string PackageCommandInvalidMinClientVersion {
            get {
                return ResourceManager.GetString("PackageCommandInvalidMinClientVersion", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to MinClientVersion 参数的值不是有效版本。.
        /// </summary>
        internal static string PackageCommandInvalidMinClientVersion_chs {
            get {
                return ResourceManager.GetString("PackageCommandInvalidMinClientVersion_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to MinClientVersion 引數的值不是有效版本。.
        /// </summary>
        internal static string PackageCommandInvalidMinClientVersion_cht {
            get {
                return ResourceManager.GetString("PackageCommandInvalidMinClientVersion_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Hodnotou argumentu MinClientVersion není platná verze..
        /// </summary>
        internal static string PackageCommandInvalidMinClientVersion_csy {
            get {
                return ResourceManager.GetString("PackageCommandInvalidMinClientVersion_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Der Wert des Arguments &quot;MinClientVersion&quot; weist keine gültige Version auf..
        /// </summary>
        internal static string PackageCommandInvalidMinClientVersion_deu {
            get {
                return ResourceManager.GetString("PackageCommandInvalidMinClientVersion_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to El valor del argumento de MinClientVersion no es una versión válida..
        /// </summary>
        internal static string PackageCommandInvalidMinClientVersion_esp {
            get {
                return ResourceManager.GetString("PackageCommandInvalidMinClientVersion_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to La version de la valeur de l&apos;argument MinClientVersion n&apos;est pas valide..
        /// </summary>
        internal static string PackageCommandInvalidMinClientVersion_fra {
            get {
                return ResourceManager.GetString("PackageCommandInvalidMinClientVersion_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Il valore dell&apos;argomento MinClientVersion non ha una versione valida..
        /// </summary>
        internal static string PackageCommandInvalidMinClientVersion_ita {
            get {
                return ResourceManager.GetString("PackageCommandInvalidMinClientVersion_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to MinClientVersion 引数の値は有効なバージョンではありません。.
        /// </summary>
        internal static string PackageCommandInvalidMinClientVersion_jpn {
            get {
                return ResourceManager.GetString("PackageCommandInvalidMinClientVersion_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to MinClientVersion 인수 값은 올바른 버전이 아닙니다..
        /// </summary>
        internal static string PackageCommandInvalidMinClientVersion_kor {
            get {
                return ResourceManager.GetString("PackageCommandInvalidMinClientVersion_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Wartość argumentu MinClientVersion nie jest prawidłową wersją..
        /// </summary>
        internal static string PackageCommandInvalidMinClientVersion_plk {
            get {
                return ResourceManager.GetString("PackageCommandInvalidMinClientVersion_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to O valor do argumento MinClientVersion não é uma versão válida..
        /// </summary>
        internal static string PackageCommandInvalidMinClientVersion_ptb {
            get {
                return ResourceManager.GetString("PackageCommandInvalidMinClientVersion_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Недопустимая версия значения аргумента MinClientVersion..
        /// </summary>
        internal static string PackageCommandInvalidMinClientVersion_rus {
            get {
                return ResourceManager.GetString("PackageCommandInvalidMinClientVersion_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to MinClientVersion bağımsız değişken değeri geçerli bir sürüm değildir..
        /// </summary>
        internal static string PackageCommandInvalidMinClientVersion_trk {
            get {
                return ResourceManager.GetString("PackageCommandInvalidMinClientVersion_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Description: {0}.
        /// </summary>
        internal static string PackageCommandIssueDescription {
            get {
                return ResourceManager.GetString("PackageCommandIssueDescription", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 说明: {0}.
        /// </summary>
        internal static string PackageCommandIssueDescription_chs {
            get {
                return ResourceManager.GetString("PackageCommandIssueDescription_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 描述: {0}.
        /// </summary>
        internal static string PackageCommandIssueDescription_cht {
            get {
                return ResourceManager.GetString("PackageCommandIssueDescription_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Popis: {0}.
        /// </summary>
        internal static string PackageCommandIssueDescription_csy {
            get {
                return ResourceManager.GetString("PackageCommandIssueDescription_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Beschreibung: {0}.
        /// </summary>
        internal static string PackageCommandIssueDescription_deu {
            get {
                return ResourceManager.GetString("PackageCommandIssueDescription_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Descripción: {0}.
        /// </summary>
        internal static string PackageCommandIssueDescription_esp {
            get {
                return ResourceManager.GetString("PackageCommandIssueDescription_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Description : {0}.
        /// </summary>
        internal static string PackageCommandIssueDescription_fra {
            get {
                return ResourceManager.GetString("PackageCommandIssueDescription_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Descrizione: {0}.
        /// </summary>
        internal static string PackageCommandIssueDescription_ita {
            get {
                return ResourceManager.GetString("PackageCommandIssueDescription_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 説明: {0}.
        /// </summary>
        internal static string PackageCommandIssueDescription_jpn {
            get {
                return ResourceManager.GetString("PackageCommandIssueDescription_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 설명: {0}.
        /// </summary>
        internal static string PackageCommandIssueDescription_kor {
            get {
                return ResourceManager.GetString("PackageCommandIssueDescription_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Opis: {0}.
        /// </summary>
        internal static string PackageCommandIssueDescription_plk {
            get {
                return ResourceManager.GetString("PackageCommandIssueDescription_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Descrição: {0}.
        /// </summary>
        internal static string PackageCommandIssueDescription_ptb {
            get {
                return ResourceManager.GetString("PackageCommandIssueDescription_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Описание: {0}.
        /// </summary>
        internal static string PackageCommandIssueDescription_rus {
            get {
                return ResourceManager.GetString("PackageCommandIssueDescription_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Açıklama: {0}.
        /// </summary>
        internal static string PackageCommandIssueDescription_trk {
            get {
                return ResourceManager.GetString("PackageCommandIssueDescription_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Solution: {0}.
        /// </summary>
        internal static string PackageCommandIssueSolution {
            get {
                return ResourceManager.GetString("PackageCommandIssueSolution", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 解决方案: {0}.
        /// </summary>
        internal static string PackageCommandIssueSolution_chs {
            get {
                return ResourceManager.GetString("PackageCommandIssueSolution_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 方案: {0}.
        /// </summary>
        internal static string PackageCommandIssueSolution_cht {
            get {
                return ResourceManager.GetString("PackageCommandIssueSolution_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Řešení: {0}.
        /// </summary>
        internal static string PackageCommandIssueSolution_csy {
            get {
                return ResourceManager.GetString("PackageCommandIssueSolution_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Projekt: {0}.
        /// </summary>
        internal static string PackageCommandIssueSolution_deu {
            get {
                return ResourceManager.GetString("PackageCommandIssueSolution_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Solución: {0}.
        /// </summary>
        internal static string PackageCommandIssueSolution_esp {
            get {
                return ResourceManager.GetString("PackageCommandIssueSolution_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Solution : {0}.
        /// </summary>
        internal static string PackageCommandIssueSolution_fra {
            get {
                return ResourceManager.GetString("PackageCommandIssueSolution_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Soluzione: {0}.
        /// </summary>
        internal static string PackageCommandIssueSolution_ita {
            get {
                return ResourceManager.GetString("PackageCommandIssueSolution_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to ソリューション: {0}.
        /// </summary>
        internal static string PackageCommandIssueSolution_jpn {
            get {
                return ResourceManager.GetString("PackageCommandIssueSolution_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 솔루션: {0}.
        /// </summary>
        internal static string PackageCommandIssueSolution_kor {
            get {
                return ResourceManager.GetString("PackageCommandIssueSolution_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Rozwiązanie: {0}.
        /// </summary>
        internal static string PackageCommandIssueSolution_plk {
            get {
                return ResourceManager.GetString("PackageCommandIssueSolution_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Solução: {0}.
        /// </summary>
        internal static string PackageCommandIssueSolution_ptb {
            get {
                return ResourceManager.GetString("PackageCommandIssueSolution_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Решение: {0}.
        /// </summary>
        internal static string PackageCommandIssueSolution_rus {
            get {
                return ResourceManager.GetString("PackageCommandIssueSolution_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Çözüm: {0}.
        /// </summary>
        internal static string PackageCommandIssueSolution_trk {
            get {
                return ResourceManager.GetString("PackageCommandIssueSolution_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Issue: {0}.
        /// </summary>
        internal static string PackageCommandIssueTitle {
            get {
                return ResourceManager.GetString("PackageCommandIssueTitle", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 问题: {0}.
        /// </summary>
        internal static string PackageCommandIssueTitle_chs {
            get {
                return ResourceManager.GetString("PackageCommandIssueTitle_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 問題: {0}.
        /// </summary>
        internal static string PackageCommandIssueTitle_cht {
            get {
                return ResourceManager.GetString("PackageCommandIssueTitle_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Problém: {0}.
        /// </summary>
        internal static string PackageCommandIssueTitle_csy {
            get {
                return ResourceManager.GetString("PackageCommandIssueTitle_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Problem: {0}.
        /// </summary>
        internal static string PackageCommandIssueTitle_deu {
            get {
                return ResourceManager.GetString("PackageCommandIssueTitle_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Problema: {0}.
        /// </summary>
        internal static string PackageCommandIssueTitle_esp {
            get {
                return ResourceManager.GetString("PackageCommandIssueTitle_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Problème : {0}.
        /// </summary>
        internal static string PackageCommandIssueTitle_fra {
            get {
                return ResourceManager.GetString("PackageCommandIssueTitle_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Versione: {0}.
        /// </summary>
        internal static string PackageCommandIssueTitle_ita {
            get {
                return ResourceManager.GetString("PackageCommandIssueTitle_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 問題: {0}.
        /// </summary>
        internal static string PackageCommandIssueTitle_jpn {
            get {
                return ResourceManager.GetString("PackageCommandIssueTitle_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 문제: {0}.
        /// </summary>
        internal static string PackageCommandIssueTitle_kor {
            get {
                return ResourceManager.GetString("PackageCommandIssueTitle_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Problem: {0}.
        /// </summary>
        internal static string PackageCommandIssueTitle_plk {
            get {
                return ResourceManager.GetString("PackageCommandIssueTitle_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Edição: {0}.
        /// </summary>
        internal static string PackageCommandIssueTitle_ptb {
            get {
                return ResourceManager.GetString("PackageCommandIssueTitle_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Проблема: {0}.
        /// </summary>
        internal static string PackageCommandIssueTitle_rus {
            get {
                return ResourceManager.GetString("PackageCommandIssueTitle_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Sorun: {0}.
        /// </summary>
        internal static string PackageCommandIssueTitle_trk {
            get {
                return ResourceManager.GetString("PackageCommandIssueTitle_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Failed to build package. Ensure &apos;{0}&apos; includes assembly files. For help on building symbols package, visit {1}..
        /// </summary>
        internal static string PackageCommandNoFilesForLibPackage {
            get {
                return ResourceManager.GetString("PackageCommandNoFilesForLibPackage", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 无法生成程序包。请确保“{0}”包括程序集文件。有关生成符号程序包的帮助，请访问 {1}。.
        /// </summary>
        internal static string PackageCommandNoFilesForLibPackage_chs {
            get {
                return ResourceManager.GetString("PackageCommandNoFilesForLibPackage_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 無法建置封裝。確定 &apos;{0}&apos; 包含組件檔案。如需建置符號封裝的說明，請造訪 {1}。.
        /// </summary>
        internal static string PackageCommandNoFilesForLibPackage_cht {
            get {
                return ResourceManager.GetString("PackageCommandNoFilesForLibPackage_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Sestavení balíčku se nezdařilo. Ověřte, zda {0} zahrnuje soubory sestavení. Nápovědu týkající se sestavení balíčku symbolů naleznete na webu {1}..
        /// </summary>
        internal static string PackageCommandNoFilesForLibPackage_csy {
            get {
                return ResourceManager.GetString("PackageCommandNoFilesForLibPackage_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Fehler beim Erstellen des Pakets. Stellen Sie sicher, dass &quot;{0}&quot; Assemblydateien enthält. Hilfe zum Erstellen eines Symbolpakets finden Sie unter {1}..
        /// </summary>
        internal static string PackageCommandNoFilesForLibPackage_deu {
            get {
                return ResourceManager.GetString("PackageCommandNoFilesForLibPackage_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Error al compilar el paquete. Asegúrese de que &apos;{0}&apos; incluye los archivos de ensamblado. Para obtener ayuda sobre la compilación de paquetes de símbolos, visite {1}..
        /// </summary>
        internal static string PackageCommandNoFilesForLibPackage_esp {
            get {
                return ResourceManager.GetString("PackageCommandNoFilesForLibPackage_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Échec de la création du package. Assurez-vous que &apos;{0}&apos; inclut les fichiers d&apos;assembly. Pour obtenir l&apos;aide relative à la création d&apos;un package de symboles, visitez {1}..
        /// </summary>
        internal static string PackageCommandNoFilesForLibPackage_fra {
            get {
                return ResourceManager.GetString("PackageCommandNoFilesForLibPackage_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Impossibile impostare package. Assicurarsi che &apos;{0}&apos; include i file di assemblaggio. Per aiuto sull&apos;impostazione dei simboli, visitare {1}..
        /// </summary>
        internal static string PackageCommandNoFilesForLibPackage_ita {
            get {
                return ResourceManager.GetString("PackageCommandNoFilesForLibPackage_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to パッケージをビルドできませんでした。&apos;{0}&apos; にアセンブリ ファイルが含まれることを確認してください。シンボル パッケージのビルド方法については、{1} を参照してください。.
        /// </summary>
        internal static string PackageCommandNoFilesForLibPackage_jpn {
            get {
                return ResourceManager.GetString("PackageCommandNoFilesForLibPackage_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 패키지를 빌드하지 못했습니다. &apos;{0}&apos;에 어셈블리 파일이 있는지 확인하십시오. 기호 패키지를 빌드하는 방법에 대한 도움말은 {1}을(를) 참조하십시오..
        /// </summary>
        internal static string PackageCommandNoFilesForLibPackage_kor {
            get {
                return ResourceManager.GetString("PackageCommandNoFilesForLibPackage_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Nie można skompilować pakietu. Upewnij się, że „{0}” zawiera pliki zestawów. Aby uzyskać pomoc na temat kompilowania pakietów symboli, odwiedź {1}..
        /// </summary>
        internal static string PackageCommandNoFilesForLibPackage_plk {
            get {
                return ResourceManager.GetString("PackageCommandNoFilesForLibPackage_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Falha ao construir pacote. Verifique se  &apos;{0}&apos; inclui arquivos de assembly. Para ajudar a construir pacotes de símbolos, visite {1}..
        /// </summary>
        internal static string PackageCommandNoFilesForLibPackage_ptb {
            get {
                return ResourceManager.GetString("PackageCommandNoFilesForLibPackage_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Не удалось выполнить сборку пакета. Убедитесь, что &quot;{0}&quot; содержит файлы сборки. Справку о выполнении сборки пакета символов см. по адресу: {1}..
        /// </summary>
        internal static string PackageCommandNoFilesForLibPackage_rus {
            get {
                return ResourceManager.GetString("PackageCommandNoFilesForLibPackage_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Paket oluşturulamadı. &apos;{0}&apos; öğesinin derleme dosyalarını içerdiğinden emin olun. Simge paketi oluşturma konusunda yardım almak için, {1} sayfasını ziyaret edin..
        /// </summary>
        internal static string PackageCommandNoFilesForLibPackage_trk {
            get {
                return ResourceManager.GetString("PackageCommandNoFilesForLibPackage_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Failed to build package. Ensure &apos;{0}&apos; includes source and symbol files. For help on building symbols package, visit {1}..
        /// </summary>
        internal static string PackageCommandNoFilesForSymbolsPackage {
            get {
                return ResourceManager.GetString("PackageCommandNoFilesForSymbolsPackage", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 无法生成程序包。请确保“{0}”包括源文件和符号文件。有关生成符号程序包的帮助，请访问 {1}。.
        /// </summary>
        internal static string PackageCommandNoFilesForSymbolsPackage_chs {
            get {
                return ResourceManager.GetString("PackageCommandNoFilesForSymbolsPackage_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 無法建置封裝。確定 &apos;{0}&apos; 包含來源和符號檔案。如需建置符號封裝的說明，請造訪 {1}。.
        /// </summary>
        internal static string PackageCommandNoFilesForSymbolsPackage_cht {
            get {
                return ResourceManager.GetString("PackageCommandNoFilesForSymbolsPackage_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Sestavení balíčku se nezdařilo. Ověřte, zda {0} zahrnuje zdroj a soubory symbolů. Nápovědu týkající se sestavení balíčku symbolů naleznete na webu {1}..
        /// </summary>
        internal static string PackageCommandNoFilesForSymbolsPackage_csy {
            get {
                return ResourceManager.GetString("PackageCommandNoFilesForSymbolsPackage_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Fehler beim Erstellen des Pakets. Stellen Sie sicher, dass &quot;{0}&quot; Quell- und Symboldateien enthält. Hilfe zum Erstellen eines Symbolpakets finden Sie unter {1}..
        /// </summary>
        internal static string PackageCommandNoFilesForSymbolsPackage_deu {
            get {
                return ResourceManager.GetString("PackageCommandNoFilesForSymbolsPackage_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Error al compilar el paquete. Asegúrese de que &apos;{0}&apos; incluye los archivos de símbolos y de origen. Para obtener ayuda sobre la compilación de paquetes de símbolos, visite {1}..
        /// </summary>
        internal static string PackageCommandNoFilesForSymbolsPackage_esp {
            get {
                return ResourceManager.GetString("PackageCommandNoFilesForSymbolsPackage_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Échec de la création du package. Assurez-vous que &apos;{0}&apos; inclut les fichiers source et de symboles. Pour obtenir l&apos;aide relative à la création d&apos;un package de symboles, visitez {1}..
        /// </summary>
        internal static string PackageCommandNoFilesForSymbolsPackage_fra {
            get {
                return ResourceManager.GetString("PackageCommandNoFilesForSymbolsPackage_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Impossibile impostare package. Assicurarsi che &apos;{0}&apos; include i file di assemblaggio. Per aiuto sull&apos;impostazione dei simboli, visitare {1}..
        /// </summary>
        internal static string PackageCommandNoFilesForSymbolsPackage_ita {
            get {
                return ResourceManager.GetString("PackageCommandNoFilesForSymbolsPackage_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to パッケージをビルドできませんでした。&apos;{0}&apos; にソースとシンボル ファイルが含まれることを確認してください。シンボル パッケージのビルド方法については、{1} を参照してください。.
        /// </summary>
        internal static string PackageCommandNoFilesForSymbolsPackage_jpn {
            get {
                return ResourceManager.GetString("PackageCommandNoFilesForSymbolsPackage_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 패키지를 빌드하지 못했습니다. &apos;{0}&apos;에 소스 및 기호 파일이 있는지 확인하십시오. 기호 패키지를 빌드하는 방법에 대한 도움말은 {1}을(를) 참조하십시오..
        /// </summary>
        internal static string PackageCommandNoFilesForSymbolsPackage_kor {
            get {
                return ResourceManager.GetString("PackageCommandNoFilesForSymbolsPackage_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Nie można skompilować pakietu. Upewnij się, że „{0}” zawiera pliki źródłowe i pliki symboli. Aby uzyskać pomoc na temat kompilowania pakietów symboli, odwiedź {1}..
        /// </summary>
        internal static string PackageCommandNoFilesForSymbolsPackage_plk {
            get {
                return ResourceManager.GetString("PackageCommandNoFilesForSymbolsPackage_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Falha ao construir pacote. Verifique se &apos;{0}&apos; inclui arquivos de origem e símbolo. Para ajudar a construir pacotes símbolos, visite {1}..
        /// </summary>
        internal static string PackageCommandNoFilesForSymbolsPackage_ptb {
            get {
                return ResourceManager.GetString("PackageCommandNoFilesForSymbolsPackage_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Не удалось выполнить сборку пакета. Убедитесь, что &quot;{0}&quot; содержит файлы источников и файлы символов. Справку о выполнении сборки пакета символов см. по адресу: {1}..
        /// </summary>
        internal static string PackageCommandNoFilesForSymbolsPackage_rus {
            get {
                return ResourceManager.GetString("PackageCommandNoFilesForSymbolsPackage_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Paket oluşturulamadı. &apos;{0}&apos; öğesinin kaynak ve simge dosyalarını içerdiğinden emin olun. Simge paketi oluşturma konusunda yardım almak için, {1} sayfasını ziyaret edin..
        /// </summary>
        internal static string PackageCommandNoFilesForSymbolsPackage_trk {
            get {
                return ResourceManager.GetString("PackageCommandNoFilesForSymbolsPackage_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} issue(s) found with package &apos;{1}&apos;..
        /// </summary>
        internal static string PackageCommandPackageIssueSummary {
            get {
                return ResourceManager.GetString("PackageCommandPackageIssueSummary", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 发现程序包“{1}”存在 {0} 个问题。.
        /// </summary>
        internal static string PackageCommandPackageIssueSummary_chs {
            get {
                return ResourceManager.GetString("PackageCommandPackageIssueSummary_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 找到{0} 個具有封裝 &apos;{1}&apos; 的問題。.
        /// </summary>
        internal static string PackageCommandPackageIssueSummary_cht {
            get {
                return ResourceManager.GetString("PackageCommandPackageIssueSummary_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Počet problémů zjištěných v balíčku {1}: {0}..
        /// </summary>
        internal static string PackageCommandPackageIssueSummary_csy {
            get {
                return ResourceManager.GetString("PackageCommandPackageIssueSummary_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Für das Paket &quot;{1}&quot; gefundene Probleme: {0}..
        /// </summary>
        internal static string PackageCommandPackageIssueSummary_deu {
            get {
                return ResourceManager.GetString("PackageCommandPackageIssueSummary_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Se han encontrado {0} problemas en el paquete &apos;{1}&apos;..
        /// </summary>
        internal static string PackageCommandPackageIssueSummary_esp {
            get {
                return ResourceManager.GetString("PackageCommandPackageIssueSummary_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} problème(s) trouvé(s) concernant le package &apos;{1}&apos;..
        /// </summary>
        internal static string PackageCommandPackageIssueSummary_fra {
            get {
                return ResourceManager.GetString("PackageCommandPackageIssueSummary_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} problema (i) trovati con pacchetto&apos;{1}&apos;..
        /// </summary>
        internal static string PackageCommandPackageIssueSummary_ita {
            get {
                return ResourceManager.GetString("PackageCommandPackageIssueSummary_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to パッケージ &apos;{1}&apos; に {0} 個の問題が見つかりました。.
        /// </summary>
        internal static string PackageCommandPackageIssueSummary_jpn {
            get {
                return ResourceManager.GetString("PackageCommandPackageIssueSummary_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to &apos;{1}&apos; 패키지에 문제가 {0}개 있습니다..
        /// </summary>
        internal static string PackageCommandPackageIssueSummary_kor {
            get {
                return ResourceManager.GetString("PackageCommandPackageIssueSummary_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Liczba znalezionych problemów z pakietem „{1}”: {0}..
        /// </summary>
        internal static string PackageCommandPackageIssueSummary_plk {
            get {
                return ResourceManager.GetString("PackageCommandPackageIssueSummary_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} problemas encontrados com o pacote &apos;{1}&apos;..
        /// </summary>
        internal static string PackageCommandPackageIssueSummary_ptb {
            get {
                return ResourceManager.GetString("PackageCommandPackageIssueSummary_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Обнаружено неполадок пакета &quot;{1}&quot;: {0}..
        /// </summary>
        internal static string PackageCommandPackageIssueSummary_rus {
            get {
                return ResourceManager.GetString("PackageCommandPackageIssueSummary_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {1}&apos; sorunuyla ilgili {0} sorun bulundu..
        /// </summary>
        internal static string PackageCommandPackageIssueSummary_trk {
            get {
                return ResourceManager.GetString("PackageCommandPackageIssueSummary_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Please specify a nuspec or project file to use..
        /// </summary>
        internal static string PackageCommandSpecifyInputFileError {
            get {
                return ResourceManager.GetString("PackageCommandSpecifyInputFileError", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 请指定要使用的 nuspec 文件或项目文件。.
        /// </summary>
        internal static string PackageCommandSpecifyInputFileError_chs {
            get {
                return ResourceManager.GetString("PackageCommandSpecifyInputFileError_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 請指定 nuspec 或要使用的專案檔。.
        /// </summary>
        internal static string PackageCommandSpecifyInputFileError_cht {
            get {
                return ResourceManager.GetString("PackageCommandSpecifyInputFileError_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Zadejte soubor nuspec nebo soubor projektu, který má být použit..
        /// </summary>
        internal static string PackageCommandSpecifyInputFileError_csy {
            get {
                return ResourceManager.GetString("PackageCommandSpecifyInputFileError_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Bitte geben Sie eine zu verwendende nuspec- oder Projektdatei an..
        /// </summary>
        internal static string PackageCommandSpecifyInputFileError_deu {
            get {
                return ResourceManager.GetString("PackageCommandSpecifyInputFileError_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Especifique el archivo de proyecto o nuspec que se va a usar..
        /// </summary>
        internal static string PackageCommandSpecifyInputFileError_esp {
            get {
                return ResourceManager.GetString("PackageCommandSpecifyInputFileError_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Veuillez spécifier le fichier .nuspec ou projet à utiliser..
        /// </summary>
        internal static string PackageCommandSpecifyInputFileError_fra {
            get {
                return ResourceManager.GetString("PackageCommandSpecifyInputFileError_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Specificare il file progetto o nuspec da usare..
        /// </summary>
        internal static string PackageCommandSpecifyInputFileError_ita {
            get {
                return ResourceManager.GetString("PackageCommandSpecifyInputFileError_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 使用する nuspec または project ファイルを指定してください。.
        /// </summary>
        internal static string PackageCommandSpecifyInputFileError_jpn {
            get {
                return ResourceManager.GetString("PackageCommandSpecifyInputFileError_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 사용할 nuspec 또는 프로젝트 파일을 지정하십시오..
        /// </summary>
        internal static string PackageCommandSpecifyInputFileError_kor {
            get {
                return ResourceManager.GetString("PackageCommandSpecifyInputFileError_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Określ plik nuspec lub plik projektu, który ma zostać użyty..
        /// </summary>
        internal static string PackageCommandSpecifyInputFileError_plk {
            get {
                return ResourceManager.GetString("PackageCommandSpecifyInputFileError_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Especifique um arquivo nuspec ou de projeto para usar..
        /// </summary>
        internal static string PackageCommandSpecifyInputFileError_ptb {
            get {
                return ResourceManager.GetString("PackageCommandSpecifyInputFileError_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Укажите NUSPEC-файл или файл проекта для использования..
        /// </summary>
        internal static string PackageCommandSpecifyInputFileError_rus {
            get {
                return ResourceManager.GetString("PackageCommandSpecifyInputFileError_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Lütfen kullanılan nuspec veya proje dosyasını belirtin..
        /// </summary>
        internal static string PackageCommandSpecifyInputFileError_trk {
            get {
                return ResourceManager.GetString("PackageCommandSpecifyInputFileError_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Successfully created package &apos;{0}&apos;..
        /// </summary>
        internal static string PackageCommandSuccess {
            get {
                return ResourceManager.GetString("PackageCommandSuccess", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 已成功创建程序包“{0}”。.
        /// </summary>
        internal static string PackageCommandSuccess_chs {
            get {
                return ResourceManager.GetString("PackageCommandSuccess_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 已成功建立封裝 &apos;{0}&apos;。.
        /// </summary>
        internal static string PackageCommandSuccess_cht {
            get {
                return ResourceManager.GetString("PackageCommandSuccess_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Balíček {0} byl úspěšně vytvořen..
        /// </summary>
        internal static string PackageCommandSuccess_csy {
            get {
                return ResourceManager.GetString("PackageCommandSuccess_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Das Paket &quot;{0}&quot; wurde erfolgreich erstellt..
        /// </summary>
        internal static string PackageCommandSuccess_deu {
            get {
                return ResourceManager.GetString("PackageCommandSuccess_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Paquete &apos;{0}&apos; creado correctamente..
        /// </summary>
        internal static string PackageCommandSuccess_esp {
            get {
                return ResourceManager.GetString("PackageCommandSuccess_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Package &apos;{0}&apos; correctement créé..
        /// </summary>
        internal static string PackageCommandSuccess_fra {
            get {
                return ResourceManager.GetString("PackageCommandSuccess_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Creato pacchetto &apos;{0}&apos; correttamente..
        /// </summary>
        internal static string PackageCommandSuccess_ita {
            get {
                return ResourceManager.GetString("PackageCommandSuccess_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to パッケージ &apos;{0}&apos; が正常に作成されました。.
        /// </summary>
        internal static string PackageCommandSuccess_jpn {
            get {
                return ResourceManager.GetString("PackageCommandSuccess_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to &apos;{0}&apos; 패키지를 만들었습니다..
        /// </summary>
        internal static string PackageCommandSuccess_kor {
            get {
                return ResourceManager.GetString("PackageCommandSuccess_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Pomyślnie utworzono pakiet „{0}”..
        /// </summary>
        internal static string PackageCommandSuccess_plk {
            get {
                return ResourceManager.GetString("PackageCommandSuccess_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Pacote &apos;{0}&apos; criado com êxito..
        /// </summary>
        internal static string PackageCommandSuccess_ptb {
            get {
                return ResourceManager.GetString("PackageCommandSuccess_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Пакет &quot;{0}&quot; успешно создан..
        /// </summary>
        internal static string PackageCommandSuccess_rus {
            get {
                return ResourceManager.GetString("PackageCommandSuccess_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0}&apos; paketi başarıyla oluşturuldu..
        /// </summary>
        internal static string PackageCommandSuccess_trk {
            get {
                return ResourceManager.GetString("PackageCommandSuccess_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Unable to locate &apos;{0} {1}&apos;. Make sure all packages exist in the packages folder before running update..
        /// </summary>
        internal static string PackageDoesNotExist {
            get {
                return ResourceManager.GetString("PackageDoesNotExist", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 找不到“{0} {1}”。在运行更新之前，请确保所有程序包均存在于程序包文件夹中。.
        /// </summary>
        internal static string PackageDoesNotExist_chs {
            get {
                return ResourceManager.GetString("PackageDoesNotExist_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 找不到 &apos;{0} {1}&apos;。請確定封裝資料夾中存在所有封裝，才能執行更新。.
        /// </summary>
        internal static string PackageDoesNotExist_cht {
            get {
                return ResourceManager.GetString("PackageDoesNotExist_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Nelze vyhledat {0} {1}. Před spuštěním aktualizace ověřte, zda jsou k dispozici všechny balíčky ve složce balíčků..
        /// </summary>
        internal static string PackageDoesNotExist_csy {
            get {
                return ResourceManager.GetString("PackageDoesNotExist_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to &quot;{0} {1}&quot; wurde nicht gefunden. Stellen Sie sicher, dass alle Pakete im Paketordner vorhanden sind, bevor Sie das Update ausführen..
        /// </summary>
        internal static string PackageDoesNotExist_deu {
            get {
                return ResourceManager.GetString("PackageDoesNotExist_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to No se puede ubicar &apos;{0} {1}&apos;. Asegúrese de que existen todos los paquetes en la carpeta de paquetes antes de ejecutar la actualización..
        /// </summary>
        internal static string PackageDoesNotExist_esp {
            get {
                return ResourceManager.GetString("PackageDoesNotExist_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Impossible de trouver &apos;{0} {1}&apos;. Assurez-vous que tous les packages existent dans le dossier des packages avant d&apos;exécuter la mise à jour..
        /// </summary>
        internal static string PackageDoesNotExist_fra {
            get {
                return ResourceManager.GetString("PackageDoesNotExist_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Impossibile localizzare &apos;{0} {1}&apos;. Assicurarsi che esistono tutte le cartelle pacchetti prima di eseguire l&apos;aggiornamento.
        /// </summary>
        internal static string PackageDoesNotExist_ita {
            get {
                return ResourceManager.GetString("PackageDoesNotExist_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to &apos;{0} {1}&apos; が見つかりません。更新を実行する前に、パッケージ フォルダーにすべてのパッケージが存在することを確認してください。.
        /// </summary>
        internal static string PackageDoesNotExist_jpn {
            get {
                return ResourceManager.GetString("PackageDoesNotExist_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to &apos;{0} {1}&apos;을(를) 찾을 수 없습니다. 패키지 폴더에 모든 패키지가 있는지 확인한 후 업데이트를 실행하십시오..
        /// </summary>
        internal static string PackageDoesNotExist_kor {
            get {
                return ResourceManager.GetString("PackageDoesNotExist_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Nie można znaleźć pakietu „{0} {1}”. Przed uruchomieniem aktualizacji upewnij się, że wszystkie pakiety znajdują się w folderze pakietów..
        /// </summary>
        internal static string PackageDoesNotExist_plk {
            get {
                return ResourceManager.GetString("PackageDoesNotExist_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Não foi possível localizar &apos;{0} {1}&apos;. Certifique-se que todos os pacotes estejam na pasta de pacotes antes de executar a atualização..
        /// </summary>
        internal static string PackageDoesNotExist_ptb {
            get {
                return ResourceManager.GetString("PackageDoesNotExist_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Не удалось найти &quot;{0} {1}&quot;. Прежде чем запускать обновление, убедитесь, что в папке пакетов содержатся все пакеты..
        /// </summary>
        internal static string PackageDoesNotExist_rus {
            get {
                return ResourceManager.GetString("PackageDoesNotExist_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} {1}&apos; bulunamıyor. Güncellemeyi çalıştırmadan önce tüm paketlerin paket klasöründe mevcut olduğundan emin olun..
        /// </summary>
        internal static string PackageDoesNotExist_trk {
            get {
                return ResourceManager.GetString("PackageDoesNotExist_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Packing files from &apos;{0}&apos;..
        /// </summary>
        internal static string PackagingFilesFromOutputPath {
            get {
                return ResourceManager.GetString("PackagingFilesFromOutputPath", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 正在打包“{0}”中的文件。.
        /// </summary>
        internal static string PackagingFilesFromOutputPath_chs {
            get {
                return ResourceManager.GetString("PackagingFilesFromOutputPath_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 正在封裝 &apos;{0}&apos; 中的檔案。.
        /// </summary>
        internal static string PackagingFilesFromOutputPath_cht {
            get {
                return ResourceManager.GetString("PackagingFilesFromOutputPath_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Probíhá balení souborů z {0}..
        /// </summary>
        internal static string PackagingFilesFromOutputPath_csy {
            get {
                return ResourceManager.GetString("PackagingFilesFromOutputPath_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Paketerstellung der Dateien aus &quot;{0}&quot;..
        /// </summary>
        internal static string PackagingFilesFromOutputPath_deu {
            get {
                return ResourceManager.GetString("PackagingFilesFromOutputPath_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Empaquetando archivos de proyecto de &apos;{0}&apos;..
        /// </summary>
        internal static string PackagingFilesFromOutputPath_esp {
            get {
                return ResourceManager.GetString("PackagingFilesFromOutputPath_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Compression des fichiers depuis &apos;{0}&apos;..
        /// </summary>
        internal static string PackagingFilesFromOutputPath_fra {
            get {
                return ResourceManager.GetString("PackagingFilesFromOutputPath_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Impacchettando file da &apos;{0}&apos;..
        /// </summary>
        internal static string PackagingFilesFromOutputPath_ita {
            get {
                return ResourceManager.GetString("PackagingFilesFromOutputPath_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to &apos;{0}&apos; のファイルをパックしています。.
        /// </summary>
        internal static string PackagingFilesFromOutputPath_jpn {
            get {
                return ResourceManager.GetString("PackagingFilesFromOutputPath_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to &apos;{0}&apos;에서 파일을 압축하고 있습니다..
        /// </summary>
        internal static string PackagingFilesFromOutputPath_kor {
            get {
                return ResourceManager.GetString("PackagingFilesFromOutputPath_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Pakowanie plików z „{0}”..
        /// </summary>
        internal static string PackagingFilesFromOutputPath_plk {
            get {
                return ResourceManager.GetString("PackagingFilesFromOutputPath_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Embalar arquivos de &apos;{0}&apos;..
        /// </summary>
        internal static string PackagingFilesFromOutputPath_ptb {
            get {
                return ResourceManager.GetString("PackagingFilesFromOutputPath_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Упаковка файлов из {0}..
        /// </summary>
        internal static string PackagingFilesFromOutputPath_rus {
            get {
                return ResourceManager.GetString("PackagingFilesFromOutputPath_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0}&apos; dosyaları paketleniyor..
        /// </summary>
        internal static string PackagingFilesFromOutputPath_trk {
            get {
                return ResourceManager.GetString("PackagingFilesFromOutputPath_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to There is no default source, please specify a source..
        /// </summary>
        internal static string PushCommandNoSourceError {
            get {
                return ResourceManager.GetString("PushCommandNoSourceError", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 没有默认源，请指定源。.
        /// </summary>
        internal static string PushCommandNoSourceError_chs {
            get {
                return ResourceManager.GetString("PushCommandNoSourceError_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 沒有預設的來源，請指定來源。.
        /// </summary>
        internal static string PushCommandNoSourceError_cht {
            get {
                return ResourceManager.GetString("PushCommandNoSourceError_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Není nastaven výchozí zdroj. Zadejte zdroj..
        /// </summary>
        internal static string PushCommandNoSourceError_csy {
            get {
                return ResourceManager.GetString("PushCommandNoSourceError_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Es ist keine Standardquelle vorhanden. Bitte geben Sie eine Quelle an..
        /// </summary>
        internal static string PushCommandNoSourceError_deu {
            get {
                return ResourceManager.GetString("PushCommandNoSourceError_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to No hay ningún origen predeterminado, especifique un origen..
        /// </summary>
        internal static string PushCommandNoSourceError_esp {
            get {
                return ResourceManager.GetString("PushCommandNoSourceError_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Absence de source par défaut, veuillez en spécifier une..
        /// </summary>
        internal static string PushCommandNoSourceError_fra {
            get {
                return ResourceManager.GetString("PushCommandNoSourceError_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Nessuna fonte di default, specificare una fonte..
        /// </summary>
        internal static string PushCommandNoSourceError_ita {
            get {
                return ResourceManager.GetString("PushCommandNoSourceError_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 既定のソースがありません。ソースを指定してください。.
        /// </summary>
        internal static string PushCommandNoSourceError_jpn {
            get {
                return ResourceManager.GetString("PushCommandNoSourceError_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 기본 소스가 없습니다. 소스를 지정하십시오..
        /// </summary>
        internal static string PushCommandNoSourceError_kor {
            get {
                return ResourceManager.GetString("PushCommandNoSourceError_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Brak domyślnego źródła. Określ źródło..
        /// </summary>
        internal static string PushCommandNoSourceError_plk {
            get {
                return ResourceManager.GetString("PushCommandNoSourceError_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Não há origem padrão. Especifique uma origem..
        /// </summary>
        internal static string PushCommandNoSourceError_ptb {
            get {
                return ResourceManager.GetString("PushCommandNoSourceError_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Источник по умолчанию отсутствует, укажите источник..
        /// </summary>
        internal static string PushCommandNoSourceError_rus {
            get {
                return ResourceManager.GetString("PushCommandNoSourceError_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Varsayılan herhangi bir kaynak yok, lütfen bir kaynak belirtin..
        /// </summary>
        internal static string PushCommandNoSourceError_trk {
            get {
                return ResourceManager.GetString("PushCommandNoSourceError_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Your package was pushed..
        /// </summary>
        internal static string PushCommandPackagePushed {
            get {
                return ResourceManager.GetString("PushCommandPackagePushed", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 已推送你的程序包。.
        /// </summary>
        internal static string PushCommandPackagePushed_chs {
            get {
                return ResourceManager.GetString("PushCommandPackagePushed_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 已推入您的封裝。.
        /// </summary>
        internal static string PushCommandPackagePushed_cht {
            get {
                return ResourceManager.GetString("PushCommandPackagePushed_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Balíček byl předán..
        /// </summary>
        internal static string PushCommandPackagePushed_csy {
            get {
                return ResourceManager.GetString("PushCommandPackagePushed_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Ihr Paket wurde mittels Push übertragen..
        /// </summary>
        internal static string PushCommandPackagePushed_deu {
            get {
                return ResourceManager.GetString("PushCommandPackagePushed_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Se ha insertado su proyecto..
        /// </summary>
        internal static string PushCommandPackagePushed_esp {
            get {
                return ResourceManager.GetString("PushCommandPackagePushed_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Votre package a été transmis..
        /// </summary>
        internal static string PushCommandPackagePushed_fra {
            get {
                return ResourceManager.GetString("PushCommandPackagePushed_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Il pacchetto è stato compresso..
        /// </summary>
        internal static string PushCommandPackagePushed_ita {
            get {
                return ResourceManager.GetString("PushCommandPackagePushed_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to パッケージがプッシュされました。.
        /// </summary>
        internal static string PushCommandPackagePushed_jpn {
            get {
                return ResourceManager.GetString("PushCommandPackagePushed_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 패키지가 푸시되었습니다..
        /// </summary>
        internal static string PushCommandPackagePushed_kor {
            get {
                return ResourceManager.GetString("PushCommandPackagePushed_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Pakiet został wypchnięty..
        /// </summary>
        internal static string PushCommandPackagePushed_plk {
            get {
                return ResourceManager.GetString("PushCommandPackagePushed_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to O pacote foi empurrado..
        /// </summary>
        internal static string PushCommandPackagePushed_ptb {
            get {
                return ResourceManager.GetString("PushCommandPackagePushed_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Ваш пакет был отправлен..
        /// </summary>
        internal static string PushCommandPackagePushed_rus {
            get {
                return ResourceManager.GetString("PushCommandPackagePushed_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Paketiniz iletildi..
        /// </summary>
        internal static string PushCommandPackagePushed_trk {
            get {
                return ResourceManager.GetString("PushCommandPackagePushed_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Pushing {0} to {1}....
        /// </summary>
        internal static string PushCommandPushingPackage {
            get {
                return ResourceManager.GetString("PushCommandPushingPackage", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 正在将 {0} 推送到 {1}....
        /// </summary>
        internal static string PushCommandPushingPackage_chs {
            get {
                return ResourceManager.GetString("PushCommandPushingPackage_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 正在將 {0} 推入 {1}....
        /// </summary>
        internal static string PushCommandPushingPackage_cht {
            get {
                return ResourceManager.GetString("PushCommandPushingPackage_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Probíhá předávání {0} do {1}....
        /// </summary>
        internal static string PushCommandPushingPackage_csy {
            get {
                return ResourceManager.GetString("PushCommandPushingPackage_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} wird mittels Push an {1} übertragen....
        /// </summary>
        internal static string PushCommandPushingPackage_deu {
            get {
                return ResourceManager.GetString("PushCommandPushingPackage_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Insertando {0} a {1}….
        /// </summary>
        internal static string PushCommandPushingPackage_esp {
            get {
                return ResourceManager.GetString("PushCommandPushingPackage_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Transmission de {0} à {1}….
        /// </summary>
        internal static string PushCommandPushingPackage_fra {
            get {
                return ResourceManager.GetString("PushCommandPushingPackage_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Comprimendo {0} di {1}....
        /// </summary>
        internal static string PushCommandPushingPackage_ita {
            get {
                return ResourceManager.GetString("PushCommandPushingPackage_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} を {1} にプッシュしています....
        /// </summary>
        internal static string PushCommandPushingPackage_jpn {
            get {
                return ResourceManager.GetString("PushCommandPushingPackage_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0}을(를) {1}에 푸시하는 중....
        /// </summary>
        internal static string PushCommandPushingPackage_kor {
            get {
                return ResourceManager.GetString("PushCommandPushingPackage_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Trwa wypychanie {0} do {1}....
        /// </summary>
        internal static string PushCommandPushingPackage_plk {
            get {
                return ResourceManager.GetString("PushCommandPushingPackage_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Empurrando {0} para {1} ....
        /// </summary>
        internal static string PushCommandPushingPackage_ptb {
            get {
                return ResourceManager.GetString("PushCommandPushingPackage_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Отправка {0} в {1}....
        /// </summary>
        internal static string PushCommandPushingPackage_rus {
            get {
                return ResourceManager.GetString("PushCommandPushingPackage_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} öğesi {1} öğesine iletiliyor....
        /// </summary>
        internal static string PushCommandPushingPackage_trk {
            get {
                return ResourceManager.GetString("PushCommandPushingPackage_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to All.
        /// </summary>
        internal static string ReservedPackageNameAll {
            get {
                return ResourceManager.GetString("ReservedPackageNameAll", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 全部.
        /// </summary>
        internal static string ReservedPackageNameAll_chs {
            get {
                return ResourceManager.GetString("ReservedPackageNameAll_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 全部.
        /// </summary>
        internal static string ReservedPackageNameAll_cht {
            get {
                return ResourceManager.GetString("ReservedPackageNameAll_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Vše.
        /// </summary>
        internal static string ReservedPackageNameAll_csy {
            get {
                return ResourceManager.GetString("ReservedPackageNameAll_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Alle.
        /// </summary>
        internal static string ReservedPackageNameAll_deu {
            get {
                return ResourceManager.GetString("ReservedPackageNameAll_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Todos.
        /// </summary>
        internal static string ReservedPackageNameAll_esp {
            get {
                return ResourceManager.GetString("ReservedPackageNameAll_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Tous.
        /// </summary>
        internal static string ReservedPackageNameAll_fra {
            get {
                return ResourceManager.GetString("ReservedPackageNameAll_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Tutto.
        /// </summary>
        internal static string ReservedPackageNameAll_ita {
            get {
                return ResourceManager.GetString("ReservedPackageNameAll_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to すべて.
        /// </summary>
        internal static string ReservedPackageNameAll_jpn {
            get {
                return ResourceManager.GetString("ReservedPackageNameAll_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 모두.
        /// </summary>
        internal static string ReservedPackageNameAll_kor {
            get {
                return ResourceManager.GetString("ReservedPackageNameAll_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Wszystkie.
        /// </summary>
        internal static string ReservedPackageNameAll_plk {
            get {
                return ResourceManager.GetString("ReservedPackageNameAll_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Tudo.
        /// </summary>
        internal static string ReservedPackageNameAll_ptb {
            get {
                return ResourceManager.GetString("ReservedPackageNameAll_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Все.
        /// </summary>
        internal static string ReservedPackageNameAll_rus {
            get {
                return ResourceManager.GetString("ReservedPackageNameAll_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Tümü.
        /// </summary>
        internal static string ReservedPackageNameAll_trk {
            get {
                return ResourceManager.GetString("ReservedPackageNameAll_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to The property &apos;{0}&apos; on resource type &apos;{1}&apos; is not a type of ResourceManager..
        /// </summary>
        internal static string ResourcePropertyIncorrectType {
            get {
                return ResourceManager.GetString("ResourcePropertyIncorrectType", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 资源类型“{1}”的属性“{0}”不是 ResourceManager 类型。.
        /// </summary>
        internal static string ResourcePropertyIncorrectType_chs {
            get {
                return ResourceManager.GetString("ResourcePropertyIncorrectType_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 資源類型 &apos;{1}&apos; 上的屬性 &apos;{0}&apos; 不是 ResourceManager 類型。.
        /// </summary>
        internal static string ResourcePropertyIncorrectType_cht {
            get {
                return ResourceManager.GetString("ResourcePropertyIncorrectType_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Vlastnost {0} typu prostředku {1} není typu ResourceManager..
        /// </summary>
        internal static string ResourcePropertyIncorrectType_csy {
            get {
                return ResourceManager.GetString("ResourcePropertyIncorrectType_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Die Eigenschaft &quot;{0}&quot; für den Ressourcentyp &quot;{1}&quot; ist kein Typ von &quot;ResourceManager&quot;..
        /// </summary>
        internal static string ResourcePropertyIncorrectType_deu {
            get {
                return ResourceManager.GetString("ResourcePropertyIncorrectType_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to La propiedad &apos;{0}&apos; del tipo de recurso &apos;{1}&apos; no es un tipo de ResourceManager..
        /// </summary>
        internal static string ResourcePropertyIncorrectType_esp {
            get {
                return ResourceManager.GetString("ResourcePropertyIncorrectType_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to La propriété &apos;{0}&apos; du type de ressource &apos;{1}&apos; n&apos;est pas du type ResourceManager..
        /// </summary>
        internal static string ResourcePropertyIncorrectType_fra {
            get {
                return ResourceManager.GetString("ResourcePropertyIncorrectType_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to La proprietà &apos;{0}&apos; sulla risorsa tipo &apos;{1}&apos; non è un tipo in ResourceManager..
        /// </summary>
        internal static string ResourcePropertyIncorrectType_ita {
            get {
                return ResourceManager.GetString("ResourcePropertyIncorrectType_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to リソースの種類 &apos;{1}&apos; のプロパティ &apos;{0}&apos; が ResourceManager 型ではありません。.
        /// </summary>
        internal static string ResourcePropertyIncorrectType_jpn {
            get {
                return ResourceManager.GetString("ResourcePropertyIncorrectType_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 리소스 형식 &apos;{1}&apos;의 &apos;{0}&apos; 속성이 ResourceManager 형식이 아닙니다..
        /// </summary>
        internal static string ResourcePropertyIncorrectType_kor {
            get {
                return ResourceManager.GetString("ResourcePropertyIncorrectType_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Właściwość „{0}” w typie zasobu „{1}” nie jest typu ResourceManager..
        /// </summary>
        internal static string ResourcePropertyIncorrectType_plk {
            get {
                return ResourceManager.GetString("ResourcePropertyIncorrectType_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to A propriedade &apos;{0}&apos; no tipo de recurso &apos;{1}&apos; não é um tipo de ResourceManager..
        /// </summary>
        internal static string ResourcePropertyIncorrectType_ptb {
            get {
                return ResourceManager.GetString("ResourcePropertyIncorrectType_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Типом свойства &quot;{0}&quot; типа ресурса &quot;{1}&quot; не является ResourceManager..
        /// </summary>
        internal static string ResourcePropertyIncorrectType_rus {
            get {
                return ResourceManager.GetString("ResourcePropertyIncorrectType_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {1}&apos; kaynak türündeki &apos;{0}&apos; özelliği bir ResourceManager türü değildir..
        /// </summary>
        internal static string ResourcePropertyIncorrectType_trk {
            get {
                return ResourceManager.GetString("ResourcePropertyIncorrectType_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to The resource type &apos;{0}&apos; does not have an accessible static property named &apos;{1}&apos;..
        /// </summary>
        internal static string ResourceTypeDoesNotHaveProperty {
            get {
                return ResourceManager.GetString("ResourceTypeDoesNotHaveProperty", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 资源类型“{0}”没有名为“{1}”的可访问静态属性。.
        /// </summary>
        internal static string ResourceTypeDoesNotHaveProperty_chs {
            get {
                return ResourceManager.GetString("ResourceTypeDoesNotHaveProperty_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 資源類型 &apos;{0}&apos; 沒有可存取的靜態屬性名為 &apos;{1}&apos;。.
        /// </summary>
        internal static string ResourceTypeDoesNotHaveProperty_cht {
            get {
                return ResourceManager.GetString("ResourceTypeDoesNotHaveProperty_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Typ prostředku {0} neobsahuje přístupnou statickou vlastnost s názvem {1}..
        /// </summary>
        internal static string ResourceTypeDoesNotHaveProperty_csy {
            get {
                return ResourceManager.GetString("ResourceTypeDoesNotHaveProperty_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Der Ressourcentyp &quot;{0}&quot; besitzt keine statische Eigenschaft namens &quot;{1}&quot;, auf die zugegriffen werden kann..
        /// </summary>
        internal static string ResourceTypeDoesNotHaveProperty_deu {
            get {
                return ResourceManager.GetString("ResourceTypeDoesNotHaveProperty_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to El tipo de recurso &apos;{0}&apos; no tiene una propiedad estática accesible denominada &apos;{1}&apos;..
        /// </summary>
        internal static string ResourceTypeDoesNotHaveProperty_esp {
            get {
                return ResourceManager.GetString("ResourceTypeDoesNotHaveProperty_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Le type de ressource &apos;{0}&apos; ne dispose pas d&apos;une propriété statique accessible nommée &apos;{1}&apos;..
        /// </summary>
        internal static string ResourceTypeDoesNotHaveProperty_fra {
            get {
                return ResourceManager.GetString("ResourceTypeDoesNotHaveProperty_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Il tipo di risorsa &apos;{0}&apos;non ha una proprietà statica accessibile di nome&apos;{1}&apos;..
        /// </summary>
        internal static string ResourceTypeDoesNotHaveProperty_ita {
            get {
                return ResourceManager.GetString("ResourceTypeDoesNotHaveProperty_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to リソースの種類 &apos;{0}&apos; に、&apos;{1}&apos; というアクセスできる静的プロパティがありません。.
        /// </summary>
        internal static string ResourceTypeDoesNotHaveProperty_jpn {
            get {
                return ResourceManager.GetString("ResourceTypeDoesNotHaveProperty_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 리소스 형식 &apos;{0}&apos;에 액세스 가능한 &apos;{1}&apos; 이름의 정적 속성이 없습니다..
        /// </summary>
        internal static string ResourceTypeDoesNotHaveProperty_kor {
            get {
                return ResourceManager.GetString("ResourceTypeDoesNotHaveProperty_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Typ zasobu „{0}” nie ma dostępnej właściwości statycznej o nazwie „{1}”..
        /// </summary>
        internal static string ResourceTypeDoesNotHaveProperty_plk {
            get {
                return ResourceManager.GetString("ResourceTypeDoesNotHaveProperty_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to O tipo de recurso &apos;{0}&apos; não tem uma propriedade estática acessível chamada &apos;{1}&apos;..
        /// </summary>
        internal static string ResourceTypeDoesNotHaveProperty_ptb {
            get {
                return ResourceManager.GetString("ResourceTypeDoesNotHaveProperty_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to У типа ресурса &quot;{0}&quot; отсутствует доступное статическое свойство с именем &quot;{1}&quot;..
        /// </summary>
        internal static string ResourceTypeDoesNotHaveProperty_rus {
            get {
                return ResourceManager.GetString("ResourceTypeDoesNotHaveProperty_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0}&apos; kaynak türünün erişilebilir &apos;{1}&apos; adlı erişilebilir statik bir özelliği yok..
        /// </summary>
        internal static string ResourceTypeDoesNotHaveProperty_trk {
            get {
                return ResourceManager.GetString("ResourceTypeDoesNotHaveProperty_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Cannot determine the packages folder to restore NuGet packages. Please specify either -PackagesDirectory or -SolutionDirectory..
        /// </summary>
        internal static string RestoreCommandCannotDeterminePackagesFolder {
            get {
                return ResourceManager.GetString("RestoreCommandCannotDeterminePackagesFolder", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 无法确定用于还原 NuGet 程序包的程序包文件夹。请指定 -PackagesDirectory 或 -SolutionDirectory。.
        /// </summary>
        internal static string RestoreCommandCannotDeterminePackagesFolder_chs {
            get {
                return ResourceManager.GetString("RestoreCommandCannotDeterminePackagesFolder_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 無法判斷要還原 NuGet 封裝的封裝資料夾。請指定 -PackagesDirectory 或 -SolutionDirectory。.
        /// </summary>
        internal static string RestoreCommandCannotDeterminePackagesFolder_cht {
            get {
                return ResourceManager.GetString("RestoreCommandCannotDeterminePackagesFolder_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Nelze určit složku balíčků pro obnovení balíčků NuGet. Zadejte parametr -PackagesDirectory nebo -SolutionDirectory..
        /// </summary>
        internal static string RestoreCommandCannotDeterminePackagesFolder_csy {
            get {
                return ResourceManager.GetString("RestoreCommandCannotDeterminePackagesFolder_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Der Paketordner zum Widerherstellen von NuGet-Paketen konnte nicht ermittelt werden. Bitte geben Sie &quot;-PackagesDirectory&quot; oder &quot;-SolutionDirectory&quot; an..
        /// </summary>
        internal static string RestoreCommandCannotDeterminePackagesFolder_deu {
            get {
                return ResourceManager.GetString("RestoreCommandCannotDeterminePackagesFolder_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to No se puede determinar la carpeta de paquetes para restaurar los paquetes NuGet. Especifique PackagesDirectory o SolutionDirectory..
        /// </summary>
        internal static string RestoreCommandCannotDeterminePackagesFolder_esp {
            get {
                return ResourceManager.GetString("RestoreCommandCannotDeterminePackagesFolder_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Impossible de déterminer le dossier de packages pour restaurer les packages NuGet. Veuillez spécifier soit -PackagesDirectory, soit -SolutionDirectory..
        /// </summary>
        internal static string RestoreCommandCannotDeterminePackagesFolder_fra {
            get {
                return ResourceManager.GetString("RestoreCommandCannotDeterminePackagesFolder_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Impossibile determinare le cartelle dei pacchetti per ripristinare i pacchetti NuGet. Specificare o PackagesDirectory o SolutionDirectory..
        /// </summary>
        internal static string RestoreCommandCannotDeterminePackagesFolder_ita {
            get {
                return ResourceManager.GetString("RestoreCommandCannotDeterminePackagesFolder_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to NuGet パッケージを復元するパッケージ フォルダーを特定できません。-PackagesDirectory または -SolutionDirectory を指定してください。.
        /// </summary>
        internal static string RestoreCommandCannotDeterminePackagesFolder_jpn {
            get {
                return ResourceManager.GetString("RestoreCommandCannotDeterminePackagesFolder_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to NuGet 패키지를 복원하는 데 필요한 패키지 폴더를 확인할 수 없습니다. -PackagesDirectory 또는 -SolutionDirectory를 지정하십시오..
        /// </summary>
        internal static string RestoreCommandCannotDeterminePackagesFolder_kor {
            get {
                return ResourceManager.GetString("RestoreCommandCannotDeterminePackagesFolder_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Nie można ustalić folderu pakietów w celu przywrócenia pakietów NuGet. Określ opcję -PackagesDirectory lub opcję -SolutionDirectory..
        /// </summary>
        internal static string RestoreCommandCannotDeterminePackagesFolder_plk {
            get {
                return ResourceManager.GetString("RestoreCommandCannotDeterminePackagesFolder_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Não é possível determinar os pacotes de pasta para restaurar pacotes NuGet. Especifique SolutionDirectory ou PackagesDirectory..
        /// </summary>
        internal static string RestoreCommandCannotDeterminePackagesFolder_ptb {
            get {
                return ResourceManager.GetString("RestoreCommandCannotDeterminePackagesFolder_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Не удается определить папку пакетов для восстановления пакетов NuGet. Укажите -PackagesDirectory или -SolutionDirectory..
        /// </summary>
        internal static string RestoreCommandCannotDeterminePackagesFolder_rus {
            get {
                return ResourceManager.GetString("RestoreCommandCannotDeterminePackagesFolder_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to NuGet paketlerinin geri yükleneceği paket klasörü belirlenemiyor. Lütfen -PackagesDirectory veya -SolutionDirectory seçimini belirtin..
        /// </summary>
        internal static string RestoreCommandCannotDeterminePackagesFolder_trk {
            get {
                return ResourceManager.GetString("RestoreCommandCannotDeterminePackagesFolder_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Input file does not exist: {0}..
        /// </summary>
        internal static string RestoreCommandFileNotFound {
            get {
                return ResourceManager.GetString("RestoreCommandFileNotFound", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 输入文件不存在: {0}。.
        /// </summary>
        internal static string RestoreCommandFileNotFound_chs {
            get {
                return ResourceManager.GetString("RestoreCommandFileNotFound_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 輸入檔不存在: {0}。.
        /// </summary>
        internal static string RestoreCommandFileNotFound_cht {
            get {
                return ResourceManager.GetString("RestoreCommandFileNotFound_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Vstupní soubor neexistuje: {0}..
        /// </summary>
        internal static string RestoreCommandFileNotFound_csy {
            get {
                return ResourceManager.GetString("RestoreCommandFileNotFound_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Die Eingabedatei ist nicht vorhanden: {0}..
        /// </summary>
        internal static string RestoreCommandFileNotFound_deu {
            get {
                return ResourceManager.GetString("RestoreCommandFileNotFound_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to El archivo de entrada no existe: {0}..
        /// </summary>
        internal static string RestoreCommandFileNotFound_esp {
            get {
                return ResourceManager.GetString("RestoreCommandFileNotFound_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Le fichier d&apos;entrée n&apos;existe pas : {0}..
        /// </summary>
        internal static string RestoreCommandFileNotFound_fra {
            get {
                return ResourceManager.GetString("RestoreCommandFileNotFound_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Input file inesistente: {0}..
        /// </summary>
        internal static string RestoreCommandFileNotFound_ita {
            get {
                return ResourceManager.GetString("RestoreCommandFileNotFound_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 入力ファイルが存在しません: {0}..
        /// </summary>
        internal static string RestoreCommandFileNotFound_jpn {
            get {
                return ResourceManager.GetString("RestoreCommandFileNotFound_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 입력 파일이 없습니다. {0}..
        /// </summary>
        internal static string RestoreCommandFileNotFound_kor {
            get {
                return ResourceManager.GetString("RestoreCommandFileNotFound_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Plik wejściowy nie istnieje: {0}..
        /// </summary>
        internal static string RestoreCommandFileNotFound_plk {
            get {
                return ResourceManager.GetString("RestoreCommandFileNotFound_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to O arquivo de entrada não existe: {0}..
        /// </summary>
        internal static string RestoreCommandFileNotFound_ptb {
            get {
                return ResourceManager.GetString("RestoreCommandFileNotFound_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Входной файл не существует: {0}..
        /// </summary>
        internal static string RestoreCommandFileNotFound_rus {
            get {
                return ResourceManager.GetString("RestoreCommandFileNotFound_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Girdi dosyası mevcut değil: {0}..
        /// </summary>
        internal static string RestoreCommandFileNotFound_trk {
            get {
                return ResourceManager.GetString("RestoreCommandFileNotFound_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Option -SolutionDirectory is not valid when restoring packages for a solution..
        /// </summary>
        internal static string RestoreCommandOptionSolutionDirectoryIsInvalid {
            get {
                return ResourceManager.GetString("RestoreCommandOptionSolutionDirectoryIsInvalid", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 为解决方案还原程序包时，选项 -SolutionDirectory 无效。.
        /// </summary>
        internal static string RestoreCommandOptionSolutionDirectoryIsInvalid_chs {
            get {
                return ResourceManager.GetString("RestoreCommandOptionSolutionDirectoryIsInvalid_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 為方案還原封裝時 Option -SolutionDirectory 為無效。.
        /// </summary>
        internal static string RestoreCommandOptionSolutionDirectoryIsInvalid_cht {
            get {
                return ResourceManager.GetString("RestoreCommandOptionSolutionDirectoryIsInvalid_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Možnost -SolutionDirectory není platná při obnovování balíčků pro řešení..
        /// </summary>
        internal static string RestoreCommandOptionSolutionDirectoryIsInvalid_csy {
            get {
                return ResourceManager.GetString("RestoreCommandOptionSolutionDirectoryIsInvalid_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Die Option &quot;-SolutionDirectory&quot; ist beim Wiederherstellen von Paketen für ein Projekt nicht gültig..
        /// </summary>
        internal static string RestoreCommandOptionSolutionDirectoryIsInvalid_deu {
            get {
                return ResourceManager.GetString("RestoreCommandOptionSolutionDirectoryIsInvalid_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to La opción SolutionDirectory no es válida cuando restaura paquetes para una solución..
        /// </summary>
        internal static string RestoreCommandOptionSolutionDirectoryIsInvalid_esp {
            get {
                return ResourceManager.GetString("RestoreCommandOptionSolutionDirectoryIsInvalid_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to L&apos;option -SolutionDirectory n&apos;est pas valide lors de la restauration de packages pour une solution..
        /// </summary>
        internal static string RestoreCommandOptionSolutionDirectoryIsInvalid_fra {
            get {
                return ResourceManager.GetString("RestoreCommandOptionSolutionDirectoryIsInvalid_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Option -SolutionDirectory non è valida quando si ripristina pacchetti per una soluzione..
        /// </summary>
        internal static string RestoreCommandOptionSolutionDirectoryIsInvalid_ita {
            get {
                return ResourceManager.GetString("RestoreCommandOptionSolutionDirectoryIsInvalid_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to ソリューションのパッケージを復元する場合、オプション -SolutionDirectory は無効です。.
        /// </summary>
        internal static string RestoreCommandOptionSolutionDirectoryIsInvalid_jpn {
            get {
                return ResourceManager.GetString("RestoreCommandOptionSolutionDirectoryIsInvalid_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 솔루션 패키지를 복원하는 경우 -SolutionDirectory 옵션은 사용할 수 없습니다..
        /// </summary>
        internal static string RestoreCommandOptionSolutionDirectoryIsInvalid_kor {
            get {
                return ResourceManager.GetString("RestoreCommandOptionSolutionDirectoryIsInvalid_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Opcja -SolutionDirectory jest nieprawidłowa podczas przywracania pakietów dla rozwiązania..
        /// </summary>
        internal static string RestoreCommandOptionSolutionDirectoryIsInvalid_plk {
            get {
                return ResourceManager.GetString("RestoreCommandOptionSolutionDirectoryIsInvalid_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Option-SolutionDirectory não é válido ao restaurar pacotes para uma solução..
        /// </summary>
        internal static string RestoreCommandOptionSolutionDirectoryIsInvalid_ptb {
            get {
                return ResourceManager.GetString("RestoreCommandOptionSolutionDirectoryIsInvalid_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to При восстановлении пакетов для решения параметр -SolutionDirectory является недопустимым..
        /// </summary>
        internal static string RestoreCommandOptionSolutionDirectoryIsInvalid_rus {
            get {
                return ResourceManager.GetString("RestoreCommandOptionSolutionDirectoryIsInvalid_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Bir çözüme ait paketler geri yüklenirken -SolutionDirectory seçeneği geçerli değildir..
        /// </summary>
        internal static string RestoreCommandOptionSolutionDirectoryIsInvalid_trk {
            get {
                return ResourceManager.GetString("RestoreCommandOptionSolutionDirectoryIsInvalid_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Restoring NuGet packages...
        ///To prevent NuGet from downloading packages during build, open the Visual Studio Options dialog, click on the Package Manager node and uncheck &apos;{0}&apos;..
        /// </summary>
        internal static string RestoreCommandPackageRestoreOptOutMessage {
            get {
                return ResourceManager.GetString("RestoreCommandPackageRestoreOptOutMessage", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 正在还原 NuGet 程序包...
        ///若要防止 NuGet 在生成期间下载程序包，请打开“Visual Studio 选项”对话框，单击“程序包管理器”节点并取消选中“{0}”。.
        /// </summary>
        internal static string RestoreCommandPackageRestoreOptOutMessage_chs {
            get {
                return ResourceManager.GetString("RestoreCommandPackageRestoreOptOutMessage_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 正在還原 NuGet packages...
        ///若要防止 NuGet 在建置時下載封裝，按一下 [封裝管理員] 節點並取消勾選 &apos;{0}&apos;。.
        /// </summary>
        internal static string RestoreCommandPackageRestoreOptOutMessage_cht {
            get {
                return ResourceManager.GetString("RestoreCommandPackageRestoreOptOutMessage_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Probíhá obnovení balíčků NuGet...
        ///Chcete-li systému NuGet zabránit ve stahování balíčků během sestavování, otevřete dialogové okno Možnosti sady Visual Studio, klikněte na uzel Správce balíčků a zrušte zaškrtnutí položky {0}..
        /// </summary>
        internal static string RestoreCommandPackageRestoreOptOutMessage_csy {
            get {
                return ResourceManager.GetString("RestoreCommandPackageRestoreOptOutMessage_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to NuGet-Pakete werden wiederhergestellt...
        ///Damit verhindert wird, dass NuGet Pakete während des Buildvorgangs herunterlädt, öffnen Sie das Dialogfeld &quot;Optionen&quot; von Visual Studio, klicken Sie auf den Knoten &quot;Paket-Manager&quot;, und deaktivieren Sie dann &quot;{0}&quot;..
        /// </summary>
        internal static string RestoreCommandPackageRestoreOptOutMessage_deu {
            get {
                return ResourceManager.GetString("RestoreCommandPackageRestoreOptOutMessage_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Restaurando paquetes NuGet…  
        ///Para impedir que NuGet descargue paquetes durante la compilación, abra el cuadro de diálogo Opciones de Visual Studio, haga clic en el nodo del Administrador de paquetes y desactive &apos;{0}&apos;. .
        /// </summary>
        internal static string RestoreCommandPackageRestoreOptOutMessage_esp {
            get {
                return ResourceManager.GetString("RestoreCommandPackageRestoreOptOutMessage_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Restauration des packages NuGet…
        ///Pour empêcher NuGet de télécharger des packages lors de la création, ouvrez la boîte de dialogue Options Visual Studio, cliquez sur le nœud Gestionnaire de package et décochez &apos;{0}&apos;..
        /// </summary>
        internal static string RestoreCommandPackageRestoreOptOutMessage_fra {
            get {
                return ResourceManager.GetString("RestoreCommandPackageRestoreOptOutMessage_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Ripristinando pacchetti NuGet…                               Per evitare che NuGet scarichi pacchetti durante il build, aprire la finestra di dialogo Visual Studio Options dialog, fare clic su Package Manager e deselezionare &apos;{0}&apos;..
        /// </summary>
        internal static string RestoreCommandPackageRestoreOptOutMessage_ita {
            get {
                return ResourceManager.GetString("RestoreCommandPackageRestoreOptOutMessage_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to NuGet パッケージを復元しています...
        ///ビルド中に NuGet がパッケージをダウンロードしないようにするには、Visual Studio の [オプション] ダイアログを開き、パッケージ マネージャー ノードをクリックし、&apos;{0}&apos; をオフにします。.
        /// </summary>
        internal static string RestoreCommandPackageRestoreOptOutMessage_jpn {
            get {
                return ResourceManager.GetString("RestoreCommandPackageRestoreOptOutMessage_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to NuGet 패키지를 복원하는 중...
        ///빌드 시 NuGet이 패키지를 다운로드하지 않도록 하려면 [Visual Studio 옵션] 대화 상자를 열고 [패키지 관리자] 노드를 클릭한 후 &apos;{0}&apos;을(를) 선택 취소합니다..
        /// </summary>
        internal static string RestoreCommandPackageRestoreOptOutMessage_kor {
            get {
                return ResourceManager.GetString("RestoreCommandPackageRestoreOptOutMessage_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Trwa przywracanie pakietów NuGet...
        ///Aby uniemożliwić pobieranie pakietów NuGet podczas kompilowania, otwórz okno dialogowe opcji programu Visual Studio, kliknij węzeł Menedżera pakietów i usuń zaznaczenie pozycji „{0}”..
        /// </summary>
        internal static string RestoreCommandPackageRestoreOptOutMessage_plk {
            get {
                return ResourceManager.GetString("RestoreCommandPackageRestoreOptOutMessage_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Restaurando pacotes NuGet ...
        ///Para evitar que o NuGet baixe pacotes durante a construção, abra a caixa de diálogo Opções do Visual Studio, clique no nó Gerenciador de Pacotes e desmarque a opção &apos;{0}&apos;..
        /// </summary>
        internal static string RestoreCommandPackageRestoreOptOutMessage_ptb {
            get {
                return ResourceManager.GetString("RestoreCommandPackageRestoreOptOutMessage_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Восстановление пакетов NuGet…
        ///Чтобы предотвратить загрузку пакетов NuGet во время выполнения сборки, в Visual Studio откройте диалоговое окно &quot;Параметры&quot;, выберите узел &quot;Диспетчер пакетов&quot; и снимите флажок &quot;{0}&quot;..
        /// </summary>
        internal static string RestoreCommandPackageRestoreOptOutMessage_rus {
            get {
                return ResourceManager.GetString("RestoreCommandPackageRestoreOptOutMessage_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to NuGet paketleri geri yükleniyor...
        ///Oluşturma sırasında NuGet&apos;in paketleri indirmesini önlemek için, Visual Studio Seçenekleri iletişim kutusunu açın, Paket Yöneticisi düğümünü tıklayıp &apos;{0}&apos; seçimini işaretleyin..
        /// </summary>
        internal static string RestoreCommandPackageRestoreOptOutMessage_trk {
            get {
                return ResourceManager.GetString("RestoreCommandPackageRestoreOptOutMessage_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Project file {0} cannot be found..
        /// </summary>
        internal static string RestoreCommandProjectNotFound {
            get {
                return ResourceManager.GetString("RestoreCommandProjectNotFound", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 找不到项目文件 {0}。.
        /// </summary>
        internal static string RestoreCommandProjectNotFound_chs {
            get {
                return ResourceManager.GetString("RestoreCommandProjectNotFound_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 找不到專案檔 {0}。.
        /// </summary>
        internal static string RestoreCommandProjectNotFound_cht {
            get {
                return ResourceManager.GetString("RestoreCommandProjectNotFound_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Soubor projektu {0} nebyl nalezen..
        /// </summary>
        internal static string RestoreCommandProjectNotFound_csy {
            get {
                return ResourceManager.GetString("RestoreCommandProjectNotFound_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Die Projektdatei &quot;{0}&quot; wurde nicht gefunden..
        /// </summary>
        internal static string RestoreCommandProjectNotFound_deu {
            get {
                return ResourceManager.GetString("RestoreCommandProjectNotFound_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to No se puede encontrar el archivo de proyecto {0}..
        /// </summary>
        internal static string RestoreCommandProjectNotFound_esp {
            get {
                return ResourceManager.GetString("RestoreCommandProjectNotFound_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Le fichier projet {0} est introuvable..
        /// </summary>
        internal static string RestoreCommandProjectNotFound_fra {
            get {
                return ResourceManager.GetString("RestoreCommandProjectNotFound_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Impossibile trovare file progetto {0}..
        /// </summary>
        internal static string RestoreCommandProjectNotFound_ita {
            get {
                return ResourceManager.GetString("RestoreCommandProjectNotFound_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to プロジェクト ファイル {0} が見つかりません。.
        /// </summary>
        internal static string RestoreCommandProjectNotFound_jpn {
            get {
                return ResourceManager.GetString("RestoreCommandProjectNotFound_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 프로젝트 파일 {0}을(를) 찾을 수 없습니다..
        /// </summary>
        internal static string RestoreCommandProjectNotFound_kor {
            get {
                return ResourceManager.GetString("RestoreCommandProjectNotFound_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Nie można znaleźć pliku projektu {0}..
        /// </summary>
        internal static string RestoreCommandProjectNotFound_plk {
            get {
                return ResourceManager.GetString("RestoreCommandProjectNotFound_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Não foi possível localizar o arquivo de projeto {0}..
        /// </summary>
        internal static string RestoreCommandProjectNotFound_ptb {
            get {
                return ResourceManager.GetString("RestoreCommandProjectNotFound_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Не удалось найти файл проекта {0}..
        /// </summary>
        internal static string RestoreCommandProjectNotFound_rus {
            get {
                return ResourceManager.GetString("RestoreCommandProjectNotFound_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} proje dosyası bulunamadı..
        /// </summary>
        internal static string RestoreCommandProjectNotFound_trk {
            get {
                return ResourceManager.GetString("RestoreCommandProjectNotFound_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Restoring NuGet packages for solution {0}..
        /// </summary>
        internal static string RestoreCommandRestoringPackagesForSolution {
            get {
                return ResourceManager.GetString("RestoreCommandRestoringPackagesForSolution", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 正在为解决方案 {0} 还原 NuGet 程序包。.
        /// </summary>
        internal static string RestoreCommandRestoringPackagesForSolution_chs {
            get {
                return ResourceManager.GetString("RestoreCommandRestoringPackagesForSolution_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 正在為方案 {0} 還原 NuGet 封裝。.
        /// </summary>
        internal static string RestoreCommandRestoringPackagesForSolution_cht {
            get {
                return ResourceManager.GetString("RestoreCommandRestoringPackagesForSolution_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Probíhá obnovení balíčků NuGet pro řešení {0}..
        /// </summary>
        internal static string RestoreCommandRestoringPackagesForSolution_csy {
            get {
                return ResourceManager.GetString("RestoreCommandRestoringPackagesForSolution_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to NuGet-Pakete für das Projekt &quot;{0}&quot; werden wiederhergestellt..
        /// </summary>
        internal static string RestoreCommandRestoringPackagesForSolution_deu {
            get {
                return ResourceManager.GetString("RestoreCommandRestoringPackagesForSolution_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Restaurando paquetes NuGet para la solución {0}..
        /// </summary>
        internal static string RestoreCommandRestoringPackagesForSolution_esp {
            get {
                return ResourceManager.GetString("RestoreCommandRestoringPackagesForSolution_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Restauration des packages NuGet pour la solution {0}..
        /// </summary>
        internal static string RestoreCommandRestoringPackagesForSolution_fra {
            get {
                return ResourceManager.GetString("RestoreCommandRestoringPackagesForSolution_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Ripristinando pacchetti NuGet per soluzione{0}..
        /// </summary>
        internal static string RestoreCommandRestoringPackagesForSolution_ita {
            get {
                return ResourceManager.GetString("RestoreCommandRestoringPackagesForSolution_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to NuGet パッケージをソリューション {0} 用に復元しています。.
        /// </summary>
        internal static string RestoreCommandRestoringPackagesForSolution_jpn {
            get {
                return ResourceManager.GetString("RestoreCommandRestoringPackagesForSolution_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} 솔루션의 NuGet 패키지를 복원하고 있습니다..
        /// </summary>
        internal static string RestoreCommandRestoringPackagesForSolution_kor {
            get {
                return ResourceManager.GetString("RestoreCommandRestoringPackagesForSolution_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Przywracanie pakietów NuGet dla rozwiązania {0}..
        /// </summary>
        internal static string RestoreCommandRestoringPackagesForSolution_plk {
            get {
                return ResourceManager.GetString("RestoreCommandRestoringPackagesForSolution_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Restaurar pacotes NuGet para a solução {0}..
        /// </summary>
        internal static string RestoreCommandRestoringPackagesForSolution_ptb {
            get {
                return ResourceManager.GetString("RestoreCommandRestoringPackagesForSolution_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Восстановление пакетов NuGet для решения {0}..
        /// </summary>
        internal static string RestoreCommandRestoringPackagesForSolution_rus {
            get {
                return ResourceManager.GetString("RestoreCommandRestoringPackagesForSolution_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} çözümü için NuGet paketleri geri yükleniyor..
        /// </summary>
        internal static string RestoreCommandRestoringPackagesForSolution_trk {
            get {
                return ResourceManager.GetString("RestoreCommandRestoringPackagesForSolution_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Restoring NuGet packages listed in packages.config file..
        /// </summary>
        internal static string RestoreCommandRestoringPackagesFromPackagesConfigFile {
            get {
                return ResourceManager.GetString("RestoreCommandRestoringPackagesFromPackagesConfigFile", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 正在还原 packages.config 文件中列出的 NuGet 程序包。.
        /// </summary>
        internal static string RestoreCommandRestoringPackagesFromPackagesConfigFile_chs {
            get {
                return ResourceManager.GetString("RestoreCommandRestoringPackagesFromPackagesConfigFile_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 正在還原列在 packages.config 檔案中的 NuGet 封裝。.
        /// </summary>
        internal static string RestoreCommandRestoringPackagesFromPackagesConfigFile_cht {
            get {
                return ResourceManager.GetString("RestoreCommandRestoringPackagesFromPackagesConfigFile_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Probíhá obnovení balíčků NuGet uvedených v souboru packages.config..
        /// </summary>
        internal static string RestoreCommandRestoringPackagesFromPackagesConfigFile_csy {
            get {
                return ResourceManager.GetString("RestoreCommandRestoringPackagesFromPackagesConfigFile_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Die in der Datei &quot;packages.config&quot; aufgelisteten NuGet-Pakete werden wiederhergestellt..
        /// </summary>
        internal static string RestoreCommandRestoringPackagesFromPackagesConfigFile_deu {
            get {
                return ResourceManager.GetString("RestoreCommandRestoringPackagesFromPackagesConfigFile_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Restaurando paquetes NuGet mostrados en el archivo packages.config..
        /// </summary>
        internal static string RestoreCommandRestoringPackagesFromPackagesConfigFile_esp {
            get {
                return ResourceManager.GetString("RestoreCommandRestoringPackagesFromPackagesConfigFile_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Restauration des packages NuGet répertoriés dans le fichier packages.config..
        /// </summary>
        internal static string RestoreCommandRestoringPackagesFromPackagesConfigFile_fra {
            get {
                return ResourceManager.GetString("RestoreCommandRestoringPackagesFromPackagesConfigFile_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Ripristinando pacchetti NuGet elencati in packages.config file..
        /// </summary>
        internal static string RestoreCommandRestoringPackagesFromPackagesConfigFile_ita {
            get {
                return ResourceManager.GetString("RestoreCommandRestoringPackagesFromPackagesConfigFile_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to packages.config ファイルに含まれる NuGet パッケージを復元しています。.
        /// </summary>
        internal static string RestoreCommandRestoringPackagesFromPackagesConfigFile_jpn {
            get {
                return ResourceManager.GetString("RestoreCommandRestoringPackagesFromPackagesConfigFile_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to packages.config 파일에 나열된 NuGet 패키지를 복원하고 있습니다..
        /// </summary>
        internal static string RestoreCommandRestoringPackagesFromPackagesConfigFile_kor {
            get {
                return ResourceManager.GetString("RestoreCommandRestoringPackagesFromPackagesConfigFile_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Przywracanie pakietów NuGet wymienionych w pliku packages.config..
        /// </summary>
        internal static string RestoreCommandRestoringPackagesFromPackagesConfigFile_plk {
            get {
                return ResourceManager.GetString("RestoreCommandRestoringPackagesFromPackagesConfigFile_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Restaurar pacotes NuGet listados no arquivo de packages.config..
        /// </summary>
        internal static string RestoreCommandRestoringPackagesFromPackagesConfigFile_ptb {
            get {
                return ResourceManager.GetString("RestoreCommandRestoringPackagesFromPackagesConfigFile_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Восстановление пакетов NuGet, перечисленных в файле packages.config..
        /// </summary>
        internal static string RestoreCommandRestoringPackagesFromPackagesConfigFile_rus {
            get {
                return ResourceManager.GetString("RestoreCommandRestoringPackagesFromPackagesConfigFile_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Packages.config dosyasında listelenen NuGet paketleri geri yükleniyor..
        /// </summary>
        internal static string RestoreCommandRestoringPackagesFromPackagesConfigFile_trk {
            get {
                return ResourceManager.GetString("RestoreCommandRestoringPackagesFromPackagesConfigFile_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Restoring NuGet packages listed in file {0}..
        /// </summary>
        internal static string RestoreCommandRestoringPackagesListedInFile {
            get {
                return ResourceManager.GetString("RestoreCommandRestoringPackagesListedInFile", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 正在还原文件 {0} 中列出的 NuGet 程序包。.
        /// </summary>
        internal static string RestoreCommandRestoringPackagesListedInFile_chs {
            get {
                return ResourceManager.GetString("RestoreCommandRestoringPackagesListedInFile_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 正在還原列在檔案 {0} 中的 NuGet 封裝。.
        /// </summary>
        internal static string RestoreCommandRestoringPackagesListedInFile_cht {
            get {
                return ResourceManager.GetString("RestoreCommandRestoringPackagesListedInFile_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Probíhá obnovení balíčků NuGet uvedených v souboru {0}..
        /// </summary>
        internal static string RestoreCommandRestoringPackagesListedInFile_csy {
            get {
                return ResourceManager.GetString("RestoreCommandRestoringPackagesListedInFile_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Die in der Datei &quot;{0}&quot; aufgelisteten NuGet-Pakete werden wiederhergestellt..
        /// </summary>
        internal static string RestoreCommandRestoringPackagesListedInFile_deu {
            get {
                return ResourceManager.GetString("RestoreCommandRestoringPackagesListedInFile_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Restaurando paquetes NuGet mostrados en el archivo {0}..
        /// </summary>
        internal static string RestoreCommandRestoringPackagesListedInFile_esp {
            get {
                return ResourceManager.GetString("RestoreCommandRestoringPackagesListedInFile_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Restauration des packages NuGet répertoriés dans le fichier {0}..
        /// </summary>
        internal static string RestoreCommandRestoringPackagesListedInFile_fra {
            get {
                return ResourceManager.GetString("RestoreCommandRestoringPackagesListedInFile_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Ripristinando pacchetti NuGet elencati in file {0}..
        /// </summary>
        internal static string RestoreCommandRestoringPackagesListedInFile_ita {
            get {
                return ResourceManager.GetString("RestoreCommandRestoringPackagesListedInFile_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to ファイル {0} に含まれる NuGet パッケージを復元しています。.
        /// </summary>
        internal static string RestoreCommandRestoringPackagesListedInFile_jpn {
            get {
                return ResourceManager.GetString("RestoreCommandRestoringPackagesListedInFile_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} 파일에 나열된 NuGet 패키지를 복원하고 있습니다..
        /// </summary>
        internal static string RestoreCommandRestoringPackagesListedInFile_kor {
            get {
                return ResourceManager.GetString("RestoreCommandRestoringPackagesListedInFile_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Przywracanie pakietów NuGet wymienionych w pliku {0}..
        /// </summary>
        internal static string RestoreCommandRestoringPackagesListedInFile_plk {
            get {
                return ResourceManager.GetString("RestoreCommandRestoringPackagesListedInFile_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Restaurar pacotes NuGet listados no arquivo {0}..
        /// </summary>
        internal static string RestoreCommandRestoringPackagesListedInFile_ptb {
            get {
                return ResourceManager.GetString("RestoreCommandRestoringPackagesListedInFile_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Восстановление пакетов NuGet, перечисленных в файле {0}..
        /// </summary>
        internal static string RestoreCommandRestoringPackagesListedInFile_rus {
            get {
                return ResourceManager.GetString("RestoreCommandRestoringPackagesListedInFile_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} içinde listelenen NuGet paketleri geri yükleniyor..
        /// </summary>
        internal static string RestoreCommandRestoringPackagesListedInFile_trk {
            get {
                return ResourceManager.GetString("RestoreCommandRestoringPackagesListedInFile_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Scanning for projects....
        /// </summary>
        internal static string ScanningForProjects {
            get {
                return ResourceManager.GetString("ScanningForProjects", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 正在扫描项目....
        /// </summary>
        internal static string ScanningForProjects_chs {
            get {
                return ResourceManager.GetString("ScanningForProjects_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 正在掃瞄專案....
        /// </summary>
        internal static string ScanningForProjects_cht {
            get {
                return ResourceManager.GetString("ScanningForProjects_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Probíhá hledání projektů....
        /// </summary>
        internal static string ScanningForProjects_csy {
            get {
                return ResourceManager.GetString("ScanningForProjects_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Suchen nach Projekten....
        /// </summary>
        internal static string ScanningForProjects_deu {
            get {
                return ResourceManager.GetString("ScanningForProjects_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Examinando proyectos….
        /// </summary>
        internal static string ScanningForProjects_esp {
            get {
                return ResourceManager.GetString("ScanningForProjects_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Recherche de projets en cours….
        /// </summary>
        internal static string ScanningForProjects_fra {
            get {
                return ResourceManager.GetString("ScanningForProjects_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Ricercando progetti....
        /// </summary>
        internal static string ScanningForProjects_ita {
            get {
                return ResourceManager.GetString("ScanningForProjects_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to プロジェクトをスキャンしています....
        /// </summary>
        internal static string ScanningForProjects_jpn {
            get {
                return ResourceManager.GetString("ScanningForProjects_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 프로젝트를 스캔하는 중....
        /// </summary>
        internal static string ScanningForProjects_kor {
            get {
                return ResourceManager.GetString("ScanningForProjects_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Trwa skanowanie w poszukiwaniu projektów....
        /// </summary>
        internal static string ScanningForProjects_plk {
            get {
                return ResourceManager.GetString("ScanningForProjects_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Verificando projetos ....
        /// </summary>
        internal static string ScanningForProjects_ptb {
            get {
                return ResourceManager.GetString("ScanningForProjects_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Сканирование проектов….
        /// </summary>
        internal static string ScanningForProjects_rus {
            get {
                return ResourceManager.GetString("ScanningForProjects_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Projeler taranıyor….
        /// </summary>
        internal static string ScanningForProjects_trk {
            get {
                return ResourceManager.GetString("ScanningForProjects_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to The API Key &apos;{0}&apos; was saved for {1}..
        /// </summary>
        internal static string SetApiKeyCommandApiKeySaved {
            get {
                return ResourceManager.GetString("SetApiKeyCommandApiKeySaved", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 已保存 {1} 的 API 密钥“{0}”。.
        /// </summary>
        internal static string SetApiKeyCommandApiKeySaved_chs {
            get {
                return ResourceManager.GetString("SetApiKeyCommandApiKeySaved_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 已為 {1} 儲存 API 索引鍵 &apos;{0}&apos;。.
        /// </summary>
        internal static string SetApiKeyCommandApiKeySaved_cht {
            get {
                return ResourceManager.GetString("SetApiKeyCommandApiKeySaved_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Klíč API {0} byl uložen pro {1}..
        /// </summary>
        internal static string SetApiKeyCommandApiKeySaved_csy {
            get {
                return ResourceManager.GetString("SetApiKeyCommandApiKeySaved_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Der API-Schlüssel &quot;{0}&quot; wurde für &quot;{1}&quot; gespeichert..
        /// </summary>
        internal static string SetApiKeyCommandApiKeySaved_deu {
            get {
                return ResourceManager.GetString("SetApiKeyCommandApiKeySaved_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to La clave API &apos;{0}&apos; se ha guardado para {1}..
        /// </summary>
        internal static string SetApiKeyCommandApiKeySaved_esp {
            get {
                return ResourceManager.GetString("SetApiKeyCommandApiKeySaved_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to La clé API &apos;{0}&apos; a été enregistrée pour {1}..
        /// </summary>
        internal static string SetApiKeyCommandApiKeySaved_fra {
            get {
                return ResourceManager.GetString("SetApiKeyCommandApiKeySaved_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to API Key &apos;{0}&apos; salvato per {1}..
        /// </summary>
        internal static string SetApiKeyCommandApiKeySaved_ita {
            get {
                return ResourceManager.GetString("SetApiKeyCommandApiKeySaved_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {1} の API キー &apos;{0}&apos; が保存されました。.
        /// </summary>
        internal static string SetApiKeyCommandApiKeySaved_jpn {
            get {
                return ResourceManager.GetString("SetApiKeyCommandApiKeySaved_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to API 키 &apos;{0}&apos;이(가) {1}에 저장되었습니다..
        /// </summary>
        internal static string SetApiKeyCommandApiKeySaved_kor {
            get {
                return ResourceManager.GetString("SetApiKeyCommandApiKeySaved_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Klucz interfejsu API „{0}” został zapisany dla {1}..
        /// </summary>
        internal static string SetApiKeyCommandApiKeySaved_plk {
            get {
                return ResourceManager.GetString("SetApiKeyCommandApiKeySaved_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to A chave de API &apos;{0}&apos; foi salva para {1}..
        /// </summary>
        internal static string SetApiKeyCommandApiKeySaved_ptb {
            get {
                return ResourceManager.GetString("SetApiKeyCommandApiKeySaved_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Был сохранен ключ API &quot;{0}&quot; для {1}..
        /// </summary>
        internal static string SetApiKeyCommandApiKeySaved_rus {
            get {
                return ResourceManager.GetString("SetApiKeyCommandApiKeySaved_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {1} için &apos;{0}&apos; API Anahtarı kaydedildi..
        /// </summary>
        internal static string SetApiKeyCommandApiKeySaved_trk {
            get {
                return ResourceManager.GetString("SetApiKeyCommandApiKeySaved_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to The API Key &apos;{0}&apos; was saved for {1} and {2}..
        /// </summary>
        internal static string SetApiKeyCommandDefaultApiKeysSaved {
            get {
                return ResourceManager.GetString("SetApiKeyCommandDefaultApiKeysSaved", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 已保存 {1} 和 {2} 的 API 密钥“{0}”。.
        /// </summary>
        internal static string SetApiKeyCommandDefaultApiKeysSaved_chs {
            get {
                return ResourceManager.GetString("SetApiKeyCommandDefaultApiKeysSaved_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 已為 {1} 和 {2} 儲存 API 索引鍵 &apos;{0}&apos;。.
        /// </summary>
        internal static string SetApiKeyCommandDefaultApiKeysSaved_cht {
            get {
                return ResourceManager.GetString("SetApiKeyCommandDefaultApiKeysSaved_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Klíč API {0} byl uložen pro {1} a {2}..
        /// </summary>
        internal static string SetApiKeyCommandDefaultApiKeysSaved_csy {
            get {
                return ResourceManager.GetString("SetApiKeyCommandDefaultApiKeysSaved_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Der API-Schlüssel &quot;{0}&quot; wurde für &quot;{1}&quot; und &quot;{2}&quot; gespeichert..
        /// </summary>
        internal static string SetApiKeyCommandDefaultApiKeysSaved_deu {
            get {
                return ResourceManager.GetString("SetApiKeyCommandDefaultApiKeysSaved_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to La clave API &apos;{0}&apos; se ha guardado para {1} y {2}..
        /// </summary>
        internal static string SetApiKeyCommandDefaultApiKeysSaved_esp {
            get {
                return ResourceManager.GetString("SetApiKeyCommandDefaultApiKeysSaved_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to La clé API &apos;{0}&apos; a été enregistrée pour {1} et {2}..
        /// </summary>
        internal static string SetApiKeyCommandDefaultApiKeysSaved_fra {
            get {
                return ResourceManager.GetString("SetApiKeyCommandDefaultApiKeysSaved_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to API Key &apos;{0}&apos;salvato per {1} e {2}..
        /// </summary>
        internal static string SetApiKeyCommandDefaultApiKeysSaved_ita {
            get {
                return ResourceManager.GetString("SetApiKeyCommandDefaultApiKeysSaved_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {1} と {2} の API キー &apos;{0}&apos; が保存されました。.
        /// </summary>
        internal static string SetApiKeyCommandDefaultApiKeysSaved_jpn {
            get {
                return ResourceManager.GetString("SetApiKeyCommandDefaultApiKeysSaved_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to API 키 &apos;{0}&apos;이(가) {1} 및 {2}에 저장되었습니다..
        /// </summary>
        internal static string SetApiKeyCommandDefaultApiKeysSaved_kor {
            get {
                return ResourceManager.GetString("SetApiKeyCommandDefaultApiKeysSaved_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Klucz interfejsu API „{0}” został zapisany dla {1} i {2}..
        /// </summary>
        internal static string SetApiKeyCommandDefaultApiKeysSaved_plk {
            get {
                return ResourceManager.GetString("SetApiKeyCommandDefaultApiKeysSaved_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to A chave de API &apos;{0}&apos; foi salva para {1} e {2}..
        /// </summary>
        internal static string SetApiKeyCommandDefaultApiKeysSaved_ptb {
            get {
                return ResourceManager.GetString("SetApiKeyCommandDefaultApiKeysSaved_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Был сохранен ключ API &quot;{0}&quot; для {1} и {2}..
        /// </summary>
        internal static string SetApiKeyCommandDefaultApiKeysSaved_rus {
            get {
                return ResourceManager.GetString("SetApiKeyCommandDefaultApiKeysSaved_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {1} ve {2} için &apos;{0}&apos; API Anahtarı kaydedildi..
        /// </summary>
        internal static string SetApiKeyCommandDefaultApiKeysSaved_trk {
            get {
                return ResourceManager.GetString("SetApiKeyCommandDefaultApiKeysSaved_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Package source name &apos;All&apos; is a reserved name..
        /// </summary>
        internal static string SourcesCommandAllNameIsReserved {
            get {
                return ResourceManager.GetString("SourcesCommandAllNameIsReserved", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 程序包源名称 &quot;All&quot; 是保留名称。.
        /// </summary>
        internal static string SourcesCommandAllNameIsReserved_chs {
            get {
                return ResourceManager.GetString("SourcesCommandAllNameIsReserved_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 封裝來源名稱 &apos;All&apos; 為保留的名稱。.
        /// </summary>
        internal static string SourcesCommandAllNameIsReserved_cht {
            get {
                return ResourceManager.GetString("SourcesCommandAllNameIsReserved_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Název zdroje balíčku All je vyhrazený název..
        /// </summary>
        internal static string SourcesCommandAllNameIsReserved_csy {
            get {
                return ResourceManager.GetString("SourcesCommandAllNameIsReserved_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Der Paketquellname &quot;All&quot; ist ein reservierter Name..
        /// </summary>
        internal static string SourcesCommandAllNameIsReserved_deu {
            get {
                return ResourceManager.GetString("SourcesCommandAllNameIsReserved_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to El nombre de origen del paquete &apos;Todo&apos; es un nombre reservado..
        /// </summary>
        internal static string SourcesCommandAllNameIsReserved_esp {
            get {
                return ResourceManager.GetString("SourcesCommandAllNameIsReserved_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Le nom de source de package &apos;All&apos; est réservé..
        /// </summary>
        internal static string SourcesCommandAllNameIsReserved_fra {
            get {
                return ResourceManager.GetString("SourcesCommandAllNameIsReserved_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Nome pacchetto &apos;Tutti&apos; è un nome riservato..
        /// </summary>
        internal static string SourcesCommandAllNameIsReserved_ita {
            get {
                return ResourceManager.GetString("SourcesCommandAllNameIsReserved_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to パッケージ ソース名 &apos;All&apos; は予約名です。.
        /// </summary>
        internal static string SourcesCommandAllNameIsReserved_jpn {
            get {
                return ResourceManager.GetString("SourcesCommandAllNameIsReserved_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 패키지 소스 이름 &apos;All&apos;은 예약된 이름입니다..
        /// </summary>
        internal static string SourcesCommandAllNameIsReserved_kor {
            get {
                return ResourceManager.GetString("SourcesCommandAllNameIsReserved_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Nazwa źródła pakietów „Wszystkie” jest nazwą zarezerwowaną..
        /// </summary>
        internal static string SourcesCommandAllNameIsReserved_plk {
            get {
                return ResourceManager.GetString("SourcesCommandAllNameIsReserved_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to O nome de origem do pacote &apos;Todos&apos; é um nome reservado..
        /// </summary>
        internal static string SourcesCommandAllNameIsReserved_ptb {
            get {
                return ResourceManager.GetString("SourcesCommandAllNameIsReserved_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Имя источника пакетов &quot;All&quot; является зарезервированным именем..
        /// </summary>
        internal static string SourcesCommandAllNameIsReserved_rus {
            get {
                return ResourceManager.GetString("SourcesCommandAllNameIsReserved_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Tümü&apos; paket kaynak adı ayrılmış bir addır..
        /// </summary>
        internal static string SourcesCommandAllNameIsReserved_trk {
            get {
                return ResourceManager.GetString("SourcesCommandAllNameIsReserved_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Both UserName and Password must be specified..
        /// </summary>
        internal static string SourcesCommandCredentialsRequired {
            get {
                return ResourceManager.GetString("SourcesCommandCredentialsRequired", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 必须同时指定用户名和密码。.
        /// </summary>
        internal static string SourcesCommandCredentialsRequired_chs {
            get {
                return ResourceManager.GetString("SourcesCommandCredentialsRequired_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 必須指定使用者名稱和密碼。.
        /// </summary>
        internal static string SourcesCommandCredentialsRequired_cht {
            get {
                return ResourceManager.GetString("SourcesCommandCredentialsRequired_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Je třeba zadat uživatelské jméno i heslo..
        /// </summary>
        internal static string SourcesCommandCredentialsRequired_csy {
            get {
                return ResourceManager.GetString("SourcesCommandCredentialsRequired_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to &quot;UserName&quot; und &quot;Password&quot; müssen angegeben werden..
        /// </summary>
        internal static string SourcesCommandCredentialsRequired_deu {
            get {
                return ResourceManager.GetString("SourcesCommandCredentialsRequired_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Se deben especificar tanto el nombre de usuario como la contraseña..
        /// </summary>
        internal static string SourcesCommandCredentialsRequired_esp {
            get {
                return ResourceManager.GetString("SourcesCommandCredentialsRequired_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Le @@@nom d&apos;utilisateur et le mot de passe doivent être spécifiés..
        /// </summary>
        internal static string SourcesCommandCredentialsRequired_fra {
            get {
                return ResourceManager.GetString("SourcesCommandCredentialsRequired_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Specificare nome utente e password..
        /// </summary>
        internal static string SourcesCommandCredentialsRequired_ita {
            get {
                return ResourceManager.GetString("SourcesCommandCredentialsRequired_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to UserName と Password の両方を指定する必要があります。.
        /// </summary>
        internal static string SourcesCommandCredentialsRequired_jpn {
            get {
                return ResourceManager.GetString("SourcesCommandCredentialsRequired_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 사용자 이름 및 암호를 모두 지정해야 합니다..
        /// </summary>
        internal static string SourcesCommandCredentialsRequired_kor {
            get {
                return ResourceManager.GetString("SourcesCommandCredentialsRequired_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Należy określić zarówno nazwę użytkownika, jak i hasło..
        /// </summary>
        internal static string SourcesCommandCredentialsRequired_plk {
            get {
                return ResourceManager.GetString("SourcesCommandCredentialsRequired_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to O Nome de usuário e senha devem ser especificados..
        /// </summary>
        internal static string SourcesCommandCredentialsRequired_ptb {
            get {
                return ResourceManager.GetString("SourcesCommandCredentialsRequired_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Следует указать имя пользователя и пароль..
        /// </summary>
        internal static string SourcesCommandCredentialsRequired_rus {
            get {
                return ResourceManager.GetString("SourcesCommandCredentialsRequired_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to KullanıcıAdı ve Parolanın belirtilmesi gerekir..
        /// </summary>
        internal static string SourcesCommandCredentialsRequired_trk {
            get {
                return ResourceManager.GetString("SourcesCommandCredentialsRequired_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Disabled.
        /// </summary>
        internal static string SourcesCommandDisabled {
            get {
                return ResourceManager.GetString("SourcesCommandDisabled", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 已禁用.
        /// </summary>
        internal static string SourcesCommandDisabled_chs {
            get {
                return ResourceManager.GetString("SourcesCommandDisabled_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 已停用.
        /// </summary>
        internal static string SourcesCommandDisabled_cht {
            get {
                return ResourceManager.GetString("SourcesCommandDisabled_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Zakázáno.
        /// </summary>
        internal static string SourcesCommandDisabled_csy {
            get {
                return ResourceManager.GetString("SourcesCommandDisabled_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Deaktiviert.
        /// </summary>
        internal static string SourcesCommandDisabled_deu {
            get {
                return ResourceManager.GetString("SourcesCommandDisabled_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Deshabilitado.
        /// </summary>
        internal static string SourcesCommandDisabled_esp {
            get {
                return ResourceManager.GetString("SourcesCommandDisabled_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Désactivé.
        /// </summary>
        internal static string SourcesCommandDisabled_fra {
            get {
                return ResourceManager.GetString("SourcesCommandDisabled_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Disabilitato.
        /// </summary>
        internal static string SourcesCommandDisabled_ita {
            get {
                return ResourceManager.GetString("SourcesCommandDisabled_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 無効.
        /// </summary>
        internal static string SourcesCommandDisabled_jpn {
            get {
                return ResourceManager.GetString("SourcesCommandDisabled_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 사용 안 함.
        /// </summary>
        internal static string SourcesCommandDisabled_kor {
            get {
                return ResourceManager.GetString("SourcesCommandDisabled_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Wyłączone.
        /// </summary>
        internal static string SourcesCommandDisabled_plk {
            get {
                return ResourceManager.GetString("SourcesCommandDisabled_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Desativado.
        /// </summary>
        internal static string SourcesCommandDisabled_ptb {
            get {
                return ResourceManager.GetString("SourcesCommandDisabled_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Отключено.
        /// </summary>
        internal static string SourcesCommandDisabled_rus {
            get {
                return ResourceManager.GetString("SourcesCommandDisabled_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Devre dışı.
        /// </summary>
        internal static string SourcesCommandDisabled_trk {
            get {
                return ResourceManager.GetString("SourcesCommandDisabled_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Enabled.
        /// </summary>
        internal static string SourcesCommandEnabled {
            get {
                return ResourceManager.GetString("SourcesCommandEnabled", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 已启用.
        /// </summary>
        internal static string SourcesCommandEnabled_chs {
            get {
                return ResourceManager.GetString("SourcesCommandEnabled_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 已啟用.
        /// </summary>
        internal static string SourcesCommandEnabled_cht {
            get {
                return ResourceManager.GetString("SourcesCommandEnabled_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Povoleno.
        /// </summary>
        internal static string SourcesCommandEnabled_csy {
            get {
                return ResourceManager.GetString("SourcesCommandEnabled_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Aktiviert.
        /// </summary>
        internal static string SourcesCommandEnabled_deu {
            get {
                return ResourceManager.GetString("SourcesCommandEnabled_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Habilitado.
        /// </summary>
        internal static string SourcesCommandEnabled_esp {
            get {
                return ResourceManager.GetString("SourcesCommandEnabled_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Activé.
        /// </summary>
        internal static string SourcesCommandEnabled_fra {
            get {
                return ResourceManager.GetString("SourcesCommandEnabled_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Abilitato.
        /// </summary>
        internal static string SourcesCommandEnabled_ita {
            get {
                return ResourceManager.GetString("SourcesCommandEnabled_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 有効.
        /// </summary>
        internal static string SourcesCommandEnabled_jpn {
            get {
                return ResourceManager.GetString("SourcesCommandEnabled_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 사용.
        /// </summary>
        internal static string SourcesCommandEnabled_kor {
            get {
                return ResourceManager.GetString("SourcesCommandEnabled_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Włączone.
        /// </summary>
        internal static string SourcesCommandEnabled_plk {
            get {
                return ResourceManager.GetString("SourcesCommandEnabled_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Ativado.
        /// </summary>
        internal static string SourcesCommandEnabled_ptb {
            get {
                return ResourceManager.GetString("SourcesCommandEnabled_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Включено.
        /// </summary>
        internal static string SourcesCommandEnabled_rus {
            get {
                return ResourceManager.GetString("SourcesCommandEnabled_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Etkin.
        /// </summary>
        internal static string SourcesCommandEnabled_trk {
            get {
                return ResourceManager.GetString("SourcesCommandEnabled_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to The source specified is invalid. Please provide a valid source..
        /// </summary>
        internal static string SourcesCommandInvalidSource {
            get {
                return ResourceManager.GetString("SourcesCommandInvalidSource", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 指定的源无效。请提供有效源。.
        /// </summary>
        internal static string SourcesCommandInvalidSource_chs {
            get {
                return ResourceManager.GetString("SourcesCommandInvalidSource_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 指定的來源無效。請提供有效來源。.
        /// </summary>
        internal static string SourcesCommandInvalidSource_cht {
            get {
                return ResourceManager.GetString("SourcesCommandInvalidSource_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Zadaný zdroj je neplatný. Zadejte platný zdroj..
        /// </summary>
        internal static string SourcesCommandInvalidSource_csy {
            get {
                return ResourceManager.GetString("SourcesCommandInvalidSource_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Die angegebene Quelle ist ungültig. Bitte geben Sie eine gültige Quelle an..
        /// </summary>
        internal static string SourcesCommandInvalidSource_deu {
            get {
                return ResourceManager.GetString("SourcesCommandInvalidSource_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to El origen especificado no es válido. Proporcione un origen válido..
        /// </summary>
        internal static string SourcesCommandInvalidSource_esp {
            get {
                return ResourceManager.GetString("SourcesCommandInvalidSource_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to La source spécifiée n&apos;est pas valide. Veuillez fournir une source valide..
        /// </summary>
        internal static string SourcesCommandInvalidSource_fra {
            get {
                return ResourceManager.GetString("SourcesCommandInvalidSource_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to La fonte specificata non è valida. Fornire una fonte valida..
        /// </summary>
        internal static string SourcesCommandInvalidSource_ita {
            get {
                return ResourceManager.GetString("SourcesCommandInvalidSource_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 指定されたソースは無効です。有効なソースを指定してください。.
        /// </summary>
        internal static string SourcesCommandInvalidSource_jpn {
            get {
                return ResourceManager.GetString("SourcesCommandInvalidSource_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 지정한 소스가 잘못되었습니다. 올바른 소스를 제공하십시오..
        /// </summary>
        internal static string SourcesCommandInvalidSource_kor {
            get {
                return ResourceManager.GetString("SourcesCommandInvalidSource_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Podane źródło jest nieprawidłowe. Określ prawidłowe źródło..
        /// </summary>
        internal static string SourcesCommandInvalidSource_plk {
            get {
                return ResourceManager.GetString("SourcesCommandInvalidSource_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to A origem especificada é inválida. Forneça uma origem válida..
        /// </summary>
        internal static string SourcesCommandInvalidSource_ptb {
            get {
                return ResourceManager.GetString("SourcesCommandInvalidSource_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Указан недопустимый источник. Укажите допустимый источник..
        /// </summary>
        internal static string SourcesCommandInvalidSource_rus {
            get {
                return ResourceManager.GetString("SourcesCommandInvalidSource_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Belirtilen kaynak geçersiz. Lütfen geçerli bir kaynak belirtin..
        /// </summary>
        internal static string SourcesCommandInvalidSource_trk {
            get {
                return ResourceManager.GetString("SourcesCommandInvalidSource_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to The name specified cannot be empty. Please provide a valid name..
        /// </summary>
        internal static string SourcesCommandNameRequired {
            get {
                return ResourceManager.GetString("SourcesCommandNameRequired", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 指定的名称不能为空。请提供有效的名称。.
        /// </summary>
        internal static string SourcesCommandNameRequired_chs {
            get {
                return ResourceManager.GetString("SourcesCommandNameRequired_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 指定的名稱不能為空。請提供有效名稱。.
        /// </summary>
        internal static string SourcesCommandNameRequired_cht {
            get {
                return ResourceManager.GetString("SourcesCommandNameRequired_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Zadaný název nesmí být prázdný. Zadejte platný název..
        /// </summary>
        internal static string SourcesCommandNameRequired_csy {
            get {
                return ResourceManager.GetString("SourcesCommandNameRequired_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Der angegebene Name darf nicht leer sein. Bitte geben Sie einen gültigen Namen an..
        /// </summary>
        internal static string SourcesCommandNameRequired_deu {
            get {
                return ResourceManager.GetString("SourcesCommandNameRequired_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to El nombre especificado no puede estar vacío. Proporcione un nombre válido..
        /// </summary>
        internal static string SourcesCommandNameRequired_esp {
            get {
                return ResourceManager.GetString("SourcesCommandNameRequired_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Le nom spécifié ne peut pas être vide. Veuillez fournir un nom valide..
        /// </summary>
        internal static string SourcesCommandNameRequired_fra {
            get {
                return ResourceManager.GetString("SourcesCommandNameRequired_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Il nome specificato non può essere vuoto. Fornire un nome valido..
        /// </summary>
        internal static string SourcesCommandNameRequired_ita {
            get {
                return ResourceManager.GetString("SourcesCommandNameRequired_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 空の名前は指定できません。有効な名前を指定してください。.
        /// </summary>
        internal static string SourcesCommandNameRequired_jpn {
            get {
                return ResourceManager.GetString("SourcesCommandNameRequired_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 지정한 이름은 비워둘 수 없습니다. 올바른 이름을 제공하십시오..
        /// </summary>
        internal static string SourcesCommandNameRequired_kor {
            get {
                return ResourceManager.GetString("SourcesCommandNameRequired_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Nazwa nie może być pusta. Określ prawidłową nazwę..
        /// </summary>
        internal static string SourcesCommandNameRequired_plk {
            get {
                return ResourceManager.GetString("SourcesCommandNameRequired_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to O nome especificado não pode ficar vazio. Forneça um nome válido..
        /// </summary>
        internal static string SourcesCommandNameRequired_ptb {
            get {
                return ResourceManager.GetString("SourcesCommandNameRequired_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Указанное имя не может быть пустым. Укажите допустимое имя..
        /// </summary>
        internal static string SourcesCommandNameRequired_rus {
            get {
                return ResourceManager.GetString("SourcesCommandNameRequired_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Belirtilen ad boş olamaz. Lütfen geçerli bir ad belirtin..
        /// </summary>
        internal static string SourcesCommandNameRequired_trk {
            get {
                return ResourceManager.GetString("SourcesCommandNameRequired_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Unable to find any package source(s) matching name: {0}..
        /// </summary>
        internal static string SourcesCommandNoMatchingSourcesFound {
            get {
                return ResourceManager.GetString("SourcesCommandNoMatchingSourcesFound", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 找不到与名称 {0} 匹配的任何程序包源。.
        /// </summary>
        internal static string SourcesCommandNoMatchingSourcesFound_chs {
            get {
                return ResourceManager.GetString("SourcesCommandNoMatchingSourcesFound_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 找不到入封裝來源相符名稱: {0}..
        /// </summary>
        internal static string SourcesCommandNoMatchingSourcesFound_cht {
            get {
                return ResourceManager.GetString("SourcesCommandNoMatchingSourcesFound_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Nebyly nalezen žádné zdroje balíčků odpovídající názvu: {0}..
        /// </summary>
        internal static string SourcesCommandNoMatchingSourcesFound_csy {
            get {
                return ResourceManager.GetString("SourcesCommandNoMatchingSourcesFound_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Es wurden keine Paketquellen gefunden, die mit dem folgenden Namen übereinstimmen: {0}..
        /// </summary>
        internal static string SourcesCommandNoMatchingSourcesFound_deu {
            get {
                return ResourceManager.GetString("SourcesCommandNoMatchingSourcesFound_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to No se puede encontrar ningún nombre que coincida con los orígenes del paquete: {0}..
        /// </summary>
        internal static string SourcesCommandNoMatchingSourcesFound_esp {
            get {
                return ResourceManager.GetString("SourcesCommandNoMatchingSourcesFound_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Impossible de trouver un nom de source de package correspondant : {0}..
        /// </summary>
        internal static string SourcesCommandNoMatchingSourcesFound_fra {
            get {
                return ResourceManager.GetString("SourcesCommandNoMatchingSourcesFound_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Impossibile trovare il nome del pacchetto: {0}..
        /// </summary>
        internal static string SourcesCommandNoMatchingSourcesFound_ita {
            get {
                return ResourceManager.GetString("SourcesCommandNoMatchingSourcesFound_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 名前と一致するパッケージ ソースが見つかりません: {0}..
        /// </summary>
        internal static string SourcesCommandNoMatchingSourcesFound_jpn {
            get {
                return ResourceManager.GetString("SourcesCommandNoMatchingSourcesFound_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} 이름과 일치하는 패키지 소스를 찾을 수 없습니다..
        /// </summary>
        internal static string SourcesCommandNoMatchingSourcesFound_kor {
            get {
                return ResourceManager.GetString("SourcesCommandNoMatchingSourcesFound_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Nie można odnaleźć żadnych zgodnych nazw źródeł pakietów: {0}..
        /// </summary>
        internal static string SourcesCommandNoMatchingSourcesFound_plk {
            get {
                return ResourceManager.GetString("SourcesCommandNoMatchingSourcesFound_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Não é possível encontrar nenhum nome correspondente de origem de pacote: {0}..
        /// </summary>
        internal static string SourcesCommandNoMatchingSourcesFound_ptb {
            get {
                return ResourceManager.GetString("SourcesCommandNoMatchingSourcesFound_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Не удалось найти источники пакетов, соответствующие имени: {0}..
        /// </summary>
        internal static string SourcesCommandNoMatchingSourcesFound_rus {
            get {
                return ResourceManager.GetString("SourcesCommandNoMatchingSourcesFound_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Bu adla eşleşen herhangi bir paket kaynağı bulunamadı: {0}..
        /// </summary>
        internal static string SourcesCommandNoMatchingSourcesFound_trk {
            get {
                return ResourceManager.GetString("SourcesCommandNoMatchingSourcesFound_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to No sources found..
        /// </summary>
        internal static string SourcesCommandNoSources {
            get {
                return ResourceManager.GetString("SourcesCommandNoSources", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 找不到源。.
        /// </summary>
        internal static string SourcesCommandNoSources_chs {
            get {
                return ResourceManager.GetString("SourcesCommandNoSources_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 找不到來源。.
        /// </summary>
        internal static string SourcesCommandNoSources_cht {
            get {
                return ResourceManager.GetString("SourcesCommandNoSources_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Nebyly nalezeny žádné zdroje..
        /// </summary>
        internal static string SourcesCommandNoSources_csy {
            get {
                return ResourceManager.GetString("SourcesCommandNoSources_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Es wurden keine Quellen gefunden..
        /// </summary>
        internal static string SourcesCommandNoSources_deu {
            get {
                return ResourceManager.GetString("SourcesCommandNoSources_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to No se ha encontrado ningún origen..
        /// </summary>
        internal static string SourcesCommandNoSources_esp {
            get {
                return ResourceManager.GetString("SourcesCommandNoSources_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Aucune source trouvée..
        /// </summary>
        internal static string SourcesCommandNoSources_fra {
            get {
                return ResourceManager.GetString("SourcesCommandNoSources_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Nessuna fonte trovata..
        /// </summary>
        internal static string SourcesCommandNoSources_ita {
            get {
                return ResourceManager.GetString("SourcesCommandNoSources_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to ソースが見つかりません。.
        /// </summary>
        internal static string SourcesCommandNoSources_jpn {
            get {
                return ResourceManager.GetString("SourcesCommandNoSources_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 소스를 찾을 수 없습니다..
        /// </summary>
        internal static string SourcesCommandNoSources_kor {
            get {
                return ResourceManager.GetString("SourcesCommandNoSources_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Nie znaleziono żadnych źródeł..
        /// </summary>
        internal static string SourcesCommandNoSources_plk {
            get {
                return ResourceManager.GetString("SourcesCommandNoSources_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Nenhuma origem encontrada..
        /// </summary>
        internal static string SourcesCommandNoSources_ptb {
            get {
                return ResourceManager.GetString("SourcesCommandNoSources_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Источники не обнаружены..
        /// </summary>
        internal static string SourcesCommandNoSources_rus {
            get {
                return ResourceManager.GetString("SourcesCommandNoSources_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Hiçbir kaynak bulunamadı..
        /// </summary>
        internal static string SourcesCommandNoSources_trk {
            get {
                return ResourceManager.GetString("SourcesCommandNoSources_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Registered Sources:.
        /// </summary>
        internal static string SourcesCommandRegisteredSources {
            get {
                return ResourceManager.GetString("SourcesCommandRegisteredSources", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 已注册的源:.
        /// </summary>
        internal static string SourcesCommandRegisteredSources_chs {
            get {
                return ResourceManager.GetString("SourcesCommandRegisteredSources_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 已註冊來源:.
        /// </summary>
        internal static string SourcesCommandRegisteredSources_cht {
            get {
                return ResourceManager.GetString("SourcesCommandRegisteredSources_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Registrované zdroje:.
        /// </summary>
        internal static string SourcesCommandRegisteredSources_csy {
            get {
                return ResourceManager.GetString("SourcesCommandRegisteredSources_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Registrierte Quellen:.
        /// </summary>
        internal static string SourcesCommandRegisteredSources_deu {
            get {
                return ResourceManager.GetString("SourcesCommandRegisteredSources_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Orígenes registrados:.
        /// </summary>
        internal static string SourcesCommandRegisteredSources_esp {
            get {
                return ResourceManager.GetString("SourcesCommandRegisteredSources_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Sources inscrites :.
        /// </summary>
        internal static string SourcesCommandRegisteredSources_fra {
            get {
                return ResourceManager.GetString("SourcesCommandRegisteredSources_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Fonti registrate:.
        /// </summary>
        internal static string SourcesCommandRegisteredSources_ita {
            get {
                return ResourceManager.GetString("SourcesCommandRegisteredSources_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 登録されているソース:.
        /// </summary>
        internal static string SourcesCommandRegisteredSources_jpn {
            get {
                return ResourceManager.GetString("SourcesCommandRegisteredSources_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 등록된 소스:.
        /// </summary>
        internal static string SourcesCommandRegisteredSources_kor {
            get {
                return ResourceManager.GetString("SourcesCommandRegisteredSources_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Zarejestrowane źródła:.
        /// </summary>
        internal static string SourcesCommandRegisteredSources_plk {
            get {
                return ResourceManager.GetString("SourcesCommandRegisteredSources_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Origens registradas:.
        /// </summary>
        internal static string SourcesCommandRegisteredSources_ptb {
            get {
                return ResourceManager.GetString("SourcesCommandRegisteredSources_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Зарегистрированные источники:.
        /// </summary>
        internal static string SourcesCommandRegisteredSources_rus {
            get {
                return ResourceManager.GetString("SourcesCommandRegisteredSources_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Kayıtlı Kaynaklar:.
        /// </summary>
        internal static string SourcesCommandRegisteredSources_trk {
            get {
                return ResourceManager.GetString("SourcesCommandRegisteredSources_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Package Source with Name: {0} added successfully..
        /// </summary>
        internal static string SourcesCommandSourceAddedSuccessfully {
            get {
                return ResourceManager.GetString("SourcesCommandSourceAddedSuccessfully", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 已成功添加名为 {0} 的程序包源。.
        /// </summary>
        internal static string SourcesCommandSourceAddedSuccessfully_chs {
            get {
                return ResourceManager.GetString("SourcesCommandSourceAddedSuccessfully_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 具有名稱的封裝來源: {0} 已成功新增。.
        /// </summary>
        internal static string SourcesCommandSourceAddedSuccessfully_cht {
            get {
                return ResourceManager.GetString("SourcesCommandSourceAddedSuccessfully_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Zdroj balíčku s názvem {0} byl úspěšně přidán..
        /// </summary>
        internal static string SourcesCommandSourceAddedSuccessfully_csy {
            get {
                return ResourceManager.GetString("SourcesCommandSourceAddedSuccessfully_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Die Paketquelle mit dem Namen &quot;{0}&quot; wurde erfolgreich hinzugefügt..
        /// </summary>
        internal static string SourcesCommandSourceAddedSuccessfully_deu {
            get {
                return ResourceManager.GetString("SourcesCommandSourceAddedSuccessfully_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Origen del paquete con nombre: {0} agregado correctamente..
        /// </summary>
        internal static string SourcesCommandSourceAddedSuccessfully_esp {
            get {
                return ResourceManager.GetString("SourcesCommandSourceAddedSuccessfully_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Source de package avec nom : {0} ajouté correctement..
        /// </summary>
        internal static string SourcesCommandSourceAddedSuccessfully_fra {
            get {
                return ResourceManager.GetString("SourcesCommandSourceAddedSuccessfully_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Fonte pacchetto con nome: {0} aggiunto correttamente..
        /// </summary>
        internal static string SourcesCommandSourceAddedSuccessfully_ita {
            get {
                return ResourceManager.GetString("SourcesCommandSourceAddedSuccessfully_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 次の名前のパッケージ ソース:{0} が正常に追加されました。.
        /// </summary>
        internal static string SourcesCommandSourceAddedSuccessfully_jpn {
            get {
                return ResourceManager.GetString("SourcesCommandSourceAddedSuccessfully_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} 이름의 패키지 소스가 추가되었습니다..
        /// </summary>
        internal static string SourcesCommandSourceAddedSuccessfully_kor {
            get {
                return ResourceManager.GetString("SourcesCommandSourceAddedSuccessfully_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Pomyślnie dodano źródło pakietów o nazwie: {0}..
        /// </summary>
        internal static string SourcesCommandSourceAddedSuccessfully_plk {
            get {
                return ResourceManager.GetString("SourcesCommandSourceAddedSuccessfully_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Origem de pacote com o nome: {0} adicionada com sucesso..
        /// </summary>
        internal static string SourcesCommandSourceAddedSuccessfully_ptb {
            get {
                return ResourceManager.GetString("SourcesCommandSourceAddedSuccessfully_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Источник пакетов с именем: {0} успешно добавлен..
        /// </summary>
        internal static string SourcesCommandSourceAddedSuccessfully_rus {
            get {
                return ResourceManager.GetString("SourcesCommandSourceAddedSuccessfully_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} Adlı Paket Kaynağı başarıyla eklendi..
        /// </summary>
        internal static string SourcesCommandSourceAddedSuccessfully_trk {
            get {
                return ResourceManager.GetString("SourcesCommandSourceAddedSuccessfully_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Package source with Name: {0} disabled successfully..
        /// </summary>
        internal static string SourcesCommandSourceDisabledSuccessfully {
            get {
                return ResourceManager.GetString("SourcesCommandSourceDisabledSuccessfully", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 已成功禁用名为 {0} 的程序包源。.
        /// </summary>
        internal static string SourcesCommandSourceDisabledSuccessfully_chs {
            get {
                return ResourceManager.GetString("SourcesCommandSourceDisabledSuccessfully_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 具有名稱的封裝來源: {1} 已成功停用。.
        /// </summary>
        internal static string SourcesCommandSourceDisabledSuccessfully_cht {
            get {
                return ResourceManager.GetString("SourcesCommandSourceDisabledSuccessfully_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Zdroj balíčku s názvem {0} byl úspěšně zakázán..
        /// </summary>
        internal static string SourcesCommandSourceDisabledSuccessfully_csy {
            get {
                return ResourceManager.GetString("SourcesCommandSourceDisabledSuccessfully_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Die Paketquelle mit dem Namen &quot;{0}&quot; wurde erfolgreich deaktiviert..
        /// </summary>
        internal static string SourcesCommandSourceDisabledSuccessfully_deu {
            get {
                return ResourceManager.GetString("SourcesCommandSourceDisabledSuccessfully_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Origen del paquete con nombre: {0} deshabilitado correctamente..
        /// </summary>
        internal static string SourcesCommandSourceDisabledSuccessfully_esp {
            get {
                return ResourceManager.GetString("SourcesCommandSourceDisabledSuccessfully_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Source de package avec nom : {0} désactivé correctement..
        /// </summary>
        internal static string SourcesCommandSourceDisabledSuccessfully_fra {
            get {
                return ResourceManager.GetString("SourcesCommandSourceDisabledSuccessfully_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Fonte pacchetto con nome: {0} disabilitata correttamente..
        /// </summary>
        internal static string SourcesCommandSourceDisabledSuccessfully_ita {
            get {
                return ResourceManager.GetString("SourcesCommandSourceDisabledSuccessfully_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 次の名前のパッケージ ソース:{0} は正常に無効になりました。.
        /// </summary>
        internal static string SourcesCommandSourceDisabledSuccessfully_jpn {
            get {
                return ResourceManager.GetString("SourcesCommandSourceDisabledSuccessfully_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} 이름의 패키지 소스가 사용되지 않도록 설정되었습니다..
        /// </summary>
        internal static string SourcesCommandSourceDisabledSuccessfully_kor {
            get {
                return ResourceManager.GetString("SourcesCommandSourceDisabledSuccessfully_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Pomyślnie wyłączono źródło pakietów o nazwie: {0}..
        /// </summary>
        internal static string SourcesCommandSourceDisabledSuccessfully_plk {
            get {
                return ResourceManager.GetString("SourcesCommandSourceDisabledSuccessfully_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Origem de pacote com o nome: {0} desativada com sucesso..
        /// </summary>
        internal static string SourcesCommandSourceDisabledSuccessfully_ptb {
            get {
                return ResourceManager.GetString("SourcesCommandSourceDisabledSuccessfully_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Источник пакетов с именем: {0} успешно отключен..
        /// </summary>
        internal static string SourcesCommandSourceDisabledSuccessfully_rus {
            get {
                return ResourceManager.GetString("SourcesCommandSourceDisabledSuccessfully_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} Adlı Paket Kaynağı başarıyla devre dışı bırakıldı..
        /// </summary>
        internal static string SourcesCommandSourceDisabledSuccessfully_trk {
            get {
                return ResourceManager.GetString("SourcesCommandSourceDisabledSuccessfully_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Package source with Name: {0} enabled successfully..
        /// </summary>
        internal static string SourcesCommandSourceEnabledSuccessfully {
            get {
                return ResourceManager.GetString("SourcesCommandSourceEnabledSuccessfully", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 已成功启用名为 {0} 的程序包源。.
        /// </summary>
        internal static string SourcesCommandSourceEnabledSuccessfully_chs {
            get {
                return ResourceManager.GetString("SourcesCommandSourceEnabledSuccessfully_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 具有名稱的封裝來源: {2} 已成功啟用。.
        /// </summary>
        internal static string SourcesCommandSourceEnabledSuccessfully_cht {
            get {
                return ResourceManager.GetString("SourcesCommandSourceEnabledSuccessfully_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Zdroj balíčku s názvem {0} byl úspěšně povolen..
        /// </summary>
        internal static string SourcesCommandSourceEnabledSuccessfully_csy {
            get {
                return ResourceManager.GetString("SourcesCommandSourceEnabledSuccessfully_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Die Paketquelle mit dem Namen &quot;{0}&quot; wurde erfolgreich aktiviert..
        /// </summary>
        internal static string SourcesCommandSourceEnabledSuccessfully_deu {
            get {
                return ResourceManager.GetString("SourcesCommandSourceEnabledSuccessfully_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Origen del paquete con nombre: {0} habilitado correctamente..
        /// </summary>
        internal static string SourcesCommandSourceEnabledSuccessfully_esp {
            get {
                return ResourceManager.GetString("SourcesCommandSourceEnabledSuccessfully_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Source de package avec nom : {0} activé correctement..
        /// </summary>
        internal static string SourcesCommandSourceEnabledSuccessfully_fra {
            get {
                return ResourceManager.GetString("SourcesCommandSourceEnabledSuccessfully_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Fonte pacchetto con nome: {0} abilitata correttamente..
        /// </summary>
        internal static string SourcesCommandSourceEnabledSuccessfully_ita {
            get {
                return ResourceManager.GetString("SourcesCommandSourceEnabledSuccessfully_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 次の名前のパッケージ ソース:{0} は正常に有効になりました。.
        /// </summary>
        internal static string SourcesCommandSourceEnabledSuccessfully_jpn {
            get {
                return ResourceManager.GetString("SourcesCommandSourceEnabledSuccessfully_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} 이름의 패키지 소스가 사용되도록 설정되었습니다..
        /// </summary>
        internal static string SourcesCommandSourceEnabledSuccessfully_kor {
            get {
                return ResourceManager.GetString("SourcesCommandSourceEnabledSuccessfully_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Pomyślnie włączono źródło pakietów o nazwie: {0}..
        /// </summary>
        internal static string SourcesCommandSourceEnabledSuccessfully_plk {
            get {
                return ResourceManager.GetString("SourcesCommandSourceEnabledSuccessfully_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Origem de pacote com o nome: {0} ativada com sucesso..
        /// </summary>
        internal static string SourcesCommandSourceEnabledSuccessfully_ptb {
            get {
                return ResourceManager.GetString("SourcesCommandSourceEnabledSuccessfully_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Источник пакетов с именем: {0} успешно включен..
        /// </summary>
        internal static string SourcesCommandSourceEnabledSuccessfully_rus {
            get {
                return ResourceManager.GetString("SourcesCommandSourceEnabledSuccessfully_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} Adlı Paket Kaynağı başarıyla etkinleştirildi..
        /// </summary>
        internal static string SourcesCommandSourceEnabledSuccessfully_trk {
            get {
                return ResourceManager.GetString("SourcesCommandSourceEnabledSuccessfully_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Package source with Name: {0} removed successfully..
        /// </summary>
        internal static string SourcesCommandSourceRemovedSuccessfully {
            get {
                return ResourceManager.GetString("SourcesCommandSourceRemovedSuccessfully", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 已成功删除名为 {0} 的程序包源。.
        /// </summary>
        internal static string SourcesCommandSourceRemovedSuccessfully_chs {
            get {
                return ResourceManager.GetString("SourcesCommandSourceRemovedSuccessfully_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 具有名稱的封裝來源: {3} 已成功移除。.
        /// </summary>
        internal static string SourcesCommandSourceRemovedSuccessfully_cht {
            get {
                return ResourceManager.GetString("SourcesCommandSourceRemovedSuccessfully_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Zdroj balíčku s názvem {0} byl úspěšně odebrán..
        /// </summary>
        internal static string SourcesCommandSourceRemovedSuccessfully_csy {
            get {
                return ResourceManager.GetString("SourcesCommandSourceRemovedSuccessfully_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Die Paketquelle mit dem Namen &quot;{0}&quot; wurde erfolgreich entfernt..
        /// </summary>
        internal static string SourcesCommandSourceRemovedSuccessfully_deu {
            get {
                return ResourceManager.GetString("SourcesCommandSourceRemovedSuccessfully_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Origen del paquete con nombre: {0} quitado correctamente..
        /// </summary>
        internal static string SourcesCommandSourceRemovedSuccessfully_esp {
            get {
                return ResourceManager.GetString("SourcesCommandSourceRemovedSuccessfully_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Source de package avec nom : {0} supprimé correctement..
        /// </summary>
        internal static string SourcesCommandSourceRemovedSuccessfully_fra {
            get {
                return ResourceManager.GetString("SourcesCommandSourceRemovedSuccessfully_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Fonte pacchetto con nome: {0} rimossa correttamente...
        /// </summary>
        internal static string SourcesCommandSourceRemovedSuccessfully_ita {
            get {
                return ResourceManager.GetString("SourcesCommandSourceRemovedSuccessfully_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 次の名前のパッケージ ソース:{0} は正常に削除されました。.
        /// </summary>
        internal static string SourcesCommandSourceRemovedSuccessfully_jpn {
            get {
                return ResourceManager.GetString("SourcesCommandSourceRemovedSuccessfully_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} 이름의 패키지 소스가 제거되었습니다..
        /// </summary>
        internal static string SourcesCommandSourceRemovedSuccessfully_kor {
            get {
                return ResourceManager.GetString("SourcesCommandSourceRemovedSuccessfully_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Pomyślnie usunięto źródło pakietów o nazwie: {0}..
        /// </summary>
        internal static string SourcesCommandSourceRemovedSuccessfully_plk {
            get {
                return ResourceManager.GetString("SourcesCommandSourceRemovedSuccessfully_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Origem do pacote com o nome: {0} removida com sucesso..
        /// </summary>
        internal static string SourcesCommandSourceRemovedSuccessfully_ptb {
            get {
                return ResourceManager.GetString("SourcesCommandSourceRemovedSuccessfully_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Источник пакетов с именем: {0} успешно удален..
        /// </summary>
        internal static string SourcesCommandSourceRemovedSuccessfully_rus {
            get {
                return ResourceManager.GetString("SourcesCommandSourceRemovedSuccessfully_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} Adlı Paket Kaynağı başarıyla kaldırıldı..
        /// </summary>
        internal static string SourcesCommandSourceRemovedSuccessfully_trk {
            get {
                return ResourceManager.GetString("SourcesCommandSourceRemovedSuccessfully_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to The source specified cannot be empty. Please provide a valid source..
        /// </summary>
        internal static string SourcesCommandSourceRequired {
            get {
                return ResourceManager.GetString("SourcesCommandSourceRequired", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 指定的源不能为空。请提供有效源。.
        /// </summary>
        internal static string SourcesCommandSourceRequired_chs {
            get {
                return ResourceManager.GetString("SourcesCommandSourceRequired_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 指定的來源不能為空。請提供有效的來源。.
        /// </summary>
        internal static string SourcesCommandSourceRequired_cht {
            get {
                return ResourceManager.GetString("SourcesCommandSourceRequired_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Zadaný zdroj nesmí být prázdný. Zadejte platný zdroj..
        /// </summary>
        internal static string SourcesCommandSourceRequired_csy {
            get {
                return ResourceManager.GetString("SourcesCommandSourceRequired_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Die angegebene Quelle darf nicht leer sein. Bitte geben Sie eine gültige Quelle an..
        /// </summary>
        internal static string SourcesCommandSourceRequired_deu {
            get {
                return ResourceManager.GetString("SourcesCommandSourceRequired_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to El origen especificado no puede estar vacío. Proporcione un origen válido..
        /// </summary>
        internal static string SourcesCommandSourceRequired_esp {
            get {
                return ResourceManager.GetString("SourcesCommandSourceRequired_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to La source spécifiée ne peut pas être vide. Veuillez fournir une source valide..
        /// </summary>
        internal static string SourcesCommandSourceRequired_fra {
            get {
                return ResourceManager.GetString("SourcesCommandSourceRequired_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to La fonte specificata non può essere vuota. Fornire una fonte valida..
        /// </summary>
        internal static string SourcesCommandSourceRequired_ita {
            get {
                return ResourceManager.GetString("SourcesCommandSourceRequired_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 空のソースは指定できません。有効なソースを指定してください。.
        /// </summary>
        internal static string SourcesCommandSourceRequired_jpn {
            get {
                return ResourceManager.GetString("SourcesCommandSourceRequired_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 지정한 소스는 비워둘 수 없습니다. 올바른 소스를 제공하십시오..
        /// </summary>
        internal static string SourcesCommandSourceRequired_kor {
            get {
                return ResourceManager.GetString("SourcesCommandSourceRequired_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Źródło nie może być puste. Podaj prawidłowe źródło..
        /// </summary>
        internal static string SourcesCommandSourceRequired_plk {
            get {
                return ResourceManager.GetString("SourcesCommandSourceRequired_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to A origem especificada não ficar vazia. Forneça uma origem válida..
        /// </summary>
        internal static string SourcesCommandSourceRequired_ptb {
            get {
                return ResourceManager.GetString("SourcesCommandSourceRequired_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Указанный источник не может быть пустым. Укажите допустимый источник..
        /// </summary>
        internal static string SourcesCommandSourceRequired_rus {
            get {
                return ResourceManager.GetString("SourcesCommandSourceRequired_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Belirtilen kaynak boş olamaz. Lütfen geçerli bir kaynak belirtin..
        /// </summary>
        internal static string SourcesCommandSourceRequired_trk {
            get {
                return ResourceManager.GetString("SourcesCommandSourceRequired_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to The name specified has already been added to the list of available package sources. Please provide a unique name..
        /// </summary>
        internal static string SourcesCommandUniqueName {
            get {
                return ResourceManager.GetString("SourcesCommandUniqueName", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 指定的名称已添加到可用程序包源列表。请提供唯一名称。.
        /// </summary>
        internal static string SourcesCommandUniqueName_chs {
            get {
                return ResourceManager.GetString("SourcesCommandUniqueName_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 指定的名稱已新增到可用封裝來源清單。請提供唯一名稱。.
        /// </summary>
        internal static string SourcesCommandUniqueName_cht {
            get {
                return ResourceManager.GetString("SourcesCommandUniqueName_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Zadaný název již byl přidán do seznamu dostupných zdrojů balíčků. Zadejte jedinečný název..
        /// </summary>
        internal static string SourcesCommandUniqueName_csy {
            get {
                return ResourceManager.GetString("SourcesCommandUniqueName_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Der angegebene Name wurde der Liste der verfügbaren Paketquellen bereits hinzugefügt. Bitte geben Sie einen eindeutigen Namen an..
        /// </summary>
        internal static string SourcesCommandUniqueName_deu {
            get {
                return ResourceManager.GetString("SourcesCommandUniqueName_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to El nombre especificado ya se ha agregado a la lista de orígenes de paquete disponibles. Proporcione un nombre único..
        /// </summary>
        internal static string SourcesCommandUniqueName_esp {
            get {
                return ResourceManager.GetString("SourcesCommandUniqueName_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Le nom spécifié a déjà été ajouté à la liste de sources de package disponibles. Veuillez fournir un nom unique..
        /// </summary>
        internal static string SourcesCommandUniqueName_fra {
            get {
                return ResourceManager.GetString("SourcesCommandUniqueName_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Il nome specificato è stato aggiunto alla lista di fonti pacchetto disponibili. Fornire un nome unico.
        /// </summary>
        internal static string SourcesCommandUniqueName_ita {
            get {
                return ResourceManager.GetString("SourcesCommandUniqueName_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 指定された名前は既に利用可能なパッケージ ソースの一覧に追加されています。一意の名前を指定してください。.
        /// </summary>
        internal static string SourcesCommandUniqueName_jpn {
            get {
                return ResourceManager.GetString("SourcesCommandUniqueName_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 지정한 이름은 이미 사용 가능한 패키지 소스 목록에 추가되어 있습니다. 고유한 이름을 제공하십시오..
        /// </summary>
        internal static string SourcesCommandUniqueName_kor {
            get {
                return ResourceManager.GetString("SourcesCommandUniqueName_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Określona nazwa została już dodana do listy dostępnych źródeł pakietów. Podaj unikatową nazwę..
        /// </summary>
        internal static string SourcesCommandUniqueName_plk {
            get {
                return ResourceManager.GetString("SourcesCommandUniqueName_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to O nome especificado já foi adicionado à lista de origens de pacotes disponíveis. Forneça um nome exclusivo..
        /// </summary>
        internal static string SourcesCommandUniqueName_ptb {
            get {
                return ResourceManager.GetString("SourcesCommandUniqueName_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Указанное имя уже было добавлено в список доступных источников пакетов. Укажите уникальное имя..
        /// </summary>
        internal static string SourcesCommandUniqueName_rus {
            get {
                return ResourceManager.GetString("SourcesCommandUniqueName_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Belirtilen ad zaten kullanılabilir paket kaynakları listesine eklenmiş. Lütfen benzersiz bir ad belirtin..
        /// </summary>
        internal static string SourcesCommandUniqueName_trk {
            get {
                return ResourceManager.GetString("SourcesCommandUniqueName_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to The source specified has already been added to the list of available package sources. Please provide a unique source..
        /// </summary>
        internal static string SourcesCommandUniqueSource {
            get {
                return ResourceManager.GetString("SourcesCommandUniqueSource", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 指定的源已添加到可用程序包源列表。请提供唯一源。.
        /// </summary>
        internal static string SourcesCommandUniqueSource_chs {
            get {
                return ResourceManager.GetString("SourcesCommandUniqueSource_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 指定的來源已新增到可用封裝來源清單。請提供唯一名稱。.
        /// </summary>
        internal static string SourcesCommandUniqueSource_cht {
            get {
                return ResourceManager.GetString("SourcesCommandUniqueSource_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Zadaný zdroj již byl přidán do seznamu dostupných zdrojů balíčků. Zadejte jedinečný zdroj..
        /// </summary>
        internal static string SourcesCommandUniqueSource_csy {
            get {
                return ResourceManager.GetString("SourcesCommandUniqueSource_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Der angegebene Name wurde der Liste der verfügbaren Paketquellen bereits hinzugefügt. Bitte geben Sie eine eindeutige Quelle an..
        /// </summary>
        internal static string SourcesCommandUniqueSource_deu {
            get {
                return ResourceManager.GetString("SourcesCommandUniqueSource_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to El origen especificado ya se ha agregado a la lista de orígenes de paquete disponibles. Proporcione un nombre único..
        /// </summary>
        internal static string SourcesCommandUniqueSource_esp {
            get {
                return ResourceManager.GetString("SourcesCommandUniqueSource_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to La source spécifiée a déjà été ajoutée à la liste de sources de package disponibles. Veuillez fournir une source unique..
        /// </summary>
        internal static string SourcesCommandUniqueSource_fra {
            get {
                return ResourceManager.GetString("SourcesCommandUniqueSource_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to La fonte specificata è stato aggiunta alla lista di fonti pacchetto disponibili. Fornire una fonte unica..
        /// </summary>
        internal static string SourcesCommandUniqueSource_ita {
            get {
                return ResourceManager.GetString("SourcesCommandUniqueSource_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 指定されたソースは既に利用可能なパッケージ ソースの一覧に追加されています。一意のソースを指定してください。.
        /// </summary>
        internal static string SourcesCommandUniqueSource_jpn {
            get {
                return ResourceManager.GetString("SourcesCommandUniqueSource_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 지정한 소스는 이미 사용 가능한 패키지 소스 목록에 추가되어 있습니다. 고유한 소스를 제공하십시오..
        /// </summary>
        internal static string SourcesCommandUniqueSource_kor {
            get {
                return ResourceManager.GetString("SourcesCommandUniqueSource_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Określone źródło zostało już dodane do listy dostępnych źródeł pakietów. Podaj unikatowe źródło..
        /// </summary>
        internal static string SourcesCommandUniqueSource_plk {
            get {
                return ResourceManager.GetString("SourcesCommandUniqueSource_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to A origem especificada já foi adicionada à lista de origens de pacotes disponíveis. Forneça uma origem única..
        /// </summary>
        internal static string SourcesCommandUniqueSource_ptb {
            get {
                return ResourceManager.GetString("SourcesCommandUniqueSource_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Указанный источник имя уже был добавлен в список доступных источников пакетов. Укажите уникальный источник..
        /// </summary>
        internal static string SourcesCommandUniqueSource_rus {
            get {
                return ResourceManager.GetString("SourcesCommandUniqueSource_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Belirtilen kaynak zaten kullanılabilir paket kaynakları listesine eklenmiş. Lütfen benzersiz bir kaynak belirtin..
        /// </summary>
        internal static string SourcesCommandUniqueSource_trk {
            get {
                return ResourceManager.GetString("SourcesCommandUniqueSource_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Package source &quot;{0}&quot; was successfully updated..
        /// </summary>
        internal static string SourcesCommandUpdateSuccessful {
            get {
                return ResourceManager.GetString("SourcesCommandUpdateSuccessful", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 已成功更新程序包源“{0}”。.
        /// </summary>
        internal static string SourcesCommandUpdateSuccessful_chs {
            get {
                return ResourceManager.GetString("SourcesCommandUpdateSuccessful_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 封裝來源 &quot;{0}&quot; 已成功更新。.
        /// </summary>
        internal static string SourcesCommandUpdateSuccessful_cht {
            get {
                return ResourceManager.GetString("SourcesCommandUpdateSuccessful_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Zdroj balíčku {0} byl úspěšně aktualizován..
        /// </summary>
        internal static string SourcesCommandUpdateSuccessful_csy {
            get {
                return ResourceManager.GetString("SourcesCommandUpdateSuccessful_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Die Paketquelle &quot;{0}&quot; wurde erfolgreich aktualisiert..
        /// </summary>
        internal static string SourcesCommandUpdateSuccessful_deu {
            get {
                return ResourceManager.GetString("SourcesCommandUpdateSuccessful_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to El origen del paquete &apos;{0}&apos; se ha actualizado correctamente..
        /// </summary>
        internal static string SourcesCommandUpdateSuccessful_esp {
            get {
                return ResourceManager.GetString("SourcesCommandUpdateSuccessful_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to La source de package « {0} » a été mise à jour correctement..
        /// </summary>
        internal static string SourcesCommandUpdateSuccessful_fra {
            get {
                return ResourceManager.GetString("SourcesCommandUpdateSuccessful_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Fonte pacchetto &quot;{0}&quot; aggiornata correttamente..
        /// </summary>
        internal static string SourcesCommandUpdateSuccessful_ita {
            get {
                return ResourceManager.GetString("SourcesCommandUpdateSuccessful_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to パッケージ ソース &quot;{0}&quot; は正常に更新されました。.
        /// </summary>
        internal static string SourcesCommandUpdateSuccessful_jpn {
            get {
                return ResourceManager.GetString("SourcesCommandUpdateSuccessful_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 패키지 소스 &quot;{0}&quot;이(가) 업데이트되었습니다..
        /// </summary>
        internal static string SourcesCommandUpdateSuccessful_kor {
            get {
                return ResourceManager.GetString("SourcesCommandUpdateSuccessful_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Źródło pakietów „{0}” zostało pomyślnie zaktualizowane..
        /// </summary>
        internal static string SourcesCommandUpdateSuccessful_plk {
            get {
                return ResourceManager.GetString("SourcesCommandUpdateSuccessful_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to A origem de pacote &apos;{0}&apos; foi atualizada com sucesso..
        /// </summary>
        internal static string SourcesCommandUpdateSuccessful_ptb {
            get {
                return ResourceManager.GetString("SourcesCommandUpdateSuccessful_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Источник пакетов &quot;{0}&quot; успешно обновлен..
        /// </summary>
        internal static string SourcesCommandUpdateSuccessful_rus {
            get {
                return ResourceManager.GetString("SourcesCommandUpdateSuccessful_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to &quot;{0}&quot; paket kaynağı başarıyla güncellendi..
        /// </summary>
        internal static string SourcesCommandUpdateSuccessful_trk {
            get {
                return ResourceManager.GetString("SourcesCommandUpdateSuccessful_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Created &apos;{0}&apos; successfully..
        /// </summary>
        internal static string SpecCommandCreatedNuSpec {
            get {
                return ResourceManager.GetString("SpecCommandCreatedNuSpec", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 已成功创建“{0}”。.
        /// </summary>
        internal static string SpecCommandCreatedNuSpec_chs {
            get {
                return ResourceManager.GetString("SpecCommandCreatedNuSpec_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 已成功建立 &apos;{0}&apos;。.
        /// </summary>
        internal static string SpecCommandCreatedNuSpec_cht {
            get {
                return ResourceManager.GetString("SpecCommandCreatedNuSpec_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} úspěšně vytvořeno..
        /// </summary>
        internal static string SpecCommandCreatedNuSpec_csy {
            get {
                return ResourceManager.GetString("SpecCommandCreatedNuSpec_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to &quot;{0}&quot; wurde erfolgreich erstellt..
        /// </summary>
        internal static string SpecCommandCreatedNuSpec_deu {
            get {
                return ResourceManager.GetString("SpecCommandCreatedNuSpec_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0}&apos; creado correctamente..
        /// </summary>
        internal static string SpecCommandCreatedNuSpec_esp {
            get {
                return ResourceManager.GetString("SpecCommandCreatedNuSpec_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to &apos;{0}&apos; créé correctement..
        /// </summary>
        internal static string SpecCommandCreatedNuSpec_fra {
            get {
                return ResourceManager.GetString("SpecCommandCreatedNuSpec_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Creato &apos;{0}&apos; correttamente..
        /// </summary>
        internal static string SpecCommandCreatedNuSpec_ita {
            get {
                return ResourceManager.GetString("SpecCommandCreatedNuSpec_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to &apos;{0}&apos; は正常に作成されました。.
        /// </summary>
        internal static string SpecCommandCreatedNuSpec_jpn {
            get {
                return ResourceManager.GetString("SpecCommandCreatedNuSpec_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to &apos;{0}&apos;을(를) 만들었습니다..
        /// </summary>
        internal static string SpecCommandCreatedNuSpec_kor {
            get {
                return ResourceManager.GetString("SpecCommandCreatedNuSpec_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Pomyślnie utworzono „{0}”..
        /// </summary>
        internal static string SpecCommandCreatedNuSpec_plk {
            get {
                return ResourceManager.GetString("SpecCommandCreatedNuSpec_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Criado &apos;{0}&apos; com sucesso..
        /// </summary>
        internal static string SpecCommandCreatedNuSpec_ptb {
            get {
                return ResourceManager.GetString("SpecCommandCreatedNuSpec_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to &quot;{0}&quot; успешно создан..
        /// </summary>
        internal static string SpecCommandCreatedNuSpec_rus {
            get {
                return ResourceManager.GetString("SpecCommandCreatedNuSpec_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0}&apos; başarıyla oluşturuldu..
        /// </summary>
        internal static string SpecCommandCreatedNuSpec_trk {
            get {
                return ResourceManager.GetString("SpecCommandCreatedNuSpec_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to &apos;{0}&apos; already exists, use -f to overwrite it..
        /// </summary>
        internal static string SpecCommandFileExists {
            get {
                return ResourceManager.GetString("SpecCommandFileExists", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to “{0}”已存在，请使用 -f 覆盖它。.
        /// </summary>
        internal static string SpecCommandFileExists_chs {
            get {
                return ResourceManager.GetString("SpecCommandFileExists_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0}&apos; 己存在，使用 -f 加以覆寫。.
        /// </summary>
        internal static string SpecCommandFileExists_cht {
            get {
                return ResourceManager.GetString("SpecCommandFileExists_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} již existuje, k přepsání použijte -f..
        /// </summary>
        internal static string SpecCommandFileExists_csy {
            get {
                return ResourceManager.GetString("SpecCommandFileExists_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to &quot;{0}&quot; ist bereits vorhanden. Verwenden Sie &quot;-f&quot; zum Überschreiben..
        /// </summary>
        internal static string SpecCommandFileExists_deu {
            get {
                return ResourceManager.GetString("SpecCommandFileExists_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0}&apos; ya existe, use -f para sobrescribirlo..
        /// </summary>
        internal static string SpecCommandFileExists_esp {
            get {
                return ResourceManager.GetString("SpecCommandFileExists_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to &apos;{0}&apos; existe déjà. Utilisez -f pour le remplacer..
        /// </summary>
        internal static string SpecCommandFileExists_fra {
            get {
                return ResourceManager.GetString("SpecCommandFileExists_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0}&apos; già esistente, usere  -f per sostituire..
        /// </summary>
        internal static string SpecCommandFileExists_ita {
            get {
                return ResourceManager.GetString("SpecCommandFileExists_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to &apos;{0}&apos; は既に存在します。上書きするには、-f を使用してください。.
        /// </summary>
        internal static string SpecCommandFileExists_jpn {
            get {
                return ResourceManager.GetString("SpecCommandFileExists_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to &apos;{0}&apos;이(가) 이미 있습니다. 덮어쓰려면 -f를 사용하십시오..
        /// </summary>
        internal static string SpecCommandFileExists_kor {
            get {
                return ResourceManager.GetString("SpecCommandFileExists_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Plik „{0}” już istnieje, użyj argumentu -f, aby go zastąpić..
        /// </summary>
        internal static string SpecCommandFileExists_plk {
            get {
                return ResourceManager.GetString("SpecCommandFileExists_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0}&apos; já existe, use -f para substituí-lo..
        /// </summary>
        internal static string SpecCommandFileExists_ptb {
            get {
                return ResourceManager.GetString("SpecCommandFileExists_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to &quot;{0}&quot; уже существует. Для перезаписи используйте параметр -f..
        /// </summary>
        internal static string SpecCommandFileExists_rus {
            get {
                return ResourceManager.GetString("SpecCommandFileExists_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0}&apos; zaten mevcut, geçersiz kılmak için -f seçimini kullanın..
        /// </summary>
        internal static string SpecCommandFileExists_trk {
            get {
                return ResourceManager.GetString("SpecCommandFileExists_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Unable to change from type &apos;{0}&apos; to &apos;{1}&apos;..
        /// </summary>
        internal static string UnableToConvertTypeError {
            get {
                return ResourceManager.GetString("UnableToConvertTypeError", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 无法从类型“{0}”更改为“{1}”。.
        /// </summary>
        internal static string UnableToConvertTypeError_chs {
            get {
                return ResourceManager.GetString("UnableToConvertTypeError_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 無法變更類型 &apos;{0}&apos; 為 &apos;{1}&apos;。.
        /// </summary>
        internal static string UnableToConvertTypeError_cht {
            get {
                return ResourceManager.GetString("UnableToConvertTypeError_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Nelze provést změnu z typu {0} na typ {1}..
        /// </summary>
        internal static string UnableToConvertTypeError_csy {
            get {
                return ResourceManager.GetString("UnableToConvertTypeError_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Der Typ kann nicht aus &quot;{0}&quot; in &quot;{1}&quot; geändert werden..
        /// </summary>
        internal static string UnableToConvertTypeError_deu {
            get {
                return ResourceManager.GetString("UnableToConvertTypeError_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to No se puede cambiar del tipo &apos;{0}&apos; a &apos;{1}&apos;..
        /// </summary>
        internal static string UnableToConvertTypeError_esp {
            get {
                return ResourceManager.GetString("UnableToConvertTypeError_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Impossible de basculer du type &apos;{0}&apos; au type &apos;{1}&apos;..
        /// </summary>
        internal static string UnableToConvertTypeError_fra {
            get {
                return ResourceManager.GetString("UnableToConvertTypeError_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Impossibile modificare da &apos;{0}&apos; a &apos;{1}&apos;..
        /// </summary>
        internal static string UnableToConvertTypeError_ita {
            get {
                return ResourceManager.GetString("UnableToConvertTypeError_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 型 &apos;{0}&apos; から &apos;{1}&apos; に変更できません。.
        /// </summary>
        internal static string UnableToConvertTypeError_jpn {
            get {
                return ResourceManager.GetString("UnableToConvertTypeError_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to &apos;{0}&apos;에서 &apos;{1}&apos;(으)로 형식을 변경할 수 없습니다..
        /// </summary>
        internal static string UnableToConvertTypeError_kor {
            get {
                return ResourceManager.GetString("UnableToConvertTypeError_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Nie można zmienić typu „{0}” na „{1}”..
        /// </summary>
        internal static string UnableToConvertTypeError_plk {
            get {
                return ResourceManager.GetString("UnableToConvertTypeError_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Não é possível mudar do tipo &apos;{0}&apos; para &apos;{1}&apos;..
        /// </summary>
        internal static string UnableToConvertTypeError_ptb {
            get {
                return ResourceManager.GetString("UnableToConvertTypeError_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Не удалось изменить тип &quot;{0}&quot; на &quot;{1}&quot;..
        /// </summary>
        internal static string UnableToConvertTypeError_rus {
            get {
                return ResourceManager.GetString("UnableToConvertTypeError_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0}&apos; türünden &apos;{1}&apos; türüne değiştirilemiyor..
        /// </summary>
        internal static string UnableToConvertTypeError_trk {
            get {
                return ResourceManager.GetString("UnableToConvertTypeError_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Unable to extract metadata from &apos;{0}&apos;..
        /// </summary>
        internal static string UnableToExtractAssemblyMetadata {
            get {
                return ResourceManager.GetString("UnableToExtractAssemblyMetadata", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 无法从“{0}”中提取元数据。.
        /// </summary>
        internal static string UnableToExtractAssemblyMetadata_chs {
            get {
                return ResourceManager.GetString("UnableToExtractAssemblyMetadata_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 無法從 &apos;{0}&apos; 擷取中繼資料。.
        /// </summary>
        internal static string UnableToExtractAssemblyMetadata_cht {
            get {
                return ResourceManager.GetString("UnableToExtractAssemblyMetadata_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Nelze extrahovat metadata z {0}..
        /// </summary>
        internal static string UnableToExtractAssemblyMetadata_csy {
            get {
                return ResourceManager.GetString("UnableToExtractAssemblyMetadata_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Aus &quot;{0}&quot; können keine Metadaten extrahiert werden..
        /// </summary>
        internal static string UnableToExtractAssemblyMetadata_deu {
            get {
                return ResourceManager.GetString("UnableToExtractAssemblyMetadata_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to No se pueden extraer metadatos desde &apos;{0}&apos;..
        /// </summary>
        internal static string UnableToExtractAssemblyMetadata_esp {
            get {
                return ResourceManager.GetString("UnableToExtractAssemblyMetadata_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Impossible d&apos;extraire les métadonnées de &apos;{0}&apos;..
        /// </summary>
        internal static string UnableToExtractAssemblyMetadata_fra {
            get {
                return ResourceManager.GetString("UnableToExtractAssemblyMetadata_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Impossibile estrarre metadati da &apos;{0}&apos;..
        /// </summary>
        internal static string UnableToExtractAssemblyMetadata_ita {
            get {
                return ResourceManager.GetString("UnableToExtractAssemblyMetadata_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to &apos;{0}&apos; からメタデータを抽出できません。.
        /// </summary>
        internal static string UnableToExtractAssemblyMetadata_jpn {
            get {
                return ResourceManager.GetString("UnableToExtractAssemblyMetadata_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to &apos;{0}&apos;에서 메타데이터를 추출할 수 없습니다..
        /// </summary>
        internal static string UnableToExtractAssemblyMetadata_kor {
            get {
                return ResourceManager.GetString("UnableToExtractAssemblyMetadata_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Nie można wyodrębnić metadanych z „{0}”..
        /// </summary>
        internal static string UnableToExtractAssemblyMetadata_plk {
            get {
                return ResourceManager.GetString("UnableToExtractAssemblyMetadata_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Não foi possível extrair os metadados de &apos;{0}&apos;..
        /// </summary>
        internal static string UnableToExtractAssemblyMetadata_ptb {
            get {
                return ResourceManager.GetString("UnableToExtractAssemblyMetadata_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Не удалось извлечь метаданные из &quot;{0}&quot;..
        /// </summary>
        internal static string UnableToExtractAssemblyMetadata_rus {
            get {
                return ResourceManager.GetString("UnableToExtractAssemblyMetadata_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Meta veriler &apos;{0}&apos; konumundan ayıklanamıyor..
        /// </summary>
        internal static string UnableToExtractAssemblyMetadata_trk {
            get {
                return ResourceManager.GetString("UnableToExtractAssemblyMetadata_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Unable to find &apos;{0}&apos;. Make sure the project has been built..
        /// </summary>
        internal static string UnableToFindBuildOutput {
            get {
                return ResourceManager.GetString("UnableToFindBuildOutput", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 找不到“{0}”。请确保已生成项目。.
        /// </summary>
        internal static string UnableToFindBuildOutput_chs {
            get {
                return ResourceManager.GetString("UnableToFindBuildOutput_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 找不到 &apos;{0}&apos;。確定已建置專案。.
        /// </summary>
        internal static string UnableToFindBuildOutput_cht {
            get {
                return ResourceManager.GetString("UnableToFindBuildOutput_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Nelze najít {0}. Ověřte, zda tento projekt byl sestaven..
        /// </summary>
        internal static string UnableToFindBuildOutput_csy {
            get {
                return ResourceManager.GetString("UnableToFindBuildOutput_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to &quot;{0}&quot; wurde nicht gefunden. Stellen Sie sicher, dass das Projekt erstellt wurde..
        /// </summary>
        internal static string UnableToFindBuildOutput_deu {
            get {
                return ResourceManager.GetString("UnableToFindBuildOutput_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to No se puede encontrar &apos;{0}&apos;. Asegúrese de que el proyecto se ha compilado..
        /// </summary>
        internal static string UnableToFindBuildOutput_esp {
            get {
                return ResourceManager.GetString("UnableToFindBuildOutput_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Impossible de trouver &apos;{0}&apos;. Assurez-vous que le projet a été créé..
        /// </summary>
        internal static string UnableToFindBuildOutput_fra {
            get {
                return ResourceManager.GetString("UnableToFindBuildOutput_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Impossibile trovare &apos;{0}&apos;. Assicurarsi che il progetto è stato creato..
        /// </summary>
        internal static string UnableToFindBuildOutput_ita {
            get {
                return ResourceManager.GetString("UnableToFindBuildOutput_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to &apos;{0}&apos; が見つかりません。プロジェクトが構築されたことを確認してください。.
        /// </summary>
        internal static string UnableToFindBuildOutput_jpn {
            get {
                return ResourceManager.GetString("UnableToFindBuildOutput_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to &apos;{0}&apos;을(를) 찾을 수 없습니다. 프로젝트가 빌드되었는지 확인하십시오..
        /// </summary>
        internal static string UnableToFindBuildOutput_kor {
            get {
                return ResourceManager.GetString("UnableToFindBuildOutput_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Nie można znaleźć „{0}”. Upewnij się, że projekt został skompilowany..
        /// </summary>
        internal static string UnableToFindBuildOutput_plk {
            get {
                return ResourceManager.GetString("UnableToFindBuildOutput_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Não é possível encontrar o &apos;{0}&apos;. Verifique se o projeto foi construído..
        /// </summary>
        internal static string UnableToFindBuildOutput_ptb {
            get {
                return ResourceManager.GetString("UnableToFindBuildOutput_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Не удалось найти &quot;{0}&quot;. Убедитесь, что была выполнена сборка проекта..
        /// </summary>
        internal static string UnableToFindBuildOutput_rus {
            get {
                return ResourceManager.GetString("UnableToFindBuildOutput_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0}&apos; bulunamıyor. Projenin oluşturulduğundan emin olun..
        /// </summary>
        internal static string UnableToFindBuildOutput_trk {
            get {
                return ResourceManager.GetString("UnableToFindBuildOutput_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to File does not exist ({0})..
        /// </summary>
        internal static string UnableToFindFile {
            get {
                return ResourceManager.GetString("UnableToFindFile", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 文件不存在({0})。.
        /// </summary>
        internal static string UnableToFindFile_chs {
            get {
                return ResourceManager.GetString("UnableToFindFile_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 檔案不存在 ({0})。.
        /// </summary>
        internal static string UnableToFindFile_cht {
            get {
                return ResourceManager.GetString("UnableToFindFile_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Soubor neexistuje ({0})..
        /// </summary>
        internal static string UnableToFindFile_csy {
            get {
                return ResourceManager.GetString("UnableToFindFile_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Die Datei ist nicht vorhanden (&quot;{0}&quot;)..
        /// </summary>
        internal static string UnableToFindFile_deu {
            get {
                return ResourceManager.GetString("UnableToFindFile_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to No existe el archivo ({0})..
        /// </summary>
        internal static string UnableToFindFile_esp {
            get {
                return ResourceManager.GetString("UnableToFindFile_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Le fichier n&apos;existe pas ({0})..
        /// </summary>
        internal static string UnableToFindFile_fra {
            get {
                return ResourceManager.GetString("UnableToFindFile_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Il file non esiste ({0})..
        /// </summary>
        internal static string UnableToFindFile_ita {
            get {
                return ResourceManager.GetString("UnableToFindFile_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to ファイルは存在しません ({0})。.
        /// </summary>
        internal static string UnableToFindFile_jpn {
            get {
                return ResourceManager.GetString("UnableToFindFile_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 파일이 없습니다({0})..
        /// </summary>
        internal static string UnableToFindFile_kor {
            get {
                return ResourceManager.GetString("UnableToFindFile_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Plik nie istnieje ({0})..
        /// </summary>
        internal static string UnableToFindFile_plk {
            get {
                return ResourceManager.GetString("UnableToFindFile_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to O arquivo não existe ({0})..
        /// </summary>
        internal static string UnableToFindFile_ptb {
            get {
                return ResourceManager.GetString("UnableToFindFile_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Файл не существует ({0})..
        /// </summary>
        internal static string UnableToFindFile_rus {
            get {
                return ResourceManager.GetString("UnableToFindFile_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Dosya mevcut değil ({0})..
        /// </summary>
        internal static string UnableToFindFile_trk {
            get {
                return ResourceManager.GetString("UnableToFindFile_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Unable to find &apos;{0}&apos;. Make sure they are specified in packages.config..
        /// </summary>
        internal static string UnableToFindPackages {
            get {
                return ResourceManager.GetString("UnableToFindPackages", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 找不到“{0}”。请确保已在 packages.config 中指定它们。.
        /// </summary>
        internal static string UnableToFindPackages_chs {
            get {
                return ResourceManager.GetString("UnableToFindPackages_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 找不到 &apos;{0}&apos;。確定已在 packages.config 中指定。.
        /// </summary>
        internal static string UnableToFindPackages_cht {
            get {
                return ResourceManager.GetString("UnableToFindPackages_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Nelze najít {0}. Ověřte, zda jsou zadány v souboru packages.config..
        /// </summary>
        internal static string UnableToFindPackages_csy {
            get {
                return ResourceManager.GetString("UnableToFindPackages_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to &quot;{0}&quot; wurde nicht gefunden. Stellen Sie sicher, dass das Element in &quot;packages.config&quot; spezifisch ist..
        /// </summary>
        internal static string UnableToFindPackages_deu {
            get {
                return ResourceManager.GetString("UnableToFindPackages_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to No se puede encontrar &apos;{0}&apos;. Asegúrese de que se han especificado en packages.config..
        /// </summary>
        internal static string UnableToFindPackages_esp {
            get {
                return ResourceManager.GetString("UnableToFindPackages_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Impossible de trouver &apos;{0}&apos;. Assurez-vous qu&apos;ils sont spécifiés dans packages.config..
        /// </summary>
        internal static string UnableToFindPackages_fra {
            get {
                return ResourceManager.GetString("UnableToFindPackages_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Impossibile trovare &apos;{0}&apos;.Assicurarsi che siano specificati in packages.config..
        /// </summary>
        internal static string UnableToFindPackages_ita {
            get {
                return ResourceManager.GetString("UnableToFindPackages_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to &apos;{0}&apos; が見つかりません。packages.config に指定されていることを確認してください。.
        /// </summary>
        internal static string UnableToFindPackages_jpn {
            get {
                return ResourceManager.GetString("UnableToFindPackages_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to &apos;{0}&apos;을(를) 찾을 수 없습니다. packages.config에 지정되어 있는지 확인하십시오..
        /// </summary>
        internal static string UnableToFindPackages_kor {
            get {
                return ResourceManager.GetString("UnableToFindPackages_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Nie można odnaleźć pakietów „{0}”. Upewnij się, że zostały one określone w pliku packages.config..
        /// </summary>
        internal static string UnableToFindPackages_plk {
            get {
                return ResourceManager.GetString("UnableToFindPackages_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Não foi possível encontrar o &apos;{0}&apos;. Verifique se eles estão especificados em packages.config..
        /// </summary>
        internal static string UnableToFindPackages_ptb {
            get {
                return ResourceManager.GetString("UnableToFindPackages_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Не удалось найти &quot;{0}&quot;. Убедитесь, что они указаны в файле packages.config..
        /// </summary>
        internal static string UnableToFindPackages_rus {
            get {
                return ResourceManager.GetString("UnableToFindPackages_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0}&apos; bulunamıyor. Packages.config içinde belirtildiğinden emin olun..
        /// </summary>
        internal static string UnableToFindPackages_trk {
            get {
                return ResourceManager.GetString("UnableToFindPackages_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Unable to find project &apos;{0}&apos;..
        /// </summary>
        internal static string UnableToFindProject {
            get {
                return ResourceManager.GetString("UnableToFindProject", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Unable to find solution &apos;{0}&apos;..
        /// </summary>
        internal static string UnableToFindSolution {
            get {
                return ResourceManager.GetString("UnableToFindSolution", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 找不到解决方案“{0}”。.
        /// </summary>
        internal static string UnableToFindSolution_chs {
            get {
                return ResourceManager.GetString("UnableToFindSolution_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 找不到方案 &apos;{0}&apos;。.
        /// </summary>
        internal static string UnableToFindSolution_cht {
            get {
                return ResourceManager.GetString("UnableToFindSolution_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Nelze najít řešení {0}..
        /// </summary>
        internal static string UnableToFindSolution_csy {
            get {
                return ResourceManager.GetString("UnableToFindSolution_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Das Projekt &quot;{0}&quot; wurde nicht gefunden..
        /// </summary>
        internal static string UnableToFindSolution_deu {
            get {
                return ResourceManager.GetString("UnableToFindSolution_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to No se puede encontrar la solución &apos;{0}&apos;..
        /// </summary>
        internal static string UnableToFindSolution_esp {
            get {
                return ResourceManager.GetString("UnableToFindSolution_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Impossible de trouver la solution &apos;{0}&apos;..
        /// </summary>
        internal static string UnableToFindSolution_fra {
            get {
                return ResourceManager.GetString("UnableToFindSolution_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Impossibile trovare soluzione &apos;{0}&apos;..
        /// </summary>
        internal static string UnableToFindSolution_ita {
            get {
                return ResourceManager.GetString("UnableToFindSolution_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to ソリューション &apos;{0}&apos; が見つかりません。.
        /// </summary>
        internal static string UnableToFindSolution_jpn {
            get {
                return ResourceManager.GetString("UnableToFindSolution_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to &apos;{0}&apos; 솔루션을 찾을 수 없습니다..
        /// </summary>
        internal static string UnableToFindSolution_kor {
            get {
                return ResourceManager.GetString("UnableToFindSolution_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Nie można znaleźć rozwiązania „{0}”..
        /// </summary>
        internal static string UnableToFindSolution_plk {
            get {
                return ResourceManager.GetString("UnableToFindSolution_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Não foi possível encontrar uma solução &apos;{0}&apos;..
        /// </summary>
        internal static string UnableToFindSolution_ptb {
            get {
                return ResourceManager.GetString("UnableToFindSolution_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Не удалось найти решение &quot;{0}&quot;..
        /// </summary>
        internal static string UnableToFindSolution_rus {
            get {
                return ResourceManager.GetString("UnableToFindSolution_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0}&apos; çözümü bulunamıyor..
        /// </summary>
        internal static string UnableToFindSolution_trk {
            get {
                return ResourceManager.GetString("UnableToFindSolution_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Unable to locate the packages folder. Try specifying it using the repositoryPath switch..
        /// </summary>
        internal static string UnableToLocatePackagesFolder {
            get {
                return ResourceManager.GetString("UnableToLocatePackagesFolder", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 找不到程序包文件夹。请尝试使用 repositoryPath 开关指定它。.
        /// </summary>
        internal static string UnableToLocatePackagesFolder_chs {
            get {
                return ResourceManager.GetString("UnableToLocatePackagesFolder_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 找不到封裝資料夾。嘗試使用 repositoryPath 切換指定。.
        /// </summary>
        internal static string UnableToLocatePackagesFolder_cht {
            get {
                return ResourceManager.GetString("UnableToLocatePackagesFolder_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Složka balíčků nebyla nalezena. Zkuste ji zadat pomocí přepínače repositoryPath..
        /// </summary>
        internal static string UnableToLocatePackagesFolder_csy {
            get {
                return ResourceManager.GetString("UnableToLocatePackagesFolder_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Der Paketordner wurde nicht gefunden. Versuchen Sie, ihn mithilfe des Schalters &quot;repositoryPath &quot; anzugeben..
        /// </summary>
        internal static string UnableToLocatePackagesFolder_deu {
            get {
                return ResourceManager.GetString("UnableToLocatePackagesFolder_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to No se puede ubicar la carpeta de los paquetes. Pruebe de especificarla usando el conmutador repositoryPath..
        /// </summary>
        internal static string UnableToLocatePackagesFolder_esp {
            get {
                return ResourceManager.GetString("UnableToLocatePackagesFolder_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Impossible de trouver le dossier de packages. Essayez de le spécifier à l&apos;aide du commutateur repositoryPath..
        /// </summary>
        internal static string UnableToLocatePackagesFolder_fra {
            get {
                return ResourceManager.GetString("UnableToLocatePackagesFolder_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Impossibile localizzare la cartella pacchetto. Provare a specificarla usando repositoryPath switch..
        /// </summary>
        internal static string UnableToLocatePackagesFolder_ita {
            get {
                return ResourceManager.GetString("UnableToLocatePackagesFolder_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to packages フォルダーが見つかりません。repositoryPath スイッチを使用して指定してください。.
        /// </summary>
        internal static string UnableToLocatePackagesFolder_jpn {
            get {
                return ResourceManager.GetString("UnableToLocatePackagesFolder_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 패키지 폴더를 찾을 수 없습니다. repositoryPath 스위치를 사용하여 지정해 보십시오..
        /// </summary>
        internal static string UnableToLocatePackagesFolder_kor {
            get {
                return ResourceManager.GetString("UnableToLocatePackagesFolder_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Nie można zlokalizować folderu pakietów. Spróbuj go określić przy użyciu przełącznika repositoryPath..
        /// </summary>
        internal static string UnableToLocatePackagesFolder_plk {
            get {
                return ResourceManager.GetString("UnableToLocatePackagesFolder_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Não foi possível localizar a pasta de pacotes. Tente especificá-la usando a opção repositoryPath..
        /// </summary>
        internal static string UnableToLocatePackagesFolder_ptb {
            get {
                return ResourceManager.GetString("UnableToLocatePackagesFolder_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Не удалось найти папку пакетов. Попробуйте указать ее с помощью параметра repositoryPath..
        /// </summary>
        internal static string UnableToLocatePackagesFolder_rus {
            get {
                return ResourceManager.GetString("UnableToLocatePackagesFolder_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Paket klasörü bulunamıyor. RepositoryPath anahtarını kullanarak belirlemeyi deneyin..
        /// </summary>
        internal static string UnableToLocatePackagesFolder_trk {
            get {
                return ResourceManager.GetString("UnableToLocatePackagesFolder_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Unable to locate project file for &apos;{0}&apos;..
        /// </summary>
        internal static string UnableToLocateProjectFile {
            get {
                return ResourceManager.GetString("UnableToLocateProjectFile", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 找不到“{0}”的项目文件。.
        /// </summary>
        internal static string UnableToLocateProjectFile_chs {
            get {
                return ResourceManager.GetString("UnableToLocateProjectFile_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 找不到 &apos;{0}&apos; 的專案檔。.
        /// </summary>
        internal static string UnableToLocateProjectFile_cht {
            get {
                return ResourceManager.GetString("UnableToLocateProjectFile_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Soubor projektu pro {0} nebyl nalezen..
        /// </summary>
        internal static string UnableToLocateProjectFile_csy {
            get {
                return ResourceManager.GetString("UnableToLocateProjectFile_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Die Projektdatei für &quot;{0}&quot; wurde nicht gefunden..
        /// </summary>
        internal static string UnableToLocateProjectFile_deu {
            get {
                return ResourceManager.GetString("UnableToLocateProjectFile_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to No se puede ubicar el archivo de proyecto para &apos;{0}&apos;..
        /// </summary>
        internal static string UnableToLocateProjectFile_esp {
            get {
                return ResourceManager.GetString("UnableToLocateProjectFile_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Impossible de trouver le fichier projet pour &apos;{0}&apos;..
        /// </summary>
        internal static string UnableToLocateProjectFile_fra {
            get {
                return ResourceManager.GetString("UnableToLocateProjectFile_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Impossibile localizzare file progetto per &apos;{0}&apos;..
        /// </summary>
        internal static string UnableToLocateProjectFile_ita {
            get {
                return ResourceManager.GetString("UnableToLocateProjectFile_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to &apos;{0}&apos; のプロジェクト ファイルが見つかりません。.
        /// </summary>
        internal static string UnableToLocateProjectFile_jpn {
            get {
                return ResourceManager.GetString("UnableToLocateProjectFile_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to &apos;{0}&apos;에 대한 프로젝트 파일을 찾을 수 없습니다..
        /// </summary>
        internal static string UnableToLocateProjectFile_kor {
            get {
                return ResourceManager.GetString("UnableToLocateProjectFile_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Nie można zlokalizować pliku projektu dla „{0}”..
        /// </summary>
        internal static string UnableToLocateProjectFile_plk {
            get {
                return ResourceManager.GetString("UnableToLocateProjectFile_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Não foi possível localizar arquivo de projeto para &apos;{0}&apos;..
        /// </summary>
        internal static string UnableToLocateProjectFile_ptb {
            get {
                return ResourceManager.GetString("UnableToLocateProjectFile_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Не удалось найти файл проекта для &quot;{0}&quot;..
        /// </summary>
        internal static string UnableToLocateProjectFile_rus {
            get {
                return ResourceManager.GetString("UnableToLocateProjectFile_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0}&apos; için proje dosyası bulunamıyor..
        /// </summary>
        internal static string UnableToLocateProjectFile_trk {
            get {
                return ResourceManager.GetString("UnableToLocateProjectFile_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Unknown command: &apos;{0}&apos;.
        /// </summary>
        internal static string UnknowCommandError {
            get {
                return ResourceManager.GetString("UnknowCommandError", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 未知命令:“{0}”.
        /// </summary>
        internal static string UnknowCommandError_chs {
            get {
                return ResourceManager.GetString("UnknowCommandError_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 不明的命令: &apos;{0}&apos;.
        /// </summary>
        internal static string UnknowCommandError_cht {
            get {
                return ResourceManager.GetString("UnknowCommandError_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Neznámý příkaz: &apos;{0}&apos;.
        /// </summary>
        internal static string UnknowCommandError_csy {
            get {
                return ResourceManager.GetString("UnknowCommandError_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Unbekannter Befehl: &quot;{0}&quot;.
        /// </summary>
        internal static string UnknowCommandError_deu {
            get {
                return ResourceManager.GetString("UnknowCommandError_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Comando desconocido: &apos;{0}&apos;.
        /// </summary>
        internal static string UnknowCommandError_esp {
            get {
                return ResourceManager.GetString("UnknowCommandError_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Commande inconnue : &apos;{0}&apos;.
        /// </summary>
        internal static string UnknowCommandError_fra {
            get {
                return ResourceManager.GetString("UnknowCommandError_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Comando sconosciuto: &apos;{0}&apos;.
        /// </summary>
        internal static string UnknowCommandError_ita {
            get {
                return ResourceManager.GetString("UnknowCommandError_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 不明なコマンド: &apos;{0}&apos;.
        /// </summary>
        internal static string UnknowCommandError_jpn {
            get {
                return ResourceManager.GetString("UnknowCommandError_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 알 수 없는 명령: &apos;{0}&apos;.
        /// </summary>
        internal static string UnknowCommandError_kor {
            get {
                return ResourceManager.GetString("UnknowCommandError_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Nieznane polecenie: „{0}”.
        /// </summary>
        internal static string UnknowCommandError_plk {
            get {
                return ResourceManager.GetString("UnknowCommandError_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Comando desconhecido: &apos;{0}&apos;.
        /// </summary>
        internal static string UnknowCommandError_ptb {
            get {
                return ResourceManager.GetString("UnknowCommandError_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Неизвестная команда: &quot;{0}&quot;.
        /// </summary>
        internal static string UnknowCommandError_rus {
            get {
                return ResourceManager.GetString("UnknowCommandError_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Bilinmeyen komut: &apos;{0}&apos;.
        /// </summary>
        internal static string UnknowCommandError_trk {
            get {
                return ResourceManager.GetString("UnknowCommandError_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Unknown option: &apos;{0}&apos;.
        /// </summary>
        internal static string UnknownOptionError {
            get {
                return ResourceManager.GetString("UnknownOptionError", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 未知选项:“{0}”.
        /// </summary>
        internal static string UnknownOptionError_chs {
            get {
                return ResourceManager.GetString("UnknownOptionError_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 不明的選項: &apos;{0}&apos;.
        /// </summary>
        internal static string UnknownOptionError_cht {
            get {
                return ResourceManager.GetString("UnknownOptionError_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Neznámá možnost: &apos;{0}&apos;.
        /// </summary>
        internal static string UnknownOptionError_csy {
            get {
                return ResourceManager.GetString("UnknownOptionError_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Unbekannte Option: &quot;{0}&quot;.
        /// </summary>
        internal static string UnknownOptionError_deu {
            get {
                return ResourceManager.GetString("UnknownOptionError_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Opción desconocida: &apos;{0}&apos;.
        /// </summary>
        internal static string UnknownOptionError_esp {
            get {
                return ResourceManager.GetString("UnknownOptionError_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Option inconnue : &apos;{0}&apos;.
        /// </summary>
        internal static string UnknownOptionError_fra {
            get {
                return ResourceManager.GetString("UnknownOptionError_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Opzione sconosciuta: &apos;{0}&apos;.
        /// </summary>
        internal static string UnknownOptionError_ita {
            get {
                return ResourceManager.GetString("UnknownOptionError_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 不明なオプション: &apos;{0}&apos;.
        /// </summary>
        internal static string UnknownOptionError_jpn {
            get {
                return ResourceManager.GetString("UnknownOptionError_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 알 수 없는 옵션: &apos;{0}&apos;.
        /// </summary>
        internal static string UnknownOptionError_kor {
            get {
                return ResourceManager.GetString("UnknownOptionError_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Nieznana opcja: „{0}”.
        /// </summary>
        internal static string UnknownOptionError_plk {
            get {
                return ResourceManager.GetString("UnknownOptionError_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Opção desconhecida: &apos;{0}&apos;.
        /// </summary>
        internal static string UnknownOptionError_ptb {
            get {
                return ResourceManager.GetString("UnknownOptionError_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Неизвестный оператор: &quot;{0}&quot;.
        /// </summary>
        internal static string UnknownOptionError_rus {
            get {
                return ResourceManager.GetString("UnknownOptionError_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Bilinmeyen seçenek: &apos;{0}&apos;.
        /// </summary>
        internal static string UnknownOptionError_trk {
            get {
                return ResourceManager.GetString("UnknownOptionError_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Checking for updates from {0}..
        /// </summary>
        internal static string UpdateCommandCheckingForUpdates {
            get {
                return ResourceManager.GetString("UpdateCommandCheckingForUpdates", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 正在检查来自 {0} 的更新。.
        /// </summary>
        internal static string UpdateCommandCheckingForUpdates_chs {
            get {
                return ResourceManager.GetString("UpdateCommandCheckingForUpdates_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 正在檢查來自 {0} 的更新。.
        /// </summary>
        internal static string UpdateCommandCheckingForUpdates_cht {
            get {
                return ResourceManager.GetString("UpdateCommandCheckingForUpdates_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Probíhá kontrola aktualizací z {0}..
        /// </summary>
        internal static string UpdateCommandCheckingForUpdates_csy {
            get {
                return ResourceManager.GetString("UpdateCommandCheckingForUpdates_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Es wird auf Updates von {0} überprüft..
        /// </summary>
        internal static string UpdateCommandCheckingForUpdates_deu {
            get {
                return ResourceManager.GetString("UpdateCommandCheckingForUpdates_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Comprobando actualizaciones desde {0}..
        /// </summary>
        internal static string UpdateCommandCheckingForUpdates_esp {
            get {
                return ResourceManager.GetString("UpdateCommandCheckingForUpdates_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Recherche de mises à jour depuis {0}..
        /// </summary>
        internal static string UpdateCommandCheckingForUpdates_fra {
            get {
                return ResourceManager.GetString("UpdateCommandCheckingForUpdates_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Controllando aggiornamenti da {0}..
        /// </summary>
        internal static string UpdateCommandCheckingForUpdates_ita {
            get {
                return ResourceManager.GetString("UpdateCommandCheckingForUpdates_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} の更新を確認しています。.
        /// </summary>
        internal static string UpdateCommandCheckingForUpdates_jpn {
            get {
                return ResourceManager.GetString("UpdateCommandCheckingForUpdates_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0}에서 업데이트를 확인하고 있습니다..
        /// </summary>
        internal static string UpdateCommandCheckingForUpdates_kor {
            get {
                return ResourceManager.GetString("UpdateCommandCheckingForUpdates_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Sprawdzanie dostępności aktualizacji z {0}..
        /// </summary>
        internal static string UpdateCommandCheckingForUpdates_plk {
            get {
                return ResourceManager.GetString("UpdateCommandCheckingForUpdates_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Verifique se há atualizações de {0}..
        /// </summary>
        internal static string UpdateCommandCheckingForUpdates_ptb {
            get {
                return ResourceManager.GetString("UpdateCommandCheckingForUpdates_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Проверка обновлений из &quot;{0}&quot;..
        /// </summary>
        internal static string UpdateCommandCheckingForUpdates_rus {
            get {
                return ResourceManager.GetString("UpdateCommandCheckingForUpdates_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Güncellemeler {0} konumundan denetleniyor..
        /// </summary>
        internal static string UpdateCommandCheckingForUpdates_trk {
            get {
                return ResourceManager.GetString("UpdateCommandCheckingForUpdates_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Currently running NuGet.exe {0}..
        /// </summary>
        internal static string UpdateCommandCurrentlyRunningNuGetExe {
            get {
                return ResourceManager.GetString("UpdateCommandCurrentlyRunningNuGetExe", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 当前正在运行 NuGet.exe {0}。.
        /// </summary>
        internal static string UpdateCommandCurrentlyRunningNuGetExe_chs {
            get {
                return ResourceManager.GetString("UpdateCommandCurrentlyRunningNuGetExe_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 目前正在執行 NuGet.exe {0}。.
        /// </summary>
        internal static string UpdateCommandCurrentlyRunningNuGetExe_cht {
            get {
                return ResourceManager.GetString("UpdateCommandCurrentlyRunningNuGetExe_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Aktuálně je spuštěn soubor NuGet.exe {0}..
        /// </summary>
        internal static string UpdateCommandCurrentlyRunningNuGetExe_csy {
            get {
                return ResourceManager.GetString("UpdateCommandCurrentlyRunningNuGetExe_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to &quot;NuGet.exe&quot; {0} wird zurzeit ausgeführt..
        /// </summary>
        internal static string UpdateCommandCurrentlyRunningNuGetExe_deu {
            get {
                return ResourceManager.GetString("UpdateCommandCurrentlyRunningNuGetExe_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Se está ejecutando NuGet.exe {0}..
        /// </summary>
        internal static string UpdateCommandCurrentlyRunningNuGetExe_esp {
            get {
                return ResourceManager.GetString("UpdateCommandCurrentlyRunningNuGetExe_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to NuGet.exe {0} en cours d&apos;exécution..
        /// </summary>
        internal static string UpdateCommandCurrentlyRunningNuGetExe_fra {
            get {
                return ResourceManager.GetString("UpdateCommandCurrentlyRunningNuGetExe_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to  NuGet.exe {0} avviato..
        /// </summary>
        internal static string UpdateCommandCurrentlyRunningNuGetExe_ita {
            get {
                return ResourceManager.GetString("UpdateCommandCurrentlyRunningNuGetExe_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to NuGet.exe {0} は実行中です。.
        /// </summary>
        internal static string UpdateCommandCurrentlyRunningNuGetExe_jpn {
            get {
                return ResourceManager.GetString("UpdateCommandCurrentlyRunningNuGetExe_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 현재 NuGet.exe {0}을(를) 실행하고 있습니다..
        /// </summary>
        internal static string UpdateCommandCurrentlyRunningNuGetExe_kor {
            get {
                return ResourceManager.GetString("UpdateCommandCurrentlyRunningNuGetExe_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Plik NuGet.exe {0} jest obecnie uruchomiony..
        /// </summary>
        internal static string UpdateCommandCurrentlyRunningNuGetExe_plk {
            get {
                return ResourceManager.GetString("UpdateCommandCurrentlyRunningNuGetExe_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Atualmente executando NuGet.exe {0}..
        /// </summary>
        internal static string UpdateCommandCurrentlyRunningNuGetExe_ptb {
            get {
                return ResourceManager.GetString("UpdateCommandCurrentlyRunningNuGetExe_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Сейчас запущен NuGet.exe {0}..
        /// </summary>
        internal static string UpdateCommandCurrentlyRunningNuGetExe_rus {
            get {
                return ResourceManager.GetString("UpdateCommandCurrentlyRunningNuGetExe_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to NuGet.exe {0} şu anda çalıştırılıyor..
        /// </summary>
        internal static string UpdateCommandCurrentlyRunningNuGetExe_trk {
            get {
                return ResourceManager.GetString("UpdateCommandCurrentlyRunningNuGetExe_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to NuGet.exe is up to date..
        /// </summary>
        internal static string UpdateCommandNuGetUpToDate {
            get {
                return ResourceManager.GetString("UpdateCommandNuGetUpToDate", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to NuGet.exe 是最新的。.
        /// </summary>
        internal static string UpdateCommandNuGetUpToDate_chs {
            get {
                return ResourceManager.GetString("UpdateCommandNuGetUpToDate_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to NuGet.exe 是最新的。.
        /// </summary>
        internal static string UpdateCommandNuGetUpToDate_cht {
            get {
                return ResourceManager.GetString("UpdateCommandNuGetUpToDate_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to NuGet.exe je aktuální..
        /// </summary>
        internal static string UpdateCommandNuGetUpToDate_csy {
            get {
                return ResourceManager.GetString("UpdateCommandNuGetUpToDate_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to &quot;NuGet.exe&quot; ist aktuell..
        /// </summary>
        internal static string UpdateCommandNuGetUpToDate_deu {
            get {
                return ResourceManager.GetString("UpdateCommandNuGetUpToDate_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to NuGet.exe está actualizado..
        /// </summary>
        internal static string UpdateCommandNuGetUpToDate_esp {
            get {
                return ResourceManager.GetString("UpdateCommandNuGetUpToDate_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to NuGet.exe est à jour..
        /// </summary>
        internal static string UpdateCommandNuGetUpToDate_fra {
            get {
                return ResourceManager.GetString("UpdateCommandNuGetUpToDate_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to NuGet.exe è aggiornato..
        /// </summary>
        internal static string UpdateCommandNuGetUpToDate_ita {
            get {
                return ResourceManager.GetString("UpdateCommandNuGetUpToDate_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to NuGet.exe は最新です。.
        /// </summary>
        internal static string UpdateCommandNuGetUpToDate_jpn {
            get {
                return ResourceManager.GetString("UpdateCommandNuGetUpToDate_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to NuGet.exe가 최신 버전입니다..
        /// </summary>
        internal static string UpdateCommandNuGetUpToDate_kor {
            get {
                return ResourceManager.GetString("UpdateCommandNuGetUpToDate_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Plik NuGet.exe jest aktualny..
        /// </summary>
        internal static string UpdateCommandNuGetUpToDate_plk {
            get {
                return ResourceManager.GetString("UpdateCommandNuGetUpToDate_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to O NuGet.exe está atualizado..
        /// </summary>
        internal static string UpdateCommandNuGetUpToDate_ptb {
            get {
                return ResourceManager.GetString("UpdateCommandNuGetUpToDate_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Обновление NuGet.exe не требуется..
        /// </summary>
        internal static string UpdateCommandNuGetUpToDate_rus {
            get {
                return ResourceManager.GetString("UpdateCommandNuGetUpToDate_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to NuGet.exe güncel..
        /// </summary>
        internal static string UpdateCommandNuGetUpToDate_trk {
            get {
                return ResourceManager.GetString("UpdateCommandNuGetUpToDate_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Unable to find &apos;{0}&apos; package..
        /// </summary>
        internal static string UpdateCommandUnableToFindPackage {
            get {
                return ResourceManager.GetString("UpdateCommandUnableToFindPackage", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 找不到“{0}”程序包。.
        /// </summary>
        internal static string UpdateCommandUnableToFindPackage_chs {
            get {
                return ResourceManager.GetString("UpdateCommandUnableToFindPackage_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 找不到 &apos;{0}&apos; 封裝。.
        /// </summary>
        internal static string UpdateCommandUnableToFindPackage_cht {
            get {
                return ResourceManager.GetString("UpdateCommandUnableToFindPackage_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Balíček {0} nebyl nalezen..
        /// </summary>
        internal static string UpdateCommandUnableToFindPackage_csy {
            get {
                return ResourceManager.GetString("UpdateCommandUnableToFindPackage_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Das Paket &quot;{0}&quot; wurde nicht gefunden..
        /// </summary>
        internal static string UpdateCommandUnableToFindPackage_deu {
            get {
                return ResourceManager.GetString("UpdateCommandUnableToFindPackage_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to No se puede encontrar el paquete &apos;{0}&apos;..
        /// </summary>
        internal static string UpdateCommandUnableToFindPackage_esp {
            get {
                return ResourceManager.GetString("UpdateCommandUnableToFindPackage_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Impossible de trouver le package &apos;{0}&apos;..
        /// </summary>
        internal static string UpdateCommandUnableToFindPackage_fra {
            get {
                return ResourceManager.GetString("UpdateCommandUnableToFindPackage_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Impossibile trovare pacchetto &apos;{0}&apos;..
        /// </summary>
        internal static string UpdateCommandUnableToFindPackage_ita {
            get {
                return ResourceManager.GetString("UpdateCommandUnableToFindPackage_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to &apos;{0}&apos; パッケージが見つかりません。.
        /// </summary>
        internal static string UpdateCommandUnableToFindPackage_jpn {
            get {
                return ResourceManager.GetString("UpdateCommandUnableToFindPackage_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to &apos;{0}&apos; 패키지를 찾을 수 없습니다..
        /// </summary>
        internal static string UpdateCommandUnableToFindPackage_kor {
            get {
                return ResourceManager.GetString("UpdateCommandUnableToFindPackage_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Nie można odnaleźć pakietu „{0}”..
        /// </summary>
        internal static string UpdateCommandUnableToFindPackage_plk {
            get {
                return ResourceManager.GetString("UpdateCommandUnableToFindPackage_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Não foi possível encontrar o pacote &apos;{0}&apos;..
        /// </summary>
        internal static string UpdateCommandUnableToFindPackage_ptb {
            get {
                return ResourceManager.GetString("UpdateCommandUnableToFindPackage_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Не удалось найти пакет &quot;{0}&quot;..
        /// </summary>
        internal static string UpdateCommandUnableToFindPackage_rus {
            get {
                return ResourceManager.GetString("UpdateCommandUnableToFindPackage_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0}&apos; paketi bulunamıyor..
        /// </summary>
        internal static string UpdateCommandUnableToFindPackage_trk {
            get {
                return ResourceManager.GetString("UpdateCommandUnableToFindPackage_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Invalid NuGet.CommandLine package. Unable to locate NuGet.exe within the package..
        /// </summary>
        internal static string UpdateCommandUnableToLocateNuGetExe {
            get {
                return ResourceManager.GetString("UpdateCommandUnableToLocateNuGetExe", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to NuGet.CommandLine 程序包无效。在该程序包中找不到 NuGet.exe。.
        /// </summary>
        internal static string UpdateCommandUnableToLocateNuGetExe_chs {
            get {
                return ResourceManager.GetString("UpdateCommandUnableToLocateNuGetExe_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 無效的 NuGet.CommandLine 封裝。在封裝中找不到 NuGet.exe。.
        /// </summary>
        internal static string UpdateCommandUnableToLocateNuGetExe_cht {
            get {
                return ResourceManager.GetString("UpdateCommandUnableToLocateNuGetExe_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Neplatný balíček NuGet.CommandLine. V tomto balíčku nebyl nalezen soubor NuGet.exe..
        /// </summary>
        internal static string UpdateCommandUnableToLocateNuGetExe_csy {
            get {
                return ResourceManager.GetString("UpdateCommandUnableToLocateNuGetExe_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Ungültiges NuGet.CommandLine-Paket. &quot;NuGet.exe&quot; wurde im Paket nicht gefunden..
        /// </summary>
        internal static string UpdateCommandUnableToLocateNuGetExe_deu {
            get {
                return ResourceManager.GetString("UpdateCommandUnableToLocateNuGetExe_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Paquete NuGet.CommandLine no válido. No se puede ubicar NuGet.exe en el paquete..
        /// </summary>
        internal static string UpdateCommandUnableToLocateNuGetExe_esp {
            get {
                return ResourceManager.GetString("UpdateCommandUnableToLocateNuGetExe_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Package NuGet.CommandLine non valide. Impossible de trouver NuGet.exe dans le package..
        /// </summary>
        internal static string UpdateCommandUnableToLocateNuGetExe_fra {
            get {
                return ResourceManager.GetString("UpdateCommandUnableToLocateNuGetExe_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Pacchetto NuGet.CommandLine invalido. Impossibile localizzare NuGet.exe all&apos;interno del pacchetto..
        /// </summary>
        internal static string UpdateCommandUnableToLocateNuGetExe_ita {
            get {
                return ResourceManager.GetString("UpdateCommandUnableToLocateNuGetExe_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to NuGet.CommandLine パッケージが無効です。パッケージ内に NuGet.exe が見つかりません。.
        /// </summary>
        internal static string UpdateCommandUnableToLocateNuGetExe_jpn {
            get {
                return ResourceManager.GetString("UpdateCommandUnableToLocateNuGetExe_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 잘못된 NuGet.CommandLine 패키지입니다. 패키지에서 NuGet.exe를 찾을 수 없습니다..
        /// </summary>
        internal static string UpdateCommandUnableToLocateNuGetExe_kor {
            get {
                return ResourceManager.GetString("UpdateCommandUnableToLocateNuGetExe_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Nieprawidłowy pakiet NuGet.CommandLine. Nie można odnaleźć pliku NuGet.exe wewnątrz pakietu..
        /// </summary>
        internal static string UpdateCommandUnableToLocateNuGetExe_plk {
            get {
                return ResourceManager.GetString("UpdateCommandUnableToLocateNuGetExe_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Pacote NuGet.CommandLine inválido. Não foi possível localizar NuGet.exe dentro do pacote..
        /// </summary>
        internal static string UpdateCommandUnableToLocateNuGetExe_ptb {
            get {
                return ResourceManager.GetString("UpdateCommandUnableToLocateNuGetExe_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Недопустимый пакет NuGet.CommandLine. В пакете не удалось найти NuGet.exe..
        /// </summary>
        internal static string UpdateCommandUnableToLocateNuGetExe_rus {
            get {
                return ResourceManager.GetString("UpdateCommandUnableToLocateNuGetExe_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Geçersiz NuGet.CommandLine paketi. Paket içindeki NuGet.exe bulunamıyor..
        /// </summary>
        internal static string UpdateCommandUnableToLocateNuGetExe_trk {
            get {
                return ResourceManager.GetString("UpdateCommandUnableToLocateNuGetExe_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Update successful..
        /// </summary>
        internal static string UpdateCommandUpdateSuccessful {
            get {
                return ResourceManager.GetString("UpdateCommandUpdateSuccessful", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 更新成功。.
        /// </summary>
        internal static string UpdateCommandUpdateSuccessful_chs {
            get {
                return ResourceManager.GetString("UpdateCommandUpdateSuccessful_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 成功更新。.
        /// </summary>
        internal static string UpdateCommandUpdateSuccessful_cht {
            get {
                return ResourceManager.GetString("UpdateCommandUpdateSuccessful_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Aktualizace proběhla úspěšně..
        /// </summary>
        internal static string UpdateCommandUpdateSuccessful_csy {
            get {
                return ResourceManager.GetString("UpdateCommandUpdateSuccessful_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Das Update war erfolgreich..
        /// </summary>
        internal static string UpdateCommandUpdateSuccessful_deu {
            get {
                return ResourceManager.GetString("UpdateCommandUpdateSuccessful_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Actualización correcta..
        /// </summary>
        internal static string UpdateCommandUpdateSuccessful_esp {
            get {
                return ResourceManager.GetString("UpdateCommandUpdateSuccessful_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Mise à jour réussie..
        /// </summary>
        internal static string UpdateCommandUpdateSuccessful_fra {
            get {
                return ResourceManager.GetString("UpdateCommandUpdateSuccessful_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Aggiornamento riuscito.
        /// </summary>
        internal static string UpdateCommandUpdateSuccessful_ita {
            get {
                return ResourceManager.GetString("UpdateCommandUpdateSuccessful_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 正常に更新されました。.
        /// </summary>
        internal static string UpdateCommandUpdateSuccessful_jpn {
            get {
                return ResourceManager.GetString("UpdateCommandUpdateSuccessful_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 업데이트했습니다..
        /// </summary>
        internal static string UpdateCommandUpdateSuccessful_kor {
            get {
                return ResourceManager.GetString("UpdateCommandUpdateSuccessful_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Aktualizacja powiodła się..
        /// </summary>
        internal static string UpdateCommandUpdateSuccessful_plk {
            get {
                return ResourceManager.GetString("UpdateCommandUpdateSuccessful_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Atualização bem-sucedida..
        /// </summary>
        internal static string UpdateCommandUpdateSuccessful_ptb {
            get {
                return ResourceManager.GetString("UpdateCommandUpdateSuccessful_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Обновление выполнено..
        /// </summary>
        internal static string UpdateCommandUpdateSuccessful_rus {
            get {
                return ResourceManager.GetString("UpdateCommandUpdateSuccessful_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Güncelleme başarılı..
        /// </summary>
        internal static string UpdateCommandUpdateSuccessful_trk {
            get {
                return ResourceManager.GetString("UpdateCommandUpdateSuccessful_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Updating NuGet.exe to {0}..
        /// </summary>
        internal static string UpdateCommandUpdatingNuGet {
            get {
                return ResourceManager.GetString("UpdateCommandUpdatingNuGet", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 正在将 NuGet.exe 更新到 {0}。.
        /// </summary>
        internal static string UpdateCommandUpdatingNuGet_chs {
            get {
                return ResourceManager.GetString("UpdateCommandUpdatingNuGet_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 正在更新 NuGet.exe 為 {0}。.
        /// </summary>
        internal static string UpdateCommandUpdatingNuGet_cht {
            get {
                return ResourceManager.GetString("UpdateCommandUpdatingNuGet_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Probíhá aktualizace souboru NuGet.exe na {0}..
        /// </summary>
        internal static string UpdateCommandUpdatingNuGet_csy {
            get {
                return ResourceManager.GetString("UpdateCommandUpdatingNuGet_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to &quot;NuGet.exe&quot; wird auf {0} aktualisiert..
        /// </summary>
        internal static string UpdateCommandUpdatingNuGet_deu {
            get {
                return ResourceManager.GetString("UpdateCommandUpdatingNuGet_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Actualizando NuGet.exe a {0}..
        /// </summary>
        internal static string UpdateCommandUpdatingNuGet_esp {
            get {
                return ResourceManager.GetString("UpdateCommandUpdatingNuGet_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Mise à jour de NuGet.exe vers {0}..
        /// </summary>
        internal static string UpdateCommandUpdatingNuGet_fra {
            get {
                return ResourceManager.GetString("UpdateCommandUpdatingNuGet_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Aggiornando NuGet.exe a {0}..
        /// </summary>
        internal static string UpdateCommandUpdatingNuGet_ita {
            get {
                return ResourceManager.GetString("UpdateCommandUpdatingNuGet_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to NuGet.exe を {0} に更新しています。.
        /// </summary>
        internal static string UpdateCommandUpdatingNuGet_jpn {
            get {
                return ResourceManager.GetString("UpdateCommandUpdatingNuGet_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to NuGet.exe를 {0}(으)로 업데이트하고 있습니다..
        /// </summary>
        internal static string UpdateCommandUpdatingNuGet_kor {
            get {
                return ResourceManager.GetString("UpdateCommandUpdatingNuGet_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Aktualizowanie pliku NuGet.exe do {0}..
        /// </summary>
        internal static string UpdateCommandUpdatingNuGet_plk {
            get {
                return ResourceManager.GetString("UpdateCommandUpdatingNuGet_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Atualizando NuGet.exe para {0}..
        /// </summary>
        internal static string UpdateCommandUpdatingNuGet_ptb {
            get {
                return ResourceManager.GetString("UpdateCommandUpdatingNuGet_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Обновление NuGet.exe в {0}..
        /// </summary>
        internal static string UpdateCommandUpdatingNuGet_rus {
            get {
                return ResourceManager.GetString("UpdateCommandUpdatingNuGet_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to NuGet.exe {0} olarak güncelleniyor..
        /// </summary>
        internal static string UpdateCommandUpdatingNuGet_trk {
            get {
                return ResourceManager.GetString("UpdateCommandUpdatingNuGet_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Updating &apos;{0}&apos;....
        /// </summary>
        internal static string UpdatingProject {
            get {
                return ResourceManager.GetString("UpdatingProject", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 正在更新“{0}”....
        /// </summary>
        internal static string UpdatingProject_chs {
            get {
                return ResourceManager.GetString("UpdatingProject_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 正在更新 &apos;{0}&apos;....
        /// </summary>
        internal static string UpdatingProject_cht {
            get {
                return ResourceManager.GetString("UpdatingProject_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Probíhá aktualizace {0}....
        /// </summary>
        internal static string UpdatingProject_csy {
            get {
                return ResourceManager.GetString("UpdatingProject_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to &quot;{0}&quot; wird aktualisiert....
        /// </summary>
        internal static string UpdatingProject_deu {
            get {
                return ResourceManager.GetString("UpdatingProject_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Actualizando &apos;{0}&apos;….
        /// </summary>
        internal static string UpdatingProject_esp {
            get {
                return ResourceManager.GetString("UpdatingProject_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Mise à jour de &apos;{0}&apos;….
        /// </summary>
        internal static string UpdatingProject_fra {
            get {
                return ResourceManager.GetString("UpdatingProject_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Aggiornando &apos;{0}&apos;....
        /// </summary>
        internal static string UpdatingProject_ita {
            get {
                return ResourceManager.GetString("UpdatingProject_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to &apos;{0}&apos; を更新しています....
        /// </summary>
        internal static string UpdatingProject_jpn {
            get {
                return ResourceManager.GetString("UpdatingProject_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to &apos;{0}&apos;을(를) 업데이트하는 중....
        /// </summary>
        internal static string UpdatingProject_kor {
            get {
                return ResourceManager.GetString("UpdatingProject_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Trwa aktualizacja „{0}”....
        /// </summary>
        internal static string UpdatingProject_plk {
            get {
                return ResourceManager.GetString("UpdatingProject_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Atualizando &apos;{0}&apos; ....
        /// </summary>
        internal static string UpdatingProject_ptb {
            get {
                return ResourceManager.GetString("UpdatingProject_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Обновление &quot;{0}&quot;....
        /// </summary>
        internal static string UpdatingProject_rus {
            get {
                return ResourceManager.GetString("UpdatingProject_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0}&apos; güncelleniyor....
        /// </summary>
        internal static string UpdatingProject_trk {
            get {
                return ResourceManager.GetString("UpdatingProject_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Using &apos;{0}&apos; for metadata..
        /// </summary>
        internal static string UsingNuspecForMetadata {
            get {
                return ResourceManager.GetString("UsingNuspecForMetadata", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 正在对元数据使用“{0}”。.
        /// </summary>
        internal static string UsingNuspecForMetadata_chs {
            get {
                return ResourceManager.GetString("UsingNuspecForMetadata_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 正在使用中繼資料的 &apos;{0}&apos;。.
        /// </summary>
        internal static string UsingNuspecForMetadata_cht {
            get {
                return ResourceManager.GetString("UsingNuspecForMetadata_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} se používá pro metadata..
        /// </summary>
        internal static string UsingNuspecForMetadata_csy {
            get {
                return ResourceManager.GetString("UsingNuspecForMetadata_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to &quot;{0}&quot; wird für Metadaten verwendet..
        /// </summary>
        internal static string UsingNuspecForMetadata_deu {
            get {
                return ResourceManager.GetString("UsingNuspecForMetadata_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Usando &apos;{0}&apos; para metadatos..
        /// </summary>
        internal static string UsingNuspecForMetadata_esp {
            get {
                return ResourceManager.GetString("UsingNuspecForMetadata_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Utilisation de &apos;{0}&apos; pour les métadonnées..
        /// </summary>
        internal static string UsingNuspecForMetadata_fra {
            get {
                return ResourceManager.GetString("UsingNuspecForMetadata_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Usando &apos;{0}&apos; per metadati..
        /// </summary>
        internal static string UsingNuspecForMetadata_ita {
            get {
                return ResourceManager.GetString("UsingNuspecForMetadata_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to メタデータに &apos;{0}&apos; を使用しています。.
        /// </summary>
        internal static string UsingNuspecForMetadata_jpn {
            get {
                return ResourceManager.GetString("UsingNuspecForMetadata_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 메타데이터에 대한 &apos;{0}&apos;을(를) 사용하고 있습니다..
        /// </summary>
        internal static string UsingNuspecForMetadata_kor {
            get {
                return ResourceManager.GetString("UsingNuspecForMetadata_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Używanie „{0}” na potrzeby metadanych..
        /// </summary>
        internal static string UsingNuspecForMetadata_plk {
            get {
                return ResourceManager.GetString("UsingNuspecForMetadata_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Usando &apos;{0}&apos; para metadados..
        /// </summary>
        internal static string UsingNuspecForMetadata_ptb {
            get {
                return ResourceManager.GetString("UsingNuspecForMetadata_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Использование &quot;{0}&quot; для метаданных..
        /// </summary>
        internal static string UsingNuspecForMetadata_rus {
            get {
                return ResourceManager.GetString("UsingNuspecForMetadata_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Meta veriler için &apos;{0}&apos; kullanılıyor..
        /// </summary>
        internal static string UsingNuspecForMetadata_trk {
            get {
                return ResourceManager.GetString("UsingNuspecForMetadata_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Found packages.config. Using packages listed as dependencies.
        /// </summary>
        internal static string UsingPackagesConfigForDependencies {
            get {
                return ResourceManager.GetString("UsingPackagesConfigForDependencies", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 找到 packages.config。正在使用列出的程序包作为依赖关系.
        /// </summary>
        internal static string UsingPackagesConfigForDependencies_chs {
            get {
                return ResourceManager.GetString("UsingPackagesConfigForDependencies_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 找到 packages.config。使用列出的封裝做為相依性。.
        /// </summary>
        internal static string UsingPackagesConfigForDependencies_cht {
            get {
                return ResourceManager.GetString("UsingPackagesConfigForDependencies_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Byl nalezen soubor packages.config. Používají se balíčky uvedené jako závislosti..
        /// </summary>
        internal static string UsingPackagesConfigForDependencies_csy {
            get {
                return ResourceManager.GetString("UsingPackagesConfigForDependencies_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to &quot;packages.config&quot; wurde gefunden. Die aufgelisteten Pakete werden als Abhängigkeiten verwendet..
        /// </summary>
        internal static string UsingPackagesConfigForDependencies_deu {
            get {
                return ResourceManager.GetString("UsingPackagesConfigForDependencies_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Se ha encontrado packages.config. Usando paquetes mostrados como dependencias.
        /// </summary>
        internal static string UsingPackagesConfigForDependencies_esp {
            get {
                return ResourceManager.GetString("UsingPackagesConfigForDependencies_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Packages.config trouvé. Utilisation de packages répertoriés comme dépendances.
        /// </summary>
        internal static string UsingPackagesConfigForDependencies_fra {
            get {
                return ResourceManager.GetString("UsingPackagesConfigForDependencies_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Trovato packages.config. Usando pacchetti elencati come dipendenze..
        /// </summary>
        internal static string UsingPackagesConfigForDependencies_ita {
            get {
                return ResourceManager.GetString("UsingPackagesConfigForDependencies_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to packages.config が見つかりました。依存関係として登録されているパッケージを使用します.
        /// </summary>
        internal static string UsingPackagesConfigForDependencies_jpn {
            get {
                return ResourceManager.GetString("UsingPackagesConfigForDependencies_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to packages.config를 찾았습니다. 나열된 패키지를 종속성으로 사용하고 있습니다..
        /// </summary>
        internal static string UsingPackagesConfigForDependencies_kor {
            get {
                return ResourceManager.GetString("UsingPackagesConfigForDependencies_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Znaleziono plik packages.config. Wymienione pakiety zostaną użyte jako zależności.
        /// </summary>
        internal static string UsingPackagesConfigForDependencies_plk {
            get {
                return ResourceManager.GetString("UsingPackagesConfigForDependencies_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Encontrado packages.config. Usando pacotes listados como dependências.
        /// </summary>
        internal static string UsingPackagesConfigForDependencies_ptb {
            get {
                return ResourceManager.GetString("UsingPackagesConfigForDependencies_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Найден файл packages.config. Перечисленные пакеты будут использованы как зависимости.
        /// </summary>
        internal static string UsingPackagesConfigForDependencies_rus {
            get {
                return ResourceManager.GetString("UsingPackagesConfigForDependencies_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Packages.config bulundu. Paketler bağımlılıklarda listelenen şekilde kullanılıyor.
        /// </summary>
        internal static string UsingPackagesConfigForDependencies_trk {
            get {
                return ResourceManager.GetString("UsingPackagesConfigForDependencies_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to The value &quot;{0}&quot; for {1} is a sample value and should be removed..
        /// </summary>
        internal static string Warning_DefaultSpecValue {
            get {
                return ResourceManager.GetString("Warning_DefaultSpecValue", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {1} 的值“{0}”是示例值，应将其删除。.
        /// </summary>
        internal static string Warning_DefaultSpecValue_chs {
            get {
                return ResourceManager.GetString("Warning_DefaultSpecValue_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {1} 的值 &quot;{0}&quot; 為範例值且應該移除。.
        /// </summary>
        internal static string Warning_DefaultSpecValue_cht {
            get {
                return ResourceManager.GetString("Warning_DefaultSpecValue_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Hodnota {0} pro {1} je ukázková hodnota a měla by být odebrána..
        /// </summary>
        internal static string Warning_DefaultSpecValue_csy {
            get {
                return ResourceManager.GetString("Warning_DefaultSpecValue_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Der Wert &quot;{0}&quot; für {1} ist ein Beispielwert und sollte entfernt werden..
        /// </summary>
        internal static string Warning_DefaultSpecValue_deu {
            get {
                return ResourceManager.GetString("Warning_DefaultSpecValue_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to El valor &quot;{0}&quot; para {1} es un valor de ejemplo y se debería quitar..
        /// </summary>
        internal static string Warning_DefaultSpecValue_esp {
            get {
                return ResourceManager.GetString("Warning_DefaultSpecValue_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to La valeur « {0} » pour {1} est un exemple et doit être supprimée..
        /// </summary>
        internal static string Warning_DefaultSpecValue_fra {
            get {
                return ResourceManager.GetString("Warning_DefaultSpecValue_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Il valore &quot;{0}&quot; per {1} è un valore campione e debe essere rimosso..
        /// </summary>
        internal static string Warning_DefaultSpecValue_ita {
            get {
                return ResourceManager.GetString("Warning_DefaultSpecValue_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {1} の値 &quot;{0}&quot; はサンプル値なので、削除する必要があります。.
        /// </summary>
        internal static string Warning_DefaultSpecValue_jpn {
            get {
                return ResourceManager.GetString("Warning_DefaultSpecValue_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {1}에 대한 &quot;{0}&quot; 값은 샘플 값이므로 제거해야 합니다..
        /// </summary>
        internal static string Warning_DefaultSpecValue_kor {
            get {
                return ResourceManager.GetString("Warning_DefaultSpecValue_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Wartość „{0}” dla {1} jest wartością przykładową i należy ją usunąć..
        /// </summary>
        internal static string Warning_DefaultSpecValue_plk {
            get {
                return ResourceManager.GetString("Warning_DefaultSpecValue_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to O valor &quot;{0}&quot; para {1} é um valor de amostra e deve ser removido..
        /// </summary>
        internal static string Warning_DefaultSpecValue_ptb {
            get {
                return ResourceManager.GetString("Warning_DefaultSpecValue_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Значение &quot;{0}&quot; для {1} является образцом значения, и его не следует удалять..
        /// </summary>
        internal static string Warning_DefaultSpecValue_rus {
            get {
                return ResourceManager.GetString("Warning_DefaultSpecValue_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {1} için &quot;{0}&quot; örnek bir değerdir ve kaldırılması gerekir..
        /// </summary>
        internal static string Warning_DefaultSpecValue_trk {
            get {
                return ResourceManager.GetString("Warning_DefaultSpecValue_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Replace with an appropriate value or remove and it and rebuild your package..
        /// </summary>
        internal static string Warning_DefaultSpecValueSolution {
            get {
                return ResourceManager.GetString("Warning_DefaultSpecValueSolution", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 请替换为适当的值或删除它，然后重新生成程序包。.
        /// </summary>
        internal static string Warning_DefaultSpecValueSolution_chs {
            get {
                return ResourceManager.GetString("Warning_DefaultSpecValueSolution_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 以適當的值取代或移除並重新建置封裝。.
        /// </summary>
        internal static string Warning_DefaultSpecValueSolution_cht {
            get {
                return ResourceManager.GetString("Warning_DefaultSpecValueSolution_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Nahraďte ji odpovídající hodnotou nebo ji odeberte a pak znovu sestavte balíček..
        /// </summary>
        internal static string Warning_DefaultSpecValueSolution_csy {
            get {
                return ResourceManager.GetString("Warning_DefaultSpecValueSolution_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Ersetzen Sie ihn durch einen geeigneten Wert, oder entfernen Sie ihn, und erstellen Sie das Paket dann erneut..
        /// </summary>
        internal static string Warning_DefaultSpecValueSolution_deu {
            get {
                return ResourceManager.GetString("Warning_DefaultSpecValueSolution_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Reemplazar con un valor apropiado o quitar y compilar el paquete..
        /// </summary>
        internal static string Warning_DefaultSpecValueSolution_esp {
            get {
                return ResourceManager.GetString("Warning_DefaultSpecValueSolution_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Remplacez-la par une valeur adéquate, ou supprimez-la et recréez le package..
        /// </summary>
        internal static string Warning_DefaultSpecValueSolution_fra {
            get {
                return ResourceManager.GetString("Warning_DefaultSpecValueSolution_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Sostituire con un valore appropriato o rimuoverlo e ricreare il pacchetto..
        /// </summary>
        internal static string Warning_DefaultSpecValueSolution_ita {
            get {
                return ResourceManager.GetString("Warning_DefaultSpecValueSolution_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 適切な値で置き換えるか、削除して、パッケージを再度ビルドする必要があります。.
        /// </summary>
        internal static string Warning_DefaultSpecValueSolution_jpn {
            get {
                return ResourceManager.GetString("Warning_DefaultSpecValueSolution_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 적합한 값으로 바꾸거나 제거하고 패키지를 다시 빌드하십시오..
        /// </summary>
        internal static string Warning_DefaultSpecValueSolution_kor {
            get {
                return ResourceManager.GetString("Warning_DefaultSpecValueSolution_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Zastąp odpowiednią wartością lub usuń i ponownie skompiluj pakiet..
        /// </summary>
        internal static string Warning_DefaultSpecValueSolution_plk {
            get {
                return ResourceManager.GetString("Warning_DefaultSpecValueSolution_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Substitua por um valor apropriado ou remova-o e reconstrua o seu pacote..
        /// </summary>
        internal static string Warning_DefaultSpecValueSolution_ptb {
            get {
                return ResourceManager.GetString("Warning_DefaultSpecValueSolution_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Замените его соответствующим значением или удалите и повторно выполните сборку пакета..
        /// </summary>
        internal static string Warning_DefaultSpecValueSolution_rus {
            get {
                return ResourceManager.GetString("Warning_DefaultSpecValueSolution_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Uygun bir değerle değiştirin veya kaldırın ve paketinizi yeniden oluşturun..
        /// </summary>
        internal static string Warning_DefaultSpecValueSolution_trk {
            get {
                return ResourceManager.GetString("Warning_DefaultSpecValueSolution_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Remove sample nuspec values..
        /// </summary>
        internal static string Warning_DefaultSpecValueTitle {
            get {
                return ResourceManager.GetString("Warning_DefaultSpecValueTitle", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 删除示例 nuspec 值。.
        /// </summary>
        internal static string Warning_DefaultSpecValueTitle_chs {
            get {
                return ResourceManager.GetString("Warning_DefaultSpecValueTitle_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 移除範例 nuspec 值。.
        /// </summary>
        internal static string Warning_DefaultSpecValueTitle_cht {
            get {
                return ResourceManager.GetString("Warning_DefaultSpecValueTitle_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Odeberte ukázkové hodnoty nuspec..
        /// </summary>
        internal static string Warning_DefaultSpecValueTitle_csy {
            get {
                return ResourceManager.GetString("Warning_DefaultSpecValueTitle_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to nuspec-Beispielwerte entfernen..
        /// </summary>
        internal static string Warning_DefaultSpecValueTitle_deu {
            get {
                return ResourceManager.GetString("Warning_DefaultSpecValueTitle_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Quitar los valores nuspec de ejemplo..
        /// </summary>
        internal static string Warning_DefaultSpecValueTitle_esp {
            get {
                return ResourceManager.GetString("Warning_DefaultSpecValueTitle_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Supprimez les valeurs nuspec d&apos;exemple..
        /// </summary>
        internal static string Warning_DefaultSpecValueTitle_fra {
            get {
                return ResourceManager.GetString("Warning_DefaultSpecValueTitle_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Rimuovere valori campione nuspec..
        /// </summary>
        internal static string Warning_DefaultSpecValueTitle_ita {
            get {
                return ResourceManager.GetString("Warning_DefaultSpecValueTitle_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to サンプル nuspec 値を削除してください。.
        /// </summary>
        internal static string Warning_DefaultSpecValueTitle_jpn {
            get {
                return ResourceManager.GetString("Warning_DefaultSpecValueTitle_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 샘플 nuspec 값을 제거합니다..
        /// </summary>
        internal static string Warning_DefaultSpecValueTitle_kor {
            get {
                return ResourceManager.GetString("Warning_DefaultSpecValueTitle_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Usuń przykładowe wartości nuspec..
        /// </summary>
        internal static string Warning_DefaultSpecValueTitle_plk {
            get {
                return ResourceManager.GetString("Warning_DefaultSpecValueTitle_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Remova os valores nuspec de amostra..
        /// </summary>
        internal static string Warning_DefaultSpecValueTitle_ptb {
            get {
                return ResourceManager.GetString("Warning_DefaultSpecValueTitle_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Удалите образцы значений nuspec..
        /// </summary>
        internal static string Warning_DefaultSpecValueTitle_rus {
            get {
                return ResourceManager.GetString("Warning_DefaultSpecValueTitle_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Örnek nuspec değerlerini kaldırın..
        /// </summary>
        internal static string Warning_DefaultSpecValueTitle_trk {
            get {
                return ResourceManager.GetString("Warning_DefaultSpecValueTitle_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to &apos;{0}&apos; key already exists in Properties collection. Overriding value..
        /// </summary>
        internal static string Warning_DuplicatePropertyKey {
            get {
                return ResourceManager.GetString("Warning_DuplicatePropertyKey", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} 键已存在于属性集合中。正在重写值。.
        /// </summary>
        internal static string Warning_DuplicatePropertyKey_chs {
            get {
                return ResourceManager.GetString("Warning_DuplicatePropertyKey_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0}&apos; 索引鍵已存在於屬性集合中。覆寫該值。.
        /// </summary>
        internal static string Warning_DuplicatePropertyKey_cht {
            get {
                return ResourceManager.GetString("Warning_DuplicatePropertyKey_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Klíč {0} již existuje v kolekci Properties. Přepisuje se hodnota..
        /// </summary>
        internal static string Warning_DuplicatePropertyKey_csy {
            get {
                return ResourceManager.GetString("Warning_DuplicatePropertyKey_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Der Schlüssel &quot;{0}&quot; ist in der Eigenschaftenauflistung bereits vorhanden. Der Wert wird außer Kraft gesetzt..
        /// </summary>
        internal static string Warning_DuplicatePropertyKey_deu {
            get {
                return ResourceManager.GetString("Warning_DuplicatePropertyKey_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to La clave &apos;{0}&apos; ya existe en la colección Propiedades. Reemplazando valor..
        /// </summary>
        internal static string Warning_DuplicatePropertyKey_esp {
            get {
                return ResourceManager.GetString("Warning_DuplicatePropertyKey_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to La clé &apos;{0}&apos; existe déjà dans le Regroupement de propriétés. Remplacement de la valeur..
        /// </summary>
        internal static string Warning_DuplicatePropertyKey_fra {
            get {
                return ResourceManager.GetString("Warning_DuplicatePropertyKey_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0}&apos; key è già presente in Properties collection. Valore prevalente..
        /// </summary>
        internal static string Warning_DuplicatePropertyKey_ita {
            get {
                return ResourceManager.GetString("Warning_DuplicatePropertyKey_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to &apos;{0}&apos; キーは Properties コレクションに既に存在します。値を上書きします。.
        /// </summary>
        internal static string Warning_DuplicatePropertyKey_jpn {
            get {
                return ResourceManager.GetString("Warning_DuplicatePropertyKey_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to &apos;{0}&apos; 키가 Properties 컬렉션에 이미 있습니다. 값을 재정의하고 있습니다..
        /// </summary>
        internal static string Warning_DuplicatePropertyKey_kor {
            get {
                return ResourceManager.GetString("Warning_DuplicatePropertyKey_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Klucz „{0}” już istnieje w kolekcji Właściwości. Wartość zostanie przesłonięta..
        /// </summary>
        internal static string Warning_DuplicatePropertyKey_plk {
            get {
                return ResourceManager.GetString("Warning_DuplicatePropertyKey_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to A chave &apos;{0}&apos; já existe na coleção Propriedades. Substituindo valor..
        /// </summary>
        internal static string Warning_DuplicatePropertyKey_ptb {
            get {
                return ResourceManager.GetString("Warning_DuplicatePropertyKey_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Ключ &quot;{0}&quot; уже существует в коллекции свойств. Значение будет переопределено..
        /// </summary>
        internal static string Warning_DuplicatePropertyKey_rus {
            get {
                return ResourceManager.GetString("Warning_DuplicatePropertyKey_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0}&apos; anahtarı zaten Özellikler koleksiyonunda mevcut. Değer geçersiz kılınıyor..
        /// </summary>
        internal static string Warning_DuplicatePropertyKey_trk {
            get {
                return ResourceManager.GetString("Warning_DuplicatePropertyKey_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to &apos;{0}&apos; was included in the project but doesn&apos;t exist. Skipping....
        /// </summary>
        internal static string Warning_FileDoesNotExist {
            get {
                return ResourceManager.GetString("Warning_FileDoesNotExist", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to “{0}”已包含在项目中，但不存在。正在跳过....
        /// </summary>
        internal static string Warning_FileDoesNotExist_chs {
            get {
                return ResourceManager.GetString("Warning_FileDoesNotExist_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0}&apos; 包含在專案中但並不存在。正在略過....
        /// </summary>
        internal static string Warning_FileDoesNotExist_cht {
            get {
                return ResourceManager.GetString("Warning_FileDoesNotExist_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Soubor {0} byl zahrnut v projektu, ale neexistuje. Dojde k přeskočení....
        /// </summary>
        internal static string Warning_FileDoesNotExist_csy {
            get {
                return ResourceManager.GetString("Warning_FileDoesNotExist_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to &quot;{0}&quot; war im Projekt enthalten, ist jedoch nicht vorhanden. Das Element wird übersprungen....
        /// </summary>
        internal static string Warning_FileDoesNotExist_deu {
            get {
                return ResourceManager.GetString("Warning_FileDoesNotExist_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0}&apos; se ha incluido en el proyecto pero no existe. Omitiendo….
        /// </summary>
        internal static string Warning_FileDoesNotExist_esp {
            get {
                return ResourceManager.GetString("Warning_FileDoesNotExist_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to &apos;{0}&apos; a été inclus au projet, mais n&apos;existe pas. Ignorer….
        /// </summary>
        internal static string Warning_FileDoesNotExist_fra {
            get {
                return ResourceManager.GetString("Warning_FileDoesNotExist_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0}&apos; è incluso nel progetto ma non esiste. Saltare….
        /// </summary>
        internal static string Warning_FileDoesNotExist_ita {
            get {
                return ResourceManager.GetString("Warning_FileDoesNotExist_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to &apos;{0}&apos; がプロジェクトに含まれていますが、存在しません。スキップしています... Skipping....
        /// </summary>
        internal static string Warning_FileDoesNotExist_jpn {
            get {
                return ResourceManager.GetString("Warning_FileDoesNotExist_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to &apos;{0}&apos;이(가) 프로젝트에 포함되어 있지만 존재하지 않습니다. 건너뛰는 중....
        /// </summary>
        internal static string Warning_FileDoesNotExist_kor {
            get {
                return ResourceManager.GetString("Warning_FileDoesNotExist_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Plik „{0}” został uwzględniony w projekcie, ale nie istnieje. Trwa pomijanie....
        /// </summary>
        internal static string Warning_FileDoesNotExist_plk {
            get {
                return ResourceManager.GetString("Warning_FileDoesNotExist_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0}&apos; foi incluído no projeto, mas não existe. Ignorando....
        /// </summary>
        internal static string Warning_FileDoesNotExist_ptb {
            get {
                return ResourceManager.GetString("Warning_FileDoesNotExist_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to &quot;{0}&quot; был добавлен в проект, но не существует. Пропуск....
        /// </summary>
        internal static string Warning_FileDoesNotExist_rus {
            get {
                return ResourceManager.GetString("Warning_FileDoesNotExist_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0}&apos; projeye dahil edildi ancak mevcut değil. Atlanıyor....
        /// </summary>
        internal static string Warning_FileDoesNotExist_trk {
            get {
                return ResourceManager.GetString("Warning_FileDoesNotExist_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Invalid PackageSaveMode value &apos;{0}&apos;..
        /// </summary>
        internal static string Warning_InvalidPackageSaveMode {
            get {
                return ResourceManager.GetString("Warning_InvalidPackageSaveMode", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Option &apos;NoPrompt&apos; has been deprecated. Use &apos;NonInteractive&apos; instead..
        /// </summary>
        internal static string Warning_NoPromptDeprecated {
            get {
                return ResourceManager.GetString("Warning_NoPromptDeprecated", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 选项 &quot;NoPrompt&quot; 已弃用。请改用 &quot;NonInteractive&quot;。.
        /// </summary>
        internal static string Warning_NoPromptDeprecated_chs {
            get {
                return ResourceManager.GetString("Warning_NoPromptDeprecated_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 選項 &apos;NoPrompt&apos; 已過時。使用 &apos;NonInteractive&apos;。.
        /// </summary>
        internal static string Warning_NoPromptDeprecated_cht {
            get {
                return ResourceManager.GetString("Warning_NoPromptDeprecated_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Možnost NoPrompt se již nepoužívá. Použijte místo toho možnost NonInteractive..
        /// </summary>
        internal static string Warning_NoPromptDeprecated_csy {
            get {
                return ResourceManager.GetString("Warning_NoPromptDeprecated_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Die Option &quot;NoPrompt&quot; ist veraltet. Verwenden Sie stattdessen &quot;NonInteractive&quot;..
        /// </summary>
        internal static string Warning_NoPromptDeprecated_deu {
            get {
                return ResourceManager.GetString("Warning_NoPromptDeprecated_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to La opción &apos;NoPrompt&apos; se ha desusado. Use &apos;NonInteractive&apos; en su lugar..
        /// </summary>
        internal static string Warning_NoPromptDeprecated_esp {
            get {
                return ResourceManager.GetString("Warning_NoPromptDeprecated_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to L&apos;option &apos;NoPrompt&apos; a été dépréciée. Préférez plutôt &apos;NonInteractive&apos;..
        /// </summary>
        internal static string Warning_NoPromptDeprecated_fra {
            get {
                return ResourceManager.GetString("Warning_NoPromptDeprecated_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Opzione &apos;NoPrompt&apos; disapprovata. Usare &apos;NonInteractive&apos;..
        /// </summary>
        internal static string Warning_NoPromptDeprecated_ita {
            get {
                return ResourceManager.GetString("Warning_NoPromptDeprecated_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to オプション &apos;NoPrompt&apos; は使用しないでください。代わりに &apos;NonInteractive&apos; を使用してください。.
        /// </summary>
        internal static string Warning_NoPromptDeprecated_jpn {
            get {
                return ResourceManager.GetString("Warning_NoPromptDeprecated_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to &apos;NoPrompt&apos; 옵션은 사용되지 않습니다. 대신 &apos;NonInteractive&apos;를 사용하십시오..
        /// </summary>
        internal static string Warning_NoPromptDeprecated_kor {
            get {
                return ResourceManager.GetString("Warning_NoPromptDeprecated_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Opcja „NoPrompt” jest przestarzała. Zamiast niej użyj opcji „NonInteractive”..
        /// </summary>
        internal static string Warning_NoPromptDeprecated_plk {
            get {
                return ResourceManager.GetString("Warning_NoPromptDeprecated_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to A opção &apos;NoPrompt&apos; tornou-se obsoleta. Use &quot;Não interativo&quot; como alternativa..
        /// </summary>
        internal static string Warning_NoPromptDeprecated_ptb {
            get {
                return ResourceManager.GetString("Warning_NoPromptDeprecated_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Параметр &quot;NoPrompt&quot; удален из этой версии. Вместо него используйте &quot;NonInteractive&quot;..
        /// </summary>
        internal static string Warning_NoPromptDeprecated_rus {
            get {
                return ResourceManager.GetString("Warning_NoPromptDeprecated_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to NoPrompt&apos; seçeneği kullanımdan kaldırılmış. Bunun yerine &apos;NonInteractive&apos; seçeneğini kullanın..
        /// </summary>
        internal static string Warning_NoPromptDeprecated_trk {
            get {
                return ResourceManager.GetString("Warning_NoPromptDeprecated_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Version &quot;{0}&quot; does not follow semantic versioning guidelines..
        /// </summary>
        internal static string Warning_SemanticVersion {
            get {
                return ResourceManager.GetString("Warning_SemanticVersion", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 版本“{0}”未遵循语义版本控制准则。.
        /// </summary>
        internal static string Warning_SemanticVersion_chs {
            get {
                return ResourceManager.GetString("Warning_SemanticVersion_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 版本 &quot;{0}&quot; 不允許語意版本設定方針。.
        /// </summary>
        internal static string Warning_SemanticVersion_cht {
            get {
                return ResourceManager.GetString("Warning_SemanticVersion_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Verze {0} nedodržuje pokyny pro správu sémantických verzí..
        /// </summary>
        internal static string Warning_SemanticVersion_csy {
            get {
                return ResourceManager.GetString("Warning_SemanticVersion_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Die Version &quot;{0}&quot; genügt nicht den Richtlinien für semantische Versionsverwaltung..
        /// </summary>
        internal static string Warning_SemanticVersion_deu {
            get {
                return ResourceManager.GetString("Warning_SemanticVersion_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to La versión &quot;{0}&quot;no sigue las instrucciones de control de versiones semánticas..
        /// </summary>
        internal static string Warning_SemanticVersion_esp {
            get {
                return ResourceManager.GetString("Warning_SemanticVersion_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to La version « {0} » ne suit pas les instructions de contrôle de version sémantique..
        /// </summary>
        internal static string Warning_SemanticVersion_fra {
            get {
                return ResourceManager.GetString("Warning_SemanticVersion_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to La versione &quot;{0}&quot; non segue le linee guida di edizione semantica..
        /// </summary>
        internal static string Warning_SemanticVersion_ita {
            get {
                return ResourceManager.GetString("Warning_SemanticVersion_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to バージョン &quot;{0}&quot; は、セマンティック バージョンのガイドラインに従っていません。.
        /// </summary>
        internal static string Warning_SemanticVersion_jpn {
            get {
                return ResourceManager.GetString("Warning_SemanticVersion_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} 버전이 의미 체계의 버전 지정 지침을 따르지 않았습니다..
        /// </summary>
        internal static string Warning_SemanticVersion_kor {
            get {
                return ResourceManager.GetString("Warning_SemanticVersion_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Wersja „{0}” nie jest zgodna ze wskazówkami dotyczącymi semantycznego przechowywania wersji..
        /// </summary>
        internal static string Warning_SemanticVersion_plk {
            get {
                return ResourceManager.GetString("Warning_SemanticVersion_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to A versão &quot;{0}&quot; não segue as orientações de versionamento semântico..
        /// </summary>
        internal static string Warning_SemanticVersion_ptb {
            get {
                return ResourceManager.GetString("Warning_SemanticVersion_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Версия &quot;{0}&quot; не соответствует правилам управления семантическими версиями..
        /// </summary>
        internal static string Warning_SemanticVersion_rus {
            get {
                return ResourceManager.GetString("Warning_SemanticVersion_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to &quot;{0}&quot; sürümü anlamsal sürüm oluşturma kurallarına uymuyor..
        /// </summary>
        internal static string Warning_SemanticVersion_trk {
            get {
                return ResourceManager.GetString("Warning_SemanticVersion_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Update your nuspec file or use the AssemblyInformationalVersion assembly attribute to specify a semantic version as described at http://semver.org. .
        /// </summary>
        internal static string Warning_SemanticVersionSolution {
            get {
                return ResourceManager.GetString("Warning_SemanticVersionSolution", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 更新 nuspec 文件，或使用 AssemblyInformationalVersion 程序集属性指定 http://semver.org 上所述的语义版本。.
        /// </summary>
        internal static string Warning_SemanticVersionSolution_chs {
            get {
                return ResourceManager.GetString("Warning_SemanticVersionSolution_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 更新您的 nuspec 檔案或使用 AssemblyInformationalVersion 組件屬性以指定語意版本設定，如t http://semver.org 中所述。 .
        /// </summary>
        internal static string Warning_SemanticVersionSolution_cht {
            get {
                return ResourceManager.GetString("Warning_SemanticVersionSolution_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Proveďte aktualizaci souboru nuspec nebo použijte atribut sestavení AssemblyInformationalVersion a zadejte sémantickou verzi podle popisu na webu http://semver.org. .
        /// </summary>
        internal static string Warning_SemanticVersionSolution_csy {
            get {
                return ResourceManager.GetString("Warning_SemanticVersionSolution_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Aktualisieren Sie die nuspec-Datei, oder verwenden Sie das Assemblyattribut &quot;AssemblyInformationalVersion&quot;, um eine semantische Version wie unter &quot;http://semver.org&quot; beschrieben anzugeben. .
        /// </summary>
        internal static string Warning_SemanticVersionSolution_deu {
            get {
                return ResourceManager.GetString("Warning_SemanticVersionSolution_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Actualice su archivo nuspec o use el atributo de ensamblado AssemblyInformationalVersion para especificar una versión semántica como se describe en http://semver.org..
        /// </summary>
        internal static string Warning_SemanticVersionSolution_esp {
            get {
                return ResourceManager.GetString("Warning_SemanticVersionSolution_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Mettez à jour le fichier .nuspec, ou utilisez l&apos;attribut d&apos;assembly AssemblyInformationalVersion, pour spécifier la version sémantique comme décrit sur http://semver.org. .
        /// </summary>
        internal static string Warning_SemanticVersionSolution_fra {
            get {
                return ResourceManager.GetString("Warning_SemanticVersionSolution_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Aggiornare il file nuspec o usare l&apos;attributo AssemblyInformationalVersion  per specificare la versione semantica come descritto in http://semver.org. .
        /// </summary>
        internal static string Warning_SemanticVersionSolution_ita {
            get {
                return ResourceManager.GetString("Warning_SemanticVersionSolution_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to nuspec ファイルを更新するか、AssemblyInformationalVersion アセンブリ属性を使用して、http://semver.org の説明に従ってセマンティック バージョンを指定してください。.
        /// </summary>
        internal static string Warning_SemanticVersionSolution_jpn {
            get {
                return ResourceManager.GetString("Warning_SemanticVersionSolution_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to nuspec 파일을 업데이트하거나 AssemblyInformationalVersion 어셈블리 특성을 사용하여 http://semver.org에서 설명하는 것과 같이 의미 체계의 버전을 지정하십시오. .
        /// </summary>
        internal static string Warning_SemanticVersionSolution_kor {
            get {
                return ResourceManager.GetString("Warning_SemanticVersionSolution_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Zaktualizuj plik nuspec lub użyj atrybutu zestawu AssemblyInformationalVersion, aby określić wersję semantyczną zgodnie z opisem na stronie http://semver.org. .
        /// </summary>
        internal static string Warning_SemanticVersionSolution_plk {
            get {
                return ResourceManager.GetString("Warning_SemanticVersionSolution_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Atualize seu arquivo nuspec ou usar o atributo de assembly AssemblyInformationalVersion para especificar uma versão semântica, como descrito em http://semver.org..
        /// </summary>
        internal static string Warning_SemanticVersionSolution_ptb {
            get {
                return ResourceManager.GetString("Warning_SemanticVersionSolution_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Обновите ваш NUSPEC-файл или с помощью атрибута AssemblyInformationalVersion укажите семантическую версию, как описано по адресу: http://semver.org. .
        /// </summary>
        internal static string Warning_SemanticVersionSolution_rus {
            get {
                return ResourceManager.GetString("Warning_SemanticVersionSolution_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Nuspec dosyanızı güncelleyin veya http://semver.org sayfasında açıklanan şekilde bir anlamsal sürüm belirlemek için AssemblyInformationalVersion derleme özniteliğini kullanın . .
        /// </summary>
        internal static string Warning_SemanticVersionSolution_trk {
            get {
                return ResourceManager.GetString("Warning_SemanticVersionSolution_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Use semantic versioning.
        /// </summary>
        internal static string Warning_SemanticVersionTitle {
            get {
                return ResourceManager.GetString("Warning_SemanticVersionTitle", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 使用语义版本控制.
        /// </summary>
        internal static string Warning_SemanticVersionTitle_chs {
            get {
                return ResourceManager.GetString("Warning_SemanticVersionTitle_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 使用語意版本設定。.
        /// </summary>
        internal static string Warning_SemanticVersionTitle_cht {
            get {
                return ResourceManager.GetString("Warning_SemanticVersionTitle_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Použít správu sémantických verzí.
        /// </summary>
        internal static string Warning_SemanticVersionTitle_csy {
            get {
                return ResourceManager.GetString("Warning_SemanticVersionTitle_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Semantische Versionsverwaltung verwenden.
        /// </summary>
        internal static string Warning_SemanticVersionTitle_deu {
            get {
                return ResourceManager.GetString("Warning_SemanticVersionTitle_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Use el control de versiones semántico.
        /// </summary>
        internal static string Warning_SemanticVersionTitle_esp {
            get {
                return ResourceManager.GetString("Warning_SemanticVersionTitle_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Utilisation du contrôle de version sémantique.
        /// </summary>
        internal static string Warning_SemanticVersionTitle_fra {
            get {
                return ResourceManager.GetString("Warning_SemanticVersionTitle_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Usare edizione semantica.
        /// </summary>
        internal static string Warning_SemanticVersionTitle_ita {
            get {
                return ResourceManager.GetString("Warning_SemanticVersionTitle_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to セマンティック バージョンの使用.
        /// </summary>
        internal static string Warning_SemanticVersionTitle_jpn {
            get {
                return ResourceManager.GetString("Warning_SemanticVersionTitle_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 의미 체계의 버전 지정 사용.
        /// </summary>
        internal static string Warning_SemanticVersionTitle_kor {
            get {
                return ResourceManager.GetString("Warning_SemanticVersionTitle_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Zastosuj semantyczne przechowywanie wersji.
        /// </summary>
        internal static string Warning_SemanticVersionTitle_plk {
            get {
                return ResourceManager.GetString("Warning_SemanticVersionTitle_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Usar versões semânticas.
        /// </summary>
        internal static string Warning_SemanticVersionTitle_ptb {
            get {
                return ResourceManager.GetString("Warning_SemanticVersionTitle_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Использования управления семантическими версиями.
        /// </summary>
        internal static string Warning_SemanticVersionTitle_rus {
            get {
                return ResourceManager.GetString("Warning_SemanticVersionTitle_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Anlamsal sürüm oluşturmayı kullanın..
        /// </summary>
        internal static string Warning_SemanticVersionTitle_trk {
            get {
                return ResourceManager.GetString("Warning_SemanticVersionTitle_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Found symbols package &apos;{0}&apos;, but no API key was specified for the symbol server. To save an API Key, run &apos;NuGet.exe setApiKey [your API key from http://www.NuGet.org]&apos;..
        /// </summary>
        internal static string Warning_SymbolServerNotConfigured {
            get {
                return ResourceManager.GetString("Warning_SymbolServerNotConfigured", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 找到符号程序包“{0}”，但未为符号服务器指定 API 密钥。若要保存 API 密钥，请运行“NuGet.exe setApiKey [来自 http://www.NuGet.org 的 API 密钥]”。.
        /// </summary>
        internal static string Warning_SymbolServerNotConfigured_chs {
            get {
                return ResourceManager.GetString("Warning_SymbolServerNotConfigured_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 找到符號封裝 &apos;{0}&apos;，但並未為符號伺服器指定 API 索引鍵。若要儲存 API 索引鍵，請執行 &apos;NuGet.exe setApiKey [http://www.NuGet.org 中的 API 索引鍵]&apos;。.
        /// </summary>
        internal static string Warning_SymbolServerNotConfigured_cht {
            get {
                return ResourceManager.GetString("Warning_SymbolServerNotConfigured_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Byl nalezen balíček symbolů {0}, ale pro server symbolů nebyl zadán žádný klíč API. Chcete-li uložit klíč API, spusťte příkaz NuGet.exe setApiKey [klíč API z webu http://www.NuGet.org]..
        /// </summary>
        internal static string Warning_SymbolServerNotConfigured_csy {
            get {
                return ResourceManager.GetString("Warning_SymbolServerNotConfigured_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Das Symbolpaket &quot;{0}&quot; wurde gefunden, es wurde jedoch kein API-Schlüssel für den Symbolserver angegeben. Führen Sie zum Speichern eines API-Schlüssels &quot;NuGet.exe setApiKey [Ihr API-Schlüssel von http://www.NuGet.org]&quot; aus..
        /// </summary>
        internal static string Warning_SymbolServerNotConfigured_deu {
            get {
                return ResourceManager.GetString("Warning_SymbolServerNotConfigured_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Se ha encontrado el paquete de símbolos &apos;{0}&apos;, pero no se ha especificado ninguna clave API para el servidor de símbolos. Para guardar una clave API, ejecute &apos;NuGet.exe setApiKey [su clave API de http://www.NuGet.org]&apos;..
        /// </summary>
        internal static string Warning_SymbolServerNotConfigured_esp {
            get {
                return ResourceManager.GetString("Warning_SymbolServerNotConfigured_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Package de symboles &apos;{0}&apos; trouvé, mais aucune clé API n&apos;a été spécifiée pour le serveur de symboles. Pour enregistrer une clé API, exécutez &apos;NuGet.exe setApiKey [votre clé API provenant de http://www.NuGet.org]&apos;..
        /// </summary>
        internal static string Warning_SymbolServerNotConfigured_fra {
            get {
                return ResourceManager.GetString("Warning_SymbolServerNotConfigured_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Trovato pacchetto simboli &apos;{0}&apos;, ma non è stata specificata alcuna API key per il symbol server. Per salvare una API Key, eseguire &apos;NuGet.exe setApiKey [ API key da http://www.NuGet.org]&apos;..
        /// </summary>
        internal static string Warning_SymbolServerNotConfigured_ita {
            get {
                return ResourceManager.GetString("Warning_SymbolServerNotConfigured_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to シンボル パッケージ &apos;{0}&apos; が見つかりましたが、シンボル サーバーの API キーが指定されていません。API キーを保存するには、&apos;NuGet.exe setApiKey [your API key from http://www.NuGet.org]&apos; を実行してください。.
        /// </summary>
        internal static string Warning_SymbolServerNotConfigured_jpn {
            get {
                return ResourceManager.GetString("Warning_SymbolServerNotConfigured_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 기호 패키지 &apos;{0}&apos;을(를) 찾았지만 기호 서버에 대한 API 키가 지정되지 않았습니다. API 키를 저장하려면 &apos;NuGet.exe setApiKey [http://www.NuGet.org의 API 키]&apos;를 실행하십시오..
        /// </summary>
        internal static string Warning_SymbolServerNotConfigured_kor {
            get {
                return ResourceManager.GetString("Warning_SymbolServerNotConfigured_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Odnaleziono pakiet symboli „{0}”, ale dla serwera symboli nie został określony klucz interfejsu API. Aby zapisać klucz interfejsu API, uruchom polecenie „NuGet.exe setApiKey [Twój klucz interfejsu API ze strony http://www.NuGet.org]”..
        /// </summary>
        internal static string Warning_SymbolServerNotConfigured_plk {
            get {
                return ResourceManager.GetString("Warning_SymbolServerNotConfigured_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to O pacote símbolos &apos;{0}&apos; foi localizado, mas nenhuma chave de API foi especificada para o servidor de símbolos. Para salvar uma chave de API,  execute &quot;NuGet.exe setApiKey [sua chave de API de http://www.NuGet.org]&quot;..
        /// </summary>
        internal static string Warning_SymbolServerNotConfigured_ptb {
            get {
                return ResourceManager.GetString("Warning_SymbolServerNotConfigured_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Обнаружен пакет символов &quot;{0}&quot;, но для сервера символов не был указан ключ API. Чтобы сохранить ключ API, выполните команду &quot;NuGet.exe setApiKey [ваш ключ API с сайта http://www.NuGet.org]&quot;..
        /// </summary>
        internal static string Warning_SymbolServerNotConfigured_rus {
            get {
                return ResourceManager.GetString("Warning_SymbolServerNotConfigured_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0}&apos; simge paketi bulundu ancak simge sunucusu için hiçbir API anahtarı belirtilmemiş. Bir API Anahtarını kaydetmek için &apos;NuGet.exe setApiKey [http://www.NuGet.org sayfasından aldığınız API anahtarı]&apos; öğesini çalıştırın..
        /// </summary>
        internal static string Warning_SymbolServerNotConfigured_trk {
            get {
                return ResourceManager.GetString("Warning_SymbolServerNotConfigured_trk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to The version of dependency &apos;{0}&apos; is not specified..
        /// </summary>
        internal static string Warning_UnspecifiedDependencyVersion {
            get {
                return ResourceManager.GetString("Warning_UnspecifiedDependencyVersion", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Specifiy the version of dependency and rebuild your package..
        /// </summary>
        internal static string Warning_UnspecifiedDependencyVersionSolution {
            get {
                return ResourceManager.GetString("Warning_UnspecifiedDependencyVersionSolution", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Specify version of dependencies..
        /// </summary>
        internal static string Warning_UnspecifiedDependencyVersionTitle {
            get {
                return ResourceManager.GetString("Warning_UnspecifiedDependencyVersionTitle", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} was not specified. Using &apos;{1}&apos;..
        /// </summary>
        internal static string Warning_UnspecifiedField {
            get {
                return ResourceManager.GetString("Warning_UnspecifiedField", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 未指定 {0}。正在使用“{1}”。.
        /// </summary>
        internal static string Warning_UnspecifiedField_chs {
            get {
                return ResourceManager.GetString("Warning_UnspecifiedField_chs", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to 並未指定 {0}。使用 &apos;{1}&apos;。.
        /// </summary>
        internal static string Warning_UnspecifiedField_cht {
            get {
                return ResourceManager.GetString("Warning_UnspecifiedField_cht", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Hodnota {0} nebyla zadána. Použije se {1}..
        /// </summary>
        internal static string Warning_UnspecifiedField_csy {
            get {
                return ResourceManager.GetString("Warning_UnspecifiedField_csy", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to &quot;{0}&quot; wurde nicht angegeben. &quot;{1}&quot; wird verwendet..
        /// </summary>
        internal static string Warning_UnspecifiedField_deu {
            get {
                return ResourceManager.GetString("Warning_UnspecifiedField_deu", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} no se ha especificado. Se está usando &apos;{1}&apos;..
        /// </summary>
        internal static string Warning_UnspecifiedField_esp {
            get {
                return ResourceManager.GetString("Warning_UnspecifiedField_esp", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} n&apos;a pas été spécifié. Utilisation de &apos;{1}&apos;..
        /// </summary>
        internal static string Warning_UnspecifiedField_fra {
            get {
                return ResourceManager.GetString("Warning_UnspecifiedField_fra", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} non specificato. Usare &apos;{1}&apos;..
        /// </summary>
        internal static string Warning_UnspecifiedField_ita {
            get {
                return ResourceManager.GetString("Warning_UnspecifiedField_ita", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} が指定されませんでした。&apos;{1}&apos; を使用しています。.
        /// </summary>
        internal static string Warning_UnspecifiedField_jpn {
            get {
                return ResourceManager.GetString("Warning_UnspecifiedField_jpn", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0}이(가) 지정되지 않았습니다. &apos;{1}&apos;을(를) 사용하고 있습니다..
        /// </summary>
        internal static string Warning_UnspecifiedField_kor {
            get {
                return ResourceManager.GetString("Warning_UnspecifiedField_kor", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Nie określono {0}. Zostanie użyta wartość „{1}”..
        /// </summary>
        internal static string Warning_UnspecifiedField_plk {
            get {
                return ResourceManager.GetString("Warning_UnspecifiedField_plk", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} não foi especificado. Usando &apos;{1}&apos;..
        /// </summary>
        internal static string Warning_UnspecifiedField_ptb {
            get {
                return ResourceManager.GetString("Warning_UnspecifiedField_ptb", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} не указан. Будет использован &quot;{1}&quot;..
        /// </summary>
        internal static string Warning_UnspecifiedField_rus {
            get {
                return ResourceManager.GetString("Warning_UnspecifiedField_rus", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to {0} belirtilmemiş. &apos;{1}&apos; kullanılıyor..
        /// </summary>
        internal static string Warning_UnspecifiedField_trk {
            get {
                return ResourceManager.GetString("Warning_UnspecifiedField_trk", resourceCulture);
            }
        }
    }
}
